/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// BitStream.h: interface for the CBitStream class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_)
#define AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class EEC_WritingLimitReached
{
public:
	EEC_WritingLimitReached() {};
};

class EEC_ReadingLimitReached
{
public:
	EEC_ReadingLimitReached() {};
};

class CBitStream  
{
public:
	CBitStream(CFile *stream, UINT op, int limit=-1);
	~CBitStream();
	
	enum { OP_READ = 1, OP_WRITE = 2 };

  // warning: Flush will alter the bitcount
  // if you would like to know the exact bitcount
  // you should call GetBitCount() before Flush()
  // the reason for that is that if other write operations follow
  // the GetBitCount should return the full size of the written stream
	void Flush();
  void Reset();

	int  GetBitCount() const { return bit_count; };
	int  GetBitLimit() const { return limit; };
  BYTE GetRemainingValue() const;
  BYTE GetBitPosition() const { return m_bit; };
  void SetBitPosition(BYTE pos);

	void ReadBit(BYTE*);
	void WriteBit(BYTE);

	void ReadBits(BYTE*, BYTE count);   // count <= 8
	void WriteBits(BYTE, BYTE count);   // count <= 8

	void ReadBits(WORD*, BYTE count);   // count <= 16
	void WriteBits(WORD, BYTE count);   // count <= 16

	void ReadBits(DWORD*, BYTE count);  // count <= 32
	void WriteBits(DWORD, BYTE count);  // count <= 32

	void ReadData(BYTE*);
	void WriteData(BYTE);

	void ReadData(WORD*);
	void WriteData(WORD);

	void ReadData(DWORD*);
	void WriteData(DWORD);

	int GetOp() const { return op; };

protected:
	int			op;
	CFile *	m_stream;
	BYTE		m_buffer;
	BYTE		m_bit;
	int			bit_count;
	int			limit;
	bool		limit_reached;
};

#endif // !defined(AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_)
