/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGGrid.h: interface for the CMGGrid class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MGGRID_H__355967AE_1BCF_40A2_9D1D_2E8BBC745DFC__INCLUDED_)
#define AFX_MGGRID_H__355967AE_1BCF_40A2_9D1D_2E8BBC745DFC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasicTypes.h"
#include "MGErrMsg.h"

typedef CFPoint3DT *			GridU;
typedef CFPoint3DT **		  GridUV;
typedef CFPoint3DT ***	  GridUVW;

enum { DIR_U, DIR_V, DIR_W };

enum {
  MIN_SPLIT_SIZE = 9,     // minimum cube size that can be further split
  MIN_DEC_SIZE = 3        // minimum cube size that consists of several subbands
};

class CMGStreamParser;

////////////////////////////////////////////////////////////////////////
//class CMGGrid
////////////////////////////////////////////////////////////////////////

class CMGGrid  
{
public:
	CMGGrid();
	virtual ~CMGGrid();

	struct Pos2D { int i, v; };

  enum
  {
    LINEAR_FILTER = 0,
    SMOOTH_FILTER = 1,
  };

	// Implementation
public:
	bool		Init(CFPoint3DT *pts, bool defaultInit, const CPointUVW &nrSlices, const CPointUVW &nrLevels, CString &error);
	bool		Init(CFPoint3DT *pts, bool defaultInit, CMGStreamParser *parser, CString &error);
  int     GetNrSlicesDir(int level, int dir);
  int     GetNrLevelsDir(int dir);
  void    ComputeLevelSlices(BYTE *levelSlices, int dir);
	void		ComputeNrSlicesLevel(const CPointUVW &nrLevels);
	void		FillSlicesLevel(const CPointUVW &nrLevels);
	void		DeleteContent();

	void		ExportCorners(CStdioFile &file);
	void		Export(CStdioFile &file, int level);

	void		InterpolBox();

  bool    SetFilterType(int filterType);
  int     GetFilterType() { return m_filterType; };
  bool    SetHierarchicalMode(int hierarchicalLevel);
  bool    InitDetails();
  bool    InitBackup();
  void    DeleteDetails();
  void    DeleteBackup();
  void    BackupGrid();
  void    ComputeHomogeneousSteps(CPointUVW &step, int baseLevel);
  void    ComputeHomogeneousSteps(CPointUVW &step, CPointUVW &step_2, CPointUVW &levelDiff, int baseLevel);
  void    ComputeDetails();
  void    ComputeDetailsSmooth(int currentLevel);
  void    ComputeDetailsLinear(int currentLevel);
  bool    CheckModifiedLevel(int level);
  void    Distribute(int baseLevel);
  void    Interpolate(int baseLevel);
  void    InterpolateWithDetail(int baseLevel);
  void    InterpolateWithDetailBackup();
  void    InterpolateSmooth(int currentLevel);
  void    InterpolateLinear(int currentLevel);
  void    InterpolateSmoothWithDetail(int currentLevel);
  void    InterpolateLinearWithDetail(int currentLevel);
  void    InterpolateSmoothWithDetailBackup(int level);
  void    InterpolateLinearWithDetailBackup(int level);

	bool		IsHomogenious(int val) const;

  static int  ComputeNrSlicesDown(int sz, int levelDiff);
  static void ComputeNrSlicesDown(CPointUVW &sz, int levelDiff);
  static void ComputeNrSlicesDown(CPointUVW &sz, const CPointUVW &levelDiff);
  static int  ComputePositionUp(int pos, int level);
  static int  ComputeDimensionUp(int sz, int levelDiff);
  static void ComputeNrSlicesLevel(int level, CPointUVW nrLevels, CPointUVW &nrSlices);

  void    ComputePositionUp(CPointUVW &pos, int level, const CPointUVW &levelDiff);
  void    ComputeLevelDiffDown(CPointUVW &levelDiff, int level);
  int     ComputePositionDown(int pos, int levelDiff);
  int     ComputeDimensionDown(int sz, int levelDiff);
  void    ComputePositionDown(CPointUVW &sz, const CPointUVW &levelDiff);
  void    ComputeDimensionDown(CPointUVW &sz, const CPointUVW &levelDiff);

	BYTE	* GetLevelU() const { return m_levelU; };
	BYTE	* GetLevelV() const { return m_levelV; };
	BYTE	* GetLevelW() const { return m_levelW; };

	BYTE		GetLevelU(int idx) const { return m_levelU[idx]; };
	BYTE		GetLevelV(int idx) const { return m_levelV[idx]; };
	BYTE		GetLevelW(int idx) const { return m_levelW[idx]; };

	CPointUVW * GetNrSlices() const { return m_nrSlices; };
	const CPointUVW & GetMaxNrSlices() const { return m_maxNrSlices; };
	const CPointUVW & GetNrSlices(int level) const { return m_nrSlices[level]; };

	int			GetNrSlicesU() const { return m_maxNrSlices.u; };
	int			GetNrSlicesV() const { return m_maxNrSlices.v; };
	int			GetNrSlicesW() const { return m_maxNrSlices.w; };

	int			GetNrSlicesU(int level) const { return m_nrSlices[level].u; };
	int			GetNrSlicesV(int level) const { return m_nrSlices[level].v; };
	int			GetNrSlicesW(int level) const { return m_nrSlices[level].w; };

	int			GetPrevSliceU(int u, BYTE level, bool isCyclic = false) const;
	int			GetNextSliceU(int u, BYTE level, bool isCyclic = false) const;
	int			GetPrevSliceV(int v, BYTE level, bool isCyclic = false) const;
	int			GetNextSliceV(int v, BYTE level, bool isCyclic = false) const;
	int			GetPrevSliceW(int w, BYTE level, bool isCyclic = false) const;
	int			GetNextSliceW(int w, BYTE level, bool isCyclic = false) const;

	void		SetMaxNrLevels(int level) { ASSERT(level >= 1); m_maxNrLevels = level; };
  int     GetMaxNrLevels() const { return m_maxNrLevels; };
	const CPointUVW & GetNrLevelsUVW() const { return m_nrLevels; };

	GridUV  &operator [](int w) { return m_gridUVW[w]; };
  GridUVW &GetGridPoints() { return m_gridUVW; };

  void    Dump(CStdioFile &file);

  int     GetCyclicMode() const { return m_cyclicMode; };
  void    SetCyclicMode(int mode) { m_cyclicMode = mode; };

	// Implementation
protected:
	CMGErrMsg			m_errorMsg;						// initilalise the error messages

  int           m_cyclicMode;         // specifies the cyclic behaviour of the mesh 
  int           m_filterType;         // type of the filter used for the interpolation
	int						m_maxNrLevels;				// number resolution levels of the mesh-grid
	CPointUVW		  m_nrLevels;						// number resolution levels for each u,v,w direction
  CPointUVW     m_maxNrSlices;        // number of slices corresponding to the last resolution level
	CPointUVW 	* m_nrSlices;           // number of slices for each resolution level

	BYTE				* m_levelU;							// level of the slicing planes
	BYTE				* m_levelV;							// level of the slicing planes
	BYTE				* m_levelW;							// level of the slicing planes
	GridUVW				m_gridUVW;            // the grid points
	GridUVW				m_detailUVW;          // computed details of the grid points
	GridUVW				m_backupUVW;          // backup of the grid points
};

inline bool CMGGrid::	IsHomogenious(int val) const
{
	ASSERT(val > 0);
	int level = 0, tmp = val;
	while (tmp > 1) { tmp >>= 1;		++level; }

	return (val == (1 << level) + 1);
}

inline int CMGGrid::GetPrevSliceU(int u, BYTE level, bool isCyclic) const
{
	if (u == 0)
  {
    if (isCyclic)
      return GetPrevSliceU(GetNrSlicesU() - 1, level);  // skip last
    else
      return GetNrSlicesU() - 1;                        // last
  }

	while (m_levelU[--u] > level);
	ASSERT(u >= 0);
	
	return u;
}

inline int CMGGrid::GetNextSliceU(int u, BYTE level, bool isCyclic) const
{
  if (isCyclic)
  {
    int uprev = GetPrevSliceU(GetNrSlicesU() - 1 , level);
	  if (u == uprev)  return 0;                    // first
  }
  else
  {
	  if (u == GetNrSlicesU() - 1)  return 0;       // first
  }

	while (m_levelU[++u] > level);

#ifdef _DEBUG
  if (isCyclic)
	  ASSERT(u < GetNrSlicesU() - 1);
  else
	  ASSERT(u < GetNrSlicesU());
#endif

	return u;
}

inline int CMGGrid::GetPrevSliceV(int v, BYTE level, bool isCyclic) const
{
	if (v == 0)
  {
    if (isCyclic)
      return GetPrevSliceV(GetNrSlicesV() - 1, level);  // skip last
    else
      return GetNrSlicesV() - 1;                        // last
  }

	while (m_levelV[--v] > level);
	ASSERT(v >= 0);
	
	return v;
}

inline int CMGGrid::GetNextSliceV(int v, BYTE level, bool isCyclic) const
{
  if (isCyclic)
  {
    int vprev = GetPrevSliceV(GetNrSlicesV() - 1, level);
	  if (v == vprev)  return 0;                    // first
  }
  else
  {
	  if (v == GetNrSlicesV() - 1)  return 0;       // first
  }

	while (m_levelV[++v] > level);

#ifdef _DEBUG
  if (isCyclic)
	  ASSERT(v < GetNrSlicesV() - 1);
  else
	  ASSERT(v < GetNrSlicesV());
#endif

	return v;
}

inline int CMGGrid::GetPrevSliceW(int w, BYTE level, bool isCyclic) const
{
	if (w == 0)
  {
    if (isCyclic)
      return GetPrevSliceW(GetNrSlicesW() - 1, level);  // skip last
    else
      return GetNrSlicesW() - 1;                        // last
  }

	while (m_levelW[--w] > level);
	ASSERT(w >= 0);
	
	return w;
}

inline int CMGGrid::GetNextSliceW(int w, BYTE level, bool isCyclic) const
{
  if (isCyclic)
  {
    int wprev = GetPrevSliceW(GetNrSlicesW() - 1, level);
	  if (w == wprev)  return 0;                    // first
  }
  else
  {
	  if (w == GetNrSlicesW() - 1)  return 0;       // first
  }

	while (m_levelW[++w] > level);

#ifdef _DEBUG
  if (isCyclic)
	  ASSERT(w < GetNrSlicesW() - 1);
  else
	  ASSERT(w < GetNrSlicesW());
#endif

	return w;
}

inline void CMGGrid::ComputeNrSlicesDown(CPointUVW &sz, int levelDiff)
{
  sz.u = ComputeNrSlicesDown(sz.u, levelDiff);
  sz.v = ComputeNrSlicesDown(sz.v, levelDiff);
  sz.w = ComputeNrSlicesDown(sz.w, levelDiff);
}

inline void CMGGrid::ComputeNrSlicesDown(CPointUVW &sz, const CPointUVW &levelDiff)
{
  sz.u = ComputeNrSlicesDown(sz.u, levelDiff.u);
  sz.v = ComputeNrSlicesDown(sz.v, levelDiff.v);
  sz.w = ComputeNrSlicesDown(sz.w, levelDiff.w);
}

inline int CMGGrid::ComputeNrSlicesDown(int sz, int levelDiff)
{
  while (levelDiff-- > 0 && sz >= MIN_DEC_SIZE)
    sz = (sz + 1) >> 1;

  return sz;
}

inline void CMGGrid::ComputeLevelDiffDown(CPointUVW &levelDiff, int level)
{
  int diff = m_maxNrLevels - 1 - level;
  levelDiff.u = min(m_nrLevels.u - 1, diff);
  levelDiff.v = min(m_nrLevels.v - 1, diff);
  levelDiff.w = min(m_nrLevels.w - 1, diff);
}

inline int CMGGrid::ComputePositionUp(int pos, int levelDiff)
{
  return pos << levelDiff;
}

inline void CMGGrid::ComputePositionDown(CPointUVW &sz, const CPointUVW &levelDiff)
{
  sz.u = ComputePositionDown(sz.u, levelDiff.u);
  sz.v = ComputePositionDown(sz.v, levelDiff.v);
  sz.w = ComputePositionDown(sz.w, levelDiff.w);
}

inline int CMGGrid::ComputePositionDown(int pos, int levelDiff)
{
  while (levelDiff-- > 0) pos = pos >> 1;

  return pos;
}

inline void CMGGrid::ComputePositionUp(CPointUVW &pos, int level, const CPointUVW &levelDiff)
{
  if (pos.u == GetNrSlicesU(level) - 1)
    pos.u = GetNrSlicesU() - 1;
  else
    pos.u = ComputePositionUp(pos.u, levelDiff.u);

  if (pos.v == GetNrSlicesV(level) - 1)
    pos.v = GetNrSlicesV() - 1;
  else
    pos.v = ComputePositionUp(pos.v, levelDiff.v);

  if (pos.w == GetNrSlicesW(level) - 1)
    pos.w = GetNrSlicesW() - 1;
  else
    pos.w = ComputePositionUp(pos.w, levelDiff.w);
}

inline int CMGGrid::ComputeDimensionUp(int sz, int levelDiff)
{
  return ((sz - 1) << levelDiff) + 1;
}

inline void CMGGrid::ComputeDimensionDown(CPointUVW &sz, const CPointUVW &levelDiff)
{
  sz.u = ComputeDimensionDown(sz.u, levelDiff.u);
  sz.v = ComputeDimensionDown(sz.v, levelDiff.v);
  sz.w = ComputeDimensionDown(sz.w, levelDiff.w);
}

inline int CMGGrid::ComputeDimensionDown(int sz, int levelDiff)
{
  while (levelDiff-- > 0) sz = (sz + 1) >> 1;

  return sz;
}

#endif // !defined(AFX_MGGRID_H__355967AE_1BCF_40A2_9D1D_2E8BBC745DFC__INCLUDED_)
