/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGStreamDescription.h: interface for the CMGStreamDescription class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_)
#define AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasicTypes.h"
#include "BitStream.h"

class RGCoding
{
public:
  RGCoding();

public:
  int          gridCorners[8];     // the 8 corners of the grid
  int          scale;              // the scaling for the coded coefficients
};

class MStreamLevel
{
public:
  MStreamLevel();

  int   GetHeaderSize() const { return m_headerSz; };                                               // retrieve the size of the stream header
  void  SetHeaderSize(int size) { m_headerSz = size; };                                             // set the size of the stream header

  int   GetHeaderPosition() const { return m_headerPos; };                                          // retrieve the position of the header in the stream
  void  SetHeaderPosition(int pos) { m_headerPos = pos; };                                          // set the position of the header in the stream

  const CPointUVW &GetNrSlices() const { return m_nrSlices; };
  void  SetNrSlices(const CPointUVW &nrSlices) { m_nrSlices = nrSlices; };

  int   Init(const CPointUVW & nrROIs);                                                             // set the number of ROIs and allocate the arrays elements for each level
  const CPointUVW &GetNrROIs() const { return m_nrROIs; };                                          // retrieve the number of ROIs for a level
  int   GetTotalNrROIs() const { return m_nrROIs.u * m_nrROIs.v * m_nrROIs.w; };                    // retrieve the total number of ROIs for a level

  int   GetRepositionBitsMode() const { return m_reposMode; };                                      // retrieve the vertices' reposition mode
  void  SetRepositionBitsMode(int mode) { m_reposMode = mode; };                                    // set the vertices' reposition mode

  const CPointUVW & GetROISize() const { return m_szROI; };                                         // retrieve the size of the ROI
  void  SetROISize(const CPointUVW &szROI, const CPointUVW &nrBits);                                // set the size of the ROI

  int   GetNrBitsROIField() const { return m_nrBitsROIField; };                                     // retrieve the number of bits allocated for the ROI field
  void  SetNrBitsROIField(BYTE nrBits) { m_nrBitsROIField = nrBits; };                              // set the number of bits allocated for the ROI field

  int   GetNrBitsROIIndex() const { return m_nrBitsROIIndex; };                                     // retrieve the number of bits allocated for the ROI index
  void  SetNrBitsROIIndex(BYTE nrBits) { m_nrBitsROIIndex = nrBits; };                              // set the number of bits allocated for the ROI index

  int   GetCodingLookup(int roiIdx) const { return m_codingLookup[roiIdx]; };                       // retrieve the coding lookup for each ROI: false means no coding info
  void  SetCodingLookup(int roiIdx, bool flag) { m_codingLookup[roiIdx] = flag; };                  // set the coding lookup for each ROI: false means no coding info
  CBoolArray & GetCodingLookup()  { return m_codingLookup; };                                       // retrieve the coding lookup for each ROI: false means no coding info

  const CPointUVW & GetNrBitsVertPosField() const { return m_nrBitsVertPos;  };                     // retrieve the nr of bits per counter holding the size of the ROI (x, y, z)
  void  SetNrBitsVertPosField(const CPointUVW &nrBits) { m_nrBitsVertPos = nrBits; };               // set the nr of bits per counter holding the size of the ROI (x, y, z)
  
  int   GetNrBitsConnectivityCounterField() const { return m_nrBitsConnetivityCounter; };           // retrieve the number of bits allocated for the connectivity counter field
  void  SetNrBitsConnectivityCounterField(BYTE nrBits) { m_nrBitsConnetivityCounter = nrBits; };    // set the number of bits allocated for the connectivity counter field
  
  int   GetNrBitsVertsReposCounterField() const { return m_nrBitsVertsReposCounter; };              // retrieve the number of bits allocated for the vertex reposition counter field
  void  SetNrBitsVertsReposCounterField(BYTE nrBits) { m_nrBitsVertsReposCounter = nrBits; };       // set the number of bits allocated for the vertex reposition counter field
  
  int   GetNrBitsVertsRefineCounterField() const { return m_nrBitsVertsRefineCounter; };            // retrieve the number of bits allocated for the vertex refinement counter field
  void  SetNrBitsVertsRefineCounterField(BYTE nrBits) { m_nrBitsVertsRefineCounter = nrBits; };     // set the number of bits allocated for the vertex refinement counter field

  int   GetNrBytesConnectivityCoding(int roiIdx) const { return m_connectivityCount[roiIdx]; };         // retrieve the number of vertices from the ROI
  void  SetNrBytesConnectivityCoding(int roiIdx, int count) { m_connectivityCount[roiIdx] = count; };   // set the number of vertices from the ROI
  CIntArray & GetNrBytesConnectivityCoding() { return m_connectivityCount; };                           // retrieve the number of vertices from the ROI

  int   GetNrBytesVertsReposCoding(int roiIdx) const { return m_vertsReposCount[roiIdx]; };             // retrieve the number of vertices from the ROI
  void  SetNrBytesVertsReposCoding(int roiIdx, int count) { m_vertsReposCount[roiIdx] = count; };       // set the number of vertices from the ROI
  CIntArray & GetNrBytesVertsReposCoding() { return m_vertsReposCount; };                               // retrieve the number of vertices from the ROI

  int   GetNrBytesVertsRefineCoding(int roiIdx) const { return m_vertsRefineCount[roiIdx]; };           // retrieve the number of vertices from the ROI
  void  SetNrBytesVertsRefineCoding(int roiIdx, int count) { m_vertsRefineCount[roiIdx] = count; };     // set the number of vertices from the ROI
  CIntArray & GetNrBytesVertsRefineCoding() { return m_vertsRefineCount; };                             // retrieve the number of vertices from the ROI

  int   GetPosStreamConnectivity(int roiIdx) const { return m_posStreamConnectivity[roiIdx]; };     // retrieve the stream position of the mesh connectivity bits
  void  SetPosStreamConnectivity(int roiIdx, int pos) { m_posStreamConnectivity[roiIdx] = pos; };   // set the stream position of the mesh connectivity bits
  CIntArray & GetPosStreamConnectivity() { return m_posStreamConnectivity; };                       // retrieve the stream position of the mesh connectivity bits

  int   GetPosStreamRefine(int roiIdx) const { return m_posStreamRefine[roiIdx]; };                 // retrieve the stream position of the vertices' refine bits
  void  SetPosStreamRefine(int roiIdx, int pos) { m_posStreamRefine[roiIdx] = pos; };               // set the stream position of the vertices' refine bits
  CIntArray & GetPosStreamRefine() { return m_posStreamRefine; };                                   // retrieve the stream position of the vertices' refine bits 

  int   GetPosStreamReposition(int roiIdx) const { return m_posStreamReposition[roiIdx]; };         // retrieve the stream position of the vertices' reposition bits
  void  SetPosStreamReposition(int roiIdx, int pos) { m_posStreamReposition[roiIdx] = pos; };       // set the stream position of the vertices' reposition bits
  CIntArray & GetPosStreamReposition() { return m_posStreamReposition; };                           // retrieve the stream position of the vertices' reposition bits 

  void  DumpStream(CStringArray &msgs);                                                             // dump the description about the MeshGrid stream

  bool  IsDoneROIConnectivity(int roiIdx) const { return m_cacheROI[roiIdx]; };                     // check whether the ROI connectivity has been recovered
  bool  IsDoneROIRefine(int roiIdx) const { return m_cacheRefine[roiIdx]; };                        // check whether the ROI Refine bits have been recovered
  bool  IsDoneROIReposition(int roiIdx) const { return m_cacheReposition[roiIdx]; };                // check whether the ROI Reposition bits have been recovered
  void  DoneROIConnectivity(int roiIdx, bool flag) { m_cacheROI[roiIdx] = flag; };                  // set the recovered flag for the ROI connectivity
  void  DoneROIRefine(int roiIdx, bool flag) { m_cacheRefine[roiIdx] = flag; };                     // set the recovered flag for the ROI Refine bits
  void  DoneROIReposition(int roiIdx, bool flag) { m_cacheReposition[roiIdx] = flag; };             // set the recovered flag for the ROI Reposition bits

  bool  IsRequested() const { return m_bRequested; };                                                 
  bool  IsDecoded() const { return m_bDecoded; };
  void  SetRequested() { m_bRequested = true; };                                                 
  void  SetDecoded() { m_bDecoded = true; };

protected:
  int   AddMsg(LPCTSTR lpszFormat, ...);                                                            // add a message to the message list
  int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };                                    // add a message to the message list
  void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                                              // delete a message from the list

public:
  int          m_headerSz;         // size of the stream header
  int          m_headerPos;        // position of the header in the stream
  CPointUVW    m_szROI;            // size of the ROI when LOD enabled, per level
  CPointUVW    m_nrROIs;           // nr of ROIs per level
  CPointUVW    m_nrSlices;         // nr of slices corresponding to this level
  int          m_reposMode;        // specifies the default resposition mode for vertices

protected:
  bool           m_bRequested;                // flag specifying that the corresponding information has been requested to the server
  bool           m_bDecoded;                  // flag specifying that the corresponding information has been decoded
  int            m_nrBitsROIField;            // nr of bits per counter holding the size of the ROI (x, y, z)
  int            m_nrBitsROIIndex;            // nr of bits per counter holding the index of the ROI
  CPointUVW      m_nrBitsVertPos;             // nr of bits per variable holding the position of a starting vertex
  int            m_nrBitsConnetivityCounter;  // nr of bits per counter holding the size of the coded connectivity
  int            m_nrBitsVertsReposCounter;   // nr of bits per counter holding the size of the coded vertices reposition bits
  int            m_nrBitsVertsRefineCounter;  // nr of bits per counter holding the size of the coded vertices refinement bits
  CBoolArray     m_codingLookup;              // lookup holding the coding status of ROIs: false means no coding bits
  CIntArray      m_connectivityCount;         // counter holding the size of the coded mesh connectivity info, per roi
  CIntArray      m_vertsReposCount;           // counter holding the size of the coded vertices reposition info, per roi
  CIntArray      m_vertsRefineCount;          // counter holding the size of the coded vertices refine info, per roi
  CIntArray      m_posStreamConnectivity;     // position in the stream of the mesh connectivity bits for each ROI
  CIntArray      m_posStreamRefine;           // position in the stream of the vertices' refinement bits for each ROI
  CIntArray      m_posStreamReposition;       // position in the stream of the vertices' reposition bits for each ROI
  CStringArray * m_msgs;                      // used for dumping the stream
  CBoolArray     m_cacheROI;                  // mark the ROIs (vertices) that have been decoded (vertices)
  CBoolArray     m_cacheRefine;               // mark the ROIs (vertices) that have been refined
  CBoolArray     m_cacheReposition;           // mark the ROIs (vertices) that have been repositioned
};

class GStreamLevel
{
public:
  GStreamLevel();

  int   GetHeaderSize() const { return m_headerSz; };                                                       // retrieve the size of the stream header
  void  SetHeaderSize(int size) { m_headerSz = size; };                                                     // set the size of the stream header

  int   GetHeaderPosition() const { return m_headerPos; };                                                  // retrieve the position of the header in the stream
  void  SetHeaderPosition(int pos) { m_headerPos = pos; };                                                  // set the position of the header in the stream

  int   InitTiles(const CPointUVW &nrTiles, int initValue);                                                // allocate the arrays elements for each level
  const CPointUVW &GetNrTiles() const { return m_nrTiles; };                                               // retrieve the number of Tiles for a level
  int   GetTotalNrTiles() const { return m_nrTiles.u * m_nrTiles.v * m_nrTiles.w; };                        // retrieve the total number of Tiles for a level
  void  SetNrTiles(const CPointUVW &val) { m_nrTiles = val; };                                             // set the number of Tiles
  void  SetNrTiles(int val) { m_nrTiles = val; };                                                           // set the number of Tiles

  bool  CanSplit() const { return m_bSplit; };                                                              // retrieve the split-into-subbands flag
  void  SetSplit(bool flag) { m_bSplit = flag; };                                                           // set the split-into-subbands flag

  int   GetNrBitsTileField() const { return m_nrBitsFieldTile; };                                           // retrieve the number of bits allocated for the Tile field
  void  SetNrBitsTileField(BYTE nrBits) { m_nrBitsFieldTile = nrBits; };                                    // set the number of bits allocated for the Tile field

  int   GetNrBitsGridCounterField() const { return m_nrBitsGridCounter; };                                  // retrieve the number of bits allocated for the grid counter field
  void  SetNrBitsGridCounterField(BYTE nrBits) { m_nrBitsGridCounter = nrBits; };                           // set the number of bits allocated for the grid counter field
  
  int   GetNrBytesTileCoding(int dir, int tileIdx) const { return m_tilesCount[dir][tileIdx]; };            // retrieve the number of bits allocated for the grid counter field
  void  SetNrBytesTileCoding(int dir, int tileIdx, int nrBytes) { m_tilesCount[dir][tileIdx] = nrBytes; };  // set the number of bits allocated for the grid counter field
  CIntArray & GetNrBytesTileCoding(int dir) { return m_tilesCount[dir]; };                                  // retrieve the number of bits allocated for the grid counter field

  int   GetPosStreamTile(int dir, int tileIdx) const { return m_posStreamTile[dir][tileIdx]; };             // retrieve the stream position of the mesh coding
  void  SetPosStreamTile(int dir, int tileIdx, int pos) { m_posStreamTile[dir][tileIdx] = pos; };           // set the stream position of the mesh coding
  CIntArray & GetPosStreamTile(int dir) { return m_posStreamTile[dir]; };                                   // retrieve the stream position of the mesh coding

  int   GetThreshold(int dir) const { return m_threshold[dir]; };                                           // retrieve the threshold for the specified dir
  int  *GetThreshold() { return m_threshold; };                                                             // retrieve the thresholds
  void  SetThreshold(int dir, int threshold) { m_threshold[dir] = threshold; };                             // set the threshold for the specified dir

  void  DumpStream(CStringArray &msgs);                                                                     // dump the description about the MeshGrid stream

  bool  IsTileStreamed(int dir, int tileIdx) const { return m_cacheTile[dir][tileIdx]; };                   // check whether the Tile has been streamed/recovered to/from the stream
  void  SetStreamedTile(int dir, int tileIdx, bool flag) { m_cacheTile[dir][tileIdx] = flag; };             // set the streamed/recovered flag for the Tile

  bool  IsRequested() const { return m_bRequested; };                                                 
  bool  IsDecoded() const { return m_bDecoded; };
  void  SetRequested() { m_bRequested = true; };                                                 
  void  SetDecoded() { m_bDecoded = true; };

protected:
  int   AddMsg(LPCTSTR lpszFormat, ...);                                                                    // add a message to the message list
  int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };                                            // add a message to the message list
  void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                                                      // delete a message from the list

public:
  int          m_headerSz;              // size of the stream header
  int          m_headerPos;             // position of the header in the stream
  bool         m_bSplit;                // flag = true if grid size is large enough to be split into subbands
  CPointUVW    m_nrTiles;               // nr of tiles per sublevel
  int          m_threshold[3];          // the threshold for each direction

protected:
  bool           m_bRequested;            // flag specifying that the corresponding information has been requested to the server
  bool           m_bDecoded;              // flag specifying that the corresponding information has been decoded
  int            m_nrBitsFieldTile;       // nr of bits per counter holding the size of the Tile (x, y, z)
  int            m_nrBitsGridCounter;     // nr of bits per counter holding the size of the coded grid
	CIntArray      m_tilesCount[3];         // size in bits of each of the coded tiles
  CIntArray      m_posStreamTile[3];      // position in the stream of the coded tiles
  CStringArray * m_msgs;                  // used for dumping the stream
  CBoolArray     m_cacheTile[3];          // cache the coded/decoded tiles to/from the stream
};

typedef CDataArray<MStreamLevel, MStreamLevel&> MStreamLevels;
typedef CDataArray<GStreamLevel, GStreamLevel&> GStreamLevels;

class MGStreamDescr
{
public:
  MGStreamDescr();

  int   GetHeaderSize() const { return m_headerSz; };                         // retrieve the size of the stream header
  void  SetHeaderSize(int size) { m_headerSz = size; };                       // set the size of the stream header

  BYTE  GetMaxNrLevelsMesh() const { return m_maxNrLevelsM; };                // retrieve the max number of levels;
  const CPointUVW &GetNrLevelsMesh() const { return m_nrLevelsM; };          // retrieve the number of levels;
  void  InitMeshLevels(const CPointUVW &nrLevels);                           // set the nr of levels of the mesh and initialise the levels objects

  BYTE  GetMaxNrLevelsGrid() const { return m_maxNrLevelsG; };                // retrieve the max number of levels of the grid
  void  InitGridLevels(BYTE nrLevels);                                        // set the max nr of levels of the grid and initialise the levels objects

  int   GetStreamSize() const { return m_streamSize; };                       // retrieve the size of the stream
  void  SetStreamSize(int size) { m_streamSize = size; };                     // set the size of the stream

  const CPointUVW & GetMaxNrSlices() const { return m_maxNrSlices; };         // retrieve the max number of slices
  void  SetMaxNrSlices(const CPointUVW &nrSlices);                            // set the max number of slices

  bool  IsSingleLayered() const { return m_bSingleLayered; };                 // retrieve whether the number of slices is 1
  void  SetSingleLayered(bool flag) { m_bSingleLayered = flag; };             // set whether the number of slices is 1

  bool  HasConnectivityInfo() const { return m_bHasConnectivityInfo; };       // retrieve the connectivity availability flag
  void  HasConnectivityInfo(bool flag) { m_bHasConnectivityInfo = flag; };    // set the connectivity availability flag

  bool  HasRefineInfo() const { return m_bHasRefineInfo; };                   // retrieve the refine availability flag
  void  HasRefineInfo(bool flag) { m_bHasRefineInfo = flag; };                // set the refine availability flag

  bool  HasRepositionInfo() const { return m_bHasRepositionInfo; };           // retrieve the reposition availability flag
  void  HasRepositionInfo(bool flag) { m_bHasRepositionInfo = flag; };        // set the reposition availability flag

  bool  HasGridInfo() const { return m_bHasGridInfo; };                       // retrieve the grid availability flag
  void  HasGridInfo(bool flag) { m_bHasGridInfo = flag; };                    // set the grid availability flag

  bool  HasNormalsInfo() const { return m_bHasNormalsInfo; };                 // retrieve the normals availability flag
  void  HasNormalsInfo(bool flag) { m_bHasNormalsInfo = flag; };              // set the normals availability flag

  bool  IsConnectivityInfoAvailable(int level) const;                         // retrieve whether the connectivity links are / have to be stored in the stream
  bool  IsRefinementInfoAvailable(int level) const;                           // retrieve whether the refine bits are / have to be stored in the stream
  bool  IsRepositionInfoAvailable(int level) const;                           // retrieve whether the reposition bits are / have to be stored in the stream
  bool  IsGridInfoAvailable(int lavel) const;                                 // retrieve whether the grid coefficients are / have to be stored in the stream

  BYTE  GetNrRefineBitsVertex() const { return m_nrRefBitsVert; };            // retrieve the number of refine bits for the vertices
  void  SetNrRefineBitsVertex(BYTE nrBits) { m_nrRefBitsVert = nrBits; };     // set the number of refine bits for the vertices

  bool  InterleaveRefine() const { return m_bInterleaveRefine; };             // retrieve the interleave vertices refine bits flag
  void  SetInterleaveRefine(bool flag) { m_bInterleaveRefine = flag; };       // set the interleave vertices refine bits flag

  int   GetMeshType() const { return m_meshType; };                           // retrieve the type of mesh
  void  SetMeshType(int type) { m_meshType = type; };                         // set the type of mesh

  bool  IsSameConnectivity() const { return m_bSameConnect; };                // retrieve the same connectivity flag
  void  SetSameConnectivity(bool bSameConnect) { m_bSameConnect = bSameConnect; }; // set the same connectivity flag

  bool  IsAutoResolution() const { return m_bAutoResolution; }                // retrieve the auto resolution mode
  void  SetAutoResolution(bool flag) { m_bAutoResolution = flag; };           // set the auto resolution mode

  int   GetCyclicMode() const { return m_cyclicMode; }                        // retrieve the cyclic mode
  void  SetCyclicMode(int mode) { m_cyclicMode = mode; };                     // set the auto cyclic mode

  int   GetOffsetAmplitude() const { return m_offsetAmp; };                   // retrieve the amplitude of the offset
  int   SetOffsetAmplitude(int offsetAmp) { return m_offsetAmp = offsetAmp; };// retrieve the amplitude of the offset

  int   GetFilterType() const { return m_filterType; };                       // retrieve the filter type
  void  SetFilterType(int filterType) { m_filterType = filterType; };         // set the filter type

  int   GetNrQuantBitsGrid() const { return m_nrQuantBits; };                 // retrieve the number of quantization bits
  void  SetNrQuantBitsGrid(int nrBits) { m_nrQuantBits = nrBits; };           // retrieve the number of quantization bits

  RGCoding * GetGridCodingParam() { return m_gridCoding; };                   // retrieve the coding parameters for the grid
  RGCoding & GetGridCodingParam(int dir) { return m_gridCoding[dir]; };       // retrieve the coding parameters for the grid

  MStreamLevel &GetMeshLevel(int level) { return m_streamLevelM[level]; };    // retrieve a resolution level from the mesh coding
  MStreamLevels &GetMeshLevel() { return m_streamLevelM; };                   // retrieve the resolution levels from the mesh coding

  GStreamLevel &GetGridLevel(int level) { return m_streamLevelG[level]; };    // retrieve a resolution level from the grid coding
  GStreamLevels &GetGridLevel() { return m_streamLevelG; };                   // retrieve the resolution levels from the grid coding

  int   GetNrBitsSlicesField() const { return m_nrBitsFieldSlices; };         // retrieve the number of bits allocated for the slices field
  void  SetNrBitsSlicesField(BYTE nrBits) { m_nrBitsFieldSlices = nrBits; };  // set the number of bits allocated for the slices field

  int   GetNrBitsROIField() const { return m_nrBitsFieldROI; };               // retrieve the number of bits allocated for the ROI field
  void  SetNrBitsROIField(BYTE nrBits) { m_nrBitsFieldROI = nrBits; };        // set the number of bits allocated for the ROI field

  int   GetNrBitsTileField() const { return m_nrBitsFieldROI; };              // retrieve the number of bits allocated for the Tile field
  void  SetNrBitsTileField(BYTE nrBits) { m_nrBitsFieldTile = nrBits; };       // set the number of bits allocated for the Tile field

  int   GetNrBitsGridParam() const { return m_nrBitsFieldGrid; };             // retrieve the number of bits allocated for the grid param field
  void  SetNrBitsGridParam(BYTE nrBits) { m_nrBitsFieldGrid = nrBits; };      // set the number of bits allocated for the grid param field

  void  SetThresholds(int dir, const CIntArray &thresholds);                  // set the thresholds for the grid resolution levels

  void  DumpStream(CStringArray &msgs);                                       // dump the description about the MeshGrid stream

protected:
  int   AddMsg(LPCTSTR lpszFormat, ...);                                      // add a message to the message list
  int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };               // add a message to the message list
  void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                         // delete a message from the list

public:
  int            m_streamSize;          // the number of parts that make the stream
  int            m_headerSz;            // size of the stream header
  BYTE           m_maxNrLevelsM;        // max number of resolution levels of the mesh
  BYTE           m_maxNrLevelsG;        // max number of resolution levels of the grid
  CPointUVW	     m_nrLevelsM;				    // number of resolution levels of the mesh
  CPointUVW	     m_maxNrSlices;				  // max nr of slices of the mesh grid
  bool           m_bSingleLayered;      // whether the number of slices is 1

  bool           m_bHasConnectivityInfo;// whether the connectivity info is available in the stream
  bool           m_bHasRefineInfo;      // whether the refine info is available in the stream
  bool           m_bHasRepositionInfo;  // whether the reposition info is available in the stream
  bool           m_bHasGridInfo;        // whether the grid info is available in the stream
  bool           m_bHasNormalsInfo;     // whether the normals info is available in the stream
  bool           m_bInterleaveRefine;   // 0 - no interleave, 1 - interleave vertices refine bits
  int            m_meshType;            // store the type of mesh
  bool           m_bSameConnect;        // if true the connectivity type between the vertices
                                        // of the quadrilateral mesh is the same
  bool           m_bAutoResolution;     // whether the uniform subdivision can be applied to
                                        // obtain higher resolution levels
  int            m_offsetAmp;           // amplitude of the offset
  int            m_cyclicMode;          // store whether the mesh is cyclic and the cyclic orienation
  BYTE           m_nrRefBitsVert;  	    // nr of refine bits for the vertices offsets 

  int            m_filterType;          // type of the filter used for encoding the grid
  int            m_nrQuantBits;         // nr of bits to quantize the grid values
  RGCoding       m_gridCoding[3];       // the coding parametres of the grid

protected:
  int            m_nrBitsFieldSlices;   // nr of bits per counter holding the nr of slices (x, y, z)
  int            m_nrBitsFieldROI;      // nr of bits per counter holding the size of the ROI (x, y, z)
  int            m_nrBitsFieldTile;     // nr of bits per counter holding the size of the Tile (x, y, z)

  int            m_nrBitsFieldGrid;     // nr of bits per counter holding the coded grid parameters
  CStringArray * m_msgs;                // used for dumping the stream

public:
  MStreamLevels  m_streamLevelM;        // the level description for the mesh coding
  GStreamLevels  m_streamLevelG;        // the level description for the grid coding
};

class MGStream
{
public:
  MGStream();
  ~MGStream();

  void  Initialise(CFile *file, int op);                                            // initialise the file and attach the stream
  void  DeleteStream() { delete m_pStream;   m_pStream = NULL; };                   // detach and delete the stream

  CBitStream *InitStream(int limit);                                                // initialise and attach the stream
  CBitStream *GetStream() { return m_pStream; };                                    // retrieve the bitream

  void  Flush() { m_pStream->Flush(); };                                            // empty the stream buffers
  void  SetPosition(int pos);                                                       // position the file and reset the stream
  void  SetPositionAndLimit(int pos, int limit, int op);                            // position the file and create new stream
  void  MovePosition(int pos);                                                      // position the file and reset the stream
  int   GetPosition() const { return m_pFile->GetPosition(); };                 // get the current file position
  int   GetBitCount() const { return m_pStream->GetBitCount(); };                   // get the bit count from the stream

  BYTE  GetRemainingValue() const { return m_pStream->GetRemainingValue(); };
  BYTE  GetBitPosition() const { return m_pStream->GetBitPosition(); };
  void  SetBitPosition(BYTE bitPos) {m_pStream->SetBitPosition(bitPos); };

  void  Write(BYTE *buffer, int sz) { m_pFile->Write(buffer, sz); };                // write raw data to the file

  void  ReadData(BYTE *res) { m_pStream->ReadData(res); };                          // read a BYTE from the stream
  void  WriteData(BYTE value) { m_pStream->WriteData(value); };                     // write a BYTE to the stream

  void  ReadBit(BYTE *res) { m_pStream->ReadBit(res); };                            // read a BIT to the stream
  void  ReadBits(BYTE *res, BYTE count) { m_pStream->ReadBits(res, count); };       // read a BYTE to the stream
  void  ReadBits(WORD *res, BYTE count) { m_pStream->ReadBits(res, count); };       // read a WORD to the stream
  void  ReadBits(DWORD *res, BYTE count) { m_pStream->ReadBits(res, count); };      // read a DWORD to the stream
  void  WriteBit(BYTE value) { m_pStream->WriteBit(value); };                       // write a BIT to the stream
  void  WriteBits(BYTE value, BYTE count) { m_pStream->WriteBits(value, count); };  // write a BYTE to the stream
  void  WriteBits(WORD value, BYTE count) { m_pStream->WriteBits(value, count); };  // write a WORD to the stream
  void  WriteBits(DWORD value, BYTE count) { m_pStream->WriteBits(value, count); }; // write a DWORD to the stream

  bool  RawRead(BYTE *buf, int pos, int count);                                     // read raw data from the stream
  int   Read(BYTE *buffer, int sz) { return m_pFile->Read(buffer, sz); };           // read raw data from the file

  void  SeekToBegin() { m_pFile->SeekToBegin(); };                                  // position the stream file at the begining
  void  ResetStream() { m_pStream->Reset(); };                                      // reset the stream without affecting the file  

public:
  CBitStream		* m_pStream; 		    // stream for coding
  CFile         * m_pFile;          // buffer for coding
};

#endif // !defined(AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_)
