/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGGrid.h: interface for the CMGGrid class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_MG_DATA_DEFS)
#define _MG_DATA_DEFS

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class MGDataExchange
{
public:
  enum { HEADER_SIZE = 2*sizeof(int) };
  enum {
    TAG_GRID_COORD,
    TAG_VERTEX_COORD,
    TAG_TRIANGLES,
    TAG_OFFSETS,
    TAG_NO_VERTICES
  };

  MGDataExchange(BYTE *pData, bool bInit, int size = 0)
  {
    m_pBuffer = pData;
    if (bInit)
    {
      m_size = *((int *) m_pBuffer);
      if (m_size > 8)
        m_noTags = *((int *) (m_pBuffer + sizeof(int)));
      else
        m_noTags = 0;
    }
    else
    {
      m_size = size;
      m_noTags = 0;
    }

    ASSERT(m_size >= 4);
    m_idxTag = 0;
    m_lastSize = 0;

    // skip the size field
    m_pLastBuffer = pData + HEADER_SIZE;
  };

  int ParseNoTags() { m_noTags; }

  bool StoreNoTags(int noTags)
  {
    if (m_pBuffer == NULL || m_size < 8)
      return false;

    // skip the size field
    *((int *) (m_pBuffer + sizeof(int))) = noTags;
    return true;
  };

  void * FetchNext(BYTE &tag, DWORD &size)
  {
    ASSERT((m_lastSize + sizeof(BYTE) + sizeof(DWORD)) < m_size);

    if (m_pLastBuffer == NULL || ++m_idxTag > m_noTags)
      return NULL;

    // read the tag and move the buffer pointer
    tag = *((BYTE *)m_pLastBuffer);
    m_pLastBuffer += sizeof(BYTE);

    // read the size of the part and move the buffer pointer
    size = *((DWORD *)m_pLastBuffer);
    m_pLastBuffer += sizeof(DWORD);

    // update the counter
    m_lastSize += sizeof(BYTE) + sizeof(DWORD) + size;

    return m_pLastBuffer;
  };

  bool StoreNext(BYTE tag, DWORD size, void *pData)
  {
    if (m_pLastBuffer == NULL ||
        (m_lastSize + sizeof(BYTE) + sizeof(DWORD) + size) > m_size)
      return false;

    // write the tag and move the buffer pointer
    *((BYTE *)m_pLastBuffer) = tag;
    m_pLastBuffer += sizeof(BYTE);

    // write the size of the part and move the buffer pointer
    *((DWORD *)m_pLastBuffer) = size;
    m_pLastBuffer += sizeof(DWORD);

    // write the data
    memcpy(m_pLastBuffer, pData, size);
    m_lastSize += sizeof(BYTE) + sizeof(DWORD) + size;

    return true;
  };

  int Finish()
  {
    return *((int *) m_pBuffer) = m_lastSize + HEADER_SIZE;
  };

protected:
  DWORD   m_size;
  DWORD   m_lastSize;
  int     m_noTags;
  int     m_idxTag;
  BYTE  * m_pBuffer;
  BYTE  * m_pLastBuffer;
};

#endif // !defined(_MG_DATA_FEFS)
