/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Patrick Gioia at patrick.gioia@rd.francetelecom.com
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       WaveSurfDec.cpp
AUTHOR:     Patrick Gioia (France Telecom R & D)
OVERVIEW:   Implementation of WaveSurfDecoder.
-------------------------------------------------------------------------*/
#include "StdAfx.h"
#include "ODP.h"
#include "DataDefs.h"
#include "WaveSurfDec.h"
#include "zerotree\ZTEncDec.h"
//#include "opengl/AFX/WaveSurfProxy.h"



static char dumpFileName [] = "Stream0.dmp";


WaveSurfDecoder :: WaveSurfDecoder () {
		m_isDecoded = FALSE;
		m_BM=NULL;
}

/*-------------------------------------------------------------
Set up the decoder
-------------------------------------------------------------*/
BOOL WaveSurfDecoder :: Setup (
	AFXExtDescriptor *pSpecificInfo
)
{
	/*------------
	Outband Setup
	------------*/

	ZTED = new ZTEncDec();
	ZTED->ZT=new zerotree();
	m_pDecSpecificInfo = (WMDecoderSpecificInfo*)pSpecificInfo;

	ZTED->ZT->m_EC = new EncoderConfig(
		m_pDecSpecificInfo->hasScaleCoeff,
		m_pDecSpecificInfo->NbBpSC,
		m_pDecSpecificInfo->NbBPX,
		m_pDecSpecificInfo->NbBPY,
		m_pDecSpecificInfo->NbBPZ,
		m_pDecSpecificInfo->Wtype,
		m_pDecSpecificInfo->Wtype,
		m_pDecSpecificInfo->lift,
		m_pDecSpecificInfo->NumberOfLevels,
		TRUE,
		m_pDecSpecificInfo->isInLocalCoordinates,
		m_pDecSpecificInfo->LengthNbBits,
		m_pDecSpecificInfo->isPartial,
		0,
		m_pDecSpecificInfo->NbBPX,
		1,
		1,
		1,
		m_pDecSpecificInfo->Xmax,
		m_pDecSpecificInfo->Ymax,
		m_pDecSpecificInfo->Zmax)
	;

	ZTED->ZT->m_depth=m_pDecSpecificInfo->NumberOfLevels;



	return TRUE;
}

BOOL WaveSurfDecoder :: Setup ()
{

	/*------------
	Inband Setup
	------------*/

	ZTED = new ZTEncDec();
	ZTED->ZT=new zerotree();

	ZTED->ZT->m_EC = new EncoderConfig();

	return TRUE;
}


/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int WaveSurfDecoder :: GetOptimalOutputSize ()
{
	/*---------------------------------------------------
	Here compute the size of the resulting indexedFaceSet
	----------------------------------------------------*/
	return 400000;
}


BOOL WaveSurfDecoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{

	CWMesh *wmesh = new CWMesh;





	/*------------------------------------------
	Don't decode if output buffer is too small
	--------------------------------------------*/
/*	if (rnOutputLength < nInputLength)
	{	rnOutputLength = nInputLength;
		return FALSE;
	}
*/
	/*--------
	Attachment
	--------*/
	wmesh->SetBaseMesh(m_BM);
	wmesh->MakeConnectivity();
	ZTED->SetBaseMesh(wmesh->m_baseMesh);

	

	/*-----------------------------------------------------
	Setup the zerotree decoder and launch zerotree decoder 
	-------------------------------------------------------*/
	int lbp = MAX(ZTED->ZT->m_EC->NbBPX, ZTED->ZT->m_EC->NbBPY);
	lbp = MAX(lbp, ZTED->ZT->m_EC->NbBPZ);
	


	/*-----------
	Decode stream
	-----------*/
	ZTED->ZTDecode_loc(pInput, nInputLength*8, 10000);
	ZTED->MoveLocZToWV(wmesh);


	/*------------
	Reconstruction
	------------*/
	switch (ZTED->ZT->GetWType()) {
	case 0://Midpoint
		wmesh->receptionCoeffLazy(NULL, wmesh->m_baseMesh->NbVertex, wmesh->m_pWr->NbCoeff, 0);
		break;
	case 1://Butterfly
		wmesh->receptionCoeffLazy(NULL, wmesh->m_baseMesh->NbVertex, wmesh->m_pWr->NbCoeff, 1);

	}
	evaluateQueue(wmesh->m_Q, &m_RecMesh);
	render(wmesh->m_Q, &m_RecMesh);
	//End reconstruction. Reconstructed mesh is available for rendering in m_RecMesh.
	////////////////////

FILE* g = fopen("out.wrl", "w");
ecriture(&m_RecMesh, g);
fclose(g);


	return TRUE;
}

int WaveSurfDecoder :: SetBaseMesh(MediaObject* WaveSurfNode) {



  //WaveletSubdivisionSurface* pObject2 = (WaveletSubdivisionSurface*)WaveSurfNode;
	//IndexedFaceSet* IFS = (IndexedFaceSet*)pObject2->baseMesh.GetNode();
	//MediaObject* IFS = pObject2->baseMesh.GetNode();
//	Coordinate* CRD = (Coordinate*)IFS->coord.GetNode();
//	ZArray<Vec3f>* AR = (ZArray<Vec3f>*)(&(CRD->point.GetArray()));
//	Vec3f TRPL = (*AR)[5];

	//MF<int> CRDIND = (MF<int>)IFS->coordIndex;//.GetNode();
	//FTEST

	
	return 0;
}

