
#include <stdio.h>
#include "3d.h"
#include <math.h>
#include "data.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


double MAX(double x, double y) {
	if (x>y) return x;
	else return y;
}

double MIN(double x, double y) {
	if (x<y) return x;
	else return y;
}

void egale(double u[3], double v[3]) {

	u[0]=v[0];
	u[1]=v[1];
	u[2]=v[2];
}

double dot(double u[3], double v[3]) {
	/* Produit scalaire */
	int i;
	double x;

	for (x=0, i=0; i<3; i++)
		x+=u[i]*v[i];
	return x;
}

double norm(double u[3]) {
	/* Norme euclidienne */
	return sqrt(dot(u,u));
}

void plus(double u[3], double v[3], double pluS[3]) {
	/* Addition de deux vecteurs */
	int i;
	for (i=0; i<3; i++) 
		pluS[i]=u[i]+v[i];
}

void moins(double u[3], double v[3], double pluS[3]) {
	/* Soustraction de deux vecteurs */
	int i;
	for (i=0; i<3; i++) 
		pluS[i]=u[i]-v[i];
}

void vect(double u[3], double v[3], double vecT[3]) {
	/* Produit vectoriel */
	vecT[0]=u[1]*v[2]-u[2]*v[1];
	vecT[1]=u[2]*v[0]-u[0]*v[2];
	vecT[2]=u[0]*v[1]-u[1]*v[0];
}

double angle(double u[3], double v[3]) {
	/* Angle entre deux vecteurs */
	/* Entre 0 et pi */
	double x, y, z;
	x=dot(u,v);
	y=norm(u);
	z=norm(v);
	return acos(x/y/z);
}

double angle2(double x[3], double y[3], double z[3]) {
	double u[3], v[3];
	moins (x, y, u);
	moins(z, y, v);
	return angle(u, v);
}

void mult (double lambda, double u[3], double v[3]) {
	int i;
	for (i=0; i<3; i++) v[i]=lambda*u[i];
}

void init(double u[3]) {
	u[0]=0;
	u[1]=0;
	u[2]=0;
}

int deuxPuissance(int j) {

	if (j==0) return 1;
	else return 2*deuxPuissance(j-1);
}

int log2(int n) {

	/* n doit etre une puissance de deux */

	if (n==1) return 0;
	else return 1+log2(n/2);
}

void normalize(double u[3]) {

	double n=norm(u);

	u[0]/=n;
	u[1]/=n;
	u[2]/=n;
}

int rem(int p, int q) {
	return p-(p/q)*q;
}

double dist(double x[3], double y[3]) {

	/* Distance euclidienne entre deux points */

	double u[3];

	moins(x, y, u);
	return norm(u);

}

double distPointSegment(double x[3], double a[3], double b[3]) {

	/* Distance euclidienne entre un point x et un segment [ab] */

	double u[3], v[3], w[3], y[3], r;

	if (dist(a, b)<epsilon) return dist(x, a);
	else {
		moins(b, a, u);
		normalize(u);
		moins(x, a, v);
		r=dot(u, v);
		mult(r, u, v);
		plus(a, v, y);
		moins(a, v, u);
		moins(b, v, w);
		if (dot(u, w)>0) { /* Le minimum est realise sur le bord */
			if (dist(x, a)>dist(x, b)) return dist(x, b);
			else return dist(x, a);
		}
		else return dist(x, y);
	}
}

int memePlace(double x[3], double y[3]) {

	return (dist(x, y)<epsilon*epsilon);
	//return ((x[0]==y[0])&&(x[1]==y[1])&&(x[2]==y[2]));
}

int estDansTriangle(double x[3], double a[3], double b[3], double c[3]) {
	
	double u[3], v[3], w[3], al[3], be[3], ga[3];

	moins(a, x, u);
	moins(b, x, v);
	moins(c, x, w);
	vect(u, v, al);
	vect(v, w, be);
	vect(w, u, ga);

	return ((dot(al, be)>=-0.00001)&&(dot(be, ga)>=-0.00001)&&(dot(ga, al)>=-0.00001));
}

double aire(double x[3], double y[3], double z[3]) {
	
	double u[3], v[3], w[3];

	moins(z, x, u);
	moins(z, y, v);
	vect(u, v, w);

	return norm(w);
}

void bary(double point[3], double a[3], double b[3], double c[3], double *alpha, double *beta, double *gamma) {
	
	//Calcule les coordonnes barycentriques de point dans le repere abc s'il est dans le plan du triangle

	double s;

	(*alpha)=aire(point, b, c);
	(*beta)=aire(point, a, c);
	(*gamma)=aire(point, a, b);

	s=(*alpha)+(*beta)+(*gamma);
	if (s==0) {
		(*alpha)=1./3;
		(*beta)=1./3;
		(*gamma)=1./3;
	}
	else {
		(*alpha)/=s;
		(*beta)/=s;
		(*gamma)/=s;
	}

}

void projetePointSegment(double x[3], double a[3], double b[3], double proj[3]) {

	double u[3], v[3], y[3], w[3], z[3], xu;
	
	if (dist(a, b)>epsilon) {
		moins(b, a, u);
		normalize(u);
		moins(x, a, v);
		xu=dot(u, v);
		mult(xu, u, y);
		plus(a, y, y);
		moins(a, y, w);
		moins(b, y, z);
		if (dot(z, w)>0) {
			if (dist(x, a)<dist(x, b)) 
				egale(proj, a);
			else 
				egale(proj, b);
		}
		else egale(proj, y);
	}
	else egale(proj, a);
}

void projetePointTriangle(double x[3], double a[3], double b[3], double c[3], double proj[3], double plafond) {

	double y[3], u[3], v[3], w[3], n[3], cmp1[3], cmp2[3], cmp3[3], xn, d1, d2, d3;

	if (aire(a, b, c)>epsilon) { 

		moins(b, a, u);
		moins(c, a, v);
		moins(x, a, w);
		vect(u, v, n);
		normalize(n);
		xn=dot(n, w);
		mult(xn, n, y);
		moins(w, y, y);
		plus(y, a, y);

		if (dist(x, y)>plafond) {
			mult(1000000, y, y);
			egale(proj, y);
		}
		else {
			if (!estDansTriangle(y, a, b, c)) {
				projetePointSegment(y, a, b, cmp1);
				d1=dist(x, cmp1);
				projetePointSegment(y, b, c, cmp2);
				d2=dist(x, cmp2);
				projetePointSegment(y, c, a, cmp3);
				d3=dist(x, cmp3);
				if ((d1<=d2)&&(d1<=d3)) egale(proj, cmp1);
				if ((d2<=d1)&&(d2<=d3)) egale(proj, cmp2);
				if ((d3<=d1)&&(d3<=d2)) egale(proj, cmp3);
			}
			else egale(proj, y);
		}
	}
	else {
		if (dist(a, b)>epsilon) projetePointSegment(x, a, b, proj);
		else projetePointSegment(x, a, c, proj);
	}
}

void projetePointMaillage(Mesh *mesh, double x[3], double proj[3]) {
	
	/* projete un point sur un maillage */
	/* Projet dans proj */

	double buf, projt1[3], projt2[3];
	int i;

	buf=1000000;
	for (i=0; i<mesh->NbFace; i++) {

		projetePointTriangle(x, mesh->Vertex[mesh->Face[i][0]], mesh->Vertex[mesh->Face[i][1]],
							mesh->Vertex[mesh->Face[i][2]], projt1, buf);
		if (dist(x, projt1)<buf) {
			buf=dist(x, projt1);
			egale(projt2, projt1);
		}
	}
	egale(proj, projt2);
}

int projetePointMaillageDiscret(Mesh *mesh, double x[3]) {
	
	// proj renvoit le sommet le plus pres de x

	double buf;
	int i, proj;

	proj=-1;
	buf=1000000;
	for (i=0; i<mesh->NbVertex; i++) {
		if (dist(x, mesh->Vertex[i])<buf) {
			buf=dist(x, mesh->Vertex[i]);
			proj=i;
		}
	}
	return proj;
}

void projetePointHM(Mesh *mesh, double x[3], double proj[3]) {
	
	/* projete un point sur un HM */
	/* Projet dans proj */

	double buf, projt1[3], projt2[3], alpha, beta, gamma;
	int i, triangle;

	triangle=-1;
	buf=1000000;
	for (i=0; i<mesh->NbFace; i++) {
if (i==200 && x[0]==-40.74) {
	i=i;

}
		projetePointTriangle(x, mesh->HM[mesh->Face[i][0]], mesh->HM[mesh->Face[i][1]],
							mesh->HM[mesh->Face[i][2]], projt1, buf);
		if (dist(x, projt1)<buf) {
			buf=dist(x, projt1);
			egale(projt2, projt1);
			triangle=i;
		}
	}

	//Calcul des coordonnes barycentriques
	bary(projt2, mesh->HM[mesh->Face[triangle][0]], mesh->HM[mesh->Face[triangle][1]], mesh->HM[mesh->Face[triangle][2]], &alpha, &beta, &gamma);

	for (i=0; i<3; i++) {
		projt2[i]=alpha*mesh->Vertex[mesh->Face[triangle][0]][i]+beta*mesh->Vertex[mesh->Face[triangle][1]][i]
			+gamma*mesh->Vertex[mesh->Face[triangle][2]][i];
	}

	egale(proj, projt2);
}

int memeFacette(Mesh *mesh, int x, int y, int z, int t) {
	/* Renvoit 0 ssi les points x, y, z et t ne sont pas sur la meme facette */

	int i, j, ok=0, present;
	for (i=0; (i<mesh->NbFace)&&(!ok); i++) {
		present=0;
		for (j=0; j<3; j++) {
			if (x==mesh->Face[i][j]) present++;
			if (y==mesh->Face[i][j]) present++;
			if (z==mesh->Face[i][j]) present++;
			if (t==mesh->Face[i][j]) present++;
		}
		ok=(present==4);
	}
	return ok;
}

int formentUnTriangle(int a, int b, int x, int y, int i, int j) {

	int ok=1, romeo=1;

	ok=ok&&((x==a)||(x==b)||(x==i)||(x==j));
	ok=ok&&((y==a)||(y==b)||(y==i)||(y==j));
	ok=ok&&((a==x)||(a==y)||(a==i)||(a==j));
	ok=ok&&((b==x)||(b==y)||(b==i)||(b==j));
	ok=ok&&((i==a)||(i==b)||(i==x)||(i==y));
	ok=ok&&((j==a)||(j==b)||(j==x)||(j==y));

	return ok;
}

int romeoJuliette(MRmesh *mrm, int j, int k) {

	/* Renvoit 0 ssi i et j ne sont pas a l'interieur de facettes differentes */

	return (mrm->Tab[j]->alpha*mrm->Tab[j]->beta*mrm->Tab[j]->gamma!=0)&&
		(mrm->Tab[k]->alpha*mrm->Tab[k]->beta*mrm->Tab[k]->gamma!=0)&&
		(mrm->Tab[j]->F0!=mrm->Tab[k]->F0);
}

void projChemIso(Chemin *chemin, double milieu[3]) {
	
	/* Renvoit un point de chemin qui rend le triangle associ presque isocele */
	/* Le chemin ne doit pas etre reduit a un segment */

	Chemin *parcours;
	double depart[3], arrivee[3];

	if (chemin->suivant->suivant==NULL) fprintf(stderr, "projChemIso : BUG. Chemin reduit a un segment.");

	//recherche des bornes
	egale(depart, chemin->pos);
	parcours=chemin;
	while (parcours!=NULL) {
		egale(arrivee, parcours->pos);
		parcours=parcours->suivant;
	}

	//Initialisation du milieu
	egale(milieu, chemin->suivant->pos);

	//On va le plus loin possible
	parcours=chemin->suivant;
	while (parcours!=NULL) {
		if (dist(depart, parcours->pos)<=dist(arrivee, parcours->pos)) 
			egale(milieu, parcours->pos);
		parcours=parcours->suivant;
	}
}

int voisinTour(int sens, Mesh *mesh, int face, int s) {

	/* N'a a voir qu'avec resteSimplConnexe */

	Lint *liste=mesh->FaceVoisine[s];
	int b, c;

	if (face==-1) return -1;
	while (liste!=NULL) {
		//Sens direct
		if (sens==1) {
			if (s==mesh->Face[face][0]) b=mesh->Face[face][2];
			if (s==mesh->Face[face][1]) b=mesh->Face[face][0];
			if (s==mesh->Face[face][2]) b=mesh->Face[face][1];

			if (s==mesh->Face[liste->facette][0]) c=mesh->Face[liste->facette][1];
			if (s==mesh->Face[liste->facette][1]) c=mesh->Face[liste->facette][2];
			if (s==mesh->Face[liste->facette][2]) c=mesh->Face[liste->facette][0];
			
			if (b==c) return liste->facette;
		}
		//Sens retrograde
		if (sens==2) {
			if (s==mesh->Face[face][0]) b=mesh->Face[face][1];
			if (s==mesh->Face[face][1]) b=mesh->Face[face][2];
			if (s==mesh->Face[face][2]) b=mesh->Face[face][0];

			if (s==mesh->Face[liste->facette][0]) c=mesh->Face[liste->facette][2];
			if (s==mesh->Face[liste->facette][1]) c=mesh->Face[liste->facette][0];
			if (s==mesh->Face[liste->facette][2]) c=mesh->Face[liste->facette][1];
			
			if (b==c) return liste->facette;
		}
		liste=liste->suivant;
	}
	return -1;
	
}

int resteSimplConnexe(Mesh *mesh, int face, int couleur) {

	/* Renvoit 0 ssi la composante couleur ne sera plus simplement connexe apres */
	/* admission de la facette face */

	int s, eclaireur1, eclaireur2, RSC=0, j, encore1, encore2;

	for (j=0; j<3; j++) {
	//Pour chaque sommet de face
		s=mesh->Face[face][j];
		encore1=encore2=1;
		eclaireur1=eclaireur2=face;
		//On fait de tour de s des deux cots
		while (encore1||encore2) {
			if (encore1) eclaireur1=voisinTour(1, mesh, eclaireur1, s);
			if (eclaireur1==-1 || mesh->FacePresente[eclaireur1]!=couleur) encore1=0;

			if (eclaireur1==eclaireur2) encore1=encore2=0;

			if (encore2) eclaireur2=voisinTour(2, mesh, eclaireur2, s);
			if (eclaireur2==-1 || mesh->FacePresente[eclaireur2]!=couleur) encore2=0;

			if (eclaireur1==eclaireur2) encore1=encore2=0;
		}
		
		//Encerclement
		encore1=encore2=1;
		while (encore1||encore2) {
			
			//Tests d'arrret
			if (eclaireur1==eclaireur2) encore1=encore2=0;
			if (mesh->FacePresente[eclaireur1]==couleur || mesh->FacePresente[eclaireur2]==couleur) return 0;
			
			//propagation
			if (encore1) eclaireur1=voisinTour(1, mesh, eclaireur1, s);
			if (eclaireur1==-1) encore1=0;
			
			if (eclaireur1==eclaireur2) encore1=encore2=0;
			
			
			if (encore2) eclaireur2=voisinTour(2, mesh, eclaireur2, s);
			if (eclaireur2==-1) encore2=0;
		}
	//Repeter pour chaque sommet
	}
	return 1;
}

double Delta(Mesh *m1, Mesh *m2) {

	//Calcule une approximation de la distance de Hausdorff entre m1 et m2
	
	int i;
	double prj[3], tmp, buf=0;

	for (i=0; i<m1->NbVertex; i++) {
		projetePointMaillage(m2, m1->Vertex[i], prj);
		tmp=dist(m1->Vertex[i], prj);
		if (tmp>buf) buf=tmp;
	}
	for (i=0; i<m2->NbVertex; i++) {
		projetePointMaillage(m1, m2->Vertex[i], prj);
		tmp=dist(m2->Vertex[i], prj);
		if (tmp>buf) buf=tmp;
//fprintf(stderr, "%f", buf);
	}

	return buf;

}

double dist2D(double x[2], double y[2]) {

	return sqrt((x[0]-y[0])*(x[0]-y[0])+(x[1]-y[1])*(x[1]-y[1]));
}

int VALANCE_F(Mesh *mesh, int s) {
	/* Renvoit le nombre de faces voisines d'un sommet */

	int tab[NbVertexMax], top=0, i, trouve;
	Lint *liste;

	liste=mesh->FaceVoisine[s];
	trouve=0;
	while (liste!=NULL) {
		for (i=0; i<top; i++) {
			trouve=trouve||(tab[i]==liste->facette);
		}
		if (!trouve) {
			tab[top]=liste->facette;
			top++;
		}
		liste=liste->suivant;
	}
	return top;
}

Chemin* endOfChemin(Chemin *chemin) {

	Chemin *runningChemin=chemin;

	while (runningChemin!=NULL) {

		if (runningChemin->suivant==NULL)
			return runningChemin;

		runningChemin=runningChemin->suivant;
	}

	return NULL;

}

double DistanceToSegment(Chemin *chemin) {
	//Calcule la distance entre le chemin et le segment form par ses extrmits

	Chemin *runningChemin=chemin;
	double MaxChemin = 0;
	double tmp;

	while (runningChemin!=NULL) {

		tmp=distPointSegment(runningChemin->pos, chemin->pos, endOfChemin(chemin)->pos);
		if (MaxChemin < tmp)
			MaxChemin=tmp;

		runningChemin=runningChemin->suivant;
	}

	
	

	return MaxChemin;
	
}

double DistanceToSegment(Chemin *chemin, double a[3], double b[3]) {
	//Calcule la distance entre le chemin et le segment form par ses extrmits

	Chemin *runningChemin=chemin;
	double MaxChemin = 0;
	double tmp;

	while (runningChemin!=NULL) {

		tmp=distPointSegment(runningChemin->pos, a, b);
		if (MaxChemin < tmp)
			MaxChemin=tmp;

		runningChemin=runningChemin->suivant;
	}

	
	

	return MaxChemin;
	
}

int CheminNbVertex(Chemin *chemin) {

	Chemin *runningChemin=chemin;
	int resu=0;

	while (runningChemin!=NULL) {

		resu++;

		runningChemin=runningChemin->suivant;
	}

	return resu;
}

double CheminLongueur(Chemin *chemin) {

	return dist(chemin->pos, endOfChemin(chemin)->pos);

}

double DistanceToBrokenSegment(Chemin *chemin, Chemin *inter) {

	double max1, max2;

	Chemin *runningChemin=chemin;
	Chemin *fin;
	double MaxChemin = 0;
	double tmp;

	while (runningChemin!=inter) {

		tmp=distPointSegment(runningChemin->pos, chemin->pos, inter->pos);
		if (MaxChemin < tmp)
			MaxChemin=tmp;

		runningChemin=runningChemin->suivant;
	}

	max1=MaxChemin;

	MaxChemin=0;
	fin=endOfChemin(chemin);
	while (runningChemin!=NULL) {

		tmp=distPointSegment(runningChemin->pos, inter->pos, fin->pos);
		if (MaxChemin < tmp)
			MaxChemin=tmp;

		runningChemin=runningChemin->suivant;
	}

	max2=MaxChemin;
	
	if (max1>max2)
		return max1;
	else
		return max2;
}

