#include "CWMesh.h"


/*----------
Constructor
----------*/
CWMesh::CWMesh() {

	m_Mesh=NULL;
	m_Q=NULL;
	m_baseMesh=(Mesh*)malloc(sizeof(Mesh));
	m_pVPC=(VertexPerCoeff*)malloc(sizeof(VertexPerCoeff));
	DM=NULL;
	m_pWr=NULL;
}

bool CWMesh::AllocateAll(bool M, bool BM, bool WR, bool Q, bool VPC, bool DMb) {


	if (M && m_Mesh==NULL)
		m_Mesh=(Mesh*)calloc(1, sizeof(Mesh));
	if (BM && m_baseMesh==NULL)
		m_baseMesh=(Mesh*)calloc(1, sizeof(m_baseMesh));
	if (WR && m_pWr==NULL)
		m_pWr=(WaveRep*)calloc(1, sizeof(WaveRep));
	if (Q && m_Q==NULL)
		m_Q=(Qtree*)calloc(1, sizeof(Qtree));
	if (VPC && m_pVPC==NULL)
		m_pVPC=(VertexPerCoeff*)calloc(1, sizeof(VertexPerCoeff));
	if (DMb && DM==NULL)
		DM=(DatMesh*)calloc(1, sizeof(DatMesh));
	

	return true;
}

/*-------------------------
Initialize from a file name
-------------------------*/
bool CWMesh::init(char *lpszPathName) {

	lisCoeffs(lpszPathName);
	initQuadTree();
	if (fabs((m_baseMesh->Vertex[0][0]-m_pWr->tab[0]->x)+(m_baseMesh->Vertex[0][1]-m_pWr->tab[0]->y)+(m_baseMesh->Vertex[0][2]-m_pWr->tab[0]->z))<0.00001)
		for (int i=0; i<m_baseMesh->NbVertex; i++) 
			m_Q->VertexTable[i]->g[0]=m_Q->VertexTable[i]->g[1]=m_Q->VertexTable[i]->g[2]=0;
	evaluate();
	render(m_Q, m_Mesh);

	return true;
}

/*---------
Read a mesh
---------*/
bool CWMesh::getTheMesh(char *lpszPathName) {

	unsigned int bidon;
	char *aremplacer;

	if (m_Mesh==NULL)
		m_Mesh=(Mesh*)malloc(sizeof(Mesh));
	getMesh2(m_Mesh, lpszPathName, &bidon, aremplacer);

	//TRACE
#ifdef _DEBUG
	FILE *f=fopen("trace.txt", "a");
	fprintf(f, "Read the base mesh\n");
	for (int i=0; i<m_baseMesh->NbVertex; i++) {
		fprintf(f, "%f %f %f\n", m_baseMesh->Vertex[i][0],m_baseMesh->Vertex[i][2],m_baseMesh->Vertex[i][2]);
	}
	for (i=0; i<m_baseMesh->NbFace; i++) {
		fprintf(f, "%d %d %d\n", m_baseMesh->Face[i][0],m_baseMesh->Face[i][2],m_baseMesh->Face[i][2]);
	}
	fclose(f);
#endif
	//END TRACE

	return true;
}

bool CWMesh::readBaseMesh(char *lpszPathName) {

	unsigned int potiche;
	char deco[512];

	getMesh(m_baseMesh, lpszPathName, &potiche, deco); 

	return true;
}

/*---------------------------------------
Read wavelet coefficients in .wv format
---------------------------------------*/
bool CWMesh::lisCoeffs(char *lpszPathName) {

	if (!m_pWr)
		m_pWr=(WaveRep*)malloc(sizeof(WaveRep));
	lisCoeffNoTexture(lpszPathName, m_pWr, m_baseMesh);

	//TRACE
#ifdef _DEBUG
	FILE *f=fopen("trace.txt", "a");
	fprintf(f, "Read the wavelet coefficients\n");
	for (int i=0; i<m_pWr->NbCoeff; i++) {
		fprintf(f, "%d %d %d %d %f %f %f\n", m_pWr->tab[i]->F0, m_pWr->tab[i]->alpha, m_pWr->tab[i]->beta, m_pWr->tab[i]->gamma, m_pWr->tab[i]->x,m_pWr->tab[i]->y,m_pWr->tab[i]->z);
	}
	fclose(f);
#endif
	//END TRACE

	return true;
}

/*----------------------------------------
Initialize the quadtree with the basemesh
----------------------------------------*/
bool CWMesh::initQuadTree() {

	initQ4(m_Q, m_baseMesh);

	return true;
}

/*-----------------------------------------
get lazy coefficients from first to last
-----------------------------------------*/
bool CWMesh::receptionCoeffLazy(int *Invite, int first, int last, double dist) {

	
	receptionCoeff6(m_Q, m_pWr, Invite, first, last, dist, m_pVPC, m_Mesh, m_baseMesh);

	return true;

}

/*-----------------------------------------
get non-lifted butterfly coefficients from first to last
-----------------------------------------*/
bool CWMesh::completeRebuildNLB(int *Invite, int first, int last, double dist, int tension) {

	
	naiveRebuildNLB(m_Q, m_pWr, Invite, first, last, dist, m_pVPC, m_Mesh, m_baseMesh, tension);

	return true;

}

bool CWMesh::getDAT(char *lpszPathName) {
	
		AllocateAll(1, 1, 1, 1, 0, 1);

		loadDAT(DM, (char *)lpszPathName);
		DAT2BaseMesh(DM, m_baseMesh);
		initQuadTree();
		DAT2Q(DM, m_Q);
		evaluate();
		render(m_Q, m_Mesh);


		return true;
}

bool CWMesh::getDAT(DatMesh *newDM) {
	
		AllocateAll(1, 1, 1, 1, 0, 1);

		DM=newDM;
		DAT2BaseMesh(DM, m_baseMesh);
		initQuadTree();
		DAT2Q(DM, m_Q);
		evaluate();
		render(m_Q, m_Mesh);


		return true;
}

bool CWMesh::ecritBaseMesh(char *lpszPathName) {

	FILE *f=fopen(lpszPathName, "w");
	ecriture(m_baseMesh, f);
	fclose(f);

	return TRUE;
}

bool CWMesh::ecritMesh(char *lpszPathName) {

	FILE *f=fopen(lpszPathName, "w");
	ecriture(m_Mesh, f);
	fclose(f);

	return TRUE;
}

bool CWMesh::ecritWV(char *lpszPathName) {

	char wrl[512]="";
	char wv[512]="";

	strcpy(wrl, lpszPathName);
	strcpy(wv, lpszPathName);
	strcat(wrl, ".wrl");
	strcat(wv, ".wv");

	ecritBaseMesh(wrl);


	FILE *f=fopen(wv, "w");

	fprintf(f, "%s\n", wrl);
	for (int i=0; i<m_pWr->NbCoeff; i++) {
		fprintf(f, "%d %d %d %d %f %f %f\n",m_pWr->tab[i]->F0, m_pWr->tab[i]->alpha, m_pWr->tab[i]->beta, m_pWr->tab[i]->gamma, m_pWr->tab[i]->x, m_pWr->tab[i]->y, m_pWr->tab[i]->z);
	}
	fprintf(f, "-1");
	fclose(f);

	return true;
}

bool CWMesh::ecritDAT(char *lpszPathName) {

	FILE *g=fopen(lpszPathName, "w");
	ecritDat(DM, g);
	fclose(g);

	return true;
}

void SphRecsubdiv (Qtree *Q, Face_WSZT*f, int jmax, int j) {
	
	void divise21(Qtree*, Face_WSZT*);
	
	if (j==jmax)
		return;

	divise21(Q, f);

	SphRecsubdiv(Q, f->children[0], jmax, j+1);
	SphRecsubdiv(Q, f->children[1], jmax, j+1);
	SphRecsubdiv(Q, f->children[2], jmax, j+1);
	SphRecsubdiv(Q, f->children[3], jmax, j+1);
}

bool CWMesh::MakeSphere (char *name) {

	int i, j=0;
	double norme;

	//Get the base mesh
	readBaseMesh(name);
	initQuadTree();

	for (i=0; i<m_Q->NbRacine; i++) {
		SphRecsubdiv(m_Q, m_Q->racines[i], 3, 0);
	}

	//Normalize
	for (i=0; i<m_Q->Top; i++) {
		norme=sqrt((m_Q->VertexTable[i]->g[0]*m_Q->VertexTable[i]->g[0])+
			(m_Q->VertexTable[i]->g[1]*m_Q->VertexTable[i]->g[1])+
			(m_Q->VertexTable[i]->g[2]*m_Q->VertexTable[i]->g[2]));
		m_Q->VertexTable[i]->g[0]/=norme;
		m_Q->VertexTable[i]->g[1]/=norme;
		m_Q->VertexTable[i]->g[2]/=norme;
	}

	return true;
}

bool CWMesh::L2NormalizeCoeffs () {


	int i;

	for (i=0; i<m_pWr->NbCoeff; i++) {
		m_pWr->tab[i]->x/=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
		m_pWr->tab[i]->y/=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
		m_pWr->tab[i]->z/=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
	}

	return true;
}

bool CWMesh::L2UnNormalizeCoeffs () {

	int i;

	for (i=0; i<m_pWr->NbCoeff; i++) {
		m_pWr->tab[i]->x*=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
		m_pWr->tab[i]->y*=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
		m_pWr->tab[i]->z*=m_pWr->tab[i]->alpha+m_pWr->tab[i]->beta+m_pWr->tab[i]->gamma;
	}

	return true;
}

bool CWMesh::SetQ(Qtree *Q) {
	
/*
	int i;

	
	m_Q->NbVertex=Q->NbVertex;
	m_Q->NbRacine=Q->NbRacine;
	m_Q->Gap=Q->Gap;
	m_Q->nouveaux=Q->nouveaux;
	m_Q->Remember=Q->Remember;
	m_Q->Top=Q->Top;
		
//	m_Q->VertexTable=Q->VertexTable;

	for (i=0; i<Q->NbRacine; i++)
		m_Q->racines[i]=Q->racines[i];
*/
	return true;
}

bool CWMesh::WriteTriangles(char *input, char *output) {

	getTheMesh(input);

	FILE *g=fopen(output,"w");

	for (int i=0; i<m_Mesh->NbFace; i++) {
		fprintf(g, "%f %f %f\n", m_Mesh->Vertex[m_Mesh->Face[i][0]][0], m_Mesh->Vertex[m_Mesh->Face[i][0]][1], m_Mesh->Vertex[m_Mesh->Face[i][0]][2]);
		fprintf(g, "%f %f %f\n", m_Mesh->Vertex[m_Mesh->Face[i][1]][0], m_Mesh->Vertex[m_Mesh->Face[i][1]][1], m_Mesh->Vertex[m_Mesh->Face[i][1]][2]);
		fprintf(g, "%f %f %f\n\n", m_Mesh->Vertex[m_Mesh->Face[i][2]][0], m_Mesh->Vertex[m_Mesh->Face[i][2]][1], m_Mesh->Vertex[m_Mesh->Face[i][2]][2]);

	}

	fclose(g);

	return true;
}



int CWMesh::LazyAnalysis() {

	if (m_pWr==NULL)
		m_pWr=(WaveRep*)malloc(sizeof(WaveRep));

	if (m_Q==NULL)
		m_Q=(Qtree*)malloc(sizeof(Qtree));
	
	//Analyse

	fillCoeffsButterfly(m_Q, m_baseMesh, 0);

	//copie des coefficients dans wr

	for (int i=0; i<m_Q->Top; i++) {
		m_pWr->tab[i]=(WaveCo*)malloc(sizeof(WaveCo));
		m_pWr->tab[i]->x=m_Q->VertexTable[i]->g[0];
		m_pWr->tab[i]->y=m_Q->VertexTable[i]->g[1];
		m_pWr->tab[i]->z=m_Q->VertexTable[i]->g[2];
	}
	copyQVerticesToWr(m_Q, m_pWr, m_baseMesh);

	return 0;
}




int CWMesh::ZDAnalysis() {

	return 0;
}
int CWMesh::ButterflyAnalysis() {

	return 0;
}
int CWMesh::LiftedButterflyAnalysis() {

	return 0;
}
int CWMesh::LoopAnalysis() {

	return 0;
}

bool CWMesh::copyMesh(Mesh *mesh) {

	int i;

	if (m_Mesh!=NULL)
		free (m_Mesh);

	m_Mesh=(Mesh*)calloc(1, sizeof(Mesh));

	for (i=0; i<mesh->NbVertex; i++) 
		egale(m_Mesh->Vertex[i], mesh->Vertex[i]);

	m_Mesh->NbVertex=mesh->NbVertex;

	for (i=0; i<mesh->NbFace; i++) {
		m_Mesh->Face[i][0]=mesh->Face[i][0];
		m_Mesh->Face[i][1]=mesh->Face[i][1];
		m_Mesh->Face[i][2]=mesh->Face[i][2];
	}

	m_Mesh->NbFace = mesh->NbFace;


	return true;
}

bool CWMesh::moveQToDM() {

	if (DM==NULL)
		DM = (DatMesh*)malloc(sizeof(DatMesh));
	DumpQ2Datmesh(m_Q, DM);

	return true;
}

void CWMesh::SetBaseMesh(Mesh *mesh) {

	m_baseMesh = mesh;
}


int  CWMesh::MakeConnectivity() {

	//remplit le champ facevoisine

	int i;

	if (m_baseMesh==NULL) 
		return -1;

	for (i=0; i<m_baseMesh->NbVertex; i++)
		m_baseMesh->FaceVoisine[i]=NULL;

	for (i=0; i<m_baseMesh->NbFace; i++) {
		//Ajout au premier sommet
		ajouteFace(m_baseMesh, i, m_baseMesh->Face[i][0]);

		//Ajout au deuxime sommet
		ajouteFace(m_baseMesh, i, m_baseMesh->Face[i][1]);

		//Ajout au troisime sommet
		ajouteFace(m_baseMesh, i, m_baseMesh->Face[i][2]);
	}

	return 0;

}
