#if !defined(_DATA_H_INCLUDED)
#define _DATA_H_INCLUDED

#define epsilon 0.0005
//#define HUGE 1000000000
#define seuil 2
#define pi 3.141592

#define TRUE 1
#define FALSE 0

#define NbVertexMax 200000
#define NbFaceMax 200000
#define NbCornerMax 3
#define NbNiveauMax 10
#define NbCoeffMax 600000

#define InnerU 7./3
#define InnerV .25
#define InnerW 1
#define InnerRegular 0.243183

#define NMAX 3000
#define NBITER 100

typedef struct LINT {
	int facette;
	struct LINT *suivant;
} Lint; 

typedef struct PILEDEFACES {// Tous la meme chose (pour pas mlanger les choux et les carrottes)
	int face;
	struct PILEDEFACES *next;
} PileDeFaces;

typedef struct MESH {
	int NbVertex, NbFace;
	double Vertex[NbVertexMax][3];
	int Face[NbFaceMax][NbCornerMax]; 
	double Normale[NbFaceMax][3];
	int SommetPresent[NbVertexMax];
	int FacePresente[NbFaceMax];
	int couleur[NbFaceMax];
	Lint *FaceVoisine[NbVertexMax];
	double HM[NbVertexMax][3];
	double TexCoord[NbVertexMax][2];
	PileDeFaces *FaceGap; 
} Mesh;

typedef struct WAVECO {
	unsigned int F0;
	
	unsigned int alpha;
	unsigned int beta;
	unsigned int gamma;

	float x;
	float y;
	float z;

	float u;
	float v;
} WaveCo;

typedef struct WAVEREP {
	unsigned int NbCoeff;
	char mzero[512];
	WaveCo *tab[NbCoeffMax];
} WaveRep;

typedef struct SOMMET {
	int level;
	struct SOMMET *parentV[2];
	struct FACE *parentF[2];
	double fGeom[3];
	double g[3];
	int indice;
	double hGeom[NbNiveauMax][3];
	double rem[3]; // Sert  remplacer hGeom, qui prend trop de place. ( dgager)
	double u, v;
} Sommet;

typedef struct CL{
	int coeff;
	struct CL *next;
} cl;

typedef struct FACE {
	int level;
	struct FACE *children[4];
	Sommet *sommetCoin[3];
	Sommet *sommetArete[3];
	struct FACE *Voisins[3];	
	int indice; // N'apparait que lorsque la face devient une feuille de Qtree
	struct CL *Coeff_list; // Nouveau (incremental)					)
	int Tricoeff[3]; // Encore plus nouveau...Alternative a CL		}  dgager...
					 // Contiendra -1 si pas de coeffs.				)
} Face_WSZT;

typedef struct PILEDESOMMETS {// Tous la meme chose (pour pas mlanger les choux et les carrottes)
	int sommet;
	struct PILEDESOMMETS *next;
} PileDeSommets;

typedef struct PILEDENTIERS {// Tous la meme chose (pour pas mlanger les choux et les carrottes)
	int entier;
	struct PILEDENTIERS *next;
} PileDEntiers;

typedef struct QTREE {
	Face_WSZT *racines[NbFaceMax];
	Sommet *VertexTable[NbVertexMax];
	int Valance[NbVertexMax];
	int NbRacine;
	int NbVertex;
	int Top; /* Nombre de sommets courants */
	int Remember;
	PileDeSommets *Gap;
	PileDeSommets *nouveaux;
} Qtree;

typedef struct RETURNPOINTER {
	int face;
	struct RETURNPOINTER *next;
} ReturnPointer;

typedef struct LISTEDECOEFF {
	int coeff;
	ReturnPointer *rp;
} ListeDeCoeff;

typedef struct LISTEDESOMMETS {
	int id;
	double alpha;
	struct LISTEDESOMMETS *next;
} ListeDeSommets;

typedef struct VERTEXPERCOEFF { 
	ListeDeSommets* LDS[NbCoeffMax]; 
} VertexPerCoeff;

typedef struct LISTEDEFACE {
	Face_WSZT *F;
	struct LISTEDEFACE *next;
} ListeDeFace;

typedef struct QUEUEDEFACE {
	struct LISTEDEFACE *debut;
	struct LISTEDEFACE *fin;
} QueueDeFace;

typedef struct DATMESH {
	int NbVertices, NbFaces;
	float V[NbVertexMax][4];
	int F[NbVertexMax][5];

} DatMesh;




/*-----------------
 Remesh structures
-----------------*/
typedef struct FACETTE {
	Face_WSZT *face;
	struct FACETTE *suivant;
} Facette;

typedef struct QUEUE {
	Facette *debut;
} Queue;

typedef struct CHEMIN {
	double pos[3];
	int indice;
	struct CHEMIN *suivant;
} Chemin;

typedef struct CHEMINS {
	Chemin *chemin[NbVertexMax];
	int Top;
} Chemins;

typedef struct NODE {
	int voisin;
	struct NODE *arc;
} Node;

typedef struct FACEGRAPH {
	int NbNoeuds;
	Node *noeud[NbFaceMax];
	int marque[NbFaceMax];
} FaceGraph;

typedef struct CNOEUD {
	int noeud;
	struct CNOEUD *suivant;
} CNoeud;

typedef struct COMPGRAPH {
	int Top;
	int cardinal[NbFaceMax];
	int couleur[NbFaceMax];
	double normale[NbFaceMax][3];
	Node *noeud[NbFaceMax];
	int FaceActive[NbFaceMax];
} CompGraph;

typedef struct MATRICE {
	double mat[NMAX*NMAX];
} Matrice;

typedef struct PROFILE {
	int tab[NMAX][20];
} Profile;

typedef struct MATRICECREUSE {
	double mat[NMAX][20];
	int profile[NMAX][20];
} MatriceCreuse;

typedef struct MRSOMMET {
	int level;
	int F0;
	int alpha;
	int beta;
	int gamma;
	double mj[3];
	double coord[3];
	int Voisins[2]; /* Pour 0-disk seulement */
	double coeff[3];
	double bucket[3];
	int valance;
	Chemin *way;
} MRSommet;

typedef struct ARETE {
	int arete[2];
	struct ARETE *suivant;
} Arete;

typedef struct MRMESH { /* Destin a contenir les c^j */
	int level;
	MRSommet *Tab[NbVertexMax];
	Arete *aretes;
	int Top;
} MRmesh;



#endif

