#include "data.h"
#include "3d.h"
#include "wavecore.h"
#include "zerotree.h"

/*-----------------------------------------------------
  Classe dcodeur/encodeur de maillage par ondelettes
-----------------------------------------------------*/

#ifndef _ZTEncDecDEFINED_
#define _ZTEncDecDEFINED_

class ZTEncDec {

/*----------
Attributes
----------*/
public:
	// A terme mettre a dans un CWMesh
	CWMesh *wmesh;

	zerotree *ZT;
	EncoderConfig *m_EC;
	DecSpecInfo *m_DSI;
	//Encoding / decoding parameters
	bool processType; // TRUE => decoding
	int inputType;  // 0 =>
					// 1 =>
	int outputType;

/*-----
Methods
-------*/

public:
	ZTEncDec();
	inline bool SetDefaultEncoderConfig() {
		free(m_EC);
		m_EC = new EncoderConfig();
		return true;
	}
	bool MoveQToDM();

	bool ZTEncode();
	bool ZTEncode(bool, char *inputfile);
	bool ZTEncode(unsigned char *bufferout, int bufmaxsize, int &size);
	//Local out of core encoding, with binary output plus BSD
	bool ZTEncodeFromWR(unsigned char *bufferout, int bufmaxsize, int &size, BSD*);

	bool ZTDecode();
	bool ZTDecode(char *inputfile, char *DSIfile);
	bool ZTDecode(unsigned char *inputstream, int streamlength);//zerotree must already have a DSI
	bool ZTDecode_loc(unsigned char *inputstream, int streamlength, int NbCoeffs);//zerotree must already have a DSI
	int ZTDecode_generic(unsigned char *inputstream, int streamlength, int NbCoeffs);//zerotree must already have a DSI

	//Reconstruction
	bool MoveZToMesh(CWMesh *);
	int MoveLocZToWV(CWMesh *);

	//GetSet
	inline void SetBaseMesh(Mesh *mesh) {
		if (wmesh==NULL)
			wmesh = new CWMesh;
		wmesh->SetBaseMesh(mesh);
	}
	bool SetDSI(char *filename);

	//IO
	bool LoadMesh(char *);
	bool writeEncoded(char *filename_wo_ext, unsigned char *stream, int nbbytes);
	bool writeEncoded(char *filename_wo_ext, Mesh *BM, unsigned char *stream, int nbbytes);
	int MakeBitsreamFromBinaryAndBSD(unsigned char* inputbinary, BSD* BSDesc, unsigned char* outputstream, int& bufsize);
	int MakeBitsreamFromBinaryAndBSDPartial(int ztid, unsigned char* inputbinary, BSD* BSDesc, unsigned char* outputstream, int& bufsize);

	int getComponent(unsigned short ztid, unsigned short composante, unsigned int& nbCoeff, unsigned char& bitplane, int *baryCoord[4], float* Z);

};
#endif