//#include "proto.h"

#include <stdio.h>
#include <stdlib.h>
#include "datlib.h"
#include "math.h"
#include "3d.h"
#include "Q.h"
#include "WaveDec.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void chCoord3f(int alpha, int beta, int gamma, int F0a[3], int F0b[3], int *alphares, int *betares, int *gammares) {
	
	// Changement de coordonnes affines entre faces partageant au moins un sommet
	// I
	// alpha, beta, gamma : coordonnes barycentriques entieres
	// F0a, F0b : tableaux[5] des faces de M0 en format DAT
	// O
	//alphares, betares, gammares :coordonnes changes
	
	int ind1, ind2;

	//Cas trivial
	if (F0a[0]==F0b[0] && F0a[1]==F0b[1] && F0a[2]==F0b[2]) {
		*alphares=alpha;
		*betares=beta;
		*gammares=gamma;
		return;
	}
		

	if (alpha+beta+gamma==1) { // Sommet de niveau 0
		ind1=2*(alpha==1)+3*(beta==1)+4*(gamma==1)-2;
		ind2=(F0a[ind1]==F0b[0])+2*(F0a[ind1]==F0b[1])+3*(F0a[ind1]==F0b[2]);
		switch(ind2) {
		case 1:
			*alphares=1;
			*betares=0;
			*gammares=0;
			return;
		case 2:
			*alphares=0;
			*betares=1;
			*gammares=0;
			return;
		case 3:
			*alphares=0;
			*betares=0;
			*gammares=1;
			return;
		}
		return;
	} else {
		if (F0a[0]==F0b[0] && F0a[1]==F0b[2]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[0]==F0b[1] && F0a[1]==F0b[0]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[0]==F0b[2] && F0a[1]==F0b[1]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[1]==F0b[0] && F0a[2]==F0b[2]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[1]==F0b[2] && F0a[2]==F0b[1]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[1]==F0b[1] && F0a[2]==F0b[0]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[0]==F0b[0] && F0a[2]==F0b[1]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[0]==F0b[2] && F0a[2]==F0b[0]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[0]==F0b[1] && F0a[2]==F0b[2]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		return;
	}
}

void chCoord3s(int alpha, int beta, int gamma, int F0a[3], int F0b[3],  char *alphares,  char *betares,  char *gammares) {
	
	// Changement de coordonnes affines entre faces partageant au moins un sommet
	// I
	// alpha, beta, gamma : coordonnes barycentriques entieres
	// F0a, F0b : tableaux[5] des faces de M0 en format DAT
	// O
	//alphares, betares, gammares :coordonnes changes
	
	int ind1, ind2;

	//Cas trivial
	if (F0a[0]==F0b[0] && F0a[1]==F0b[1] && F0a[2]==F0b[2]) {
		*alphares=alpha;
		*betares=beta;
		*gammares=gamma;
		return;
	}
		

	if (alpha+beta+gamma==1) { // Sommet de niveau 0
		ind1=2*(alpha==1)+3*(beta==1)+4*(gamma==1)-2;
		ind2=(F0a[ind1]==F0b[0])+2*(F0a[ind1]==F0b[1])+3*(F0a[ind1]==F0b[2]);
		switch(ind2) {
		case 1:
			*alphares=1;
			*betares=0;
			*gammares=0;
			return;
		case 2:
			*alphares=0;
			*betares=1;
			*gammares=0;
			return;
		case 3:
			*alphares=0;
			*betares=0;
			*gammares=1;
			return;
		}
		return;
	} else {
		if (F0a[0]==F0b[0] && F0a[1]==F0b[2]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[0]==F0b[1] && F0a[1]==F0b[0]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[0]==F0b[2] && F0a[1]==F0b[1]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[1]==F0b[0] && F0a[2]==F0b[2]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[1]==F0b[2] && F0a[2]==F0b[1]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[1]==F0b[1] && F0a[2]==F0b[0]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[0]==F0b[0] && F0a[2]==F0b[1]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[0]==F0b[2] && F0a[2]==F0b[0]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[0]==F0b[1] && F0a[2]==F0b[2]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		return;
	}
}

void chCoord(int alpha, int beta, int gamma, int *F0a, int *F0b, int *alphares, int *betares, int *gammares) {
	
	// Changement de coordonnes affines entre faces partageant au moins un sommet
	// I
	// alpha, beta, gamma : coordonnes barycentriques entieres
	// F0a, F0b : tableaux[5] des faces de M0 en format DAT
	// O
	//alphares, betares, gammares :coordonnes changes
	
	int ind1, ind2;

	if (alpha+beta+gamma==1) { // Sommet de niveau 0
		ind1=2*(alpha==1)+3*(beta==1)+4*(gamma==1);
		ind2=(F0a[ind1]==F0b[2])+2*(F0a[ind1]==F0b[3])+3*(F0a[ind1]==F0b[4]);
		switch(ind2) {
		case 1:
			*alphares=1;
			*betares=0;
			*gammares=0;
			return;
		case 2:
			*alphares=0;
			*betares=1;
			*gammares=0;
			return;
		case 3:
			*alphares=0;
			*betares=0;
			*gammares=1;
			return;
		}
		fprintf(stderr, "\n Incoherence dans changement de coordonnes : chCoord() sommet de niveau 0 sans voisinage\n");
		exit(0);
	} else {
		if (F0a[2]==F0b[2] && F0a[3]==F0b[4]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[2]==F0b[3] && F0a[3]==F0b[2]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[2]==F0b[4] && F0a[3]==F0b[3]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[3]==F0b[2] && F0a[4]==F0b[4]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		if (F0a[3]==F0b[4] && F0a[4]==F0b[3]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[3]==F0b[3] && F0a[4]==F0b[2]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[2]==F0b[2] && F0a[4]==F0b[3]) {
			*alphares=alpha;
			*betares=gamma;
			*gammares=beta;
			return;
		}
		if (F0a[2]==F0b[4] && F0a[4]==F0b[2]) {
			*alphares=gamma;
			*betares=beta;
			*gammares=alpha;
			return;
		}
		if (F0a[2]==F0b[3] && F0a[4]==F0b[4]) {
			*alphares=beta;
			*betares=alpha;
			*gammares=gamma;
			return;
		}
		fprintf(stderr, "\n Incoherence dans changement de coordonnes : chCoord() sommet de niveau >1 sans voisinage\n");
		exit(0);
	}
}

DatMesh* loadDAT2MRM(MRmesh *mrm, char *filename) {


// Remplit un MRmesh  partir d'un fichier dat
	
	FILE *f=fopen(filename, "r");
	char *tmp=(char *)malloc(512*sizeof(char));
	int level;
	int i;
	int F0=-1;
	int rem1, rem2, rem3;
	DatMesh *DM=(DatMesh*)malloc(sizeof(DatMesh));
	int *F0index=(int*)malloc(NbFaceMax*sizeof(int));



	for (i=0; i<NbFaceMax; i++) {
		F0index[i]=-1;
	}

	// Lecture du niveau
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "depth"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%d", &level);
	mrm->level=level;
	
	// Lecture des coeffs
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "Vertices"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%s", tmp);
	DM->NbVertices=0;
	while (strcmp(tmp, "Triangles")) {
		DM->V[DM->NbVertices][0]=atoi(tmp);
		fscanf(f, "%f", &DM->V[DM->NbVertices][1]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][2]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][3]);
		DM->NbVertices++;
		if (DM->NbVertices==NbVertexMax) {
			fprintf(stderr, "Erreur : trop de sommets dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}
	mrm->Top=DM->NbVertices;

	// Lecture des coeffs
	DM->NbFaces=0;
	fscanf(f, "%s", tmp); // name:
	while (strcmp(tmp, "end")) {
		fscanf(f, "%d", &DM->F[DM->NbFaces][0]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][1]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][2]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][3]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][4]);
		DM->NbFaces++;
		if (DM->NbFaces==NbFaceMax) {
			fprintf(stderr, "Erreur : trop de faces dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}

	//INIT de mrm
	for (i=0; i<DM->NbFaces; i++) {
		mrm->Tab[i]=NULL;
	}

	// Parcours des faces
	for (i=0; i<DM->NbFaces; i++) {
		if (DM->F[i][1]==1) {
			F0++;
			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][2]]==NULL) {
				MRSommet *s1=(MRSommet*)malloc(sizeof(MRSommet));
				s1->bucket[0]=s1->bucket[1]=s1->bucket[2]=0;
				s1->coeff[0]=s1->coeff[2]=s1->coeff[2]=0;
				s1->coord[0]=DM->V[DM->F[i][2]][1];
				s1->coord[1]=DM->V[DM->F[i][2]][2];
				s1->coord[2]=DM->V[DM->F[i][2]][3];
				s1->F0=F0;
				s1->alpha=1;
				s1->beta=0;
				s1->gamma=0;
				s1->Voisins[0]=-1;
				s1->Voisins[1]=-1;
				s1->level=0;
				mrm->Tab[DM->F[i][2]]=s1;
			}
			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][3]]==NULL) {														
				MRSommet *s2=(MRSommet*)malloc(sizeof(MRSommet));
				s2->bucket[0]=s2->bucket[1]=s2->bucket[2]=0;
				s2->coeff[0]=s2->coeff[2]=s2->coeff[2]=0;
				s2->coord[0]=DM->V[DM->F[i][3]][1];
				s2->coord[1]=DM->V[DM->F[i][3]][2];
				s2->coord[2]=DM->V[DM->F[i][3]][3];
				s2->F0=F0;
				s2->alpha=0;
				s2->beta=1;
				s2->gamma=0;
				s2->Voisins[0]=-1;
				s2->Voisins[1]=-1;
				s2->level=0;
				mrm->Tab[DM->F[i][3]]=s2;
			}

			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][4]]==NULL) {
				MRSommet *s3=(MRSommet *)malloc(sizeof(MRSommet));
				s3->bucket[0]=s3->bucket[1]=s3->bucket[2]=0;
				s3->coeff[0]=s3->coeff[2]=s3->coeff[2]=0;
				s3->coord[0]=DM->V[DM->F[i][4]][1];
				s3->coord[1]=DM->V[DM->F[i][4]][2];
				s3->coord[2]=DM->V[DM->F[i][4]][3];
				s3->F0=F0;
				s3->alpha=0;
				s3->beta=0;
				s3->gamma=1;
				s3->Voisins[0]=-1;
				s3->Voisins[1]=-1;
				s3->level=0;
				mrm->Tab[DM->F[i][4]]=s3;
			}

			F0index[F0]=i;
		
	
		} else if (DM->F[i][0]==0) {
			int al, be, ga, alp, bet, gam;
			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][2]]==NULL) {	
				MRSommet *s1=(MRSommet *)malloc(sizeof(MRSommet));
				s1->bucket[0]=s1->bucket[1]=s1->bucket[2]=0;
				s1->coeff[0]=s1->coeff[2]=s1->coeff[2]=0;
				s1->coord[0]=DM->V[DM->F[i][2]][1];
				s1->coord[1]=DM->V[DM->F[i][2]][2];
				s1->coord[2]=DM->V[DM->F[i][2]][3];
				s1->F0=F0; 
				s1->Voisins[0]=rem2;
				s1->Voisins[1]=rem3;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (mrm->Tab[s1->Voisins[0]]->F0!=s1->F0) {
					chCoord(mrm->Tab[s1->Voisins[0]]->alpha, mrm->Tab[s1->Voisins[0]]->beta, mrm->Tab[s1->Voisins[0]]->gamma, DM->F[F0index[mrm->Tab[s1->Voisins[0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=mrm->Tab[s1->Voisins[0]]->alpha;
					be=mrm->Tab[s1->Voisins[0]]->beta;
					ga=mrm->Tab[s1->Voisins[0]]->gamma;
				}
				if (mrm->Tab[s1->Voisins[1]]->F0!=s1->F0) {
					chCoord(mrm->Tab[s1->Voisins[1]]->alpha, mrm->Tab[s1->Voisins[1]]->beta, mrm->Tab[s1->Voisins[1]]->gamma, DM->F[F0index[mrm->Tab[s1->Voisins[1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=mrm->Tab[s1->Voisins[1]]->alpha;
					bet=mrm->Tab[s1->Voisins[1]]->beta;
					gam=mrm->Tab[s1->Voisins[1]]->gamma;
				}// Les coordonnes (al be ga) et (alp bet gam) pour 
				// respectivement voisins[0] et voisins[1] sont exprimes dans le mme repre (celui de F0)

				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				s1->alpha=al+alp;
				s1->beta=be+bet;
				s1->gamma=ga+gam;
				//Reduction par 2^n
				while (s1->alpha%2==0 && s1->beta%2==0 && s1->gamma%2==0) {
					s1->alpha/=2;
					s1->beta/=2;
					s1->gamma/=2;
				}
				//Divers
				s1->level=level+1-DM->V[DM->F[i][2]][0];
				mrm->Tab[DM->F[i][2]]=s1;
			}
			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][3]]==NULL) {
				MRSommet *s2=(MRSommet *)malloc(sizeof(MRSommet));
				s2->bucket[0]=s2->bucket[1]=s2->bucket[2]=0;
				s2->coeff[0]=s2->coeff[2]=s2->coeff[2]=0;
				s2->coord[0]=DM->V[DM->F[i][3]][1];
				s2->coord[1]=DM->V[DM->F[i][3]][2];
				s2->coord[2]=DM->V[DM->F[i][3]][3];
				s2->F0=F0;
				s2->Voisins[0]=rem3;
				s2->Voisins[1]=rem1;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (mrm->Tab[s2->Voisins[0]]->F0!=s2->F0) {
					chCoord(mrm->Tab[s2->Voisins[0]]->alpha, mrm->Tab[s2->Voisins[0]]->beta, mrm->Tab[s2->Voisins[0]]->gamma, DM->F[F0index[mrm->Tab[s2->Voisins[0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=mrm->Tab[s2->Voisins[0]]->alpha;
					be=mrm->Tab[s2->Voisins[0]]->beta;
					ga=mrm->Tab[s2->Voisins[0]]->gamma;
				}
				if (mrm->Tab[s2->Voisins[1]]->F0!=s2->F0) {
					chCoord(mrm->Tab[s2->Voisins[1]]->alpha, mrm->Tab[s2->Voisins[1]]->beta, mrm->Tab[s2->Voisins[1]]->gamma, DM->F[F0index[mrm->Tab[s2->Voisins[1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=mrm->Tab[s2->Voisins[1]]->alpha;
					bet=mrm->Tab[s2->Voisins[1]]->beta;
					gam=mrm->Tab[s2->Voisins[1]]->gamma;
				}// Les coordonnes (al be ga) et (alp bet gam) pour 
				// respectivement voisins[0] et voisins[1] sont exprimes dans le mme repre (celui de F0)

				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				s2->alpha=al+alp;
				s2->beta=be+bet;
				s2->gamma=ga+gam;
				//Reduction par 2^n
				while (s2->alpha%2==0 && s2->beta%2==0 && s2->gamma%2==0) {
					s2->alpha/=2;
					s2->beta/=2;
					s2->gamma/=2;
				}
				//Divers
				s2->level=level+1-DM->V[DM->F[i][3]][0];
				mrm->Tab[DM->F[i][3]]=s2;
			}

			//Creation du mrsommet
			if (mrm->Tab[DM->F[i][4]]==NULL) {
				MRSommet *s3=(MRSommet *)malloc(sizeof(MRSommet));
				// DEBUG
				if (DM->F[i][4]==1010) {
					i=i;	
				}
														// FDEBUG
				s3->bucket[0]=s3->bucket[1]=s3->bucket[2]=0;
				s3->coeff[0]=s3->coeff[2]=s3->coeff[2]=0;
				s3->coord[0]=DM->V[DM->F[i][4]][1];
				s3->coord[1]=DM->V[DM->F[i][4]][2];
				s3->coord[2]=DM->V[DM->F[i][4]][3];
				s3->F0=F0;
				s3->Voisins[0]=rem1;
				s3->Voisins[1]=rem2;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (mrm->Tab[s3->Voisins[0]]->F0!=s3->F0) {
					chCoord(mrm->Tab[s3->Voisins[0]]->alpha, mrm->Tab[s3->Voisins[0]]->beta, mrm->Tab[s3->Voisins[0]]->gamma, DM->F[F0index[mrm->Tab[s3->Voisins[0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=mrm->Tab[s3->Voisins[0]]->alpha;
					be=mrm->Tab[s3->Voisins[0]]->beta;
					ga=mrm->Tab[s3->Voisins[0]]->gamma;
				}
				if (mrm->Tab[s3->Voisins[1]]->F0!=s3->F0) {
					chCoord(mrm->Tab[s3->Voisins[1]]->alpha, mrm->Tab[s3->Voisins[1]]->beta, mrm->Tab[s3->Voisins[1]]->gamma, DM->F[F0index[mrm->Tab[s3->Voisins[1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=mrm->Tab[s3->Voisins[1]]->alpha;
					bet=mrm->Tab[s3->Voisins[1]]->beta;
					gam=mrm->Tab[s3->Voisins[1]]->gamma;
				}// Les coordonnes (al be ga) et (alp bet gam) pour 
				// respectivement voisins[0] et voisins[1] sont exprimes dans le mme repre (celui de F0)

				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				s3->alpha=al+alp;
				s3->beta=be+bet;
				s3->gamma=ga+gam;
				//Reduction par 2^n
				while (s3->alpha%2==0 && s3->beta%2==0 && s3->gamma%2==0) {
					s3->alpha/=2;
					s3->beta/=2;
					s3->gamma/=2;
				}
				//Divers
				s3->level=level+1-DM->V[DM->F[i][4]][0];
				mrm->Tab[DM->F[i][4]]=s3;
			}
		}
		rem1=DM->F[i][2];
		rem2=DM->F[i][3];
		rem3=DM->F[i][4];
	}


	return DM;
	

}

void loadDAT(DatMesh *DM, char *filename) {


// Remplit un MRmesh  partir d'un fichier dat
	
	FILE *f=fopen(filename, "r");
	char *tmp=(char *)malloc(512*sizeof(char));
	int level;
	int i;
	int F0=-1;
	int *F0index=(int*)malloc(NbFaceMax*sizeof(int));



	for (i=0; i<NbFaceMax; i++) {
		F0index[i]=-1;
	}

	// Lecture du niveau
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "depth"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%d", &level);
	
	// Lecture des coeffs
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "Vertices"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%s", tmp);
	DM->NbVertices=0;
	while (strcmp(tmp, "Triangles")) {
		DM->V[DM->NbVertices][0]=atoi(tmp);
		fscanf(f, "%f", &DM->V[DM->NbVertices][1]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][2]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][3]);
		DM->NbVertices++;
		if (DM->NbVertices==NbVertexMax) {
			fprintf(stderr, "Erreur : trop de sommets dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}

	// Lecture des coeffs
	DM->NbFaces=0;
	fscanf(f, "%s", tmp); // name:
	while (strcmp(tmp, "end")) {
		fscanf(f, "%d", &DM->F[DM->NbFaces][0]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][1]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][2]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][3]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][4]);
		DM->NbFaces++;
		if (DM->NbFaces==NbFaceMax) {
			fprintf(stderr, "Erreur : trop de faces dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}


}

void loadDAT(DatMesh *DM, char *filename, Mesh *M0) {


	
	FILE *f=fopen(filename, "r");
	char *tmp=(char *)malloc(512*sizeof(char));
	int level;
	int i;
	int F0=-1;
	int *F0index=(int*)malloc(NbFaceMax*sizeof(int));


	//Init M0
	M0->NbFace=M0->NbVertex=0;

	for (i=0; i<NbFaceMax; i++) {
		F0index[i]=-1;
	}

	// Lecture du niveau
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "depth"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%d", &level);
	
	// Lecture des coeffs
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "Vertices"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%s", tmp);
	DM->NbVertices=0;
	while (strcmp(tmp, "Triangles")) {
		DM->V[DM->NbVertices][0]=atoi(tmp);
		fscanf(f, "%f", &DM->V[DM->NbVertices][1]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][2]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][3]);
		//On s'occupe de M0
		if (DM->V[DM->NbVertices][0]==level+1) {
			M0->Vertex[M0->NbVertex][0]=DM->V[DM->NbVertices][1];
			M0->Vertex[M0->NbVertex][1]=DM->V[DM->NbVertices][2];
			M0->Vertex[M0->NbVertex][2]=DM->V[DM->NbVertices][3];
			M0->NbVertex++;
		}
		//
		DM->NbVertices++;
		if (DM->NbVertices==NbVertexMax) {
			fprintf(stderr, "Erreur : trop de sommets dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}

	// Lecture des coeffs
	DM->NbFaces=0;
	fscanf(f, "%s", tmp); // name:
	while (strcmp(tmp, "end")) {
		fscanf(f, "%d", &DM->F[DM->NbFaces][0]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][1]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][2]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][3]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][4]);
		//On s'occupe de M0
		if (DM->F[DM->NbFaces][1]==1) {//Root
			M0->Face[M0->NbFace][0]=DM->F[DM->NbFaces][2];
			M0->Face[M0->NbFace][1]=DM->F[DM->NbFaces][3];
			M0->Face[M0->NbFace][2]=DM->F[DM->NbFaces][4];
			M0->NbFace++;
		}
		//
		DM->NbFaces++;
		if (DM->NbFaces==NbFaceMax) {
			fprintf(stderr, "Erreur : trop de faces dans le maillage en entree. loadDAT2MRM\n");
		}
		fscanf(f, "%s", tmp);
	}


}

void ecritCoeffsWV(DatMesh *DM, MRmesh *mrm, FILE *g) {

	/* Ecrit les coefficients d'ondelettes stockes dans un maillage */
	/* multiresolution apres passage au filterbank */

	/* Le fichier doit etre ouvert */

	int i;

	for (i=0; i<mrm->Top; i++) {
		//if (fabs(mrm->Tab[i]->coeff[0])+fabs(mrm->Tab[i]->coeff[1])+fabs(mrm->Tab[i]->coeff[2])>0.00005
		//	|| mrm->Tab[i]->alpha+mrm->Tab[i]->beta+mrm->Tab[i]->gamma==1) {
			if (mrm->Tab[i]->level>0) {
				fprintf(g, "%d ", mrm->Tab[i]->F0);
				fprintf(g, "%d ", mrm->Tab[i]->alpha);
				fprintf(g, "%d ", mrm->Tab[i]->beta);
				fprintf(g, "%d ", mrm->Tab[i]->gamma);
				fprintf(g, "%f ", mrm->Tab[i]->coeff[0]);
				fprintf(g, "%f ", mrm->Tab[i]->coeff[1]);
				fprintf(g, "%f\n", mrm->Tab[i]->coeff[2]);
			} else {
				fprintf(g, "%d ", mrm->Tab[i]->F0);// La mme chose maintenant
				fprintf(g, "%d ", mrm->Tab[i]->alpha);
				fprintf(g, "%d ", mrm->Tab[i]->beta);
				fprintf(g, "%d ", mrm->Tab[i]->gamma);
				fprintf(g, "%f ", mrm->Tab[i]->coeff[0]);
				fprintf(g, "%f ", mrm->Tab[i]->coeff[1]);
				fprintf(g, "%f\n", mrm->Tab[i]->coeff[2]);
			}
		//}
	}
	fprintf(g, "-1");
}

void ecritCoeffsDat(DatMesh *DM, MRmesh *mrm, FILE *g) {
	// Ecrit les coeffs du MRmesh selon la topologie du DatMesh dans un fichier WV
	// g doit tre ouvert

	int i;

	fprintf(g, "Multires data file\n");
	fprintf(g, "depth %d\n", mrm->level);
	fprintf(g, "Vertices\n");
	for (i=0; i<mrm->Top; i++) {
		if ((int)DM->V[i][0]!=mrm->level+1)
			fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], mrm->Tab[i]->coeff[0], mrm->Tab[i]->coeff[1], 
			mrm->Tab[i]->coeff[2]);
		else
			fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], 0.0, 0.0, 0.0);
		if ((int)DM->V[i][0]!=(int)DM->V[0][0])
			fprintf(g, "\n");
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		fprintf(g, "name: %d   %d   %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}

	fprintf(g, "end");
}

void ecritM0wv(char rac[512], MRmesh *mrm, DatMesh *DM) {

	int i;
	FILE *g=fopen(strcat(rac, ".wrl"), "w");

	/* Ecriture dans un fichier VRML2 */
	
	/* Entete */

    fprintf(g, "#VRML V2.0 utf8\n");   
	fprintf(g, "\n");
	fprintf(g, "Shape {\n");
	fprintf(g, "  appearance Appearance {\n");
	fprintf(g, "    material Material {\n");
	fprintf(g, "      diffuseColor 0 1.0 0\n");
	fprintf(g, "    }\n");
	fprintf(g, "  }\n");
	fprintf(g, "  geometry IndexedFaceSet {\n");
	fprintf(g, "    coord Coordinate {\n");
	fprintf(g, "      point [\n");

	/* Sommets */

    for (i=0; (fabs(DM->V[i][0]-DM->V[0][0]))<0.5; i++) {
		
		if (i>0) fprintf(g, ",\n");
		fprintf(g, "%f %f %f", /*DM->V[i][1]-*/mrm->Tab[i]->coeff[0], /*DM->V[i][2]-*/mrm->Tab[i]->coeff[1], /*DM->V[i][3]-*/mrm->Tab[i]->coeff[2]);

	}
	fprintf(g, " \n]\n");
	fprintf(g, "    }\n");
	fprintf(g, "    coordIndex [\n");

	/* Facettes */

    for (i=0; i<DM->NbFaces; i++) {
		if (DM->F[i][1]==1) {
			if (i>0) fprintf(g, ",\n");
			fprintf(g, "%d, %d, %d, -1", DM->F[i][2], DM->F[i][3], DM->F[i][4]);	
		}
	}
	fprintf(g, " \n]\n");
	fprintf(g, "  }\n");
	fprintf(g, "}\n");  

	fclose(g);
}

void ecritM0pgc(char rac[512], MRmesh *mrm, DatMesh *DM) {

	int i;
	FILE *g=fopen(strcat(rac, ".pgc"), "w");

	fprintf(g, "Multires data file\n");
	fprintf(g, "depth 0\n");
	fprintf(g, "Vertices\n");
	for (i=0; DM->V[i][0]==DM->V[0][0]; i++) {
		fprintf(g, "1 %f %f %f\n", DM->V[i][1]-mrm->Tab[i]->coeff[0], DM->V[i][2]-mrm->Tab[i]->coeff[1], 
		/**/DM->V[i][3]-mrm->Tab[i]->coeff[2]);
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		if (DM->F[i][1]==1)
			fprintf(g, "name: %d %d %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}
	fprintf(g, "end");

	fclose(g);
}


void DAT2MJ(MRmesh *mrm, DatMesh *DM) {

	// Fait un Mj  partir d'un fichier dat
	// Non inversible evidemment

	FILE *g=fopen("out", "w");
	int i;
	int encore;


	for (i=0; i<mrm->Top; i++) {
		if (DM->V[i][0]==DM->V[0][0]) {
			mrm->Tab[i]->coeff[0]=DM->V[i][1];
			mrm->Tab[i]->coeff[1]=DM->V[i][2];
			mrm->Tab[i]->coeff[2]=DM->V[i][3];
			mrm->Tab[i]->alpha=0;
		} else {
			mrm->Tab[i]->alpha=-1;
		}
	}

	encore=1;
	while (encore) {
		encore=0;
		for (i=0; i<mrm->Top; i++) {
			if (mrm->Tab[i]->alpha==-1) {
				if (mrm->Tab[mrm->Tab[i]->Voisins[0]]->alpha!=-1 && mrm->Tab[mrm->Tab[i]->Voisins[1]]->alpha!=-1) {
					mrm->Tab[i]->alpha=0;
					mrm->Tab[i]->coeff[0]=0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[0]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[0]);
					mrm->Tab[i]->coeff[1]=0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[1]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[1]);
					mrm->Tab[i]->coeff[2]=0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[2]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[2]);
				} else
					encore=1;
					
			}
		}
	}

	// Ecriture
	fprintf(g, "Multires data file\n");
	fprintf(g, "depth 0\n");
	fprintf(g, "Vertices\n");
	for (i=0; i<DM->NbVertices; i++) {
		fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], mrm->Tab[i]->coeff[0], mrm->Tab[i]->coeff[1], 
		/**/mrm->Tab[i]->coeff[2]);
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		fprintf(g, "name: %d %d %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}
	fprintf(g, "end");

	fclose(g);

	//Vrification TBD
/*	for (i=251; i<mrm->Top; i++) {
		if (mrm->Tab[i]->alpha==-1)
			fprintf(stderr, "ERREUR : sommet -1\n");
		else {
			if ((fabs(mrm->Tab[i]->coeff[0]-0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[0]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[0]))>0.0001) ||
			(fabs(mrm->Tab[i]->coeff[1]-0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[1]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[1]))>0.0001)||
			(fabs(mrm->Tab[i]->coeff[2]-0.5*(mrm->Tab[mrm->Tab[i]->Voisins[0]]->coeff[2]+mrm->Tab[mrm->Tab[i]->Voisins[1]]->coeff[2]))>0.0001))

			fprintf(stderr, "ERREUR !\n");

		}
	}
*/



}


/*
void DAT2MJ(char filename[512]) {

	// Fait un Mj  partir d'un fichier dat
	// Non inversible evidemment

	FILE *f=fopen(filename, "r"), *g=fopen("out", "w");
	char *tmp=malloc(512*sizeof(char));
	int level;
	int i;
	int F0=-1;
	int rem1, rem2, rem3;
	DatMesh *DM=malloc(sizeof(DatMesh));
	DatMesh *DMout=malloc(sizeof(DatMesh));
	int *F0index=malloc(NbFaceMax*sizeof(int));



	for (i=0; i<NbFaceMax; i++) {
		F0index[i]=-1;
	}

	// Lecture du niveau
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "depth"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%d", &level);
	
	// Lecture des coeffs
	fscanf(f, "%s", tmp);
	while (strcmp(tmp, "Vertices"))
		fscanf(f, "%s", tmp);
	fscanf(f, "%s", tmp);
	DM->NbVertices=0;
	while (strcmp(tmp, "Triangles")) {
		DM->V[DM->NbVertices][0]=atoi(tmp);
		fscanf(f, "%f", &DM->V[DM->NbVertices][1]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][2]);
		fscanf(f, "%f", &DM->V[DM->NbVertices][3]);
		DM->NbVertices++;
		if (DM->NbVertices==NbVertexMax) {
			fprintf(stderr, "Erreur : trop de sommets dans le maillage en entree. dat2mj\n");
		}
		fscanf(f, "%s", tmp);
	}

	// Lecture des coeffs
	DM->NbFaces=0;
	fscanf(f, "%s", tmp); // name:
	while (strcmp(tmp, "end")) {
		fscanf(f, "%d", &DM->F[DM->NbFaces][0]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][1]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][2]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][3]);
		fscanf(f, "%d", &DM->F[DM->NbFaces][4]);
		DM->NbFaces++;
		if (DM->NbFaces==NbFaceMax) {
			fprintf(stderr, "Erreur : trop de faces dans le maillage en entree. dat2mj\n");
		}
		fscanf(f, "%s", tmp);
	}

	//DMout
	DMout->NbVertices=DM->NbVertices;
	DMout->NbFaces=DM->NbFaces;
	for (i=0; i<DM->NbVertices; i++) {
		DM->V[i][0]=-1;
	}
	for (i=0; i<DM->NbFaces; i++) {
		DMout->F[i][0]=DM->F[i][0];
		DMout->F[i][1]=DM->F[i][1];
		DMout->F[i][2]=DM->F[i][2];
		DMout->F[i][3]=DM->F[i][3];
		DMout->F[i][4]=DM->F[i][4];
		//Init
		if (DM->F[i][1]==1) {
			DMout->V[DMout->F[i][2]][0]=DM->V[DM->F[i][2]][0];
			DMout->V[DMout->F[i][2]][1]=DM->V[DM->F[i][2]][1];
			DMout->V[DMout->F[i][2]][2]=DM->V[DM->F[i][2]][2];

			DMout->V[DMout->F[i][3]][0]=DM->V[DM->F[i][3]][0];
			DMout->V[DMout->F[i][3]][1]=DM->V[DM->F[i][3]][1];
			DMout->V[DMout->F[i][3]][2]=DM->V[DM->F[i][3]][2];

			DMout->V[DMout->F[i][4]][0]=DM->V[DM->F[i][4]][0];
			DMout->V[DMout->F[i][4]][1]=DM->V[DM->F[i][4]][1];
			DMout->V[DMout->F[i][4]][2]=DM->V[DM->F[i][4]][2];
		}
	}

	for (i=0; i<DM->NbFaces; i++) {
		//Iter
		if (DM->F[i][1]!=1) {
			DMout->V[DMout->F[i][2]][0]=DM->V[DM->F[i][2]][0];
			DMout->V[DMout->F[i][2]][1]=DM->V[DM->F[i][2]][1];
			DMout->V[DMout->F[i][2]][2]=DM->V[DM->F[i][2]][2];
		}
	}

}
*/

int **datfaces2wvbary(int **faces, int NbVertices, int NbFaces) {
	// faces : les faces d'un aillage format DAT
	// NbVertices : nombre de sommets du maillage
	// NbFaces : nombre de Faces du maillage
	// retourne un tableau [NbVertices][4] contenant les (F0 alpha beta gamma)

	int **bary=(int **)malloc(NbVertices*sizeof(int*));
	int **parents=(int **)malloc(NbVertices*sizeof(int*));
	int i;
	int F0=-1;
	int rem1, rem2, rem3;
	int *F0index=(int *)malloc(NbFaceMax*sizeof(int));

	// Init
	for (i=0; i<NbVertices; i++) {
		bary[i]=(int *)malloc(4*sizeof(int));
		parents[i]=(int *)malloc(2*sizeof(int));
		bary[i][0]=-1;
	}

	// on y va...
	for (i=0; i< NbFaces; i++) {
		if (faces[i][1]==1) {
			F0++;
			// Et d'un
			if (bary[faces[i][2]][0]==-1) {
				bary[faces[i][2]][0]=F0;
				bary[faces[i][2]][1]=1;
				bary[faces[i][2]][2]=0;
				bary[faces[i][2]][3]=0;
				parents[faces[i][2]][0]=-1;
				parents[faces[i][2]][1]=-1;
			}
			// Et de deux
			if (bary[faces[i][3]][0]==-1) {
				bary[faces[i][3]][0]=F0;
				bary[faces[i][3]][1]=0;
				bary[faces[i][3]][2]=1;
				bary[faces[i][3]][3]=0;
				parents[faces[i][3]][0]=-1;
				parents[faces[i][3]][1]=-1;
			}
			// Et de trois
			if (bary[faces[i][4]][0]==-1) {
				bary[faces[i][4]][0]=F0;
				bary[faces[i][4]][1]=0;
				bary[faces[i][4]][2]=0;
				bary[faces[i][4]][3]=1;
				parents[faces[i][4]][0]=-1;
				parents[faces[i][4]][1]=-1;
			}
			F0index[F0]=i;
		} else if (faces[i][0]==0) {
			int al, be, ga, alp, bet, gam;
		// Et d'un
			if (bary[faces[i][2]][0]==-1) {
				bary[faces[i][2]][0]=F0;
				parents[faces[i][2]][0]=rem2;
				parents[faces[i][2]][1]=rem3;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (bary[parents[faces[i][2]][0]][0]!=F0) {
					chCoord(bary[parents[faces[i][2]][0]][1], bary[parents[faces[i][2]][0]][2], bary[parents[faces[i][2]][0]][3], faces[F0index[bary[parents[faces[i][2]][0]][0]]], faces[F0index[F0]], &al, &be, &ga);
				} else {
					al=bary[parents[faces[i][2]][0]][1];
					be=bary[parents[faces[i][2]][0]][2];
					ga=bary[parents[faces[i][2]][0]][3];
				}
				if (bary[parents[faces[i][2]][1]][0]!=F0) {
					chCoord(bary[parents[faces[i][2]][1]][1], bary[parents[faces[i][2]][1]][2], bary[parents[faces[i][2]][1]][3], faces[F0index[bary[parents[faces[i][2]][1]][0]]], faces[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=bary[parents[faces[i][2]][1]][1];
					bet=bary[parents[faces[i][2]][1]][2];
					gam=bary[parents[faces[i][2]][1]][3];
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				bary[faces[i][2]][1]=al+alp;
				bary[faces[i][2]][2]=be+bet;
				bary[faces[i][2]][3]=ga+gam;
				//Reduction par 2^n
				while (bary[faces[i][2]][1]%2==0 && bary[faces[i][2]][2]%2==0 && bary[faces[i][2]][3]%2==0) {
					bary[faces[i][2]][1]/=2;
					bary[faces[i][2]][2]/=2;
					bary[faces[i][2]][3]/=2;
				}
				
			}
			// Et de deux
			if (bary[faces[i][3]][0]==-1) {
				bary[faces[i][3]][0]=F0;
				parents[faces[i][3]][0]=rem3;
				parents[faces[i][3]][1]=rem1;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (bary[parents[faces[i][3]][0]][0]!=F0) {
					chCoord(bary[parents[faces[i][3]][0]][1], bary[parents[faces[i][3]][0]][2], bary[parents[faces[i][3]][0]][3], faces[F0index[bary[parents[faces[i][3]][0]][0]]], faces[F0index[F0]], &al, &be, &ga);
				} else {
					al=bary[parents[faces[i][3]][0]][1];
					be=bary[parents[faces[i][3]][0]][2];
					ga=bary[parents[faces[i][3]][0]][3];
				}
				if (bary[parents[faces[i][3]][1]][0]!=F0) {
					chCoord(bary[parents[faces[i][3]][1]][1], bary[parents[faces[i][3]][1]][2], bary[parents[faces[i][3]][1]][3], faces[F0index[bary[parents[faces[i][3]][1]][0]]], faces[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=bary[parents[faces[i][3]][1]][1];
					bet=bary[parents[faces[i][3]][1]][2];
					gam=bary[parents[faces[i][3]][1]][3];
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				bary[faces[i][3]][1]=al+alp;
				bary[faces[i][3]][2]=be+bet;
				bary[faces[i][3]][3]=ga+gam;
				//Reduction par 2^n
				while (bary[faces[i][3]][1]%2==0 && bary[faces[i][3]][2]%2==0 && bary[faces[i][3]][3]%2==0) {
					bary[faces[i][3]][1]/=2;
					bary[faces[i][3]][2]/=2;
					bary[faces[i][3]][3]/=2;
				}
			}
			// Et de trois
			if (bary[faces[i][4]][0]==-1) {
				bary[faces[i][4]][0]=F0;
				parents[faces[i][4]][0]=rem1;
				parents[faces[i][4]][1]=rem2;
				// Expression des coordonnes barycentriques
				// des deux voisins dans le mme repre
				if (bary[parents[faces[i][4]][0]][0]!=F0) {
					chCoord(bary[parents[faces[i][4]][0]][1], bary[parents[faces[i][4]][0]][2], bary[parents[faces[i][4]][0]][3], faces[F0index[bary[parents[faces[i][4]][0]][0]]], faces[F0index[F0]], &al, &be, &ga);
				} else {
					al=bary[parents[faces[i][4]][0]][1];
					be=bary[parents[faces[i][4]][0]][2];
					ga=bary[parents[faces[i][4]][0]][3];
				}
				if (bary[parents[faces[i][4]][1]][0]!=F0) {
					chCoord(bary[parents[faces[i][4]][1]][1], bary[parents[faces[i][4]][1]][2], bary[parents[faces[i][4]][1]][3], faces[F0index[bary[parents[faces[i][4]][1]][0]]], faces[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=bary[parents[faces[i][4]][1]][1];
					bet=bary[parents[faces[i][4]][1]][2];
					gam=bary[parents[faces[i][4]][1]][3];
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				bary[faces[i][4]][1]=al+alp;
				bary[faces[i][4]][2]=be+bet;
				bary[faces[i][4]][3]=ga+gam;
				//Reduction par 2^n
				while (bary[faces[i][4]][1]%2==0 && bary[faces[i][4]][2]%2==0 && bary[faces[i][4]][3]%2==0) {
					bary[faces[i][4]][1]/=2;
					bary[faces[i][4]][2]/=2;
					bary[faces[i][4]][3]/=2;
				}
			}
		}
		rem1=faces[i][2];
		rem2=faces[i][3];
		rem3=faces[i][4];
	}

	//Libration
	for (i=0; i<NbVertices; i++) {
		free(bary[i]);
		free(parents[i]);
	}
	free(F0index);

	return bary;
}

void ecritCoordDat(DatMesh *DM, MRmesh *mrm, FILE *g) {
	// Ecrit les coord du MRmesh selon la topologie du DatMesh dans un fichier WV
	// g doit tre ouvert

	int i;

	fprintf(g, "Multires data file\n");
	fprintf(g, "depth %d\n", mrm->level);
	fprintf(g, "Vertices\n");
	for (i=0; i<mrm->Top; i++) {
		fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], mrm->Tab[i]->coord[0], mrm->Tab[i]->coord[1], 
		mrm->Tab[i]->coord[2]);
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		fprintf(g, "name: %d   %d   %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}

	fprintf(g, "end");
}

void ecritDat(DatMesh *DM, FILE *g) {

	int i;

	fprintf(g, "Multires data file\n");
	fprintf(g, "depth %d\n", (int)DM->V[0][0]-1);
	fprintf(g, "Vertices\n");
	for (i=0; i<DM->NbVertices; i++) {
			fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], DM->V[i][1], DM->V[i][2], 
			DM->V[i][3]);
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		fprintf(g, "name: %d   %d   %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}

	fprintf(g, "end");
}

void ecritDatNormalized(DatMesh *DM, FILE *g) {

	int i, dpj;

	fprintf(g, "Multires data file\n");
	fprintf(g, "depth %d\n", DM->V[0][0]-1);
	fprintf(g, "Vertices\n");
	for (i=0; i<DM->NbVertices; i++) {
			dpj=deuxPuissance((int)DM->V[0][0]+1-(int)DM->V[i][0]);
			fprintf(g, "%d %f %f %f\n", (int)DM->V[i][0], DM->V[i][1]*dpj, DM->V[i][2]*dpj, 
			DM->V[i][3]*dpj);
	}
	fprintf(g, "Triangles\n");
	for (i=0; i<DM->NbFaces; i++) {
		fprintf(g, "name: %d   %d   %d %d %d\n", DM->F[i][0], DM->F[i][1], DM->F[i][2], DM->F[i][3], DM->F[i][4]); 
	}

	fprintf(g, "end");
}


void Identity(MRmesh *mrm) {
	// Recopie les coordonnes dans les coefficients
	// Exemple d'utilisation : conversion d'un .dat en .wv

	int i;

	for (i=0; i<mrm->Top; i++) {
		mrm->Tab[i]->coeff[0]=mrm->Tab[i]->coord[0];
		mrm->Tab[i]->coeff[1]=mrm->Tab[i]->coord[1];
		mrm->Tab[i]->coeff[2]=mrm->Tab[i]->coord[2];
	}
}

int depthrec(Face_WSZT *F, Sommet *V1, Sommet *V2, Sommet *V3, DatMesh *DM, int root, int ordre) {

	//Appel par DumpQ2DatMesh()

	Sommet *Vp1, *Vp2, *Vp3;


	//Recopie
	DM->F[DM->NbFaces][0] = ordre;
	DM->F[DM->NbFaces][1] = root;
	DM->F[DM->NbFaces][2] = V1->indice;
	DM->F[DM->NbFaces][3] = V2->indice;
	DM->F[DM->NbFaces][4] = V3->indice;
	DM->NbFaces++;

	if (F->children[0]!=NULL) {
		//Dtermination des nouveaux sommets
		if (F->children[3]->sommetCoin[0]->parentV[0]==V1 && F->children[3]->sommetCoin[0]->parentV[1]==V2 || F->children[3]->sommetCoin[0]->parentV[0]==V2 && F->children[3]->sommetCoin[0]->parentV[1]==V1)
			Vp3=F->children[3]->sommetCoin[0];
		if (F->children[3]->sommetCoin[1]->parentV[0]==V1 && F->children[3]->sommetCoin[1]->parentV[1]==V2 || F->children[3]->sommetCoin[1]->parentV[0]==V2 && F->children[3]->sommetCoin[1]->parentV[1]==V1)
			Vp3=F->children[3]->sommetCoin[1];
		if (F->children[3]->sommetCoin[2]->parentV[0]==V1 && F->children[3]->sommetCoin[2]->parentV[1]==V2 || F->children[3]->sommetCoin[2]->parentV[0]==V2 && F->children[3]->sommetCoin[2]->parentV[1]==V1)
			Vp3=F->children[3]->sommetCoin[2];
		if (F->children[3]->sommetCoin[0]->parentV[0]==V1 && F->children[3]->sommetCoin[0]->parentV[1]==V3 || F->children[3]->sommetCoin[0]->parentV[0]==V3 && F->children[3]->sommetCoin[0]->parentV[1]==V1)
			Vp2=F->children[3]->sommetCoin[0];
		if (F->children[3]->sommetCoin[1]->parentV[0]==V1 && F->children[3]->sommetCoin[1]->parentV[1]==V3 || F->children[3]->sommetCoin[1]->parentV[0]==V3 && F->children[3]->sommetCoin[1]->parentV[1]==V1)
			Vp2=F->children[3]->sommetCoin[1];
		if (F->children[3]->sommetCoin[2]->parentV[0]==V1 && F->children[3]->sommetCoin[2]->parentV[1]==V3 || F->children[3]->sommetCoin[2]->parentV[0]==V3 && F->children[3]->sommetCoin[2]->parentV[1]==V1)
			Vp2=F->children[3]->sommetCoin[2];
		if (F->children[3]->sommetCoin[0]->parentV[0]==V2 && F->children[3]->sommetCoin[0]->parentV[1]==V3 || F->children[3]->sommetCoin[0]->parentV[0]==V3 && F->children[3]->sommetCoin[0]->parentV[1]==V2)
			Vp1=F->children[3]->sommetCoin[0];
		if (F->children[3]->sommetCoin[1]->parentV[0]==V2 && F->children[3]->sommetCoin[1]->parentV[1]==V3 || F->children[3]->sommetCoin[1]->parentV[0]==V3 && F->children[3]->sommetCoin[1]->parentV[1]==V2)
			Vp1=F->children[3]->sommetCoin[1];
		if (F->children[3]->sommetCoin[2]->parentV[0]==V2 && F->children[3]->sommetCoin[2]->parentV[1]==V3 || F->children[3]->sommetCoin[2]->parentV[0]==V3 && F->children[3]->sommetCoin[2]->parentV[1]==V2)
			Vp1=F->children[3]->sommetCoin[2];

		//Rec
		depthrec(F->children[3], Vp1, Vp2, Vp3, DM, 0, 0);
		if (V1==F->children[0]->sommetCoin[0] || V1==F->children[0]->sommetCoin[1] || V1==F->children[0]->sommetCoin[2])
			depthrec(F->children[0], V1, Vp2, Vp3, DM, 0, 1);
		else if (V1==F->children[1]->sommetCoin[0] || V1==F->children[1]->sommetCoin[1] || V1==F->children[1]->sommetCoin[2])
			depthrec(F->children[1], V1, Vp2, Vp3, DM, 0, 1);
		else if (V1==F->children[2]->sommetCoin[0] || V1==F->children[2]->sommetCoin[1] || V1==F->children[2]->sommetCoin[2])
			depthrec(F->children[2], V1, Vp2, Vp3, DM, 0, 1);
		else
			return 1;

		if (V2==F->children[0]->sommetCoin[0] || V2==F->children[0]->sommetCoin[1] || V2==F->children[0]->sommetCoin[2])
			depthrec(F->children[0], Vp1, V2, Vp3, DM, 0, 2);
		else if (V2==F->children[1]->sommetCoin[0] || V2==F->children[1]->sommetCoin[1] || V2==F->children[1]->sommetCoin[2])
			depthrec(F->children[1], Vp1, V2, Vp3, DM, 0, 2);
		else if (V2==F->children[2]->sommetCoin[0] || V2==F->children[2]->sommetCoin[1] || V2==F->children[2]->sommetCoin[2])
			depthrec(F->children[2], Vp1, V2, Vp3, DM, 0, 2);
		else
			return 1;

		if (V3==F->children[0]->sommetCoin[0] || V3==F->children[0]->sommetCoin[1] || V3==F->children[0]->sommetCoin[2])
			depthrec(F->children[0], Vp1, Vp2, V3, DM, 0, 3);
		else if (V3==F->children[1]->sommetCoin[0] || V3==F->children[1]->sommetCoin[1] || V3==F->children[1]->sommetCoin[2])
			depthrec(F->children[1], Vp1, Vp2, V3, DM, 0, 3);
		else if (V3==F->children[2]->sommetCoin[0] || V3==F->children[2]->sommetCoin[1] || V3==F->children[2]->sommetCoin[2])
			depthrec(F->children[2], Vp1, Vp2, V3, DM, 0, 3);
		else
			return 1;
		
		
	}

	return 0;
}

void DumpQ2Datmesh(Qtree *Q, DatMesh *DM) {
	
	int i;
	int level;

	//Init
	DM->NbVertices = DM->NbFaces = 0;
	level=Q->VertexTable[Q->Top-1]->level;

	//Sommets
	for (i=0; i<Q->Top; i++) {
		DM->V[i][0] = level+1-Q->VertexTable[i]->level;
		DM->V[i][1] = Q->VertexTable[i]->g[0];
		DM->V[i][2] = Q->VertexTable[i]->g[1];
		DM->V[i][3] = Q->VertexTable[i]->g[2];
	}
	DM->NbVertices = i;

	//Faces
	//Pour chaque racine :
	for (i=0; i<Q->NbRacine; i++) {
		// itrer sur les filles rcursivement
		depthrec(Q->racines[i], Q->racines[i]->sommetCoin[0], Q->racines[i]->sommetCoin[1], Q->racines[i]->sommetCoin[2], DM, 1, 0);

	}
		

}	

int  DumpWR2DatMesh(WaveRep *WR, DatMesh *DM) {

	int i;

	if (DM->NbVertices != WR->NbCoeff)
		return 1;

	for (i=0; i<WR->NbCoeff; i++) {
		DM->V[i][1] = WR->tab[i]->x;
		DM->V[i][2] = WR->tab[i]->y;
		DM->V[i][3] = WR->tab[i]->z;
	}

	return 0;
	
}

int DumpBase2DatMesh(Mesh *M0, DatMesh *DM) {

	int i;

	for (i=0; i<M0->NbVertex; i++) {
		DM->V[i][1] = M0->Vertex[i][0];
		DM->V[i][2] = M0->Vertex[i][1];
		DM->V[i][3] = M0->Vertex[i][2];
	}
	return 0;
}

void Dat2Bary(DatMesh *DM, int NbVertices, int NbFaces, WaveRep *WR) {
	// faces : les faces d'un maillage format DAT
	// NbVertices : nombre de sommets du maillage
	// NbFaces : nombre de Faces du maillage
	// retourne un tableau [NbVertices][4] contenant les (F0 alpha beta gamma)

	//int **bary=(int **)malloc(NbVertices*sizeof(int*));
	int **parents=(int **)malloc(NbVertices*sizeof(int*));
	int i;
	int F0=-1;
	int rem1, rem2, rem3;
	int *F0index=(int *)malloc(NbFaceMax*sizeof(int));

	// Init
	for (i=0; i<NbVertices; i++) {
		parents[i]=(int *)malloc(2*sizeof(int));
		WR->tab[i]=(WaveCo*)malloc(sizeof(WaveCo));
		WR->tab[i]->F0=-1;
	}

	// on y va...
	for (i=0; i< NbFaces; i++) {
		if (DM->F[i][1]==1) {
			F0++;
			// Et d'un
			if (WR->tab[DM->F[i][2]]->F0==-1) {
				WR->tab[DM->F[i][2]]->F0=F0;
				WR->tab[DM->F[i][2]]->alpha=1;
				WR->tab[DM->F[i][2]]->beta=0;
				WR->tab[DM->F[i][2]]->gamma=0;
				parents[DM->F[i][2]][0]=-1;
				parents[DM->F[i][2]][1]=-1;
			}
			// Et de deux
			if (WR->tab[DM->F[i][3]]->F0==-1) {
				WR->tab[DM->F[i][3]]->F0=F0;
				WR->tab[DM->F[i][3]]->alpha=0;
				WR->tab[DM->F[i][3]]->beta=1;
				WR->tab[DM->F[i][3]]->gamma=0;
				parents[DM->F[i][3]][0]=-1;
				parents[DM->F[i][3]][1]=-1;
			}
			// Et de trois
			if (WR->tab[DM->F[i][4]]->F0==-1) {
				WR->tab[DM->F[i][4]]->F0=F0;
				WR->tab[DM->F[i][4]]->alpha=0;
				WR->tab[DM->F[i][4]]->beta=0;
				WR->tab[DM->F[i][4]]->gamma=1;
				parents[DM->F[i][4]][0]=-1;
				parents[DM->F[i][4]][1]=-1;
			}
			F0index[F0]=i;
		} else if (DM->F[i][0]==0) {
			int al, be, ga, alp, bet, gam;
		// Et d'un
			if (WR->tab[DM->F[i][2]]->F0==-1) {
				WR->tab[DM->F[i][2]]->F0=F0;
				parents[DM->F[i][2]][0]=rem2;
				parents[DM->F[i][2]][1]=rem3;
				// Expression des coordonnes WR->tabcentriques
				// des deux voisins dans le mme repre
				if (WR->tab[parents[DM->F[i][2]][0]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][2]][0]]->alpha, WR->tab[parents[DM->F[i][2]][0]]->beta, WR->tab[parents[DM->F[i][2]][0]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][2]][0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=WR->tab[parents[DM->F[i][2]][0]]->alpha;
					be=WR->tab[parents[DM->F[i][2]][0]]->beta;
					ga=WR->tab[parents[DM->F[i][2]][0]]->gamma;
				}
				if (WR->tab[parents[DM->F[i][2]][1]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][2]][1]]->alpha, WR->tab[parents[DM->F[i][2]][1]]->beta, WR->tab[parents[DM->F[i][2]][1]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][2]][1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=WR->tab[parents[DM->F[i][2]][1]]->alpha;
					bet=WR->tab[parents[DM->F[i][2]][1]]->beta;
					gam=WR->tab[parents[DM->F[i][2]][1]]->gamma;
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				WR->tab[DM->F[i][2]]->alpha=al+alp;
				WR->tab[DM->F[i][2]]->beta=be+bet;
				WR->tab[DM->F[i][2]]->gamma=ga+gam;
				//Reduction par 2^n
				while (WR->tab[DM->F[i][2]]->alpha%2==0 && WR->tab[DM->F[i][2]]->beta%2==0 && WR->tab[DM->F[i][2]]->gamma%2==0) {
					WR->tab[DM->F[i][2]]->alpha/=2;
					WR->tab[DM->F[i][2]]->beta/=2;
					WR->tab[DM->F[i][2]]->gamma/=2;
				}
				
			}
			// Et de deux
			if (WR->tab[DM->F[i][3]]->F0==-1) {
				WR->tab[DM->F[i][3]]->F0=F0;
				parents[DM->F[i][3]][0]=rem3;
				parents[DM->F[i][3]][1]=rem1;
				// Expression des coordonnes WR->tabcentriques
				// des deux voisins dans le mme repre
				if (WR->tab[parents[DM->F[i][3]][0]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][3]][0]]->alpha, WR->tab[parents[DM->F[i][3]][0]]->beta, WR->tab[parents[DM->F[i][3]][0]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][3]][0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=WR->tab[parents[DM->F[i][3]][0]]->alpha;
					be=WR->tab[parents[DM->F[i][3]][0]]->beta;
					ga=WR->tab[parents[DM->F[i][3]][0]]->gamma;
				}
				if (WR->tab[parents[DM->F[i][3]][1]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][3]][1]]->alpha, WR->tab[parents[DM->F[i][3]][1]]->beta, WR->tab[parents[DM->F[i][3]][1]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][3]][1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=WR->tab[parents[DM->F[i][3]][1]]->alpha;
					bet=WR->tab[parents[DM->F[i][3]][1]]->beta;
					gam=WR->tab[parents[DM->F[i][3]][1]]->gamma;
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				WR->tab[DM->F[i][3]]->alpha=al+alp;
				WR->tab[DM->F[i][3]]->beta=be+bet;
				WR->tab[DM->F[i][3]]->gamma=ga+gam;
				//Reduction par 2^n
				while (WR->tab[DM->F[i][3]]->alpha%2==0 && WR->tab[DM->F[i][3]]->beta%2==0 && WR->tab[DM->F[i][3]]->gamma%2==0) {
					WR->tab[DM->F[i][3]]->alpha/=2;
					WR->tab[DM->F[i][3]]->beta/=2;
					WR->tab[DM->F[i][3]]->gamma/=2;
				}
			}
			// Et de trois
			if (WR->tab[DM->F[i][4]]->F0==-1) {
				WR->tab[DM->F[i][4]]->F0=F0;
				parents[DM->F[i][4]][0]=rem1;
				parents[DM->F[i][4]][1]=rem2;
				// Expression des coordonnes WR->tabcentriques
				// des deux voisins dans le mme repre
				if (WR->tab[parents[DM->F[i][4]][0]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][4]][0]]->alpha, WR->tab[parents[DM->F[i][4]][0]]->beta, WR->tab[parents[DM->F[i][4]][0]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][4]][0]]->F0]], DM->F[F0index[F0]], &al, &be, &ga);
				} else {
					al=WR->tab[parents[DM->F[i][4]][0]]->alpha;
					be=WR->tab[parents[DM->F[i][4]][0]]->beta;
					ga=WR->tab[parents[DM->F[i][4]][0]]->gamma;
				}
				if (WR->tab[parents[DM->F[i][4]][1]]->F0!=F0) {
					chCoord(WR->tab[parents[DM->F[i][4]][1]]->alpha, WR->tab[parents[DM->F[i][4]][1]]->beta, WR->tab[parents[DM->F[i][4]][1]]->gamma, DM->F[F0index[WR->tab[parents[DM->F[i][4]][1]]->F0]], DM->F[F0index[F0]], &alp, &bet, &gam);
				} else {
					alp=WR->tab[parents[DM->F[i][4]][1]]->alpha;
					bet=WR->tab[parents[DM->F[i][4]][1]]->beta;
					gam=WR->tab[parents[DM->F[i][4]][1]]->gamma;
				}
				// Homogeneisation des puissances de deux
				while (al+be+ga!=alp+bet+gam) {
					if ((alp+bet+gam)>(al+be+ga)) {
						al*=2;
						be*=2;
						ga*=2;
					} else {
						alp*=2;
						bet*=2;
						gam*=2;
					}
				}
				// Affectation
				WR->tab[DM->F[i][4]]->alpha=al+alp;
				WR->tab[DM->F[i][4]]->beta=be+bet;
				WR->tab[DM->F[i][4]]->gamma=ga+gam;
				//Reduction par 2^n
				while (WR->tab[DM->F[i][4]]->alpha%2==0 && WR->tab[DM->F[i][4]]->beta%2==0 && WR->tab[DM->F[i][4]]->gamma%2==0) {
					WR->tab[DM->F[i][4]]->alpha/=2;
					WR->tab[DM->F[i][4]]->beta/=2;
					WR->tab[DM->F[i][4]]->gamma/=2;
				}
			}
		}
		rem1=DM->F[i][2];
		rem2=DM->F[i][3];
		rem3=DM->F[i][4];
	}

	//recopie des sommets
	for (i=0; i<NbVertices; i++) {
		int jj=WR->tab[i]->alpha+WR->tab[i]->beta+WR->tab[i]->gamma;
		WR->tab[i]->x=DM->V[i][1]*jj;
		WR->tab[i]->y=DM->V[i][2]*jj;
		WR->tab[i]->z=DM->V[i][3]*jj;
	}
	WR->NbCoeff=i;

	//Libration
/*	for (i=0; i<NbVertices; i++) {
		free(bary[i]);
		free(parents[i]);
	}
	free(F0index);
*/
}

void PermuteVertices(DatMesh *DM, VertexPerCoeff *VPC) {

	int i;

	for (i=0; i<DM->NbFaces; i++) {
		DM->F[i][2]=VPC->LDS[DM->F[i][2]]->id;
		DM->F[i][3]=VPC->LDS[DM->F[i][3]]->id;
		DM->F[i][4]=VPC->LDS[DM->F[i][4]]->id;
	}

}

void trans(int ligneFille, int first, int second, int third, int &f, int &s, int &t, int &res) {

	switch (ligneFille) {
	//1
	case 1:
		if (first==0 && second == 1 && third == 2) {
			res=0;
			f=0;
			s=2;
			t=1;
		}
		if (first==0 && second == 2 && third == 1) {
			res=0;
			f=0;
			s=1;
			t=2;
		}
		
		if (first==1 && second == 0 && third == 2) {
			res=1;
			f=2;
			s=0;
			t=1;
		}
		
		if (first==1 && second == 2 && third == 0) {
			res=2;
			f=2;
			s=1;
			t=0;
		}

		if (first==2 && second == 0 && third == 1) {
			res=1;
			f=1;
			s=0;
			t=2;
		}

		if (first==2 && second == 1 && third == 0) {
			res=2;
			f=1;
			s=2;
			t=0;
		}
		break;
		//2
		case 2:
		if (first==0 && second == 1 && third == 2) {
			res=1;
			f=2;
			s=1;
			t=0;
		}

		if (first==0 && second == 2 && third == 1) {
			res=2;
			f=2;
			s=0;
			t=1;
		}

		if (first==1 && second == 0 && third == 2) {
			res=0;
			f=1;
			s=2;
			t=0;
		}

		if (first==1 && second == 2 && third == 0) {
			res=0;
			f=1;
			s=0;
			t=2;
		}

		if (first==2 && second == 0 && third == 1) {
			res=2;
			f=0;
			s=2;
			t=1;
		}

		if (first==2 && second == 1 && third == 0) {
			res=1;
			f=0;
			s=1;
			t=2;
		}
		break;
		//3
		case 3:
		if (first==0 && second == 1 && third == 2) {
			res=2;
			f=1;
			s=0;
			t=2;
		}

		if (first==0 && second == 2 && third == 1) {
			res=1;
			f=1;
			s=2;
			t=0;
		}

		if (first==1 && second == 0 && third == 2) {
			res=2;
			f=0;
			s=1;
			t=2;
		}

		if (first==1 && second == 2 && third == 0) {
			res=1;
			f=0;
			s=2;
			t=1;
		}
	
		if (first==2 && second == 0 && third == 1) {
			res=0;
			f=2;
			s=1;
			t=0;
		}

		if (first==2 && second == 1 && third == 0) {
			res=0;
			f=2;
			s=0;
			t=1;
		}
		break;
	}
}

int recSubd(DatMesh *DM, int &ligne, Qtree *Q, Face_WSZT*f, int jcur, int jmax, int first, int second, int third) {
	
	void divise21(Qtree*, Face_WSZT*);
	static int fi, se, th, res;

	//traitement de f
	f->sommetCoin[0]->g[0]=DM->V[DM->F[ligne][2+first]][1];
	f->sommetCoin[0]->g[1]=DM->V[DM->F[ligne][2+first]][2];
	f->sommetCoin[0]->g[2]=DM->V[DM->F[ligne][2+first]][3];

	f->sommetCoin[1]->g[0]=DM->V[DM->F[ligne][2+second]][1];
	f->sommetCoin[1]->g[1]=DM->V[DM->F[ligne][2+second]][2];
	f->sommetCoin[1]->g[2]=DM->V[DM->F[ligne][2+second]][3];

	f->sommetCoin[2]->g[0]=DM->V[DM->F[ligne][2+third]][1];
	f->sommetCoin[2]->g[1]=DM->V[DM->F[ligne][2+third]][2];
	f->sommetCoin[2]->g[2]=DM->V[DM->F[ligne][2+third]][3];

	ligne++;

	//Retour conditionnel
	if (jcur==jmax)
		return 0;

	//traitement des filles
	divise21(Q, f);
	recSubd(DM, ligne, Q, f->children[3], jcur+1, jmax, third, first, second);
	trans(1, first, second, third, fi, se, th, res);
	recSubd(DM, ligne, Q, f->children[res], jcur+1, jmax, fi, se, th);
	trans(2, first, second, third, fi, se, th, res);
	recSubd(DM, ligne, Q, f->children[res], jcur+1, jmax, fi, se, th);
	trans(3, first, second, third, fi, se, th, res);
	recSubd(DM, ligne, Q, f->children[res], jcur+1, jmax, fi, se, th);

	return 0;

}
/*----------------------------------------------
Transform a DAT file into a QuadTree hierarchy
----------------------------------------------*/
bool DAT2Q(DatMesh *DM, Qtree *Q) {

	int i, j;
	Face_WSZT *f=NULL;

	//Faces
	//Mthode optimiste
	
	j=0;

	//Subdivision complete
	i=0;
	for (j=0; j<Q->NbRacine; j++) {
		recSubd(DM, i, Q, Q->racines[j], 0, (int)DM->V[0][0]-1, 0, 1, 2);
	}
	

	return true;
}


bool DAT2BaseMesh(DatMesh *DM, Mesh *BM) {

	int i=0, j, k;

	while (DM->V[i][0]==DM->V[0][0]) {
		BM->Vertex[i][0]=DM->V[i][1];
		BM->Vertex[i][1]=DM->V[i][2];
		BM->Vertex[i][2]=DM->V[i][3];
		BM->FaceVoisine[i]=NULL;
		i++;
	}
	BM->NbVertex=i;

	j=0;
	for (i=0; i<DM->NbFaces; i++) {
		if (DM->F[i][1]==1) {
			BM->FacePresente[j]=1;
			BM->couleur[i]=7;
			for (k=0; k<3; k++) {
				BM->Face[j][k]=DM->F[i][2+k];
				ajouteFace(BM, j, DM->F[i][2+k]);	/*Liste des voisins*/		
			}
			j++;
		}
	}
	BM->NbFace=j;

	return true;


}

DatMesh* makeDMsubdividedFromBaseMesh(Mesh* BM, int level) {
	
	Qtree* Q=(Qtree*)malloc(sizeof(Qtree));
	DatMesh* DM=(DatMesh*)malloc(sizeof(DatMesh));

	//transformation du BM en Qtree
	initQ4(Q, BM);

	//Subdivision du Qtree
	subdivideAtLevel(Q, level);

	//trans. du Qtree en Dat 
	DumpQ2Datmesh(Q, DM);

	return DM;
}
