#ifndef LISTEENTRY
#define LISTEENTRY
#include "structure.h"

//!une classe reprsentant une liste d'entry.
class liste_entry {
	
	//!pointeur sur la tete de liste.
	entry* m_tete;

	//!pointeur sur le fin de liste.
	entry* m_queue;

	//!iterateur.
	entry* m_iter;

	//!ancienne position de l'iterateur.
	entry* m_backiter;

public:
	//!constructeur.
	liste_entry() {
		m_tete = NULL;
		m_queue = NULL;
		m_iter = NULL;
		m_backiter = NULL;
	}

	//!destructeur.
	~liste_entry() {
		entry * e = m_tete;
		while (m_tete) {
			m_tete = m_tete->next;
			delete e;
			e = m_tete;
		}
		m_tete = NULL;
		m_queue = NULL;
	}

	//!rinitialisation de la liste.
	void reset() {
		entry * e = m_tete;
		while (m_tete) {
			m_tete = m_tete->next;
			delete e;
			e = m_tete;
		}
		m_tete = NULL;
		m_queue = NULL;
		m_iter = NULL;
		m_backiter = NULL;
	}

	//!ajout d'une entry en tete de liste.
	void add(entry* e) {
		if (m_tete) {
			e->next = m_tete;
			m_tete = e;
		}
		else {
			e->next = NULL;
			m_tete = e;
			m_queue = e;
		}
	}

	//!ajout d'une entry en fin de liste.
	void addend(entry* e) {
		if (m_tete) {
			e->next = NULL;
			m_queue->next = e;
			m_queue = e;
			if ((m_backiter)&&(m_iter==NULL)) {
				m_iter = m_queue;
			}
		}
		else {
			e->next = NULL;
			m_tete = e;
			m_queue = e;
		}
	}

	//!initialise un parcours de la liste.
	entry* startiter() {
		m_iter = m_tete;
		m_backiter = NULL;
		return m_iter;
	}

	//!iter le parcours de la liste.
	entry* iter() {
		m_backiter = m_iter;
		m_iter = m_iter->next;
		return m_iter;
	}

	//!supprime l'entry de la liste.
	entry* removeiter() {
		if (m_iter) {
			if (m_iter==m_tete) {
				m_tete = m_tete->next;
				m_iter = m_tete;
				m_backiter = NULL;
				if (m_tete==NULL) {
					m_queue = NULL;
				}
			}
			else {
				m_backiter->next = m_iter->next;
				m_iter = m_iter->next;
				if (m_iter==NULL) {
					m_queue = m_backiter;
				}
			}
		}
		return m_iter;
	}

	//!concatnation de 2 liste.
	void append(liste_entry& l) {
		if (m_tete==NULL) {
			m_tete = l.m_tete;
			m_queue = l.m_queue;
			l.m_tete = NULL;
			l.m_queue = NULL;
		}
		else {
			if (l.m_tete) {
				m_queue->next = l.m_tete;
				m_queue = l.m_queue;
				l.m_tete = NULL;
				l.m_queue = NULL;
			}
		}
		return;
	}
};

//!une classe reprsentant une liste d'entry.
class liste_entry_loc {
	
	//!pointeur sur la tete de liste.
	entry_loc* m_tete;

	//!pointeur sur le fin de liste.
	entry_loc* m_queue;

	//!iterateur.
	entry_loc* m_iter;

	//!ancienne position de l'iterateur.
	entry_loc* m_backiter;

public:
	//!constructeur.
	liste_entry_loc() {
		m_tete = NULL;
		m_queue = NULL;
		m_iter = NULL;
		m_backiter = NULL;
	}

	//!destructeur.
	~liste_entry_loc() {
		entry_loc * e = m_tete;
		while (m_tete) {
			m_tete = m_tete->next;
			delete e;
			e = m_tete;
		}
		m_tete = NULL;
		m_queue = NULL;
	}

	//!rinitialisation de la liste.
	void reset() {
		entry_loc * e = m_tete;
		while (m_tete) {
			m_tete = m_tete->next;
			delete e;
			e = m_tete;
		}
		m_tete = NULL;
		m_queue = NULL;
		m_iter = NULL;
		m_backiter = NULL;
	}

	//!ajout d'une entry en tete de liste.
	void add(entry_loc* e) {
		if (m_tete) {
			e->next = m_tete;
			m_tete = e;
		}
		else {
			e->next = NULL;
			m_tete = e;
			m_queue = e;
		}
	}

	//!ajout d'une entry en fin de liste.
	void addend(entry_loc* e) {
		if (m_tete) {
			e->next = NULL;
			m_queue->next = e;
			m_queue = e;
			if ((m_backiter)&&(m_iter==NULL)) {
				m_iter = m_queue;
			}
		}
		else {
			e->next = NULL;
			m_tete = e;
			m_queue = e;
		}
	}

	//!initialise un parcours de la liste.
	entry_loc* startiter() {
		m_iter = m_tete;
		m_backiter = NULL;
		return m_iter;
	}

	//!iter le parcours de la liste.
	entry_loc* iter() {
		m_backiter = m_iter;
		m_iter = m_iter->next;
		return m_iter;
	}

	//!supprime l'entry de la liste.
	entry_loc* removeiter() {
		if (m_iter) {
			if (m_iter==m_tete) {
				m_tete = m_tete->next;
				m_iter = m_tete;
				m_backiter = NULL;
				if (m_tete==NULL) {
					m_queue = NULL;
				}
			}
			else {
				m_backiter->next = m_iter->next;
				m_iter = m_iter->next;
				if (m_iter==NULL) {
					m_queue = m_backiter;
				}
			}
		}
		return m_iter;
	}

	//!concatnation de 2 liste.
	void append(liste_entry_loc& l) {
		if (m_tete==NULL) {
			m_tete = l.m_tete;
			m_queue = l.m_queue;
			l.m_tete = NULL;
			l.m_queue = NULL;
		}
		else {
			if (l.m_tete) {
				m_queue->next = l.m_tete;
				m_queue = l.m_queue;
				l.m_tete = NULL;
				l.m_queue = NULL;
			}
		}
		return;
	}
};

#endif