
#include <stdio.h>
#include "proto.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* Module gerant l'arrivee des coefficients d'ondelettes */
/* et leurs prise en compte au niveau de l'arbre de subdivision */

/* Dans tout le module, les ondelettes utilisees */
/* sont les fonctions d'echelles du niveau directement plus fin */
/* centrees aux nouveaux points de subdivisions */

/* On rappelle que cette representation est bi-orthogonale et que les bases duales sont formees */
/* de masses de Dirac (l'analyse se reduit a un echantillonnage) */

/* On l'utilisera aussi pour les fonctions pics dans le cas general */

void divise(Qtree *Q, Face_WSZT*face) {

	/* Pas de traitement sur les coeffs d'ondelettes: les prevoire a l'exterieur */

	Face_WSZT *fille1, *fille2, *fille3, *fille4;
	Sommet *fils1, *fils2, *fils3;
	int ticket, arete, critere, i;

	/* Allocation familliale */

	if ((fille1=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille2=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille3=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille4=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");

	/* Allocation eventuelle des nouveaux sommets, suivant qu'il ont deja ete crees */
	/* dans une autre facette ou pas */

	/* Premier sommet */

	ticket=0;
	
	if (face->Voisins[0]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[0]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils1=face->Voisins[0]->sommetArete[arete];
		}
	}
	
	if (ticket==1) {
		if ((fils1=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils1;
		fils1->indice=Q->Top;
		Q->Top++;

		fils1->parentV[0]=face->sommetCoin[0];
		fils1->parentV[1]=face->sommetCoin[1];

		fils1->parentF[0]=face;	
		
		fils1->g[0]=fils1->fGeom[0]=0;
		fils1->g[1]=fils1->fGeom[1]=0;
		fils1->g[2]=fils1->fGeom[2]=0;

		fils1->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils1->hGeom[i][0]=0; 
			fils1->hGeom[i][1]=0; 
			fils1->hGeom[i][2]=0; 
		}
	}

	/* Deuxieme sommet */

	ticket=0;
	
	if (face->Voisins[1]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[1]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils2=face->Voisins[1]->sommetArete[arete];
		}
	}

	if (ticket==1) {
		
		if ((fils2=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils2;
		fils2->indice=Q->Top;
		Q->Top++;

		fils2->parentV[0]=face->sommetCoin[1];
		fils2->parentV[1]=face->sommetCoin[2];

		fils2->parentF[0]=face;

		fils2->g[0]=fils2->fGeom[0]=0;
		fils2->g[1]=fils2->fGeom[1]=0;
		fils2->g[2]=fils2->fGeom[2]=0;

		fils2->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils2->hGeom[i][0]=0; 
			fils2->hGeom[i][1]=0; 
			fils2->hGeom[i][2]=0; 
		}
	}
	

	/* Troisieme sommet */

	ticket=0;
	
	if (face->Voisins[2]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[2]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils3=face->Voisins[2]->sommetArete[arete];
		}
	}

	if (ticket==1) {

		if ((fils3=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils3;
		fils3->indice=Q->Top;
		Q->Top++;

		fils3->parentV[0]=face->sommetCoin[2];
		fils3->parentV[1]=face->sommetCoin[0];

		fils3->parentF[0]=face;

		fils3->g[0]=fils3->fGeom[0]=0;
		fils3->g[1]=fils3->fGeom[1]=0;
		fils3->g[2]=fils3->fGeom[2]=0;

		fils3->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils3->hGeom[i][0]=0; 
			fils3->hGeom[i][1]=0; 
			fils3->hGeom[i][2]=0; 
		}
	}

	/* Mise a jour des champs des filles */

	/* Premiere fille */

	fille1->level=(face->level)+1;

	fille1->children[0]=NULL;
	fille1->children[1]=NULL;
	fille1->children[2]=NULL;
	fille1->children[3]=NULL;

	fille1->sommetCoin[0]=face->sommetCoin[0];
	fille1->sommetCoin[1]=fils1;
	fille1->sommetCoin[2]=fils3;

	fille1->sommetArete[0]=NULL;
	fille1->sommetArete[1]=NULL;
	fille1->sommetArete[2]=NULL;

		/* Voisins */

	fille1->Voisins[1]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille1->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille1->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[0]->Voisins[2]=fille1;
				break;
			case 1:
				fille1->Voisins[0]->Voisins[0]=fille1;
				break;
			case 2:
				fille1->Voisins[0]->Voisins[1]=fille1;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille1->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille1->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[2]->Voisins[0]=fille1;
				break;
			case 1:
				fille1->Voisins[2]->Voisins[1]=fille1;
				break;
			case 2:
				fille1->Voisins[2]->Voisins[2]=fille1;
				break;
			}
		}
	}


	fille2->level=(face->level)+1;

	fille2->children[0]=NULL;
	fille2->children[1]=NULL;
	fille2->children[2]=NULL;
	fille2->children[3]=NULL;

	fille2->sommetCoin[0]=fils1;
	fille2->sommetCoin[1]=face->sommetCoin[1];
	fille2->sommetCoin[2]=fils2;

	fille2->sommetArete[0]=NULL;
	fille2->sommetArete[1]=NULL;
	fille2->sommetArete[2]=NULL;

		/* Voisins */

	fille2->Voisins[2]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille2->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille2->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[0]->Voisins[0]=fille2;
				break;
			case 1:
				fille2->Voisins[0]->Voisins[1]=fille2;
				break;
			case 2:
				fille2->Voisins[0]->Voisins[2]=fille2;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[1]==NULL) fille2->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille2->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[1]->Voisins[2]=fille2;
				break;
			case 1:
				fille2->Voisins[1]->Voisins[0]=fille2;
				break;
			case 2:
				fille2->Voisins[1]->Voisins[1]=fille2;
				break;
			}
		}
	}


	fille3->level=(face->level)+1;

	fille3->children[0]=NULL;
	fille3->children[1]=NULL;
	fille3->children[2]=NULL;
	fille3->children[3]=NULL;

	fille3->sommetCoin[0]=fils3;
	fille3->sommetCoin[1]=fils2;
	fille3->sommetCoin[2]=face->sommetCoin[2];

	fille3->sommetArete[0]=NULL;
	fille3->sommetArete[1]=NULL;
	fille3->sommetArete[2]=NULL;
		
		/* Voisins */

	fille3->Voisins[0]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[1]==NULL) fille3->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille3->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[1]->Voisins[0]=fille3;
				break;
			case 1:
				fille3->Voisins[1]->Voisins[1]=fille3;
				break;
			case 2:
				fille3->Voisins[1]->Voisins[2]=fille3;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille3->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille3->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[2]->Voisins[2]=fille3;
				break;
			case 1:
				fille3->Voisins[2]->Voisins[0]=fille3;
				break;
			case 2:
				fille3->Voisins[2]->Voisins[1]=fille3;
				break;
			}
		}
	}
	/* Quatrieme fille, la plus facile avec les voisins */

	fille4->level=(face->level)+1;
		
	fille4->children[0]=NULL;
	fille4->children[1]=NULL;
	fille4->children[2]=NULL;
	fille4->children[3]=NULL;

	fille4->sommetCoin[0]=fils1;
	fille4->sommetCoin[1]=fils2;
	fille4->sommetCoin[2]=fils3;

    fille4->sommetArete[0]=NULL;
	fille4->sommetArete[1]=NULL;
	fille4->sommetArete[2]=NULL;

	fille4->Voisins[0]=fille2;
	fille4->Voisins[1]=fille3;
	fille4->Voisins[2]=fille1;

	/* Puis on met a jour la face mere */

	face->sommetArete[0]=fils1;
	face->sommetArete[1]=fils2;
	face->sommetArete[2]=fils3;

	face->children[0]=fille1;
	face->children[1]=fille2;
	face->children[2]=fille3;
	face->children[3]=fille4;


}

void divise2(Qtree *Q, Face_WSZT*face) {

	// Va avec accomodate2 (sommets etiquets avec les coefficients)

	Face_WSZT *fille1, *fille2, *fille3, *fille4;
	Sommet *fils1, *fils2, *fils3;
	int ticket, arete, critere, i;

	/* Allocation familliale */

	if ((fille1=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille2=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille3=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille4=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");

	//liste vides
	fille1->Coeff_list=fille2->Coeff_list=fille3->Coeff_list=fille4->Coeff_list=NULL;

	/* Allocation eventuelle des nouveaux sommets, suivant qu'il ont deja ete crees */
	/* dans une autre facette ou pas */

	/* Premier sommet */

	ticket=0;
	
	if (face->Voisins[0]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[0]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils1=face->Voisins[0]->sommetArete[arete];
		}
	}
	
	if (ticket==1) {
		if ((fils1=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils1;
		fils1->indice=Q->Top;
		Q->Top++;

		fils1->parentV[0]=face->sommetCoin[0];
		fils1->parentV[1]=face->sommetCoin[1];

		fils1->parentF[0]=face;	
		
		fils1->g[0]=fils1->fGeom[0]=0;
		fils1->g[1]=fils1->fGeom[1]=0;
		fils1->g[2]=fils1->fGeom[2]=0;

		fils1->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils1->hGeom[i][0]=0; 
			fils1->hGeom[i][1]=0; 
			fils1->hGeom[i][2]=0; 
		}
	}

	/* Deuxieme sommet */

	ticket=0;
	
	if (face->Voisins[1]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[1]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils2=face->Voisins[1]->sommetArete[arete];
		}
	}

	if (ticket==1) {
		
		if ((fils2=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils2;
		fils2->indice=Q->Top;
		Q->Top++;

		fils2->parentV[0]=face->sommetCoin[1];
		fils2->parentV[1]=face->sommetCoin[2];

		fils2->parentF[0]=face;

		fils2->g[0]=fils2->fGeom[0]=0;
		fils2->g[1]=fils2->fGeom[1]=0;
		fils2->g[2]=fils2->fGeom[2]=0;

		fils2->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils2->hGeom[i][0]=0; 
			fils2->hGeom[i][1]=0; 
			fils2->hGeom[i][2]=0; 
		}
	}
	

	/* Troisieme sommet */

	ticket=0;
	
	if (face->Voisins[2]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[2]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils3=face->Voisins[2]->sommetArete[arete];
		}
	}

	if (ticket==1) {

		if ((fils3=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils3;
		fils3->indice=Q->Top;
		Q->Top++;

		fils3->parentV[0]=face->sommetCoin[2];
		fils3->parentV[1]=face->sommetCoin[0];

		fils3->parentF[0]=face;

		fils3->g[0]=fils3->fGeom[0]=0;
		fils3->g[1]=fils3->fGeom[1]=0;
		fils3->g[2]=fils3->fGeom[2]=0;

		fils3->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils3->hGeom[i][0]=0; 
			fils3->hGeom[i][1]=0; 
			fils3->hGeom[i][2]=0; 
		}
	}

	/* Mise a jour des champs des filles */

	/* Premiere fille */

	fille1->level=(face->level)+1;

	fille1->children[0]=NULL;
	fille1->children[1]=NULL;
	fille1->children[2]=NULL;
	fille1->children[3]=NULL;

	fille1->sommetCoin[0]=face->sommetCoin[0];
	fille1->sommetCoin[1]=fils1;
	fille1->sommetCoin[2]=fils3;

	fille1->sommetArete[0]=NULL;
	fille1->sommetArete[1]=NULL;
	fille1->sommetArete[2]=NULL;

		/* Voisins */

	fille1->Voisins[1]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille1->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille1->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[0]->Voisins[2]=fille1;
				break;
			case 1:
				fille1->Voisins[0]->Voisins[0]=fille1;
				break;
			case 2:
				fille1->Voisins[0]->Voisins[1]=fille1;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille1->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille1->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[2]->Voisins[0]=fille1;
				break;
			case 1:
				fille1->Voisins[2]->Voisins[1]=fille1;
				break;
			case 2:
				fille1->Voisins[2]->Voisins[2]=fille1;
				break;
			}
		}
	}


	fille2->level=(face->level)+1;

	fille2->children[0]=NULL;
	fille2->children[1]=NULL;
	fille2->children[2]=NULL;
	fille2->children[3]=NULL;

	fille2->sommetCoin[0]=fils1;
	fille2->sommetCoin[1]=face->sommetCoin[1];
	fille2->sommetCoin[2]=fils2;

	fille2->sommetArete[0]=NULL;
	fille2->sommetArete[1]=NULL;
	fille2->sommetArete[2]=NULL;

		/* Voisins */

	fille2->Voisins[2]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille2->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille2->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[0]->Voisins[0]=fille2;
				break;
			case 1:
				fille2->Voisins[0]->Voisins[1]=fille2;
				break;
			case 2:
				fille2->Voisins[0]->Voisins[2]=fille2;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[1]==NULL) fille2->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille2->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[1]->Voisins[2]=fille2;
				break;
			case 1:
				fille2->Voisins[1]->Voisins[0]=fille2;
				break;
			case 2:
				fille2->Voisins[1]->Voisins[1]=fille2;
				break;
			}
		}
	}


	fille3->level=(face->level)+1;

	fille3->children[0]=NULL;
	fille3->children[1]=NULL;
	fille3->children[2]=NULL;
	fille3->children[3]=NULL;

	fille3->sommetCoin[0]=fils3;
	fille3->sommetCoin[1]=fils2;
	fille3->sommetCoin[2]=face->sommetCoin[2];

	fille3->sommetArete[0]=NULL;
	fille3->sommetArete[1]=NULL;
	fille3->sommetArete[2]=NULL;
		
		/* Voisins */

	fille3->Voisins[0]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[1]==NULL) fille3->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille3->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[1]->Voisins[0]=fille3;
				break;
			case 1:
				fille3->Voisins[1]->Voisins[1]=fille3;
				break;
			case 2:
				fille3->Voisins[1]->Voisins[2]=fille3;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille3->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille3->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[2]->Voisins[2]=fille3;
				break;
			case 1:
				fille3->Voisins[2]->Voisins[0]=fille3;
				break;
			case 2:
				fille3->Voisins[2]->Voisins[1]=fille3;
				break;
			}
		}
	}
	/* Quatrieme fille, la plus facile avec les voisins */

	fille4->level=(face->level)+1;
		
	fille4->children[0]=NULL;
	fille4->children[1]=NULL;
	fille4->children[2]=NULL;
	fille4->children[3]=NULL;

	fille4->sommetCoin[0]=fils1;
	fille4->sommetCoin[1]=fils2;
	fille4->sommetCoin[2]=fils3;

    fille4->sommetArete[0]=NULL;
	fille4->sommetArete[1]=NULL;
	fille4->sommetArete[2]=NULL;

	fille4->Voisins[0]=fille2;
	fille4->Voisins[1]=fille3;
	fille4->Voisins[2]=fille1;

	/* Puis on met a jour la face mere */

	face->sommetArete[0]=fils1;
	face->sommetArete[1]=fils2;
	face->sommetArete[2]=fils3;

	face->children[0]=fille1;
	face->children[1]=fille2;
	face->children[2]=fille3;
	face->children[3]=fille4;


}

void divise21(Qtree *Q, Face_WSZT*face) {
	//Low pass midpoint

	// (Va avec accomodate2 (sommets etiquets avec les coefficients))
	// Place les nouveaux sommets entre leurs parents

	Face_WSZT *fille1, *fille2, *fille3, *fille4;
	Sommet *fils1, *fils2, *fils3;
	int ticket, arete, critere, i;

	/* Allocation familliale */

	if ((fille1=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille2=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille3=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille4=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");

	//liste vides
	fille1->Coeff_list=fille2->Coeff_list=fille3->Coeff_list=fille4->Coeff_list=NULL;

	/* Allocation eventuelle des nouveaux sommets, suivant qu'il ont deja ete crees */
	/* dans une autre facette ou pas */

	/* Premier sommet */

	ticket=0;
	
	if (face->Voisins[0]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[0]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils1=face->Voisins[0]->sommetArete[arete];

			// Et on oublie pas de mettre  jour parentF !!!
			fils1->parentF[0]=face;
			fils1->parentF[1]=face->Voisins[0];
		}
	}

	if (ticket==1) {
		if ((fils1=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		if (Q->Gap==NULL) {
			Q->VertexTable[Q->Top]=fils1;
			fils1->indice=Q->Top;
			Q->Top++;
		} else {
			Q->VertexTable[Q->Gap->sommet]=fils1;
			fils1->indice=Q->Gap->sommet;
			// On depile Gap
			PileDeSommets *tmp=(PileDeSommets *)malloc(sizeof(PileDeSommets));
			tmp=Q->Gap;
			Q->Gap=Q->Gap->next;
			free(tmp);
		}

		// Affectation dans Q->nouveaux
		PileDeSommets *PDS=(PileDeSommets *)malloc(sizeof(PileDeSommets));
		PDS->sommet=fils1->indice;
		PDS->next=Q->nouveaux;
		Q->nouveaux=PDS;

		fils1->parentV[0]=face->sommetCoin[0];
		fils1->parentV[1]=face->sommetCoin[1];

		fils1->parentF[0]=face;	
		fils1->parentF[1]=face->Voisins[0];

		fils1->u=0.5*(fils1->parentV[0]->u+fils1->parentV[1]->u);
		fils1->v=0.5*(fils1->parentV[0]->v+fils1->parentV[1]->v);
		
		fils1->g[0]=0.5*(fils1->parentV[0]->g[0]+fils1->parentV[1]->g[0]);
		fils1->g[1]=0.5*(fils1->parentV[0]->g[1]+fils1->parentV[1]->g[1]);
		fils1->g[2]=0.5*(fils1->parentV[0]->g[2]+fils1->parentV[1]->g[2]);

		fils1->fGeom[0]=0;
		fils1->fGeom[1]=0;
		fils1->fGeom[2]=0;

		fils1->level=face->level+1;


		for (i=0; i<NbNiveauMax; i++) {
			fils1->hGeom[i][0]=0; 
			fils1->hGeom[i][1]=0; 
			fils1->hGeom[i][2]=0; 
		}
	}

	/* Deuxieme sommet */

	ticket=0;
	
	if (face->Voisins[1]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[1]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils2=face->Voisins[1]->sommetArete[arete];

			// Et on oublie pas de mettre  jour parentF !!!
			fils2->parentF[0]=face;
			fils2->parentF[1]=face->Voisins[1];

		}
	}

	if (ticket==1) {
		
		if ((fils2=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		if (Q->Gap==NULL) {
			Q->VertexTable[Q->Top]=fils2;
			fils2->indice=Q->Top;
			Q->Top++;
		} else {
			Q->VertexTable[Q->Gap->sommet]=fils2;
			fils2->indice=Q->Gap->sommet;
			// On depile Gap
			PileDeSommets *tmp=(PileDeSommets *)malloc(sizeof(PileDeSommets));
			tmp=Q->Gap;
			Q->Gap=Q->Gap->next;
			free(tmp);
		}

		// Affectation dans Q->nouveaux
		PileDeSommets *PDS=(PileDeSommets *)malloc(sizeof(PileDeSommets));
		PDS->sommet=fils2->indice;
		PDS->next=Q->nouveaux;
		Q->nouveaux=PDS;

		fils2->parentV[0]=face->sommetCoin[1];
		fils2->parentV[1]=face->sommetCoin[2];

		fils2->parentF[0]=face;
		fils2->parentF[1]=face->Voisins[1];

		fils2->u=0.5*(fils2->parentV[0]->u+fils2->parentV[1]->u);
		fils2->v=0.5*(fils2->parentV[0]->v+fils2->parentV[1]->v);

		fils2->g[0]=0.5*(fils2->parentV[0]->g[0]+fils2->parentV[1]->g[0]);
		fils2->g[1]=0.5*(fils2->parentV[0]->g[1]+fils2->parentV[1]->g[1]);
		fils2->g[2]=0.5*(fils2->parentV[0]->g[2]+fils2->parentV[1]->g[2]);

		fils2->fGeom[0]=0;
		fils2->fGeom[1]=0;
		fils2->fGeom[2]=0;

		fils2->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils2->hGeom[i][0]=0; 
			fils2->hGeom[i][1]=0; 
			fils2->hGeom[i][2]=0; 
		}
	}
	

	/* Troisieme sommet */

	ticket=0;
	
	if (face->Voisins[2]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[2]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils3=face->Voisins[2]->sommetArete[arete];

			// Et on oublie pas de mettre  jour parentF !!!
			fils3->parentF[0]=face;
			fils3->parentF[1]=face->Voisins[2];
		}
	}

	if (ticket==1) {

		if ((fils3=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		if (Q->Gap==NULL) {
			Q->VertexTable[Q->Top]=fils3;
			fils3->indice=Q->Top;
			Q->Top++;
		} else {
			Q->VertexTable[Q->Gap->sommet]=fils3;
			fils3->indice=Q->Gap->sommet;
			// On depile Gap
			PileDeSommets *tmp=(PileDeSommets *)malloc(sizeof(PileDeSommets));
			tmp=Q->Gap;
			Q->Gap=Q->Gap->next;
			free(tmp);
		}

		// Affectation dans Q->nouveaux
		PileDeSommets *PDS=(PileDeSommets *)malloc(sizeof(PileDeSommets));
		PDS->sommet=fils3->indice;
		PDS->next=Q->nouveaux;
		Q->nouveaux=PDS;

		fils3->parentV[0]=face->sommetCoin[2];
		fils3->parentV[1]=face->sommetCoin[0];

		fils3->parentF[0]=face;
		fils3->parentF[1]=face->Voisins[2];

		fils3->u=0.5*(fils3->parentV[0]->u+fils3->parentV[1]->u);
		fils3->v=0.5*(fils3->parentV[0]->v+fils3->parentV[1]->v);

		fils3->g[0]=0.5*(fils3->parentV[0]->g[0]+fils3->parentV[1]->g[0]);
		fils3->g[1]=0.5*(fils3->parentV[0]->g[1]+fils3->parentV[1]->g[1]);
		fils3->g[2]=0.5*(fils3->parentV[0]->g[2]+fils3->parentV[1]->g[2]);

		fils3->fGeom[0]=0;
		fils3->fGeom[1]=0;
		fils3->fGeom[2]=0;

		fils3->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils3->hGeom[i][0]=0; 
			fils3->hGeom[i][1]=0; 
			fils3->hGeom[i][2]=0; 
		}
	}

	/* Mise a jour des champs des filles */

	/* Premiere fille */

	fille1->level=(face->level)+1;

	fille1->children[0]=NULL;
	fille1->children[1]=NULL;
	fille1->children[2]=NULL;
	fille1->children[3]=NULL;

	fille1->sommetCoin[0]=face->sommetCoin[0];
	fille1->sommetCoin[1]=fils1;
	fille1->sommetCoin[2]=fils3;

	fille1->sommetArete[0]=NULL;
	fille1->sommetArete[1]=NULL;
	fille1->sommetArete[2]=NULL;

	fille1->indice=-1;

		/* Voisins */

	fille1->Voisins[1]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille1->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille1->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[0]->Voisins[2]=fille1;
				break;
			case 1:
				fille1->Voisins[0]->Voisins[0]=fille1;
				break;
			case 2:
				fille1->Voisins[0]->Voisins[1]=fille1;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille1->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille1->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[2]->Voisins[0]=fille1;
				break;
			case 1:
				fille1->Voisins[2]->Voisins[1]=fille1;
				break;
			case 2:
				fille1->Voisins[2]->Voisins[2]=fille1;
				break;
			}
		}
	}


	fille2->level=(face->level)+1;

	fille2->children[0]=NULL;
	fille2->children[1]=NULL;
	fille2->children[2]=NULL;
	fille2->children[3]=NULL;

	fille2->sommetCoin[0]=fils1;
	fille2->sommetCoin[1]=face->sommetCoin[1];
	fille2->sommetCoin[2]=fils2;

	fille2->sommetArete[0]=NULL;
	fille2->sommetArete[1]=NULL;
	fille2->sommetArete[2]=NULL;

	fille2->indice=-1;

		/* Voisins */

	fille2->Voisins[2]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille2->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille2->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[0]->Voisins[0]=fille2;
				break;
			case 1:
				fille2->Voisins[0]->Voisins[1]=fille2;
				break;
			case 2:
				fille2->Voisins[0]->Voisins[2]=fille2;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[1]==NULL) fille2->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille2->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[1]->Voisins[2]=fille2;
				break;
			case 1:
				fille2->Voisins[1]->Voisins[0]=fille2;
				break;
			case 2:
				fille2->Voisins[1]->Voisins[1]=fille2;
				break;
			}
		}
	}


	fille3->level=(face->level)+1;

	fille3->children[0]=NULL;
	fille3->children[1]=NULL;
	fille3->children[2]=NULL;
	fille3->children[3]=NULL;

	fille3->sommetCoin[0]=fils3;
	fille3->sommetCoin[1]=fils2;
	fille3->sommetCoin[2]=face->sommetCoin[2];

	fille3->sommetArete[0]=NULL;
	fille3->sommetArete[1]=NULL;
	fille3->sommetArete[2]=NULL;

	fille3->indice=-1;

	/* Voisins */

	fille3->Voisins[0]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[1]==NULL) fille3->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille3->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[1]->Voisins[0]=fille3;
				break;
			case 1:
				fille3->Voisins[1]->Voisins[1]=fille3;
				break;
			case 2:
				fille3->Voisins[1]->Voisins[2]=fille3;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille3->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille3->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[2]->Voisins[2]=fille3;
				break;
			case 1:
				fille3->Voisins[2]->Voisins[0]=fille3;
				break;
			case 2:
				fille3->Voisins[2]->Voisins[1]=fille3;
				break;
			}
		}
	}

	fille4->indice=-1;

	/* Quatrieme fille, la plus facile avec les voisins */

	fille4->level=(face->level)+1;
		
	fille4->children[0]=NULL;
	fille4->children[1]=NULL;
	fille4->children[2]=NULL;
	fille4->children[3]=NULL;

	fille4->sommetCoin[0]=fils1;
	fille4->sommetCoin[1]=fils2;
	fille4->sommetCoin[2]=fils3;

    fille4->sommetArete[0]=NULL;
	fille4->sommetArete[1]=NULL;
	fille4->sommetArete[2]=NULL;

	fille4->Voisins[0]=fille2;
	fille4->Voisins[1]=fille3;
	fille4->Voisins[2]=fille1;

	/* Puis on met a jour la face mere */

	face->sommetArete[0]=fils1;
	face->sommetArete[1]=fils2;
	face->sommetArete[2]=fils3;

	face->children[0]=fille1;
	face->children[1]=fille2;
	face->children[2]=fille3;
	face->children[3]=fille4;


}

void divise22(Qtree *Q, Face_WSZT*face) {

	// Comme 22 mais avec les tricoeffs

	Face_WSZT *fille1, *fille2, *fille3, *fille4;
	Sommet *fils1, *fils2, *fils3;
	int ticket, arete, critere, i;

	/* Allocation familliale */

	if ((fille1=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille2=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille3=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");
	if ((fille4=(Face_WSZT*)malloc(sizeof(Face_WSZT)))==NULL) fprintf(stderr, " divise : can't allocate memory");

	//tricoeff vide
	fille1->Tricoeff[0]=fille2->Tricoeff[0]=fille3->Tricoeff[0]=fille4->Tricoeff[0]=-1;
	fille1->Tricoeff[1]=fille2->Tricoeff[1]=fille3->Tricoeff[1]=fille4->Tricoeff[1]=-1;
	fille1->Tricoeff[2]=fille2->Tricoeff[2]=fille3->Tricoeff[2]=fille4->Tricoeff[2]=-1;

	/* Allocation eventuelle des nouveaux sommets, suivant qu'il ont deja ete crees */
	/* dans une autre facette ou pas */

	/* Premier sommet */

	ticket=0;
	
	if (face->Voisins[0]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[0]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils1=face->Voisins[0]->sommetArete[arete];
		}
	}
	
	if (ticket==1) {
		if ((fils1=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils1;
		fils1->indice=Q->Top;
		Q->Top++;

		fils1->parentV[0]=face->sommetCoin[0];
		fils1->parentV[1]=face->sommetCoin[1];

		fils1->parentF[0]=face;	
		
		fils1->g[0]=0.5*(fils1->parentV[0]->g[0]+fils1->parentV[1]->g[0]);
		fils1->g[1]=0.5*(fils1->parentV[0]->g[1]+fils1->parentV[1]->g[1]);
		fils1->g[2]=0.5*(fils1->parentV[0]->g[2]+fils1->parentV[1]->g[2]);

		fils1->fGeom[0]=0;
		fils1->fGeom[1]=0;
		fils1->fGeom[2]=0;

		fils1->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils1->hGeom[i][0]=0; 
			fils1->hGeom[i][1]=0; 
			fils1->hGeom[i][2]=0; 
		}
	}

	/* Deuxieme sommet */

	ticket=0;
	
	if (face->Voisins[1]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[1]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils2=face->Voisins[1]->sommetArete[arete];
		}
	}

	if (ticket==1) {
		
		if ((fils2=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils2;
		fils2->indice=Q->Top;
		Q->Top++;

		fils2->parentV[0]=face->sommetCoin[1];
		fils2->parentV[1]=face->sommetCoin[2];

		fils2->parentF[0]=face;

		fils2->g[0]=0.5*(fils2->parentV[0]->g[0]+fils2->parentV[1]->g[0]);
		fils2->g[1]=0.5*(fils2->parentV[0]->g[1]+fils2->parentV[1]->g[1]);
		fils2->g[2]=0.5*(fils2->parentV[0]->g[2]+fils2->parentV[1]->g[2]);

		fils2->fGeom[0]=0;
		fils2->fGeom[1]=0;
		fils2->fGeom[2]=0;

		fils2->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils2->hGeom[i][0]=0; 
			fils2->hGeom[i][1]=0; 
			fils2->hGeom[i][2]=0; 
		}
	}
	

	/* Troisieme sommet */

	ticket=0;
	
	if (face->Voisins[2]==NULL) ticket=1; /* On va creer le sommet */
	else {
		if (face->Voisins[2]->children[0]==NULL) ticket=1;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On affecte le soi disant nouveau point a sa vraie valeur */

			fils3=face->Voisins[2]->sommetArete[arete];
		}
	}

	if (ticket==1) {

		if ((fils3=(Sommet*)malloc(sizeof(Sommet)))==NULL) fprintf(stderr, " divise : can't allocate memory");

		Q->VertexTable[Q->Top]=fils3;
		fils3->indice=Q->Top;
		Q->Top++;

		fils3->parentV[0]=face->sommetCoin[2];
		fils3->parentV[1]=face->sommetCoin[0];

		fils3->parentF[0]=face;

		fils3->g[0]=0.5*(fils3->parentV[0]->g[0]+fils3->parentV[1]->g[0]);
		fils3->g[1]=0.5*(fils3->parentV[0]->g[1]+fils3->parentV[1]->g[1]);
		fils3->g[2]=0.5*(fils3->parentV[0]->g[2]+fils3->parentV[1]->g[2]);

		fils3->fGeom[0]=0;
		fils3->fGeom[1]=0;
		fils3->fGeom[2]=0;

		fils3->level=face->level+1;

		for (i=0; i<NbNiveauMax; i++) {
			fils3->hGeom[i][0]=0; 
			fils3->hGeom[i][1]=0; 
			fils3->hGeom[i][2]=0; 
		}
	}

	/* Mise a jour des champs des filles */

	/* Premiere fille */

	fille1->level=(face->level)+1;

	fille1->children[0]=NULL;
	fille1->children[1]=NULL;
	fille1->children[2]=NULL;
	fille1->children[3]=NULL;

	fille1->sommetCoin[0]=face->sommetCoin[0];
	fille1->sommetCoin[1]=fils1;
	fille1->sommetCoin[2]=fils3;

	fille1->sommetArete[0]=NULL;
	fille1->sommetArete[1]=NULL;
	fille1->sommetArete[2]=NULL;

		/* Voisins */

	fille1->Voisins[1]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille1->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille1->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[0]->Voisins[2]=fille1;
				break;
			case 1:
				fille1->Voisins[0]->Voisins[0]=fille1;
				break;
			case 2:
				fille1->Voisins[0]->Voisins[1]=fille1;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille1->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille1->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille1->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille1->Voisins[2]->Voisins[0]=fille1;
				break;
			case 1:
				fille1->Voisins[2]->Voisins[1]=fille1;
				break;
			case 2:
				fille1->Voisins[2]->Voisins[2]=fille1;
				break;
			}
		}
	}


	fille2->level=(face->level)+1;

	fille2->children[0]=NULL;
	fille2->children[1]=NULL;
	fille2->children[2]=NULL;
	fille2->children[3]=NULL;

	fille2->sommetCoin[0]=fils1;
	fille2->sommetCoin[1]=face->sommetCoin[1];
	fille2->sommetCoin[2]=fils2;

	fille2->sommetArete[0]=NULL;
	fille2->sommetArete[1]=NULL;
	fille2->sommetArete[2]=NULL;

		/* Voisins */

	fille2->Voisins[2]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[0]==NULL) fille2->Voisins[0]=NULL;
	else {
		if (face->Voisins[0]->children[0]==NULL) fille2->Voisins[0]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[0])+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[1])*2+
					(face->sommetCoin[0]==face->Voisins[0]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[0]=face->Voisins[0]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[0]->Voisins[0]=fille2;
				break;
			case 1:
				fille2->Voisins[0]->Voisins[1]=fille2;
				break;
			case 2:
				fille2->Voisins[0]->Voisins[2]=fille2;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[1]==NULL) fille2->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille2->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille2->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille2->Voisins[1]->Voisins[2]=fille2;
				break;
			case 1:
				fille2->Voisins[1]->Voisins[0]=fille2;
				break;
			case 2:
				fille2->Voisins[1]->Voisins[1]=fille2;
				break;
			}
		}
	}


	fille3->level=(face->level)+1;

	fille3->children[0]=NULL;
	fille3->children[1]=NULL;
	fille3->children[2]=NULL;
	fille3->children[3]=NULL;

	fille3->sommetCoin[0]=fils3;
	fille3->sommetCoin[1]=fils2;
	fille3->sommetCoin[2]=face->sommetCoin[2];

	fille3->sommetArete[0]=NULL;
	fille3->sommetArete[1]=NULL;
	fille3->sommetArete[2]=NULL;
		
		/* Voisins */

	fille3->Voisins[0]=fille4;

		/* Pour les autres, on regarde les orientations des voisins */

	if (face->Voisins[1]==NULL) fille3->Voisins[1]=NULL;
	else {
		if (face->Voisins[1]->children[0]==NULL) fille3->Voisins[1]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[0])+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[1])*2+
					(face->sommetCoin[1]==face->Voisins[1]->sommetCoin[2])*4;
			switch(critere) {
			case 1:
				arete=2;
				break;
			case 2:
				arete=0;
				break;
			case 4:
				arete=1;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[1]=face->Voisins[1]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[1]->Voisins[0]=fille3;
				break;
			case 1:
				fille3->Voisins[1]->Voisins[1]=fille3;
				break;
			case 2:
				fille3->Voisins[1]->Voisins[2]=fille3;
				break;
			}
		}
	}

		/* Pour l'autre face */

	if (face->Voisins[2]==NULL) fille3->Voisins[2]=NULL;
	else {
		if (face->Voisins[2]->children[0]==NULL) fille3->Voisins[2]=NULL;
		else {
			/* On regarde l'orientation du voisin de face */
			critere=(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[0])+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[1])*2+
					(face->sommetCoin[2]==face->Voisins[2]->sommetCoin[2])*4;
			switch(critere) { 
			case 1:
				arete=0;
				break;
			case 2:
				arete=1;
				break;
			case 4:
				arete=2;
				break;
			}
			/* On met en relation les soeurs retrouves */

			fille3->Voisins[2]=face->Voisins[2]->children[arete];
			switch(arete) {
			case 0:
				fille3->Voisins[2]->Voisins[2]=fille3;
				break;
			case 1:
				fille3->Voisins[2]->Voisins[0]=fille3;
				break;
			case 2:
				fille3->Voisins[2]->Voisins[1]=fille3;
				break;
			}
		}
	}
	/* Quatrieme fille, la plus facile avec les voisins */

	fille4->level=(face->level)+1;
		
	fille4->children[0]=NULL;
	fille4->children[1]=NULL;
	fille4->children[2]=NULL;
	fille4->children[3]=NULL;

	fille4->sommetCoin[0]=fils1;
	fille4->sommetCoin[1]=fils2;
	fille4->sommetCoin[2]=fils3;

    fille4->sommetArete[0]=NULL;
	fille4->sommetArete[1]=NULL;
	fille4->sommetArete[2]=NULL;

	fille4->Voisins[0]=fille2;
	fille4->Voisins[1]=fille3;
	fille4->Voisins[2]=fille1;

	/* Puis on met a jour la face mere */

	face->sommetArete[0]=fils1;
	face->sommetArete[1]=fils2;
	face->sommetArete[2]=fils3;

	face->children[0]=fille1;
	face->children[1]=fille2;
	face->children[2]=fille3;
	face->children[3]=fille4;


}

void ChCoord(int alpha, int beta, int gamma, int n, int *a, int *b, int *c) {

	/* Applique les matrices de changement de coordonnees barycentriques d'un point */ 
	/* apres subdivision de la face representant la base affine. La nouvelle base affine est formee */
	/* des sommets de face->children[n] */

	/* Evidemment, n est compris entre 0 et 3 inclus */

	/* a, b et c sont les nouvelles coordonnees , a un coefficient 2 pres */
	
	switch(n) {
		case 0 :
			*a=alpha-beta-gamma;
			*b=2*beta;
			*c=2*gamma;
			break;
		case 1 :
			*a=2*alpha;
			*b=-alpha+beta-gamma;
			*c=2*gamma;
			break;
		case 2 :
			*a=2*alpha;
			*b=2*beta;
			*c=-alpha-beta+gamma;
			break;
		case 3 :
			*a=alpha+beta-gamma;
			*b=-alpha+beta+gamma;
			*c=alpha-beta+gamma;
			break;
	}
}

void accomodate(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma) {

	/* Subdivise recursivement la facette face jusqu'a ce que la fonction pic centree au point */
	/* de coordonnees barycentriques alpha beta gamma soit linaire sur chaque face */

	/* L'algorithme est optimal, c'est a dire qu'il n'est cre que le minimum de facettes necessaires */

	/* level est la resolution de la fonction pic ( qui dfinit son support) */

	/* S'applique a tous les points, reguliers ou singuliers */

	int i, a, b, c;

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)) {


		if (face->children[0]==NULL) 
			divise(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			}
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate(Q, face->children[i], level, a, b, c);
		}
	}
}


void globalAccomodate(Qtree *Q, int face, int level, int alpha, int beta, int gamma) {
	
	/* Accomodation de M0 */
	
	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate(Q, Q->racines[face], level, alpha, beta, gamma);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate(Q, Q->racines[i], level, 1, 0, 0);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate(Q, Q->racines[i], level, 0, 1, 0);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate(Q, Q->racines[i], level, 0, 0, 1);					
				}
			}
			accomodate(Q, Q->racines[face], level, 1, 0, 0);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate(Q, Q->racines[i], level, 1, 0, 0);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate(Q, Q->racines[i], level, 0, 1, 0);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate(Q, Q->racines[i], level, 0, 0, 1);					
				}
			}
			accomodate(Q, Q->racines[face], level, 0, 1, 0);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate(Q, Q->racines[i], level, 1, 0, 0);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate(Q, Q->racines[i], level, 0, 1, 0);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate(Q, Q->racines[i], level, 0, 0, 1);					
				}
			}
			accomodate(Q, Q->racines[face], level, 0, 0, 1);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0);
			}

			accomodate(Q, Q->racines[face], level, alpha, beta, gamma);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0);
			
			}
			
			accomodate(Q, Q->racines[face], level, alpha, beta, gamma);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0);
			}
			
			accomodate(Q, Q->racines[face], level, alpha, beta, gamma);	
			break;
		}
	}
}

void accomodate2(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff) {

	/* Subdivise recursivement la facette face jusqu'a ce que la fonction pic centree au point */
	/* de coordonnees barycentriques alpha beta gamma soit linaire sur chaque face */

	/* L'algorithme est optimal, c'est a dire qu'il n'est cre que le minimum de facettes necessaires */

	/* level est la resolution de la fonction pic ( qui dfinit son support) */

	/* S'applique a tous les points, reguliers ou singuliers */

	// Nouveau : ajoute l'indice du coefficient d'ondelette aux listes des facettes

	int i, a, b, c;
	CL *tmp;
	unsigned int estDansLaListe=0;

	//estDansLaListe
	tmp=face->Coeff_list;
	while (tmp!=NULL) {
		if (tmp->coeff==coeff) {
			estDansLaListe=1;
			break;
		}
		tmp=tmp->next;
	}

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)&&(!estDansLaListe)) {

		//Ajout du coefficient a la liste
		if (tmp=(CL*)malloc(sizeof(CL)))
			fprintf(stderr, "accomodate2 : Not enough memory");
		tmp->coeff=coeff;
		tmp->next=face->Coeff_list;
		face->Coeff_list=tmp;

		if (face->children[0]==NULL) 
			divise21(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			}
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate2(Q, face->children[i], level, a, b, c, coeff);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}

void globalAccomodate2(Qtree *Q, int face, int level, int alpha, int beta, int gamma, int coeff) {
	
	/* Accomodation de M0 */
	// Nouveau : place les indices des coeffs dans les listes des facettes

	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate2(Q, Q->racines[face], level, alpha, beta, gamma, coeff);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate2(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate2(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate2(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2(Q, Q->racines[face], level, 1, 0, 0, coeff);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate2(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate2(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate2(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2(Q, Q->racines[face], level, 0, 1, 0, coeff);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate2(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate2(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate2(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2(Q, Q->racines[face], level, 0, 0, 1, coeff);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate2(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate2(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate2(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0, coeff);
			}

			accomodate2(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate2(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate2(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate2(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0, coeff);
			
			}
			
			accomodate2(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate2(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate2(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate2(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0, coeff);
			}
			
			accomodate2(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		}
	}
}

void accomodate2miroir(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff) {

	/* Subdivise recursivement la facette face jusqu'a ce que la fonction pic centree au point */
	/* de coordonnees barycentriques alpha beta gamma soit linaire sur chaque face */

	/* L'algorithme est optimal, c'est a dire qu'il n'est cre que le minimum de facettes necessaires */

	/* level est la resolution de la fonction pic ( qui dfinit son support) */

	/* S'applique a tous les points, reguliers ou singuliers */

	// Nouveau : ajoute l'indice du coefficient d'ondelette aux listes des facettes

	int i, a, b, c;
	CL *tmp;
	unsigned int estDansLaListe=0;

	//estDansLaListe
	tmp=face->Coeff_list;
	while (tmp!=NULL) {
		if (tmp->coeff==coeff) {
			estDansLaListe=1;
			break;
		}
		tmp=tmp->next;
	}

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)/*&&(!estDansLaListe)*/) {

		if (face->children[0]==NULL) 
			divise21(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			}
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate2miroir(Q, face->children[i], level, a, b, c, coeff);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}

void globalAccomodate2miroir(Qtree *Q, int face, int level, int alpha, int beta, int gamma, int coeff) {
	
	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate2miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate2miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate2miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate2miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2miroir(Q, Q->racines[face], level, 1, 0, 0, coeff);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate2miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate2miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate2miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2miroir(Q, Q->racines[face], level, 0, 1, 0, coeff);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate2miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate2miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate2miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate2miroir(Q, Q->racines[face], level, 0, 0, 1, coeff);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0, coeff);
			}

			accomodate2miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0, coeff);
			
			}
			
			accomodate2miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate2miroir(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0, coeff);
			}
			
			accomodate2miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		}
	}
}

void accomodate3(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff) {

	// Pareil que accomodate2 mais avec les tricoefffs

	int i, a, b, c;
	unsigned int estDansLaListe=0;

	// Test de presence
	estDansLaListe=(face->Tricoeff[0]==coeff) || (face->Tricoeff[1]==coeff)
		||(face->Tricoeff[2]==coeff);

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)&&(!estDansLaListe)) {

		//Mise a jour des tricoeffs
		if (face->Tricoeff[0]==-1 || face->Tricoeff[0]==coeff) 
			face->Tricoeff[0]=coeff;
		else 
			if (face->Tricoeff[1]==-1 || face->Tricoeff[1]==coeff) 
				face->Tricoeff[1]=coeff;
			else
				face->Tricoeff[2]=coeff;

		if (face->children[0]==NULL) 
			divise22(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
					if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			}	
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
					if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
					if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate3(Q, face->children[i], level, a, b, c, coeff);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}

void globalAccomodate3(Qtree *Q, int face, int level, int alpha, int beta, int gamma, int coeff) {
	
	/* Accomodation de M0 */
	// Pareil que globalaccomodate2 mais avec les tricoeffs

	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate3(Q, Q->racines[face], level, alpha, beta, gamma, coeff);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate3(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate3(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate3(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3(Q, Q->racines[face], level, 1, 0, 0, coeff);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate3(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate3(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate3(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3(Q, Q->racines[face], level, 0, 1, 0, coeff);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate3(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate3(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate3(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3(Q, Q->racines[face], level, 0, 0, 1, coeff);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate3(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate3(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate3(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0, coeff);
			}

			accomodate3(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate3(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate3(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate3(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0, coeff);
			
			}
			
			accomodate3(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate3(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate3(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate3(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0, coeff);
			}
			
			accomodate3(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		}
	}
}

void accomodate3miroir(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff) {

	/* Subdivise recursivement la facette face jusqu'a ce que la fonction pic centree au point */
	/* de coordonnees barycentriques alpha beta gamma soit linaire sur chaque face */

	/* L'algorithme est optimal, c'est a dire qu'il n'est cre que le minimum de facettes necessaires */

	/* level est la resolution de la fonction pic ( qui dfinit son support) */

	/* S'applique a tous les points, reguliers ou singuliers */

	// Nouveau : ajoute l'indice du coefficient d'ondelette aux listes des facettes

	int i, a, b, c;
	unsigned int estDansLaListe=0;

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)/*&&(!estDansLaListe)*/) {

		if (face->children[0]==NULL) 
			divise21(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			}
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate3miroir(Q, face->children[i], level, a, b, c, coeff);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}

void globalAccomodate3miroir(Qtree *Q, int face, int level, int alpha, int beta, int gamma, int coeff) {
	
	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate3miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate3miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate3miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate3miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3miroir(Q, Q->racines[face], level, 1, 0, 0, coeff);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate3miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate3miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate3miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3miroir(Q, Q->racines[face], level, 0, 1, 0, coeff);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate3miroir(Q, Q->racines[i], level, 1, 0, 0, coeff);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate3miroir(Q, Q->racines[i], level, 0, 1, 0, coeff);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate3miroir(Q, Q->racines[i], level, 0, 0, 1, coeff);					
				}
			}
			accomodate3miroir(Q, Q->racines[face], level, 0, 0, 1, coeff);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta, coeff);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0, coeff);
			}

			accomodate3miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma, coeff);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0, coeff);
			
			}
			
			accomodate3miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha, coeff);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate3miroir(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0, coeff);
			}
			
			accomodate3miroir(Q, Q->racines[face], level, alpha, beta, gamma, coeff);	
			break;
		}
	}
}

void accomodate4(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff, Mesh *mesh) {

	// Pareil que accomodate2 mais avec les tricoefffs

	int i, a, b, c;
	unsigned int estDansLaListe=0;

	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)) {

		if (face->children[0]==NULL) {
			PileDeFaces *tmp=(PileDeFaces *)malloc(sizeof(PileDeFaces));
			divise21(Q, face);
			if (mesh->FacePresente[face->indice]) {
				mesh->FacePresente[face->indice]=0;
				tmp->face=face->indice;
				tmp->next=mesh->FaceGap;
				mesh->FaceGap=tmp;
			}
		}
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)/*&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)*/) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			/*		if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			*/}	
			if ((beta==gamma)&&(alpha==0)/*&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)*/) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			/*		if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			*/}
			if ((gamma==alpha)&&(beta==0)/*&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)*/) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			/*		if (Q->VertexTable[Q->Remember]->parentF[0]==NULL)
						Q->VertexTable[Q->Remember]->parentF[0]=face;
					else
						Q->VertexTable[Q->Remember]->parentF[1]=face;
			*/}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate4(Q, face->children[i], level, a, b, c, coeff, mesh);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}

#ifdef BIDON
void NULLaccomodate4(Qtree *Q, Face_WSZT*face, int level, int alpha, int beta, int gamma, int coeff) {

	/* Subdivise recursivement la facette face jusqu'a ce que la fonction pic centree au point */
	/* de coordonnees barycentriques alpha beta gamma soit linaire sur chaque face */

	/* L'algorithme est optimal, c'est a dire qu'il n'est cre que le minimum de facettes necessaires */

	/* level est la resolution de la fonction pic ( qui dfinit son support) */

	/* S'applique a tous les points, reguliers ou singuliers */

	// Nouveau : ajoute l'indice du coefficient d'ondelette aux listes des facettes

	int i, a, b, c;
	CL *tmp;
	
	if ((alpha>=0)&&(beta>=0)&&(gamma>=0)&&(face->level<level)) {

		//Ajout du coefficient a la liste
		if (tmp=(CL*)malloc(sizeof(CL)))
			fprintf(stderr, "accomodate4 : Not enough memory");
		tmp->coeff=coeff;
		tmp->next=face->Coeff_list;
		face->Coeff_list=tmp;

		if (face->children[0]==NULL) 
			divise21(Q, face);	
			
			/* On remplit Remember pour l'affectation ulterieure des coefficients d'ondelettes */

			if ((alpha==beta)&&(gamma==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[0]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[0]->indice;
			}
			if ((beta==gamma)&&(alpha==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[1]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[1]->indice;
			}
			if ((gamma==alpha)&&(beta==0)&&(Q->Remember!=
				face->children[3]->sommetCoin[2]->indice)) { 
					Q->Remember=face->children[3]->sommetCoin[2]->indice;
			}
		

		for (i=0; i<4; i++) {
			ChCoord(alpha, beta, gamma, i, &a, &b, &c);
			accomodate4(Q, face->children[i], level, a, b, c, coeff);
		}
	}

	// ATTENTION : (pas de subdivision) => (Q->Remember==-1) 
}
#endif

void globalAccomodate4(Qtree *Q, int face, int level, int alpha, int beta, int gamma, int coeff, Mesh *mesh) {
	
	/* Accomodation de M0 */
	// Pour aller avec receptionCoeff61()

	int i, critere; 

	if (alpha*beta*gamma!=0) accomodate4(Q, Q->racines[face], level, alpha, beta, gamma, coeff, mesh);
	else { /* Effet de bord (ou pire, de coin) */
		critere=(alpha!=0)+2*(beta!=0)+4*(gamma!=0);
		switch(critere) {
		case 1 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[0]) accomodate4(Q, Q->racines[i], level, 1, 0, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[0]) accomodate4(Q, Q->racines[i], level, 0, 1, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[0]) accomodate4(Q, Q->racines[i], level, 0, 0, 1, coeff, mesh);					
				}
			}
			accomodate4(Q, Q->racines[face], level, 1, 0, 0, coeff, mesh);
			break;
		case 2 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[1]) accomodate4(Q, Q->racines[i], level, 1, 0, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[1]) accomodate4(Q, Q->racines[i], level, 0, 1, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[1]) accomodate4(Q, Q->racines[i], level, 0, 0, 1, coeff, mesh);					
				}
			}
			accomodate4(Q, Q->racines[face], level, 0, 1, 0, coeff, mesh);
			break;
		case 4 :
			for (i=0; i<Q->NbRacine; i++) {
				if (i!=face) { 
					if (Q->racines[i]->sommetCoin[0]==Q->racines[face]->sommetCoin[2]) accomodate4(Q, Q->racines[i], level, 1, 0, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[1]==Q->racines[face]->sommetCoin[2]) accomodate4(Q, Q->racines[i], level, 0, 1, 0, coeff, mesh);
					if (Q->racines[i]->sommetCoin[2]==Q->racines[face]->sommetCoin[2]) accomodate4(Q, Q->racines[i], level, 0, 0, 1, coeff, mesh);					
				}
			}
			accomodate4(Q, Q->racines[face], level, 0, 0, 1, coeff, mesh);	
			break;
		case 3 :
			if (Q->racines[face]->Voisins[0]!=NULL) {
				if ((Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1]))

				accomodate4(Q, Q->racines[face]->Voisins[0], level, 0, beta, alpha, coeff, mesh);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1]))

				accomodate4(Q, Q->racines[face]->Voisins[0], level, alpha, 0, beta, coeff, mesh);

				if ((Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0])&&
					(Q->racines[face]->Voisins[0]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1]))

				accomodate4(Q, Q->racines[face]->Voisins[0], level, beta, alpha, 0, coeff, mesh);
			}

			accomodate4(Q, Q->racines[face], level, alpha, beta, gamma, coeff, mesh);	
			break;
		case 6 :
			if (Q->racines[face]->Voisins[1]!=NULL) {
				if ((Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2]))

				accomodate4(Q, Q->racines[face]->Voisins[1], level, 0, gamma, beta, coeff, mesh);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2]))

				accomodate4(Q, Q->racines[face]->Voisins[1], level, beta, 0, gamma, coeff, mesh);

				if ((Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[1])&&
					(Q->racines[face]->Voisins[1]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2]))

				accomodate4(Q, Q->racines[face]->Voisins[1], level, gamma, beta, 0, coeff, mesh);
			
			}
			
			accomodate4(Q, Q->racines[face], level, alpha, beta, gamma, coeff, mesh);	
			
			break;
		case 5 :
			if (Q->racines[face]->Voisins[2]!=NULL) {
				if ((Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[0]!=Q->racines[face]->sommetCoin[0]))

				accomodate4(Q, Q->racines[face]->Voisins[2], level, 0, alpha, gamma, coeff, mesh);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[1]!=Q->racines[face]->sommetCoin[0]))

				accomodate4(Q, Q->racines[face]->Voisins[2], level, gamma, 0, alpha, coeff, mesh);

				if ((Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[2])&&
					(Q->racines[face]->Voisins[2]->sommetCoin[2]!=Q->racines[face]->sommetCoin[0]))

				accomodate4(Q, Q->racines[face]->Voisins[2], level, alpha, gamma, 0, coeff, mesh);
			}
			
			accomodate4(Q, Q->racines[face], level, alpha, beta, gamma, coeff, mesh);	
			break;
		}
	}
}

//Ne sert que pour SubdivAuNiveauJ()
void RecSubd(Qtree *Q, Face_WSZT*f, int j, int jmax) {

	if (j==jmax)
		return;
	else {
		divise21(Q, f);
		RecSubd(Q, f->children[0], j+1, jmax);
		RecSubd(Q, f->children[1], j+1, jmax);
		RecSubd(Q, f->children[2], j+1, jmax);
		RecSubd(Q, f->children[3], j+1, jmax);
	}
}

// Subdivise rgulirement Q en j niveaux
void SubdivAuNiveauJ(Qtree *Q, int j) {
	
	int i;
	for (i=0; i<Q->NbRacine; i++) {
		RecSubd(Q, Q->racines[i], 0, j);
	}
}