#include "stdio.h"
#include "structure.h"
#include "zerotree.h"
#include <stdlib.h>

//Wavelet_Mesh_Object_Header
Wavelet_Mesh_Object_Header::~Wavelet_Mesh_Object_Header() {
	return;
}

Wavelet_Mesh_Object_Header::Wavelet_Mesh_Object_Header() {
	return;
}

int Wavelet_Mesh_Object_Header::read(ibitstream &i) {
	i.readnbit(m_Base_id,8);
	
	i.readnbit(m_s_h.nb_sc,32);
	i.readnbit(m_s_h.NbBpC,5);
	i.readnbit(m_w_h.m_Wtype,2);
	i.readnbit(m_c_h.m_NbBP,1);
	i.readnbit((unsigned int &)m_c_h.m_Cmax,32);
	i.readnbit(m_ZT_h.m_ZTtype,3);

	return 1;
}

//Wavelet_Mesh_Object
Wavelet_Mesh_Object::~Wavelet_Mesh_Object() {
	return;
}

Wavelet_Mesh_Object::Wavelet_Mesh_Object() {
	return;
}

void Wavelet_Mesh_Object::read(ibitstream &ibs) {
	int i;
	
	//lecture 3D WMO start code
	ibs.readnbit(m_WMO,16);

	if (m_WMO!=WMO_start_code) {
		//pas de start code en dbut de bitstream
		return;
	}

	ibs.readnbit(m_type,2);

	switch (m_type) {

		case 0: //Base Layer
			m_wmoh.read(ibs);
			while (readnextWMOBL(ibs)) {
			}
			break;

		case 1: //Refinement Layer
			while (readnextWMORL(ibs)) {
			}
			break;

		case 2: //Bitplane Refinement Layer
			unsigned short int NbZT;
			ibs.readnbit(NbZT,16);
			for (i=0; i<NbZT; i++) {
				readnextWMOBRL(ibs);
			}
			break;

		default:
			break;
	}
	return;
}

int Wavelet_Mesh_Object::readnextWMOBL(ibitstream &ibs) {
	unsigned short int ztID;
	unsigned char i0, np;

	ibs.readnbit(m_WMOL,16);

	//verification Wavelet Mesh Object Base Layer
	if (m_WMOL!=WMOBL_start_code) {
		return 0;
	}

	//lecture Base Layer
	ibs.readnbit(ztID,16);
	ibs.readnbit(i0,4);
	ibs.readnbit(np,4);
	for (int i=i0; i<i0+np; i++) {
		//readbitplane(ztID,i,ibs)
	}
	return 1;
}

int Wavelet_Mesh_Object::readnextWMORL(ibitstream &ibs) {
	unsigned short int ztID;
	unsigned char i0, np;

	ibs.readnbit(m_WMOL,16);

	//verification Wavelet Mesh Object Base Layer
	if (m_WMOL!=WMORL_start_code) {
		return 0;
	}

	//lecture Refinement Layer
	ibs.readnbit(ztID,16);
	ibs.readnbit(i0,4);
	ibs.readnbit(np,4);
	for (int i=i0; i<i0+np; i++) {
		//readbitplane(ztID,i,ibs)
	}
	return 1;
}

int Wavelet_Mesh_Object::readnextWMOBRL(ibitstream &ibs) {
	unsigned short int ztID;
	unsigned char i0, np;

	//lecture Bitplane Refinement Layer 
	ibs.readnbit(ztID,16);
	ibs.readnbit(i0,4);
	ibs.readnbit(np,4);
	for (int i=i0; i<i0+np; i++) {
		//readbitplane(ztID,i,ibs)
	}

	return 1;
}

DecSpecInfo::DecSpecInfo(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, double xm, double ym, double zm) {

	hasScaleCoeff=hsc;
	NbBpSC=nb;
	NbBPX=nx;
	NbBPY=ny;
	NbBPZ=nz;
	Wtype[0]=w1;
	Wtype[1]=w2;
	lift=lft;
	NumberOfLevels=lev;

	Xmax=xm;
	Ymax=ym;
	Zmax=zm;
}


DecSpecInfo::DecSpecInfo() {
	hasScaleCoeff=0;
	NbBpSC=0;
	NbBPX=15;
	NbBPY=15;
	NbBPZ=15;
	Wtype[0]=0;
	Wtype[1]=0;
	lift=0;
	NumberOfLevels=4;

	Xmax=Ymax=Zmax=0;
}

DecSpecInfo::DecSpecInfo(char *nom) {
	
	FILE *f=fopen(nom, "r");
	
	fscanf(f, "%d", &hasScaleCoeff);
	fscanf(f, "%d", &NbBpSC);
	fscanf(f, "%d", &NbBPX);
	fscanf(f, "%d", &NbBPY);
	fscanf(f, "%d", &NbBPZ);
	fscanf(f, "%d", &NbBpSC);
	fscanf(f, "%d", &Wtype[0]);
	fscanf(f, "%d", &Wtype[1]);
	fscanf(f, "%d", &lift);
	fscanf(f, "%d", &NumberOfLevels);
	//To be added
	fscanf(f, "%d", &Xmax);
	fscanf(f, "%d", &Ymax);
	fscanf(f, "%d", &Zmax);
	
	fclose(f);
}

EncoderConfig::EncoderConfig(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, bool ib, bool iilc, short lnb, bool ip, int fbp, int lbp, bool x, bool y, bool z) {

	SetValues(hsc, nb, nx, ny, nz, w1, w2, lft, lev, 0, 0, 0);
	isBase=ib;
	isInLocalCoordinates=iilc;
	LengthNbBits=lnb;
	isPartial=ip;
	FBP=fbp;
	LBP=lbp;
	X=x; 
	Y=y; 
	Z=z;
	
}

EncoderConfig::EncoderConfig(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, bool ib, bool iilc, short lnb, bool ip, int fbp, int lbp, bool x, bool y, bool z, double xm, double ym, double zm) {

	SetValues(hsc, nb, nx, ny, nz, w1, w2, lft, lev, xm, ym, zm);
	Xmax=xm;
	Ymax=ym;
	Zmax=zm;
	isBase=ib;
	isInLocalCoordinates=iilc;
	LengthNbBits=lnb;
	isPartial=ip;
	FBP=fbp;
	LBP=lbp;
	X=x; 
	Y=y; 
	Z=z;
	
}


EncoderConfig::EncoderConfig() {

	hasScaleCoeff=0;
	NbBpSC=0;
	NbBPX=12;
	NbBPY=12;
	NbBPZ=12;	
	FBP=0;
	LBP=12;
	Wtype[0]=0;
	Wtype[1]=1;
	lift=0;
	NumberOfLevels=2;

	isBase=1;
	isInLocalCoordinates=0;
	LengthNbBits=16;
	isPartial=0;
	X=1; 
	Y=1; 
	Z=1;

	//v2
	filetype=0;
	strcpy(inputname, "");
	strcpy(outputname, "encoded");
	isInBand=0;

}

EncoderConfig::EncoderConfig(char *nom) {
	
	FILE *f=fopen(nom, "r");

	if (f == NULL) {
		EncoderConfig();
		return;
	}
	
	//DSI
	fscanf(f, "%d", &hasScaleCoeff);
	fscanf(f, "%d", &NbBpSC);
	fscanf(f, "%d", &NbBPX);
	fscanf(f, "%d", &NbBPY);
	fscanf(f, "%d", &NbBPZ);
	fscanf(f, "%d", &NbBpSC);
	fscanf(f, "%d", &Wtype[0]);
	fscanf(f, "%d", &Wtype[1]);
	fscanf(f, "%d", &lift);
	fscanf(f, "%d", &NumberOfLevels);
	//To be added
	fscanf(f, "%d", &Xmax);
	fscanf(f, "%d", &Ymax);
	fscanf(f, "%d", &Zmax);
	//EC
	fscanf(f, "%d", &isBase);
	fscanf(f, "%d", &isInLocalCoordinates);
	fscanf(f, "%d", &LengthNbBits);
	fscanf(f, "%d", &isPartial);
	fscanf(f, "%d", &FBP);
	fscanf(f, "%d", &LBP);
	fscanf(f, "%f\n", &X);
	fscanf(f, "%f\n", &Y); 
	fscanf(f, "%f\n", &Z);

	NbBPX--;
	NbBPY--;
	NbBPZ--;
	
	fclose(f);
}

EncoderConfig::EncoderConfig(char *nom, const bool DecSpecInfo) {
	
	FILE *f=fopen(nom, "r");

	if (f == NULL) {
		EncoderConfig();
		return;
	}
	
	//DSI
	fscanf(f, "%d", &hasScaleCoeff);
	fscanf(f, "%d", &NbBpSC);
	fscanf(f, "%d", &NbBPX);
	fscanf(f, "%d", &NbBPY);
	fscanf(f, "%d", &NbBPZ);
	fscanf(f, "%d", &Wtype[0]);
	fscanf(f, "%d", &Wtype[1]);
	fscanf(f, "%d", &lift);
	fscanf(f, "%d", &NumberOfLevels);
	//To be added
	fscanf(f, "%f", &Xmax);
	fscanf(f, "%f", &Ymax);
	fscanf(f, "%f", &Zmax);

	NbBPX--;
	NbBPY--;
	NbBPZ--;
	
	fclose(f);
}

/*Neudcoeff_loc---------------------------------------------------------------------------------------*/

noeudcoeff_loc::noeudcoeff_loc() {

	Age=0;
	alpha=0;
	beta=0;
	gamma=0;
	F0=0;
	coeffvalue=0;
	m_frere=NULL;
	m_fils=NULL;
	ind=0;
	quantvalue=0;
	
}

noeudcoeff_loc* noeudcoeff_loc::fils() {
	// Si this a deja un fils, le retourne, sinon le cre
	
	if (m_fils!=NULL) 
		return m_fils;
	else {

		int direction;
		int level = alpha+beta+gamma;

		//cration 
		noeudcoeff_loc* newcoeff= new noeudcoeff_loc();
		
		//determination de l'orientation (donn par la coordonne paire)
		direction = (!(alpha%2))+2*(!(beta%2))+3*(!(gamma%2));

		//determination des coordonnes barycentriques
		switch(direction) {
			case 1:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha;
				newcoeff->beta=2*beta+1;
				newcoeff->gamma=2*gamma-1;

				break;
			case 2:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha-1;
				newcoeff->beta=2*beta;
				newcoeff->gamma=2*gamma+1;

				break;
			case 3:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha+1;
				newcoeff->beta=2*beta-1;
				newcoeff->gamma=2*gamma;

		}

		//C'est le premier fils
		newcoeff->Age=1;

		//Fils et frere  NULLS
		newcoeff->m_fils=newcoeff->m_frere=NULL;

		m_fils=newcoeff;



		return m_fils;
	}
}



noeudcoeff_loc* noeudcoeff_loc::fils(int currentcoeffnumber) {
	// Si this a deja un fils, le retourne, sinon le cre
	
	if (m_fils!=NULL) 
		return m_fils;
	else {

		int direction;
		int level = alpha+beta+gamma;

		//cration 
		noeudcoeff_loc* newcoeff=new noeudcoeff_loc();
		
		//determination de l'orientation (donn par la coordonne paire)
		direction = (!(alpha%2))+2*(!(beta%2))+3*(!(gamma%2));

		//determination des coordonnes barycentriques
		switch(direction) {
			case 1:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha;
				newcoeff->beta=2*beta+1;
				newcoeff->gamma=2*gamma-1;
				newcoeff->ind=currentcoeffnumber;
				currentcoeffnumber++;
				break;

			case 2:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha-1;
				newcoeff->beta=2*beta;
				newcoeff->gamma=2*gamma+1;
				newcoeff->ind=currentcoeffnumber;
				currentcoeffnumber++;
				break;

			case 3:
				newcoeff->F0=F0;
				newcoeff->alpha=2*alpha+1;
				newcoeff->beta=2*beta-1;
				newcoeff->gamma=2*gamma;
				newcoeff->ind=currentcoeffnumber;
				currentcoeffnumber++;
		}

		//C'est le premier fils
		newcoeff->Age=1;

		//Fils et frere  NULLS
		newcoeff->m_fils=newcoeff->m_frere=NULL;

		m_fils=newcoeff;
		
		return m_fils;
	}
}



noeudcoeff_loc* noeudcoeff_loc::frere(Mesh *BM) {
	// Si this a deja un frere, le retourne, sinon le cre

	if (m_frere!=NULL) 
		return m_frere;
	else {
	
		int only3;
		int direction;
		int level = (alpha+beta+gamma)/2;//niveau du pre
		int alphapar, betapar, gammapar;

		if (Age==4 || Age==0)
			return NULL;//Pas de frre

		//cration
		noeudcoeff_loc* newcoeff=new noeudcoeff_loc();

		//determination de l'orientation (donn par la coordonne paire)
		direction = (!(alpha%2))+2*(!(beta%2))+3*(!(gamma%2));

		//determination des coordonnes du parent
		switch(direction) {
			case 1:	// tourns vers alpha
				switch(Age) {
				case 1:
					//premier fils
					alphapar=alpha/2;
					betapar=(beta-1)/2;
					gammapar=(gamma+1)/2;
					break;

				case 2:
					//deuxime fils
					alphapar=alpha/2;
					betapar=(beta+1)/2;
					gammapar=(gamma-1)/2;
					break;
				case 3:
					//troisime fils
					alphapar=(alpha-2)/2;
					betapar=(beta+1)/2;
					gammapar=(gamma+1)/2;
					break;
				}
				

			break;

			case 2:	// tourns vers beta
				switch(Age) {
				case 1:
					alphapar=(alpha+1)/2;
					betapar=beta/2;
					gammapar=(gamma-1)/2;
					break;

				case 2:
					//deuxime fils
					alphapar=(alpha-1)/2;
					betapar=beta/2;
					gammapar=(gamma+1)/2;
					break;

				case 3:
					//troisime fils
					alphapar=(alpha+1)/2;
					betapar=(beta-2)/2;
					gammapar=(gamma+1)/2;
					break;
					
				}				
			break;
	

			case 3:	// tourns vers gamma
				switch(Age) {
				case 1:
					//premier fils
					alphapar=(alpha-1)/2;
					betapar=(beta+1)/2;
					gammapar=gamma/2;
					break;
				case 2:
					//deuxime fils
					alphapar=(alpha+1)/2;
					betapar=(beta-1)/2;
					gammapar=gamma/2;
					break;
				case 3:
					//troisime fils
					alphapar=(alpha+1)/2;
					betapar=(beta+1)/2;
					gammapar=(gamma-2)/2;
					break;
				}
			break;
			
		}

		//determination des coordonnes barycentriques en fonction des coordonnes parents
		switch (direction) {
			case 1:		
				switch(Age) {
					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar;
						newcoeff->beta=2*betapar-1;
						newcoeff->gamma=2*gammapar+1;
	
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar+2;
						newcoeff->beta=2*betapar-1;
						newcoeff->gamma=2*gammapar-1;
	
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar-2;
						newcoeff->beta=2*betapar+1;
						newcoeff->gamma=2*gammapar+1;
	

				}
				break;
			case 2:		
				switch(Age) {

					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar+1;
						newcoeff->beta=2*betapar;
						newcoeff->gamma=2*gammapar-1;
		
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar-1;
						newcoeff->beta=2*betapar+2;
						newcoeff->gamma=2*gammapar-1;
	
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar+1;
						newcoeff->beta=2*betapar-2;
						newcoeff->gamma=2*gammapar+1;


				}
				break;
			case 3:		

				switch(Age) {
					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar-1;
						newcoeff->beta=2*betapar+1;
						newcoeff->gamma=2*gammapar;
		
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar-1;
						newcoeff->beta=2*betapar-1;
						newcoeff->gamma=2*gammapar+2;
		
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=2*alphapar+1;
						newcoeff->beta=2*betapar+1;
						newcoeff->gamma=2*gammapar-2;
		

				}
		}


		//changement de coordonnes ventuel
		if (newcoeff->alpha<0 || newcoeff->beta<0 || newcoeff->gamma<0) {
			int al, be, ga;
			int bary[4];
			int olny3;
			bary[0]=newcoeff->F0;
			bary[1]=newcoeff->alpha;
			bary[2]=newcoeff->beta;
			bary[3]=newcoeff->gamma;
			only3=zerotree::changeFace(bary, BM, &al, &be, &ga, &bary[0]);
			newcoeff->F0=bary[0];
			if (only3) {
				free(newcoeff);
				return NULL;
			} else {
				newcoeff->alpha=al;
				newcoeff->beta=be;
				newcoeff->gamma=ga;
			}
		}

		//Age
		newcoeff->Age=Age+1;

		//Fils et frere  NULLS
		newcoeff->m_fils=newcoeff->m_frere=NULL;

		m_frere=newcoeff;


		return m_frere;
	}

}

noeudcoeff_loc* noeudcoeff_loc::frere(Mesh *BM, int currentcoeffnumber) {
	// Si this a deja un frere, le retourne, sinon le cre

	if (m_frere!=NULL) 
		return m_frere;
	else {
	
		int only3;
		int direction;
		int level = (alpha+beta+gamma)/2;//niveau du pre

		if (Age==4 || Age==0)
			return NULL;//Pas de frre

		//cration
		noeudcoeff_loc* newcoeff=new noeudcoeff_loc();

		//determination de l'orientation (donn par la coordonne paire)
		direction = (!(alpha%2))+2*(!(beta%2))+3*(!(gamma%2));

		//determination des coordonnes barycentriques
		switch (direction) {
			case 1:		
				switch(Age) {
					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha;
						newcoeff->beta=beta-level;
						newcoeff->gamma=gamma+level;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha+level;
						newcoeff->beta=beta;
						newcoeff->gamma=gamma-level;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha-2*level;
						newcoeff->beta=beta;
						newcoeff->gamma=gamma;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;

				}
				break;
			case 2:		
				switch(Age) {
					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha+level;
						newcoeff->beta=beta;
						newcoeff->gamma=gamma-level;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha-level;
						newcoeff->beta=beta+level;
						newcoeff->gamma=gamma;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha;
						newcoeff->beta=beta-2*level;
						newcoeff->gamma=gamma;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;

				}
				break;
			case 3:		
				switch(Age) {
					case 1:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha-level;
						newcoeff->beta=beta+level;
						newcoeff->gamma=gamma;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 2:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha;
						newcoeff->beta=beta-level;
						newcoeff->gamma=gamma+level;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;
						break;
					case 3:
						newcoeff->F0=F0;
						newcoeff->alpha=alpha;
						newcoeff->beta=beta;
						newcoeff->gamma=gamma-2*level;
						newcoeff->ind=currentcoeffnumber;
						currentcoeffnumber++;

				}
		}

		//changement de coordonnes ventuel
		if (newcoeff->alpha<0 || newcoeff->beta<0 || newcoeff->gamma<0) {
			char al, be, ga;
			int bary[4];
			int olny3;
			bary[0]=newcoeff->F0;
			bary[1]=newcoeff->alpha;
			bary[2]=newcoeff->beta;
			bary[3]=newcoeff->gamma;
			only3=changeFace(bary, BM, &al, &be, &ga, &(newcoeff->F0));
			if (only3) {
				free(newcoeff);
				return NULL;
			} else {
				newcoeff->alpha=al;
				newcoeff->beta=be;
				newcoeff->gamma=ga;
			}
		}

		//Age
		newcoeff->Age=Age+1;

		//Fils et frere  NULLS
		newcoeff->m_fils=newcoeff->m_frere=NULL;

		m_frere=newcoeff;
		
		return m_frere;
	}

}

int noeudcoeff_loc::changeFace(int bary[4], Mesh *BM,  char *al,  char *be,  char *ga, unsigned short *F0) {

	int aprime, bprime, cprime;
	int b1, b2;
	int ok;

	// Reprage de l'arte
	ok = 1*(bary[1]<0)+2*(bary[2]<0)+4*(bary[3]<0);
	switch (ok) {
		case 1:
			b1=BM->Face[bary[0]][1];
			b2=BM->Face[bary[0]][2];
			break;
		case 2:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][2];
			break;
		case 4:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][1];
			break;
	}

	// Reprage des faces voisines
	Lint *run=BM->FaceVoisine[b1];

	// On parcourt la liste des voisins
	while (run!=NULL) {


		ok = ((b2==BM->Face[run->facette][0])||
			(b2==BM->Face[run->facette][1])||
			(b2==BM->Face[run->facette][2])) && run->facette!=bary[0];
		if (ok) {
			aprime=BM->Face[run->facette][0];
			bprime=BM->Face[run->facette][1];
			cprime=BM->Face[run->facette][2];
			goto facetfound;
		}
		run=run->suivant;
	}
	//Not found
	return 1;
facetfound:;

	chCoord3s(bary[1], bary[2], bary[3], BM->Face[bary[0]], BM->Face[run->facette], al, be, ga);

	//redressement des coordonnes ngatives
	if (*al<0) 
		*al=-*al;
	if (*be<0) 
		*be=-*be;
	if (*ga<0) 
		*ga=-*ga;

	*F0=run->facette;

	return 0;

}

int noeudcoeff_loc::ImplicitIndex(int level) {

//retourne l'indice implicite donn par alpha, beta, gamma et F0

	int c=(1<<level)+1;	//nombre de sommet  la base du triangle
	int n=gamma*c/(alpha+beta+gamma);	//nombre d'tages
										//on se convaincra que c'est bien un entier
	int m=beta*c/(alpha+beta+gamma);	//colonne

	return ((F0+1)*(c*(c+1))/2)-((c-n+1)*(c-n))/2+m;	//faire un dessin pour s'en convaincre

		

}

int EncoderConfig::ReadEncodingParameters(char* filename) {

	char tmp[512], rem[512];
	int tmpi, i=0, foundItem;
	FILE *f=fopen(filename, "r");

	//Error management
	if (!f) 
		return 1;

	//read the first string
	fscanf(f, "%s", tmp);

	//loop over the lines
	while (tmp[0]!=0) {
		foundItem=0;

		//Inband / Outband
		if (!strcmp(tmp, "isInBand")) {
			fscanf(f, "%d", &isInBand);
			foundItem=1;
		}
		//local coordinates
		if (!strcmp(tmp, "isInLocalCoordinates")) {
			fscanf(f, "%d", &isInLocalCoordinates);
			foundItem=1;
		}
		//Filetype
		if (!strcmp(tmp, "filetype")) {
			fscanf(f, "%d", &filetype);
			foundItem=1;
		}
		//input file
		if (!strcmp(tmp, "input")) {
			fscanf(f, "%s", inputname);
			foundItem=1;
		}
		//output file
		if (!strcmp(tmp, "output")) {
			fscanf(f, "%s", outputname);
			foundItem=1;
		}
		//scaling coefficients
		if (!strcmp(tmp, "hasScaleCoeff")) {
			fscanf(f, "%d", &hasScaleCoeff);
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "lift")) {
			fscanf(f, "%d", &lift);
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "Wtype")) {
			fscanf(f, "%d", &tmpi);
			Wtype[0]=tmpi/2;
			Wtype[1]=tmpi%2;
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "NumberOfLevels")) {
			fscanf(f, "%d", &NumberOfLevels);
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "NbBPX")) {
			fscanf(f, "%d", &NbBPX);
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "NbBPY")) {
			fscanf(f, "%d", &NbBPY);
			foundItem=1;
		}
		//lifting
		if (!strcmp(tmp, "NbBPZ")) {
			fscanf(f, "%d", &NbBPZ);
			foundItem=1;
		}



		//Next string
		strcpy(rem, tmp);
		fscanf(f, "%s", tmp);
		if (!strcmp(rem, tmp))
			tmp[0]=0;
		i++;

		//prevents unexpected loops
		if (i>15)
			return 2;

		//Parsing error
		if (!foundItem)
			return 3;

	}
	
	return 0;

}

void DecSpecInfo::SetValues(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, double x, double y, double z) {

	hasScaleCoeff=hsc;
	NbBpSC=nb;
	NbBPX=nx;
	NbBPY=ny;
	NbBPZ=nz;
	Wtype[0]=w1;
	Wtype[1]=w2;
	lift=lft;
	NumberOfLevels=lev;

	Xmax=x;
	Ymax=y;
	Zmax=z;
}


			

			



