#ifndef STRUCTURE
#define STRUCTURE
#include "bitstream.h"
#include "wavecore.h"


//!vertex
struct vertex {
	unsigned char depth;
	float x,y,z;
};

//!triangle
struct triangle {
	unsigned char tind;
	unsigned char root;
	int v1,v2,v3;
};

//!noeudcoeff
struct noeudcoeff {
	float* coeffvalue;
	int ind;
	unsigned short int quantvalue;
	noeudcoeff* fils;
	noeudcoeff* frere;

	noeudcoeff() {
		fils = NULL;
		frere = NULL;
	}
};

//!noeudcoeff
class noeudcoeff_loc {
	
public:	
	float* coeffvalue;
	int ind;
	unsigned short int quantvalue;
	unsigned short int F0;
	char alpha;
	char beta;
	char gamma;
	noeudcoeff_loc* m_fils;
	noeudcoeff_loc* m_frere;
	unsigned char Age;
	
public:
	noeudcoeff_loc* fils();
	noeudcoeff_loc* fils(int);
	noeudcoeff_loc* frere(Mesh *BM);
	noeudcoeff_loc* frere(Mesh *BM, int);
	int changeFace(int bary[4], Mesh *BM,  char *al,  char *be,  char *ga, unsigned short *F0);
	int ImplicitIndex(int level);

	noeudcoeff_loc();
};




//!entry
struct entry {
	noeudcoeff* pnc;
	int entrytype;
	entry* next;

	entry() {
		pnc = NULL;
		next = NULL;
	}
};

//!entry
struct entry_loc {
	noeudcoeff_loc* pnc;
	int entrytype;
	entry_loc* next;

	entry_loc() {
		pnc = NULL;
		next = NULL;
	}
};

struct voisins {
	int vo1, vo2, vo3;

	voisins() {
		vo1 = -1;
		vo2 = -1;
		vo3 = -1;
	}
};

//---------------------------------------------------------------------------------------------------------
#define WMO_start_code 0
#define WMOBL_start_code 1
#define WMORL_start_code 2
#define WMOBRL_start_code 3

//!scaling header
struct scaling_header {
	unsigned int nb_sc;
	unsigned char NbBpC;
};

//!wavelet header
struct wavelet_header {
	unsigned char m_Wtype;
};

//!coeff header 
struct coeff_header {
	unsigned char m_NbBP;
	float m_Cmax;
	unsigned char m_depth;
};

//!ZT header
struct ZT_header {
	unsigned char m_ZTtype;
};

class Wavelet_Mesh_Object_Header {
public:
	unsigned char m_Base_id;

	scaling_header m_s_h;
	wavelet_header m_w_h;
	coeff_header m_c_h;
	ZT_header m_ZT_h;

	int read(ibitstream &i);

	Wavelet_Mesh_Object_Header();
	~Wavelet_Mesh_Object_Header();
};

//!Wavelet Mesh Object
class Wavelet_Mesh_Object {
public:
	unsigned short int m_WMO;
	unsigned short int m_WMOL;
	unsigned char m_type;

	Wavelet_Mesh_Object_Header m_wmoh;

	void read(ibitstream &i);
	int readnextWMOBL(ibitstream & i);
	int readnextWMORL(ibitstream & i);
	int readnextWMOBRL(ibitstream & i);

	Wavelet_Mesh_Object();
	~Wavelet_Mesh_Object();
};


class DecSpecInfo {
	// See the spec for semantics
public:
	bool hasScaleCoeff;
	short NbBpSC;
	short NbBPX;
	short NbBPY;
	short NbBPZ;
	bool Wtype[2];
	bool lift;
	short NumberOfLevels;
	float Xmax, Ymax, Zmax;

	DecSpecInfo();
	DecSpecInfo(bool, short, short, short, short, bool, bool, bool, short, double, double, double);
	DecSpecInfo(char *);
	void SetValues(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, double x, double y, double z);


};

class EncoderConfig : public DecSpecInfo {
public:
	//v1
	bool isBase;
	bool isInLocalCoordinates;
	short LengthNbBits;
	bool isPartial;
	int FBP, LBP;
	bool X, Y, Z;

	//v2 
	bool isInBand;
	int filetype;
	char inputname[512], outputname[512];


	EncoderConfig();
	EncoderConfig(char *, const bool DecSpecInfo);
	EncoderConfig(char *);
	EncoderConfig(bool, short, short, short, short, bool, bool, bool, short, bool, bool, short, bool, int, int, bool, bool, bool);
	EncoderConfig::EncoderConfig(bool hsc, short nb, short nx, short ny, short nz, bool w1, bool w2, bool lft, short lev, bool ib, bool iilc, short lnb, bool ip, int fbp, int lbp, bool x, bool y, bool z, double xm, double ym, double zm);
	bool SetFields(bool,short, short, short, bool, bool, bool, bool, short, bool, bool, short, bool, int, int, bool, bool, bool);

	//v2
	int ReadEncodingParameters(char *filename);


};


#endif

