
#ifndef ZEROTREE
#define ZEROTREE
#include "structure.h"
#include "bitstream.h"
#include "liste_entry.h"
#include "proto.h"
#include "BSD.h"

#define Xbit 11
#define Ybit 8
#define Zbit 8

//#define LOGZT

//!une classe pour les zerotrees associs au diffrentes artes du maillage de base.
class sub_zerotree {
public:
	//!ID sub zerotree
	int m_ztID;

	//!indice dernier plan de bit visit
	//Cas global
	char m_lastplanbit;
	//Cas spars
	char m_lastplanbitX;
	char m_lastplanbitY;
	char m_lastplanbitZ;

	//!la liste LSP.
	liste_entry m_LSP;

	//V.2
	liste_entry m_LSPX;
	liste_entry m_LSPY;
	liste_entry m_LSPZ;

	//!liste stockant les coefficient  ajouter  la liste m_LSP.
	/*!les coefficients de cette liste sont ajouts  m_LSP prs le Refinement Pass
	*/
	liste_entry m_LSPadd;
	
	//V.2
	liste_entry m_LSPXadd;
	liste_entry m_LSPYadd;
	liste_entry m_LSPZadd;

	//!la liste LIP.
	liste_entry m_LIP;

	//V.2
	liste_entry m_LIPX;
	liste_entry m_LIPY;
	liste_entry m_LIPZ;

	//!la lsite LIS
	liste_entry m_LIS;
	
	//V.2
	liste_entry m_LISX;
	liste_entry m_LISY;
	liste_entry m_LISZ;


	//!racine arbre de coefficient
	noeudcoeff* m_ncroot;

	//!constructeur
	sub_zerotree();

	//!constructeur
	sub_zerotree(int ztID);

	//!destructeur
	~sub_zerotree();
};

class sub_zerotree_loc {
public:
	//!ID sub zerotree
	int m_ztID;

	//!indice dernier plan de bit visit
	//Cas global
	char m_lastplanbit;
	//Cas spars
	char m_lastplanbitX;
	char m_lastplanbitY;
	char m_lastplanbitZ;

	int m_currentcoeffnumber;

	//!la liste LSP.
	liste_entry_loc m_LSP;

	//V.2
	liste_entry_loc m_LSPX;
	liste_entry_loc m_LSPY;
	liste_entry_loc m_LSPZ;

	//!liste stockant les coefficient  ajouter  la liste m_LSP.
	/*!les coefficients de cette liste sont ajouts  m_LSP prs le Refinement Pass
	*/
	liste_entry m_LSPadd;
	
	//V.2
	liste_entry_loc m_LSPXadd;
	liste_entry_loc m_LSPYadd;
	liste_entry_loc m_LSPZadd;

	//!la liste LIP.
	liste_entry_loc m_LIP;

	//V.2
	liste_entry_loc m_LIPX;
	liste_entry_loc m_LIPY;
	liste_entry_loc m_LIPZ;

	//!la lsite LIS
	liste_entry_loc m_LIS;
	
	//V.2
	liste_entry_loc m_LISX;
	liste_entry_loc m_LISY;
	liste_entry_loc m_LISZ;


	//!racine arbre de coefficient
	noeudcoeff_loc* m_ncroot;

	//!constructeur
	sub_zerotree_loc();

	//!constructeur
	sub_zerotree_loc(int ztID);

	//!destructeur
	~sub_zerotree_loc();
};

//!zerotree
class zerotree {
public:
	//!nombre de plans de bits.
	char m_maxn;

	//!Nombre de triangles.du maillage de base
	int m_nbbasetriangle;

	//!nombre de vertex.
	int m_nbvertex;

	//!nombre de triangles.
	int m_nbtriangle;

	int m_statin, m_statout;

	bool m_firsttime;


	//!constructeur.
	zerotree();

	//!destructeur.
	~zerotree();

	//!rinitialisation du zerotree.
	void reset();
	
	

	//!encodage de tous les coeffients dans un seul zerotree
	/*!nomf est le nom du fichier a encoder. Cette mthode provoque le chargement du fichier<br>
	 * la construction des arbres de coefficients puis l'encodage.
	 */
	void encode_all(char* nomf);

	//! decodage de chaque zerotree associe a chacune des artes du maillage de base.
	void decode(char* nomf);

	void decodemp4(char* nomf);

	void decodemp4_conf(char*, char*, Mesh *);
	void decodemp4_conf(unsigned char*, int, Mesh *);
	void decodemp4_conf_loc(unsigned char*, int, Mesh *, int nbcoeff);
	int mp4decode(unsigned char*, int, Mesh *, int nbcoeff);

	//!decodage de tous les coeffients dans un seul zerotree.
	void decode_all(char* nomf);

	//!produit une trace pour le debug
	void dump();
	void dumpZ2Dat(DatMesh*);

	//gre les paramtre en ligne de commande.
	void Do(char* param);

	void start_decode(char* nomf);
	void start_decode_conf(Mesh *BM);
	void start_decode_conf_v2(Mesh *BM);
	void start_decode_conf_loc(Mesh *BM);

	void encodemp4(char* nomf);

	void encodemp4_conf();
	void encodemp4_conf(char*);
	void encodemp4_conf(DatMesh*);
	void encodemp4_conf(unsigned char *bufferout, int &size, int bufmaxsize, DatMesh *DM);
	int encodemp4_conf(unsigned char* bufferout, int &bufsize, int bufmaxsize);
	//Encoding from a base mesh and WaveRep
	int encodemp4_conf(Mesh *BM, WaveRep *WR, unsigned char* bufferout, int& bufsize, int bufmaxsize, BSD* BSDesc);
	int encodeAndBSD(int nzt, Mesh *BM, WaveRep *WR, mbitstream *, int &bufsize, int bufmaxsize, BSD* BSDesc);

	void reverseRoots(int);

	void ConformantOrdering(int);//ZTs and offspring get there their afx conformant ordering
	//geometric reconstruction
	void NaiveRebuild();

	void reOrderRoots();

	//TEST

	void testw();
	void testr();

	int GetWType();

	void SetName(char*);

	void writeNode();
	int createZTHierarchy(int root, WaveRep* WR, int, Mesh *BM);
	int deleteZTHierarchy(noeudcoeff* p);
	int iterbary(int bary[4], Mesh *BM, WaveRep *WR, noeudcoeff* fatherx, noeudcoeff* fathery, noeudcoeff* fatherz);
	static int changeFace(int bary[4], Mesh *BM, int *al, int *be, int *ga, int *F0);

private:
	//-------------------------------------------- Membres Privs ---------------------------------------	

	
	//!valeur aabsolue maximale des coefficients.
	float m_maxcoeff;
	//Separate cases
	float m_maxcoeffX;
	float m_maxcoeffY;
	float m_maxcoeffZ;

	

	//!nombre de sous zerotree
	unsigned short int m_nbsub_zerotree;

	//!indice prochaine bitstream  lire
	int m_nextstreamind;

	//!activation streaming
	int m_streamingON;

public:

	//!profondeur maillage.
	unsigned char m_depth;

private:

	//!indice du triangle zn cours de lecture.
	int m_ind;
	
	//!tableau dynamique de vertex.
	vertex* m_tabvertex;

	//!tableau dynamique de triangles.
	triangle* m_triangles;

	sub_zerotree* m_tabsub_zerotree;
	sub_zerotree_loc* m_tabsub_zerotree_loc;

	//!tableau dynamique d'indices des parents de chaque sommet.
	/*!pour chaque sommet m_tabvertex[i], m_vertexparent[i] est l'indice du sommet<br>
	 * parent dans m_tabvertex.
	 */
	int* m_vertexparent;

	//!nom du fichier complet en cours d'encodage/dcodage.
	char* m_nomf;

	char m_basename[200];

	//!liste des noeuds de coefficient racines, 3 pour chaque arete du maillage initial.
	noeudcoeff* m_treecoeff;
	noeudcoeff_loc* m_treecoeff_loc;

	//!stream de lecture dans un fichier au bit prs.
	obitstream m_o;

	//!stream d' criture dans un fichier au bit prs.
	ibitstream m_i;

	//!stream d'criture dans un fichier pour le debug.
	ofstream m_dumpstream;


	//!Tableau dynamique des voisins des triangles.
	voisins* m_triangles_voisins;

	int m_currentcoeffnumber;


public:
	EncoderConfig *m_EC;


	//-------------------------------------------- Mthodes Prives ------------------------------------	


	int zerotree::writeplanem_conf(int i, unsigned char n, mbitstream& obs, int flag, WaveRep* WR, Mesh* BM);

	int zerotree::sign(noeudcoeff* pnc);

	void deletefils(noeudcoeff* pnc);

	//!decodage de chaque zerotree associe a chacune des artes du maillage de base.
	//void decode();

	//void decodemp4();



	void threeD_Wavelet_Mesh_Object_Scale_coeff();

	//!quantization d'un coefficient par rapport  m_maxcoeff.
	//unsigned short int quantize(float f);
	unsigned short int quantize(float f, unsigned char nbit);
	unsigned short int quantize(float f, unsigned char nbit, int flag);

	//!quantization inverse d'une valeur quantifie.
	float unquantize(unsigned short int usi);
	float unquantize(unsigned short int usi, int flag);

	float unquantize(unsigned short int usi, unsigned char nbit);

	//!quantization inverse de toutes les valeurs quantifies d'un noeud de coefficient.
	void unquantize(noeudcoeff* pnc);
	void unquantize(noeudcoeff* pnc, int flag);

	void unquantize(noeudcoeff* pnc, unsigned char nbit);
	void unquantize(noeudcoeff_loc* pnc, int flag);

	//!criture des valeurs quantifies d'un noeud de coefficient.pour le debug
	void dumpquantize(noeudcoeff* pnc);
	void dumpquantize(noeudcoeff_loc* pnc);

	//!fonction Sn.
	int SN(int n, noeudcoeff* nc);
	int SN(int n, noeudcoeff* nc, int flag);

	//!mise  jour de la valeur quantifie du noeud de coefficient nc.
	void setSN(int n, int sn, noeudcoeff* nc);
	void setSN(int n, int sn, noeudcoeff_loc* nc);

	//!fonction Snd.
	int SND(int n, entry* e);
	int SND(int n, entry* e, int flag);

	//!fonction Snd.
	int SNL(int n, entry* e);
	int SNL(int n, entry* e, int flag);
	
	//!calcul du signe d'un noeud de coefficient pnc.
	int posorneg(noeudcoeff* pnc);

	//!mise  jour du signe de la valeur quantifie du noeud de coefficient pnc.
	void setsign(noeudcoeff* pnc, int sign);
	void setsign(noeudcoeff_loc* pnc, int sign);
	
	//!test L(i,j) vide.
	int Lijvide(entry* e);
	int Lijvide(entry_loc* e);

	//!lecture du fichier  l'encodage.
	int read_encode(char* nomf);
	int read_encode(DatMesh*);
	int read_encode_conf(char* nomf);

	//!lecture du fichier au dcodage.
	int read_decode(DatMesh*);

	//!determine le sommet parent de chaque sommet de m_tabvertex.
	/*!l'indice du sommet parent d'un sommet et enregistr dans m_vertexparent.
	 *
	 */
	void buildvertexparent();

	//!pour chaque sommet, l'indice du sommet parent dans m_vertexparent devient celui du sommet racine.
	void aplativertexparent();

	//!dtermine les sommets parents pour tous les sommets d'un triangle dun maillage de base.
	void iterbasetriangle();

	//!initialise les sommets parents pour le triangle en cours de lecture(cf m_ind).
	int iterinit(int & a, int & b, int & c);

	//!mise  jour des sommets parents d'un triangle, itration sur les sous triangles.
	void iter0(int tind, int a, int b, int c);

	//!construction des arbres de coeficients, 1 par arte du maillage de base initilal.
	int buildcoefftree();

	//!construction des arbres de coeficients, 1 par arte du maillage de base initilal.
	int buildcoefftree_conf();

	//!permute the offspring of a node in order to have an mpeg-4 conformant hierarchy
	int permuteOffspring();

	//Great name
	void zerotree::MakeConformantRootOrder(int);
	
	//!extraction du coefficient de valeur absolue maximale
	void initmaxcoeff();
	void initmaxcoeff_conf();

	//!lecture du nime plan de bit du ime subzerotree
	void readplane(int i, unsigned char n, ibitstream& ibs);

	//!lecture du nime plan de bit du ime subzerotree
	int readplane_conf(int i, unsigned char n, ibitstream& ibs, int);
	int readplane_conf(int i, unsigned char n, mbitstream *ibs, int flag);
	int readplane_conf_loc(int i, unsigned char n, mbitstream *ibs, int flag, Mesh *BM);

	//!lecture des plan de bits n0  n0-n+1 du subzerotree d'indice ztid
	void readplanes(int ztid, unsigned char n0, unsigned char n, ibitstream& ibs);

	void start_decode_conf(char* nomf, char *, Mesh *);

	//!Subdivise tous les triangles de  ce niveau.
	/*!Les triangles de niveaux infrieurs sont supposs avoir t subdiviss!!!
	 */
	void diviseniveau(int niveau);
	void diviseniveau(int niveau, unsigned char*);

	//!Mise  jour du voisinage de deux triangles, s'ils sont voisins.
	/*!tind1 et tind2 sont les indices dans m_triangles de deux triangles de mmes niveau 
	 */
	int setvoisin(int tind1, int tind2);

	//!Propagation des nouveaux sommets aux triangles voisins
	/*!tind : indice du triangle en cours de subdivision. Cette subdivision cre 4 nouveaux triangles<br>
	 * et au plus 3 nouveaux sommets qui doivent tre partags par les voinsins de tind.<br>
	 * Propage assure donc que les voinins du triangle subdivis sont galement mis  jour.<br>
	 * vind est l'indice du voisin  propager<br>
	 * ecart est la diffrence d'indice entre les sous triangles rsultant de la division.
	 */
	void propage(int tind, int vind, int ecart);

	//!Trouver un nom!!!.
	/*!Si x est l' indice d'un triangle de niveaun n dans m_triangles, f(x) est l'indice du <br>
	 * prochain triangle de niveau n : tous les triangles de mme niveau ne sont pas ncessairement<br>
	 * conscutifs
	 */
	int f(int x);

	//!Teste  l'egalit des couples 
	/*!Les couples gaux sont (a1 b1) = (a2 b2), (b1 a1) = (a2 b2), (a1 b1) = (b2 a2), (b1 a1) = (b2 a2)
	 *
	 */
	int egalecouple(int a1, int b1, int a2, int b2);

	//DecoderSpecificInfo crit par l'encodeur
	void writeDSI();
	void writeDSI(char *filename_wo_extention);

	//Recopie du maillage de base dans un Mesh
	Mesh* GetBaseMesh();

	void createRootTab(Mesh* BM);

	int dumpZ2WR(WaveRep*, Mesh*);

	void recDump2WR(noeudcoeff_loc* coeff, WaveRep*, int flag);

	inline noeudcoeff_loc* getTreeCoeffLoc() {return m_treecoeff_loc;}

	inline int GetNbRoots() {return m_nbsub_zerotree;}

	int recMoveCoeffs(noeudcoeff_loc* ncroot, int composante, int* baryCoord[4], float* Z, unsigned int& nbcoeff);

	

	int DumpZTHierarchy(char *filename);

private:
	int DumpNoeud(FILE* f, noeudcoeff* run, int indent);
	static int chCoord(int alpha, int beta, int gamma, int F0[3], int F1[3], int* al, int* be, int* ga);

	//void writetype1(int indfichier,int nbzt);

	

public:
	void readtype0();

	int readtype1(char* nomf);

	int readnextstream();

	#ifdef LOGZT
	FILE *logfile;
	#endif

};



#endif