// This software module was originally developed by Marius Preda (INT)
// in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 2003.



#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "SBVCAnimationProxy.h"
#include "BBA/SBSkinnedModelProxy.h"
#include "BBA/EulerAngles.h"
#include "Stream.h"

#include "AfxStreamProxy.h" 
#include "AfxMediaObjectProxy.h" 
#include "Im1Decoders/AFX/AFX.h" 
#include "Im1Decoders/AFX/BBA/BBADecoderLib/BBADecoderLib.h" 
#include "Im1Decoders/AFX/BBA/BBADecoderLib/BBADataExchange.h" 


void SBVCAnimationProxy::MakeLink(/*AfxStreamProxy *pProxy*/)
{
  if (!IsLinkInit())
  {
    AFXDecoder *pDecoder = (AFXDecoder *)GetSBVCAnimation()->GetDecoder();
    ASSERT(pDecoder != NULL);
    AFXTemplate *pAFXDecoder = pDecoder->GetDecoder();
    ASSERT(pAFXDecoder != NULL);

	//pAFXDecoder->m_pProxyBBA=this;

    SetDecoder(pAFXDecoder);

    SetLinkInit(true);
  }
}

void SBVCAnimationProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD )
{
  RenderDescriptionBox(pRenderer, "SBVCAnimation");
}

// Render object


BOOL SBVCAnimationProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  SBVCAnimation* poSBVCA = GetSBVCAnimation();
  int noVC = poSBVCA->virtualCharacters.GetLength();

  if (poSBVCA->GetURL().GetLength() > 0){
  
	  //AfxStreamProxy *pProxy = (AfxStreamProxy *)poSBVCA->GetRenderingObject();
	  //if (pProxy == NULL) return FALSE;
	  if (m_bIsDecodingNeeded){
		// give the decoder the pointer to the rendering object
   
		MakeLink();
		// bitstream from URL
		Start();
	    AFXDecoder *pDecoder = (AFXDecoder *)GetSBVCAnimation()->GetDecoder();
		MediaStream* poStream = pDecoder->GetOutputStream ();
		if (RenderFromStream(poStream, poSBVCA->GetID(), dwTime))
		{
		  m_bIsDecodingNeeded = false;
		  MediaStream* poStream = poSBVCA->GetStream();
		  poStream->Release();
		}
	  }
  }

	
	ZArray<MediaObjectProxy *> nodes;

  for (int i=0;i<noVC;i++){
	MediaObject * pObject = (SBSkinnedModel*) poSBVCA->virtualCharacters[i];
	MediaObjectProxy *pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());
	nodes.InsertAt(nodes.GetLength(), pProxy);
  }
	

  for (int j = 0; j < nodes.GetLength(); ++j){
	nodes[j]->CullOrRender(
		  rTransformation, 
		rCullFlag,
		pRenderer, 
		dwTime);  
  }


 return MediaObjectProxy::Render(rTransformation, rCullFlag,pRenderer, dwTime);
}

void
SBVCAnimationProxy::Select(
		const Matrix& rTransformation, 
		const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
		DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
		unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{

  SBVCAnimation* poSBVCA = GetSBVCAnimation();
  int noVC = poSBVCA->virtualCharacters.GetLength();

  ZArray<MediaObjectProxy *> nodes;
  ZArray<MediaObjectProxy *> sensors;

  for (int i=0;i<noVC;i++){
	MediaObject * pObject = (SBSkinnedModel*) poSBVCA->virtualCharacters[i];
	MediaObjectProxy *pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());
	nodes.InsertAt(nodes.GetLength(), pProxy);
	if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
  }
	

  for (int j = 0; j < nodes.GetLength(); ++j){
		nodes[j]->CullOrSelect(rTransformation, rCullFlag,pRenderer, dwTime,intersectingObject,      activatedSensors,       depth,	  localSensors);
  }
}

BOOL SBVCAnimationProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  // it is called by SBVCAnimation :: Render
  // it should retrieve from the decoder the data and update the bone and muscle fields


   SBVCAnimation *pSBVCA = GetSBVCAnimation();

  // first time get the decoder specific info from the stream
  if (!m_bInit)
  {
    m_bInit = true;
  }

  // get the decoded data buffer
  int nSize;
  DWORD dvTime;
  BOOL bEOU;

  BYTE * pOutput = (BYTE *)poStream->FetchNext(nSize, dvTime, bEOU, false);

	if (pOutput != NULL )//&& rdvTime > m_lastTime)
	{
		if (pOutput == END_OF_STREAM)
			return true;
		
		m_lastTime = dvTime;
		// retreive tha data from the stream
		BBADataExchange dataExchange(pOutput, true);
		
		BYTE tag;
		DWORD len;
		void *pData = NULL;
		InitBonesMuscles();
		while (pData = dataExchange.FetchNext(tag, len)){
			switch (tag){
			case BBADataExchange::TAG_KEY_FRAME:   GetKeyFrame(pData, len);  break;
			case BBADataExchange::TAG_NO_BONES:	  GetNoBones(pData, len);  break;
			case BBADataExchange::TAG_NO_MUSCLES:  GetNoMuscles(pData, len);  break;
			case BBADataExchange::TAG_BONES:       GetBones(pData, len);  break;
			case BBADataExchange::TAG_MUSCLES:     GetMuscles(pData, len);    break;
			default: break;//ASSERT(0); return FALSE;
			}
		}

		SBVCAnimation* poSBVCA = GetSBVCAnimation();
		int noVC = poSBVCA->virtualCharacters.GetLength();
		for (int i=0;i<noVC;i++){
			SBSkinnedModel* pSBSM = (SBSkinnedModel*) poSBVCA->virtualCharacters[i];
			SBSkinnedModelProxy *pSBSMProxy = (SBSkinnedModelProxy *) (pSBSM->GetRenderingObject());
			pSBSMProxy->UpdateVertices();
		}
		return FALSE; // TRUE
	}
	return FALSE;
}

bool SBVCAnimationProxy:: GetKeyFrame (void *pData, int size)
{
  ASSERT(size % (sizeof(int)) == 0);
  int *pointArray = (int *)pData;
  keyFrame=pointArray[0];
  return true;
}

bool SBVCAnimationProxy:: GetNoBones (void *pData, int size)
{
  ASSERT(size % (sizeof(int)) == 0);
  int *pointArray = (int *)pData;
  noBones=pointArray[0];

  return true;
}

bool SBVCAnimationProxy:: GetNoMuscles (void *pData, int size)
{
  ASSERT(size % (sizeof(int)) == 0);
  int *pointArray = (int *)pData;
  noMuscles=pointArray[0];

  return true;
}

bool SBVCAnimationProxy:: GetBones (void *pData, int size)
{
	// this function gets one bone
	// and update its tranformation in the original graph
	// update of the vertices will be called when all the bones and muscles are updated


	currentBone = (CBone*)pData;
	UpdateBoneTransform(currentBone);
	
//	firstBone = boneArray[0];
	return true;
}

bool SBVCAnimationProxy:: GetMuscles (void *pData, int size)
{
	// this function gets one muscle
	// and update its tranformation in the original graph
	// update of the vertices will be called when all the bones and muscles are updated

	currentMuscle = (CMuscle*)pData;
	UpdateMuscleTransform(currentMuscle);

	return true;
}

void SBVCAnimationProxy:: InitBonesMuscles()
{
	
	currentBone = firstBone;
	currentMuscle = firstMuscle;
}

///////////////////////////////////

bool SBVCAnimationProxy:: UpdateBoneTransform (CBone *cb)
{

	int id=cb->boneIdentifier;
	bool found=false;
	bool isPredictive=false;
	bool isDCT=false;
	SBBone *pSBBToUpdate;

	SBVCAnimation* poSBVCA = GetSBVCAnimation();
	int noVC = poSBVCA->virtualCharacters.GetLength();
	for (int i=0;i<noVC;i++){
		SBSkinnedModel* pSBSM = (SBSkinnedModel*) poSBVCA->virtualCharacters[i];
		int noBones = pSBSM->bones.GetLength();
		for (int j=0;j<noBones;j++){
			SBBone *pSBB=(SBBone *) pSBSM->bones[j];
			int bID=pSBB->boneID;
			if ( bID == id){
				pSBBToUpdate = pSBB;
				j=noBones;i=noVC;
				found=true;
			}
		}
	}


	if (found){
		if (cb->boneDCTData) isDCT=true;
		else isPredictive=true;
		float no=1e-5;
		if (isPredictive){
			// update translation, rotation, center ....
			int indexData=0;
			if (cb->boneMask.isTonXChanged || cb->boneMask.isTonYChanged || cb->boneMask.isTonZChanged){
				Vec3f tran=pSBBToUpdate->translation;
				if (cb->boneMask.isTonXChanged) tran.x= - cb->boneData.TonX*no;
				if (cb->boneMask.isTonYChanged) tran.y= cb->boneData.TonY*no;
				if (cb->boneMask.isTonZChanged) tran.z= cb->boneData.TonZ*no;
				pSBBToUpdate->translation = tran;
			}
			float flexion=0;
			float abduction=0;
			float twisting=0;
			EulerAngles eang;
			Quat q;
			if (cb->boneMask.isRonAx1Changed || cb->boneMask.isRonAx2Changed|| cb->boneMask.isRonAx3Changed){
				// update rot.x, rot.y, rot.z, rot.theta
				// MP check the coordinate system in IM1
				if (cb->boneMask.isRonAx1Changed) flexion  = cb->boneData.RonAx1*no; 
				if (cb->boneMask.isRonAx2Changed) twisting = cb->boneData.RonAx2*no;
				if (cb->boneMask.isRonAx3Changed)  abduction = cb->boneData.RonAx3*no;
				int rotOrd=pSBBToUpdate->rotationOrder;
				eang=Eul_(flexion,abduction,twisting, rotOrd);
				q=Eul_ToQuat(eang);
				Rotation rot;
				rot.x=q.x;
				rot.y=q.y;
				rot.z=q.z;
				rot.theta=q.w;
				rot.unmakeQuat();
				pSBBToUpdate->rotation=rot;
			}
			// continue with the rest of transformation of the bones: scale, scaleOrientation and center
		}
		else if (isDCT){
		// update from DCT
			// in this version only the first frame is readed
			// a temoparl buffer should be used
		// update translation, rotation, center ....
			int indexData=0;
			if (cb->boneMask.isTonXChanged || cb->boneMask.isTonYChanged || cb->boneMask.isTonZChanged){
				Vec3f tran=pSBBToUpdate->translation;
				if (cb->boneMask.isTonXChanged) tran.x= - cb->boneDCTData[0].TonX*no;
				if (cb->boneMask.isTonYChanged) tran.y= cb->boneDCTData[0].TonY*no;
				if (cb->boneMask.isTonZChanged) tran.z= cb->boneDCTData[0].TonZ*no;
				pSBBToUpdate->translation = tran;
			}
			float flexion=0;
			float abduction=0;
			float twisting=0;
			EulerAngles eang;
			Quat q;
			if (cb->boneMask.isRonAx1Changed || cb->boneMask.isRonAx2Changed|| cb->boneMask.isRonAx3Changed){
				// update rot.x, rot.y, rot.z, rot.theta
				// MP check the coordinate system in IM1
				if (cb->boneMask.isRonAx1Changed) flexion  = cb->boneDCTData[0].RonAx1*no; 
				if (cb->boneMask.isRonAx2Changed) twisting = cb->boneDCTData[0].RonAx2*no;
				if (cb->boneMask.isRonAx3Changed)  abduction = cb->boneDCTData[0].RonAx3*no;
				int rotOrd=pSBBToUpdate->rotationOrder;
				eang=Eul_(flexion,abduction,twisting, rotOrd);
				q=Eul_ToQuat(eang);
				Rotation rot;
				rot.x=q.x;
				rot.y=q.y;
				rot.z=q.z;
				rot.theta=q.w;
				rot.unmakeQuat();
				pSBBToUpdate->rotation=rot;
			}
			// continue with the rest of transformation of the bones: scale, scaleOrientation and center
		}
	}
	return true;
}

bool SBVCAnimationProxy:: UpdateMuscleTransform (CMuscle *cm)
{
	return true;
}
