/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Patrick.gioia@rd.francetelecom.com.
----------------------------------------------------------------------*/

#include "OpenGl/OpenGl.h"
#include <math.h>
#include "WaveSurfProxy.h"
#include "opengl/appearance/Material2DProxy.h"
#include "opengl/AFX/BitWrapperProxy.h"
#include "IM1Decoders/AFX/AFX.h"
#include "IM1Decoders/AFX/WaveSurf/WaveSurfDec.h"


/***************************************************************************/
// Constructor - calls the inherited constructor from Node3DProxy
WaveletSubdivisionSurfaceProxy::WaveletSubdivisionSurfaceProxy(WaveletSubdivisionSurface* WS)
  : AfxStreamProxy(WS)
{
  failedDecode = false;
  m_bInit = false;
  m_lastTime = 0;
  m_pInfo = NULL;
  m_noTriangles = 0;
  Faces = NULL;
  Vertices = NULL;
}

WaveletSubdivisionSurfaceProxy::~WaveletSubdivisionSurfaceProxy()
{
	
}

// traverse scene graph, checking for intersection

/* virtual */ BOOL
WaveletSubdivisionSurfaceProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}

// render object



BOOL WaveletSubdivisionSurfaceProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  // it is called by BitWrapper
  // it should retrieve from the decoder the data and initialise the fields

  WaveletSubdivisionSurface *pObject = GetWaveSurf();
  BitWrapper *pParent = (BitWrapper *)pObject->GetParent();
  ASSERT(pParent != NULL);

  // first time get the decoder specific info from the stream
  if (!m_bInit)
  {
	m_bInit = true;
	SetDecoderSpecificInfo(pParent);
  }

  // get the stream
  MediaStream * pStream = (MediaStream *)pParent->GetStream();
  ASSERT(pStream != NULL);

  // get the decoded data buffer
  int rnSize, rnStatus;
  DWORD rdvTime;
  BOOL rbEOU;
	BYTE * pOutput = (BYTE *)pStream->Fetch(rnSize, rdvTime, rbEOU, rnStatus);


	//Copy the decoded mesh into the composition buffer
	//DataExchange();
	

	
	// update time
	if (rdvTime > m_lastTime) {
		m_lastTime = rdvTime;
		return TRUE;
	}

	  return FALSE;
}



BOOL WaveletSubdivisionSurfaceProxy::RenderFromBuffer(SFString *buffer, int id)
{
  // it is called by BitWrapper
  // it should decode the stream from the buffer and initialize the fields
	WaveSurfDecoder* WSDec = new WaveSurfDecoder();
	WaveletSubdivisionSurface *pObject = GetWaveSurf();
	BitWrapper *pParent = (BitWrapper *)pObject->GetParent();
	//BitWrapperProxy *pParentProxy = (BitWrapperProxy *)pParent->GetRenderingObject();
	ASSERT(pParent != NULL);
	unsigned char* output = NULL;;
	int outputlength = 0;

	//Init
	if (!m_bInit) {
		m_bInit = true;
	    //SetDecoderSpecificInfo(pParent);	
		WSDec->Setup();
		SetupDecoderData(pParent, WSDec);
		m_Decoder=WSDec;
	}

	//Decode
	WSDec->Decode((LPBYTE) buffer->GetValue(), buffer->GetLength(), output, outputlength);

	//Get the decoded mesh
	//DataExchange();
	
  return TRUE;
}





void WaveletSubdivisionSurfaceProxy :: SetDecoderSpecificInfo(BitWrapper *pParent)
{
	AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
	WaveSurfDecoder  *wsDec = (WaveSurfDecoder *)afxDec->GetDecoder();
	WMDecoderSpecificInfo *m_pInfo = wsDec->m_pDecSpecificInfo;

	// Data will be moved from the WMDecoderConfig to the ZT decoder juts before decoding

}



BOOL WaveletSubdivisionSurfaceProxy::Render(DWORD dwTime)
{
	WaveletSubdivisionSurface *pObject = GetWaveSurf();
	ASSERT(m_bInit);
	BitWrapper *pParent = (BitWrapper *)pObject->GetParent();
	ASSERT(pParent != NULL);
	AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
	WaveSurfDecoder  *wsDec;

	//Inband/outband ? Where is the decoder ?
	if (pParent->GetURL().GetLength() > 0) 
		wsDec= (WaveSurfDecoder *)afxDec->GetDecoder();
	else 
		wsDec=(WaveSurfDecoder*)m_Decoder;
	Mesh* mesh = &wsDec->m_RecMesh;

	glPushAttrib(GL_POLYGON_BIT);


	SF<Vec3f> sizeB, centerB;
  MediaObjectProxy::GetBbox(sizeB, centerB);
  Vec3f size = (Vec3f)sizeB, center = (Vec3f)centerB;

  glTranslatef(center.x, center.y, center.z);
  glScalef(3000, 3000, 3000);
  glTranslatef(-center.x, -center.y-0.1, -center.z);

	for (int i = 0, k = 0; i < mesh->NbFace; i++)
  {
		glBegin(GL_TRIANGLES);
		
			glVertex3f(mesh->Vertex[mesh->Face[i][0]][0], mesh->Vertex[mesh->Face[i][0]][1], mesh->Vertex[mesh->Face[i][0]][2]);
			glVertex3f(mesh->Vertex[mesh->Face[i][1]][0], mesh->Vertex[mesh->Face[i][1]][1], mesh->Vertex[mesh->Face[i][1]][2]);
			glVertex3f(mesh->Vertex[mesh->Face[i][2]][0], mesh->Vertex[mesh->Face[i][2]][1], mesh->Vertex[mesh->Face[i][2]][2]);

		glEnd();
	}

	glPopAttrib();


  return TRUE;
}

BOOL WaveletSubdivisionSurfaceProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // called when the node is encoded with BIFS
  if (failedDecode) return FALSE;

  WaveletSubdivisionSurface *WS = GetWaveSurf();
  ASSERT(WS != NULL);

  if (!IsDecoded())
  {
    if (!Decode(WS))
    {
      failedDecode = true;
      return  FALSE;
    }

    SetDecoded(true);
  }

  // display the result
  Render(dwTime);

  return TRUE;
}

void WaveletSubdivisionSurfaceProxy::DataExchange() {

	//Copy the decoded mesh into the composition buffer

	int i;
	WaveletSubdivisionSurface *pObject = GetWaveSurf();
	BitWrapper *pParent = (BitWrapper *)pObject->GetParent();
	ASSERT(pParent != NULL);
	AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
	WaveSurfDecoder  *wsDec = (WaveSurfDecoder *)afxDec->GetDecoder();

	while (!((WaveSurfDecoder*)(afxDec->GetDecoder()))->m_isDecoded){
	}


	
	NbFaces = wsDec->m_RecMesh.NbFace;
	NbVertices = wsDec->m_RecMesh.NbVertex;
	if (Faces != NULL)
		free(Faces);
	if (Vertices != NULL)
		free(Vertices);
	Faces = (int*)malloc(sizeof(int)*NbFaces*3);
	Vertices = (float*)malloc(sizeof(float)*NbVertices*3);

	for (i=0; i<NbFaces*3; i++) {
		Faces[i] = wsDec->m_RecMesh.Face[i/3][i%3];
	}
	for (i=0; i<NbVertices*3; i++) {
		Vertices[i] = wsDec->m_RecMesh.Vertex[i/3][i%3];
	}

}

void WaveletSubdivisionSurfaceProxy::SetupDecoderData(BitWrapperProxy *pParent)
{
	int i;
	WaveletSubdivisionSurface *pObject = GetWaveSurf();
	IndexedFaceSet* IFS = (IndexedFaceSet*)pObject->baseMesh.GetNode();

  BitWrapper* poBitWrapper = pParent->GetBitWrapper();
  AFXDecoder *pDecoder = (AFXDecoder *)poBitWrapper->GetDecoder();
  ASSERT(pDecoder != NULL);
  WaveSurfDecoder *wsDec = (WaveSurfDecoder *)pDecoder->GetDecoder();
  ASSERT(wsDec != NULL);
  
	
	Coordinate* CRD = (Coordinate*)IFS->coord.GetNode();
	ZArray<Vec3f>* AR = (ZArray<Vec3f>*)(&(CRD->point.GetArray()));
	Vec3f TRPL;
	MF<int> CRDIND = (MF<int>)IFS->coordIndex;
	ZArray<int>* CAR = (ZArray<int>*)(&(CRDIND.GetArray()));
	
	if (wsDec->m_BM==NULL)
		wsDec->m_BM = (Mesh*)malloc(sizeof(Mesh));

	for (i=0; i<AR->GetLength(); i++) {
		TRPL = (*AR)[i];
		wsDec->m_BM->Vertex[i][0] = TRPL.x;
		wsDec->m_BM->Vertex[i][1] = TRPL.y;
		wsDec->m_BM->Vertex[i][2] = TRPL.z;
	}
	wsDec->m_BM->NbVertex = AR->GetLength();

	for (i=0; i<CAR->GetLength(); i++) {
		if ((i+1)%4)
			wsDec->m_BM->Face[i/4][i%4] = CRDIND[i];
	}
	wsDec->m_BM->NbFace = (CAR->GetLength())/4;
}

void WaveletSubdivisionSurfaceProxy::SetupDecoderData(BitWrapper *poBitWrapper, void *wsDecoder)
{
	int i;
	WaveletSubdivisionSurface *pObject = GetWaveSurf();
	IndexedFaceSet* IFS = (IndexedFaceSet*)pObject->baseMesh.GetNode();
  
	WaveSurfDecoder* wsDec = (WaveSurfDecoder*)wsDecoder;
	Coordinate* CRD = (Coordinate*)IFS->coord.GetNode();
	ZArray<Vec3f>* AR = (ZArray<Vec3f>*)(&(CRD->point.GetArray()));
	Vec3f TRPL;
	MF<int> CRDIND = (MF<int>)IFS->coordIndex;
	ZArray<int>* CAR = (ZArray<int>*)(&(CRDIND.GetArray()));
	
	if (wsDec->m_BM==NULL)
		wsDec->m_BM = (Mesh*)malloc(sizeof(Mesh));

	for (i=0; i<AR->GetLength(); i++) {
		TRPL = (*AR)[i];
		wsDec->m_BM->Vertex[i][0] = TRPL.x;
		wsDec->m_BM->Vertex[i][1] = TRPL.y;
		wsDec->m_BM->Vertex[i][2] = TRPL.z;
	}
	wsDec->m_BM->NbVertex = AR->GetLength();

	for (i=0; i<CAR->GetLength(); i++) {
		if ((i+1)%4)
			wsDec->m_BM->Face[i/4][i%4] = CRDIND[i];
	}
	wsDec->m_BM->NbFace = (CAR->GetLength())/4;
}


void WaveletSubdivisionSurfaceProxy::SetBaseMesh() {

	//Copy the decoded mesh into the composition buffer


}

bool WaveletSubdivisionSurfaceProxy::Decode(WaveletSubdivisionSurface *pWS)
{
  

	return TRUE;
}

