/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

#ifndef __WaveletSubdivisionSurfaceProxy_H__
#define __WaveletSubdivisionSurfaceProxy_H__

#include "AfxStreamProxy.h"

#include "Nodes.h"

#include "..\..\IM1Decoders\AFX\WaveSurf\Zerotree\data.h"

class BitWrapperProxy;
class WMDecoderSpecificInfo;

/// WaveSurf renderer node

class WaveletSubdivisionSurfaceProxy : public AfxStreamProxy
{
public:
  /// Constructor
	WaveletSubdivisionSurfaceProxy(WaveletSubdivisionSurface* WS);

  /// Destructor
	~WaveletSubdivisionSurfaceProxy();

  /// Decode object
  bool Decode(WaveletSubdivisionSurface *WS);

  /// Render object
  virtual BOOL  Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  virtual BOOL  RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);
  virtual BOOL  RenderFromBuffer(SFString *buffer, int id);
  virtual BOOL  Render(DWORD dwTime);
  virtual void  SetupDecoderData(BitWrapperProxy *pParent);
  void SetupDecoderData(BitWrapper *poBitWrapper, void *wsDec);

  /// Traverse scene graph, checking for intersection
  virtual BOOL  Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  // moved methods from WaveSurf to WaveletSubdivisionSurfaceProxy
  WaveletSubdivisionSurface * GetWaveSurf() const
  { return (WaveletSubdivisionSurface *) GetMediaObject(); }

  /// Get bounding box
  inline /* virtual */ void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const
  {  }

  void  SetDecoded(bool flag) { bDecoded = flag; };
  bool  IsDecoded() { return bDecoded; };
  void SetBaseMesh();

private:

  void DataExchange();

  void SetDecoderSpecificInfo(BitWrapper *pParent);

protected:
  bool  m_bInit;
  DWORD m_lastTime;
  WMDecoderSpecificInfo *m_pInfo;

  bool bDecoded;
  bool failedDecode;

  int m_noTriangles;

  //Rendering buffer
  int* Faces;
  int NbFaces;
  float* Vertices;
  int NbVertices;

  //Decoder
  void* m_Decoder;
};

#endif // __WaveletSubdivisionSurfaceProxy_H__
