/*
	This software module was originally developed by SGDL Systems Inc. in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license
to this software module or modifications thereof for use in hardware or software
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1).
Those intending to use this software module in hardware or software products are
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability for use of
this software module or modifications thereof in an implementation. Copyright is
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
	SGDL Systems Inc. retains full right to use the code for his/her own purpose, assign or
donate the code to a third party and to inhibit third parties from using the code
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
						Copyright  2002
*/

//*******************************************************************************
//
//  This file contains all the methods for the AdaptiveRefiner class and MyHashTable
//  class. These objects are used to refine the polygons that represent and imlicit
//  surface.
//
//  Author : Simon Lavalle, SGDL Systme Inc.
//  Date   : March 28th, 2002
//
//*******************************************************************************

#include "VolumeTreeNode/SGDLTools.h"
#include "SGDLPolygonise.h"
#include "AdaptiveRefinement.h"


//***************************************************
// Class MyHashTable
//***************************************************

//***************************************************
//	
//  MyHashTable::MyHashTable()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : 
//
//  Output : 
//
//  Desc   : Constructor
//***************************************************
MyHashTable::MyHashTable()
{
    //Initialization
    memset(m_NVTable, 0, TABLE_SIZE*TABLE_SIZE*sizeof(NewVertexList*));
}

//***************************************************
//	
//  MyHashTable::~MyHashTable()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : 
//
//  Output : 
//
//  Desc   : Destructor
//***************************************************
MyHashTable::~MyHashTable()
{
    for (int i = 0; i < TABLE_SIZE; i++)
    {
        for (int j = 0; j < TABLE_SIZE; j++)
        {
            if (m_NVTable[i][j] != 0)
            {
                //First delete all NewVertexInd allocated in the list
                for (int k = 0; k < m_NVTable[i][j]->size; k++)
                    delete ((m_NVTable[i][j])->NVPtr)[k];
                
                //Then delete the pointer list
                delete [] (m_NVTable[i][j])->NVPtr;
                //Finally delete the list struct
                delete m_NVTable[i][j];

                //Set the two deleted pointers to NULL
                m_NVTable[i][j] = 0;
                m_NVTable[j][i] = 0;
            }
        }
    }    
}


//***************************************************
//	
//  MyHashTable::GetMidPt()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : int Pt1 -> index of first  point
//           int Pt2 -> index of second point
//
//  Output : index of the midpoint of Pt1 and Pt2 or
//           -1 if the midpoint hasn't been computed yet.
//
//  Desc   :
//***************************************************

int MyHashTable::GetMidPt(int Pt1, int Pt2)
{
    NewVertexList* NVList = 0;

    if ((NVList = m_NVTable[Pt1%TABLE_SIZE][Pt2%TABLE_SIZE]) != 0)
    {
        //If a list has been allocated at this place, check all structs
        //to get the good one
        for (int i = 0; i < NVList->size; i++)
        {
            if (((Pt1 == ((NVList->NVPtr)[i])->Pt1) && (Pt2 == ((NVList->NVPtr)[i])->Pt2)) ||
                ((Pt2 == ((NVList->NVPtr)[i])->Pt1) && (Pt1 == ((NVList->NVPtr)[i])->Pt2)))
                //We found a struct that contains the midpoint we search, return the midpoint
                return ((NVList->NVPtr)[i])->MidPt;
        }
    }
    //We haven't been able to find the midpoint of Pt1 and Pt2
    //(this mean it hasn't been computed yet)
    return -1;
}

//***************************************************
//	
//  MyHashTable::InsertMidPt()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : int Pt1   -> index of the first point
//           int Pt2   -> index of the second point
//           int MidPt -> Vec3f to be inserted
//
//  Output : 
//
//  Desc   : This function insert the index of the midpoint
//           of Pt1 and Pt2 in the hashtable
//***************************************************
void MyHashTable::InsertMidPt(int Pt1, int Pt2, int MidPt)
{
    //get the pointer to the list at the correct position
    NewVertexList *NVList  = m_NVTable[Pt1%TABLE_SIZE][Pt2%TABLE_SIZE];
    NewVertexInd  **OldPtr = NULL;    

    if (NVList == NULL)
    {
        //The list is empty, so allocate a new list
        //Since the points can be passed in any order, the two
        //possible pointer list must reference the same list.
        NVList = 
            m_NVTable[Pt2%TABLE_SIZE][Pt1%TABLE_SIZE] = 
            m_NVTable[Pt1%TABLE_SIZE][Pt2%TABLE_SIZE] = new NewVertexList;
        assert(NVList != 0);
        assert(NVList != 0);
        assert(NVList != 0);

        //Allocate a pointer to the struct
        NVList->NVPtr = new NewVertexInd*;
        assert(NVList->NVPtr != 0);

        //Allocate the struct
        (NVList->NVPtr)[0]          = new NewVertexInd;
        assert((NVList->NVPtr)[0] != 0);

        //Set the new size
        NVList->size                = 1;
        //Fill the struct with the values
        ((NVList->NVPtr)[0])->Pt1   = Pt1;
        ((NVList->NVPtr)[0])->Pt2   = Pt2;
        ((NVList->NVPtr)[0])->MidPt = MidPt;
    }

    else
    {
        //Keep a ref on the old pointer
        OldPtr = NVList->NVPtr;
        //Allocate a new pointer list and increase its size by one
        NVList->NVPtr = new NewVertexInd*[++(NVList->size)];

        //Copy the pointers from the old list to the new list
        for(int i = 0; i < (NVList->size - 1); i++)
            (NVList->NVPtr)[i] = OldPtr[i];

        //Allocate a new struct and fill it with the new value
        (NVList->NVPtr)[NVList->size - 1]        = new NewVertexInd;
        (NVList->NVPtr)[NVList->size - 1]->Pt1   = Pt1;
        (NVList->NVPtr)[NVList->size - 1]->Pt2   = Pt2;
        (NVList->NVPtr)[NVList->size - 1]->MidPt = MidPt;

        //delete the old ptr list
        delete [] OldPtr;
    }
}


//***************************************************
// Class AdaptiveRefiner
//***************************************************

//***************************************************
//	
//  AdaptiveRefiner::Refine()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : Shapejb *Shape -> Pointer to a shape produced
//                             by the JB polygonizer
//  Output : 
//
//  Desc   : This function perform a first subdivision
//           of triangles depending on its normals
//           and call the recursive subdivision for each
//           of the new triangles produced.
//***************************************************
void AdaptiveRefiner::Refine(Shapejb *Shape, PointArray *ColorTable)
{
    //Flag to know which points are flat
    bool        Pt1Pt2IsFlat  = false;
    bool        Pt1Pt3IsFlat  = false;
    bool        Pt2Pt3IsFlat  = false;

    //Indexs of the three points of the triangle
    int         Pt1Ind        = -1;
    int         Pt2Ind        = -1;
    int         Pt3Ind        = -1;

    //Indexs of the midpoints of each edge
    int         Pt1Pt2Ind     = -1;
    int         Pt1Pt3Ind     = -1;
    int         Pt2Pt3Ind     = -1;

    //Color of each point
    Vec3f       *Pt1Color     = 0;
    Vec3f       *Pt2Color     = 0;
    Vec3f       *Pt3Color     = 0;    

    //The tree vertex
    VERTEX      *Pt1;
    VERTEX      *Pt2;
    VERTEX      *Pt3;

    //Index of the next vertex to check
    int         IndOfNextVert = 0;
    //The subdivision level we are
    int         SubdvLevel    = 1;
    
    //Cpt
    int         i             = 0;

    //Number of vertex before the subdivision
    int         NbOfIndex     = Shape->nbtris * nbvertbyface;
    
    //Ptr to the vertex list
    VERTICES    *VertPt       = Shape->Svertices();    
    
    //Table that keep computed vertex, to avoid computing twice the same vertex
    m_ComputedVertex          = new MyHashTable;
    
    //Copy the initial vertex array to the CArray
    m_VertexList.SetSize(VertPt->count);
    memcpy(m_VertexList.GetData(), VertPt->ptr, VertPt->count*sizeof(VERTEX));

    //nGrowBy = 1000
    m_IndexList.SetSize(0, 1000);

    //For each triangles...
    for (i = 0; i < NbOfIndex; i+=3)
    {
        //Get indexs of points in the vertex list
        Pt1Ind = Shape->Trivert(i);
        Pt2Ind = Shape->Trivert(i+1);
        Pt3Ind = Shape->Trivert(i+2);

        Pt1    = &((VertPt->ptr)[Pt1Ind]); 
        Pt2    = &((VertPt->ptr)[Pt2Ind]);
        Pt3    = &((VertPt->ptr)[Pt3Ind]);

        //Get index of next triangle in the index list
        IndOfNextVert = m_IndexList.GetSize();

        m_IndexList.Add(Pt1Ind);
        m_IndexList.Add(Pt2Ind);
        m_IndexList.Add(Pt3Ind);

        //Check for curvature between two points of the triangle
        Pt1Pt2IsFlat = IsFlat(Pt1, Pt2, Pt1Color, Pt2Color);
        Pt1Pt3IsFlat = IsFlat(Pt1, Pt3, Pt1Color, Pt3Color);
        Pt2Pt3IsFlat = IsFlat(Pt2, Pt3, Pt2Color, Pt3Color);
        
        if ((Pt1Pt2IsFlat == NOT_FLAT) || (Pt1Pt3IsFlat == NOT_FLAT) || (Pt2Pt3IsFlat == NOT_FLAT))
        {
            //The triangle isn't flat, subdivide it.
            Pt1Pt2Ind = MidPoint(Pt1Pt2IsFlat, Pt1Ind, Pt2Ind);
            Pt1Pt3Ind = MidPoint(Pt1Pt3IsFlat, Pt1Ind, Pt3Ind);
            Pt2Pt3Ind = MidPoint(Pt2Pt3IsFlat, Pt2Ind, Pt3Ind);

            //Insert the midpoint in the list
            m_IndexList.InsertAt(IndOfNextVert + 1, Pt1Pt2Ind);
            m_IndexList.InsertAt(IndOfNextVert + 3, Pt1Pt2Ind);
            m_IndexList.InsertAt(IndOfNextVert + 5, Pt1Pt2Ind);

            m_IndexList.InsertAt(IndOfNextVert + 2, Pt1Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 6, Pt1Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 8, Pt1Pt3Ind);

            m_IndexList.InsertAt(IndOfNextVert + 4,  Pt2Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 8,  Pt2Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 10, Pt2Pt3Ind);
            
            //Subdivide the four new triangles recursively
            SubdivideNewTriangles(IndOfNextVert, SubdvLevel + 1);                
        }
    }

    //Free memory
    delete m_ComputedVertex;

    //Delete the old vertex ptr and copy the new vertex list
    delete [] VertPt->ptr;
    VertPt->ptr = new VERTEX [m_VertexList.GetSize()];
    memcpy(VertPt->ptr, m_VertexList.GetData(), m_VertexList.GetSize()*sizeof(VERTEX));
    VertPt->count = m_VertexList.GetSize();

    // We don't reallocate Shape->trivert cause it is allocated automatically.
    assert(m_IndexList.GetSize() < MAX_NUMB_TRI);
    memcpy(Shape->trivert, m_IndexList.GetData(), m_IndexList.GetSize()*sizeof(int));
    Shape->nbtris  = m_IndexList.GetSize()/nbvertbyface;

    //Clear arrays
    m_VertexList.RemoveAll();
    m_VertexList.SetSize(0);

    m_IndexList.RemoveAll();
    m_IndexList.SetSize(0);
}

//***************************************************
//	
//  AdaptiveRefiner::SubdivideNewTriangles()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : IndOfNextVert -> Index of next triangle to subdivide
//           SubdvLevel    -> Which subdivision level we are
//
//  Output : 
//
//  Desc   : This function subdivide recursively the triangles
//           according to the surface curvature of the implicit
//           function (estimated by the surface normal at each 
//           point of the triangle, Vehlo 1996) and stop when 
//           the triangle is considered to be flat or the maximum 
//           subdivision level is reached.
//***************************************************
void AdaptiveRefiner::SubdivideNewTriangles(int IndOfNextVert, int SubdvLevel)
{

    //If we reached the maximum subdivision level, stop here.
    if (SubdvLevel > MAX_SUBDIVISION_LEVEL)
        return;    

    //These variable are used for the samething than AdaptiveRefiner::Refine()...
    VERTEX *Pt1;
    VERTEX *Pt2;
    VERTEX *Pt3;

    int    Pt1Ind    = -1;
    int    Pt2Ind    = -1;
    int    Pt3Ind    = -1;

    int    Pt1Pt2Ind = -1;
    int    Pt1Pt3Ind = -1;
    int    Pt2Pt3Ind = -1;

    Vec3f  *Pt1Color = 0;
    Vec3f  *Pt2Color = 0;
    Vec3f  *Pt3Color = 0;    

    bool   Pt1Pt2IsFlat = false;
    bool   Pt1Pt3IsFlat = false;
    bool   Pt2Pt3IsFlat = false;

    //Get the size of the array befor the subdivision
    int    InitialSize  = m_IndexList.GetSize();
    int    i            = 0;

    //For each new triangle...
    for(i = 0; i < 4; i++)
    {
        //Subdivide the same way as ::Refine()...
        Pt1Ind = m_IndexList[IndOfNextVert];
        Pt2Ind = m_IndexList[IndOfNextVert+1];
        Pt3Ind = m_IndexList[IndOfNextVert+2];

        Pt1 = &(m_VertexList[Pt1Ind]);
        Pt2 = &(m_VertexList[Pt2Ind]);
        Pt3 = &(m_VertexList[Pt3Ind]);
        
        Pt1Pt2IsFlat = IsFlat(Pt1, Pt2, Pt1Color, Pt2Color);
        Pt1Pt3IsFlat = IsFlat(Pt1, Pt3, Pt1Color, Pt3Color);
        Pt2Pt3IsFlat = IsFlat(Pt2, Pt3, Pt2Color, Pt3Color);

        if ((Pt1Pt2IsFlat == NOT_FLAT) || (Pt1Pt3IsFlat == NOT_FLAT) || (Pt2Pt3IsFlat == NOT_FLAT))
        {
            //The triangle isn't flat, subdivide it.
            Pt1Pt2Ind = MidPoint(Pt1Pt2IsFlat, Pt1Ind, Pt2Ind);
            Pt1Pt3Ind = MidPoint(Pt1Pt3IsFlat, Pt1Ind, Pt3Ind);
            Pt2Pt3Ind = MidPoint(Pt2Pt3IsFlat, Pt2Ind, Pt3Ind);

            //Insert the midpoint in the list
            m_IndexList.InsertAt(IndOfNextVert + 1, Pt1Pt2Ind);
            m_IndexList.InsertAt(IndOfNextVert + 3, Pt1Pt2Ind);
            m_IndexList.InsertAt(IndOfNextVert + 5, Pt1Pt2Ind);

            m_IndexList.InsertAt(IndOfNextVert + 2, Pt1Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 6, Pt1Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 8, Pt1Pt3Ind);

            m_IndexList.InsertAt(IndOfNextVert + 4,  Pt2Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 8,  Pt2Pt3Ind);
            m_IndexList.InsertAt(IndOfNextVert + 10, Pt2Pt3Ind);            

            SubdivideNewTriangles(IndOfNextVert, SubdvLevel + 1);           
        }

        //Next triangle index is equal to current index + 3 + Number of new triangles
        //generated recursively*3.
        IndOfNextVert += (m_IndexList.GetSize() - InitialSize) + 3;
        InitialSize    = m_IndexList.GetSize();
    }    
}

//***************************************************
//	
//  AdaptiveRefiner::MidPoint()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : bool IsFlat      -> Flag to tell if we have to 
//                               project the midpoint to the surface.
//           const int Pt1Ind -> First point.
//           const int Pt2Ind -> Second point.
//
//  Output : VERTEX MidPt -> The point between Pt1 and Pt2
//
//  Desc   : This function calculate the point in the 
//           middle of two points and project it to the
//           surface if needed.
//***************************************************
int AdaptiveRefiner::MidPoint(bool AreFlat, const int Pt1Ind, const int Pt2Ind)
{
    //Try to get the midpoint in the computed midpoint list
    int MidPtInd = m_ComputedVertex->GetMidPt(Pt1Ind,Pt2Ind);

    if (MidPtInd != -1)
        //This midpoint has already been computed, get out of here!
        return MidPtInd;

    //Calculate the midpoint...

    //Retrieve the two points
    const VERTEX &Pt1 = m_VertexList[Pt1Ind];
    const VERTEX &Pt2 = m_VertexList[Pt2Ind];

    //MidPoint struct, normal and color
    VERTEX MidPtVert;
    double MidPtNormal[3];
    float  MidPtColor[3] = {-1.0, -1.0, -1.0};

    //Get the midpoint of Pt1 and Pt2
    double MidPt[3]         = {(Pt1.position.x + Pt2.position.x)*0.5,
                               (Pt1.position.y + Pt2.position.y)*0.5,
                               (Pt1.position.z + Pt2.position.z)*0.5};

    double NormalSumV[3];
    double NormalSumVLength = 0.0;

    //These two variables are used to patch the GetColor function if
    //the points is outside all volume...
    double MidPtCopy[3]     = {MidPt[0], MidPt[1], MidPt[2]};
    bool   FlatAndOutside   = false;    
    
    if (!AreFlat || (FlatAndOutside = (0 && !m_TheNode->isInside(MidPt[0], MidPt[1], MidPt[2]))))
    {        
        //Get the sum of the two normals, that gives an average vector
        //between of two normals.        
        NormalSumV[0]    = Pt1.normal.x + Pt2.normal.x;
        NormalSumV[1]    = Pt1.normal.y + Pt2.normal.y;
        NormalSumV[2]    = Pt1.normal.z + Pt2.normal.z;
        
        NormalSumVLength = sqrt(NormalSumV[0]*NormalSumV[0] +
                                NormalSumV[1]*NormalSumV[1] +
                                NormalSumV[2]*NormalSumV[2]);

        if (NormalSumVLength != 0.0)
        {
            //Normalize
            NormalSumV[0] /= NormalSumVLength;
            NormalSumV[1] /= NormalSumVLength;
            NormalSumV[2] /= NormalSumVLength;

            //Project the point to the surface following 
            //the sum of the two normals.
            ProjectToSurface(NormalSumV, MidPt);
        }
      
        else
        {
            assert(NormalSumVLength != 0.0);

            //This is to prevent from crash...
            float DummyColor[3] = {0,0,0};
            CalculateNormal(MidPt, NormalSumV, DummyColor);
            ProjectToSurface(NormalSumV, MidPt);
        }
    }    

    //Get the normal from the imlicit function and the color of the point,
    //if it isn't already found.
    this->CalculateNormal(MidPt, MidPtNormal, MidPtColor);

    //Copy the position of the new point to the VERTEX struct
    MidPtVert.position.x = MidPt[0];
    MidPtVert.position.y = MidPt[1];
    MidPtVert.position.z = MidPt[2];

    //Copy the normal to the vertex struct
    MidPtVert.normal.x = MidPtNormal[0];
    MidPtVert.normal.y = MidPtNormal[1];
    MidPtVert.normal.z = MidPtNormal[2];
    
    if (AreFlat && ((DotProd(Pt1.normal, MidPtVert.normal) < DOT_TOLERANCE) ||
                    (DotProd(Pt2.normal, MidPtVert.normal) < DOT_TOLERANCE)))
    {
        //Patch - Because we found normal by checking wich surface 
        //is the nearest of the point, it can be possible that an error
        //occurs since the distance between the point an the surface
        //is approximated. In the case that the two initial points are flat, 
        //it is obvious the the midpoint should be flat with these two points.
        //So if it is not, approximate the normal with the sum of the normals
        //of the two other points.
        NormalSumV[0] = Pt1.normal.x + Pt2.normal.x;
        NormalSumV[1] = Pt1.normal.y + Pt2.normal.y;
        NormalSumV[2] = Pt1.normal.z + Pt2.normal.z;

        //Calculate the length to normalize
        NormalSumVLength = sqrt(NormalSumV[0]*NormalSumV[0] +
                                NormalSumV[1]*NormalSumV[1] +
                                NormalSumV[2]*NormalSumV[2]);

        if (NormalSumVLength != 0)
        {
            MidPtVert.normal.x = NormalSumV[0] / NormalSumVLength;
            MidPtVert.normal.y = NormalSumV[1] / NormalSumVLength;
            MidPtVert.normal.z = NormalSumV[2] / NormalSumVLength;
        }
#ifdef FIXME_OB
        else
            assert(NormalSumVLength != 0);
#endif
    }    

    //Add the new computed vertex to the VertexList
    m_VertexList.Add(MidPtVert);
    //Get it's index and put it in the table of already computed midpoint
    MidPtInd = m_VertexList.GetSize()-1;
    m_ComputedVertex->InsertMidPt(Pt1Ind, Pt2Ind, MidPtInd);
    
    //Return the index of the new midpoint
    return MidPtInd;
}

//***************************************************
//	
//  AdaptiveRefiner::ProjectToSurface()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : double Normal[3] -> The vector to project the point
//           double MidPt[3]  -> The point to be projected
//
//  Output : double MidPt[3]  -> The projected point
//
//  Desc   : This function project a point to the surface
//           by finding an inside and an outside point and
//           converging these points to the surface along the
//           Normal vector given in input.
//***************************************************
void AdaptiveRefiner::ProjectToSurface(const double Normal[3], double MidPt[3]) const
{
    double InX       = 0.0;
    double InY       = 0.0;
    double InZ       = 0.0;

    double OutX      = 0.0;
    double OutY      = 0.0;
    double OutZ      = 0.0;

    double TmpMidX   = 0.0;
    double TmpMidY   = 0.0;
    double TmpMidZ   = 0.0;

    double DeltaX    = Normal[0]*STEP;
    double DeltaY    = Normal[1]*STEP;
    double DeltaZ    = Normal[2]*STEP;

    int    i         = 0;
    int    IterCount = 0;
    
    if (m_TheNode->isInside(MidPt[0], MidPt[1], MidPt[2]))
    {
        //If the node is inside the volume, try to get an outside point
        InX  = MidPt[0];
        InY  = MidPt[1];
        InZ  = MidPt[2];

        OutX = InX + DeltaX;
        OutY = InY + DeltaY;
        OutZ = InZ + DeltaZ;

        while (m_TheNode->isInside(OutX, OutY, OutZ))
        {
            OutX += DeltaX;
            OutY += DeltaY;
            OutZ += DeltaZ;

            if (++IterCount > 30)
            {
                //Normally, we're not supposed to be here but since
                //there can be some errors when calculating the normals,
                //it is possible that we were not able to project the MidPoint
                //to the surface so we keep it's initial position. 
                //(this should be modified...) (SL 20-03-02)
                return;
            }
        }
    }

    else
    {
        //If the node is outside the volume, try to get an inside point
        OutX  = MidPt[0];
        OutY  = MidPt[1];
        OutZ  = MidPt[2];

        InX = OutX - DeltaX;
        InY = OutY - DeltaY;
        InZ = OutZ - DeltaZ;

        while (!m_TheNode->isInside(InX, InY, InZ))
        {
            InX -= DeltaX;
            InY -= DeltaY;
            InZ -= DeltaZ;

            if (++IterCount > 30)
            {
                //Normally, we're not supposed to be here but since
                //there can be some errors when calculating the normals,
                //it is possible that we were not able to project the MidPoint
                //to the surface so we keep it's initial position. 
                //(this should be modified...) (SL 20-03-02)
                return;
            }
        }
    }
    
    //Now that we've got one inside and one outside point, converge to
    //the implicit surface.
    while (i < 100        && 
          !((InX == OutX) &&
            (InY == OutY) &&
            (InZ == OutZ)))

    {
        TmpMidX = (OutX + InX) * 0.5;
        TmpMidY = (OutY + InY) * 0.5;
        TmpMidZ = (OutZ + InZ) * 0.5;

        if (m_TheNode->isInside(TmpMidX, TmpMidY, TmpMidZ))
        {
            InX = TmpMidX;
            InY = TmpMidY;
            InZ = TmpMidZ;
        }

        else
        {
            OutX = TmpMidX;
            OutY = TmpMidY;
            OutZ = TmpMidZ;
        }
        i++;
    }

    MidPt[0] = InX;
    MidPt[1] = InY;
    MidPt[2] = InZ;
}


//***************************************************
//	
//  AdaptiveRefiner::CalculateNormal()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : double Pt[3]        -> Vec3f that we have 
//                                  to get his normal
//
//  Output : double NewNormal[3] -> The normal (gradient)
//                                  of the point
//           float PtColor       -> Color of the point
//
//  Desc   : This function calculate the normal of a given
//           point. The normal is given by the function gradient
//           at this point.
//***************************************************
void AdaptiveRefiner::CalculateNormal(const double Pt[3], double Normal[3], float PtColor[3])
{
    double Distance      = 0.0;
    bool   NormalIsValid = false;
    double NormalLength  = 0.0;
    
    NormalIsValid = m_TheNode->EvaluateGradient(Pt[0], Pt[1], Pt[2], Distance, Normal);
    
    assert(NormalIsValid);

    NormalLength  = sqrt(Normal[0]*Normal[0] + 
                         Normal[1]*Normal[1] + 
                         Normal[2]*Normal[2]);
    
    if ((NormalLength != 0.0) && (NormalLength != 1.0))
    {
        //Normalize
        Normal[0] /= NormalLength;
        Normal[1] /= NormalLength;
        Normal[2] /= NormalLength;
    }

    else 
        assert(NormalLength != 0.0);  //Normal cannot be 000
    
    //TBM **** This check should be done inside ****************
    //******** Volume_Tree_Node::EvaluateGradient() ************
    if (!m_TheNode->ShowAllDensities)
    {
        if (!m_TheNode->isInside(Pt[0] - Normal[0]*(fabs(Distance) + EPSILON*0.01),
                                 Pt[1] - Normal[1]*(fabs(Distance) + EPSILON*0.01),
                                 Pt[2] - Normal[2]*(fabs(Distance) + EPSILON*0.01)) &&
             m_TheNode->isInsideWithoutDensityCheck(Pt[0] - Normal[0]*(fabs(Distance) + EPSILON*0.01),
                                                    Pt[1] - Normal[1]*(fabs(Distance) + EPSILON*0.01),
                                                    Pt[2] - Normal[2]*(fabs(Distance) + EPSILON*0.01)))
        {
            Normal[0] *= -1;
            Normal[1] *= -1;
            Normal[2] *= -1;
        }
    }
    //TBM *** END **********************************************
}


//***************************************************
//	
//  AdaptiveRefiner::IsFlat()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 20th, 2002
//
//  Input  : VERTEX Pt1         -> First point
//           VERTEX Pt2         -> Second point
//           Vec3f  *Pt1Color   -> Color of the Pt1
//           Vec3f  *Pt2Color   -> Color of the Pt2
//
//  Output : The triangle is or isn't flat.
//
//  Desc   : This function return FLAT if the divergence
//           of the two normals is less than the tolerance
//           or if the two points are very near, else 
//           it returns NOT_FLAT.
//***************************************************
bool AdaptiveRefiner::IsFlat(VERTEX* Pt1, VERTEX* Pt2, Vec3f* Pt1Color, Vec3f* Pt2Color) const
{
    //The dot product of two vectors gives the divergence between those vectors.
    //-1 = maximum divergence an 1 = minimum divergence
    double DotProduct = DotProd(Pt1->normal, Pt2->normal);

    if ((DotProduct >= DOT_TOLERANCE) || (DotProduct == -1))
    {
        //If the divergence is less than the tolerance, the points are flat.
        //But we check if they also have the same color, if not subdivide it!
        return FLAT;
     }
   
    double Distance = sqrt((Pt1->position.x - Pt2->position.x)*(Pt1->position.x - Pt2->position.x) +
                           (Pt1->position.y - Pt2->position.y)*(Pt1->position.y - Pt2->position.y) +
                           (Pt1->position.z - Pt2->position.z)*(Pt1->position.z - Pt2->position.z));

    //If the points are not flat but are very near, consider it flat.
    return (Distance > MIN_DISTANCE) ? NOT_FLAT : FLAT;
}


//***************************************************
//	
//  Dotprod()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : April 11th, 2002
//
//  Input  : const VERTEX& Pt1  -> First point
//           const VERTEX& Pt2  -> Second point
//
//  Output : double -> the dot product of the two points
//
//***************************************************
double DotProd(const POINT_J& Pt1, const POINT_J& Pt2)
{
    return Pt1.x*Pt2.x + Pt1.y*Pt2.y + Pt1.z*Pt2.z;
}

//EOF***************************************************