/*
	This software module was originally developed by SGDL Systems Inc. in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license
to this software module or modifications thereof for use in hardware or software
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1).
Those intending to use this software module in hardware or software products are
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability for use of
this software module or modifications thereof in an implementation. Copyright is
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
	SGDL Systems Inc. retains full right to use the code for his/her own purpose, assign or
donate the code to a third party and to inhibit third parties from using the code
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
						Copyright  2001
*/

// -*- c++ -*-

/* 
essai 1: juste la creation avec les parametres
essai2: avec density
essai3: VolumetreeNode
essai4: ajouter la polygonisation
essai5: rendu
*/

#include "OpenGl/OpenGl.h"
#include "ImplicitProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"

// support classes

// For use in indexedFace set generation
CString ImplicitProxy::TextureURL = "NONE";

// Constructor

ImplicitProxy::ImplicitProxy(Implicit* pImplicit)
  : GeometryProxy(pImplicit), updateTime_(-1)
{
	this->m_nDisplayList = 0;
}

// Destructor

ImplicitProxy::~ImplicitProxy()
{
  // ici destruction
}

Volume_Tree_Node *ImplicitProxy::GetVolumeTreeNode()
{    
	return ImplicitProxy::GetVolumeTreeNode(GetImplicit());
}

Volume_Tree_Node *ImplicitProxy::GetVolumeTreeNode(Implicit* obj) {
    int NodeDensities[2] = {1, 2};
	int* densities = obj->densities.GetArray().GetBuffer();
	if (obj->densities.GetLength()>0 && densities) {
		DensityMatrix::ValidateDensities(NodeDensities, densities);
	}

	float *c = obj->c.GetArray().GetBuffer();
	float coeff[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	int CoeffNumber = obj->c.GetLength();
	
    if (CoeffNumber == 4) {
		CoeffNumber = 10;
		coeff[6] = c[0];
		coeff[7] = c[1];
		coeff[8] = c[2];
		coeff[9] = c[3];
		c = coeff;
	}

	Volume_Tree_Node *to_return = new SGDLNode(c,
											   CoeffNumber,
											   obj->solid.GetValue(),
											   obj->bboxSize.GetValue(),
                                               NodeDensities);
	if (obj->dual.GetValue()) to_return = new Volume_Tree_Unary_Op(SGDLNot, to_return, obj->bboxSize.GetValue());

	return to_return;    
}

// Render object description
void ImplicitProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
  GLfloat color[] = { 1.0f, 0.0f, 1.0f, 0.75f};
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Implicit");
}

// Render object
BOOL ImplicitProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD)
{
	if (!checkUpdate()) return false;

	if (glIsEnabled(GL_LIGHTING) && !glIsEnabled(GL_LIGHT0)) glEnable(GL_LIGHT0);
	glCallList(m_nDisplayList);

	TRACE("[Implicit] DisplayList %4d \n", m_nDisplayList);

  return TRUE;
}

// Traverse scene graph, checking for intersection
BOOL
ImplicitProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}

bool ImplicitProxy::checkUpdate(){
	DWORD lastUpdate = GetImplicit()->GetUpdateTime();
	if (updateTime_!=-1 && updateTime_==lastUpdate) return true;

    int sz = GetImplicit()->c.GetLength();
    
    if ((sz == 10) || (sz == 35) || (sz == 4)) { 
		Vec3f bboxSize = GetImplicit()->bboxSize.GetValue();
        Volume_Tree_Node *TheTree = GetVolumeTreeNode();                        
		//Fields
		DensityMatrix densMatrix; 
        densMatrix.dimension[0] = bboxSize.x + 1;
        densMatrix.dimension[1] = bboxSize.y + 1;
        densMatrix.dimension[2] = bboxSize.z + 1;
        
        //Add a link to the tree
        TheTree->Ref();
		// AM a faire 
		setDisplayList(NewDLzK_coe(densMatrix,
                            Global_Grid,
                            *TheTree,
                            TextureURL,
                            DensityArray(),
                            true));
        
        //Remove the link to the tree
        TheTree->UnRef();

		this->updateTime_ = lastUpdate;
		return true;
	}
	return false;
}

// is p outside the box ? 

Bool ImplicitProxy::isOutside(const Vec3f &p)
{
    return (!GetVolumeTreeNode()->isInside(p.x,p.y,p.z));
//	return FALSE;
}

void ImplicitProxy::setDisplayList(int dspLst) {
	if (this->m_nDisplayList) {
		glDeleteLists(this->m_nDisplayList, 1);
	}
	this->m_nDisplayList = dspLst;
}
