/***********************************************************************
This software module was originally developed by Chloe Mignot,Eric Baillargeon
and Eric Paquet, (SGDL Systme Inc. ) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (SGDL Systme Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001

  Modified for IM1 integration - 2004
************************************************************************/

/***********************************************************************/
//
//						QuadricProxy.cpp
//
// This is the implementation of a new node by SGDL Systems
// Quadric node :  capable of creating a volume defined by a quadric determined
//				   by geometrica l control points
//
// created : 17-06-2001
// last modified: 17-06-2001
/***********************************************************************/

#include "OpenGL/OpenGL.h"
#include "OpenGL/base/OpenGLVisualRenderer.h"
#include "QuadricProxy.h"
//sgdl primitives
#include "z2_coo.h"

// For use in indexedFace set generation
CString QuadricProxy::TextureURL = "NONE";

//QuadricProxy//
QuadricProxy::QuadricProxy(Quadric* pQuadric)
	: GeometryProxy(pQuadric), updateTime_(-1)
{
	this->m_nDisplayList = 0;
}

Volume_Tree_Node *QuadricProxy::GetVolumeTreeNode()
{
	return QuadricProxy::GetVolumeTreeNode(GetQuadric());
}

Volume_Tree_Node* QuadricProxy::GetVolumeTreeNode(Quadric* obj) {
    int NodeDensities[2] = {1, 2};
	int* densities = obj->densities.GetArray().GetBuffer();
	if (obj->densities.GetLength() && densities) {
		DensityMatrix::ValidateDensities(NodeDensities, densities);
	}

	double coeff[10];
	Volume_Tree_Node *to_return = new SGDLNode(QuadricProxy::Get_Current_Coeff(obj, coeff),
										   10,
										   obj->solid.GetValue(),
										   obj->bboxSize.GetValue(),
                                           NodeDensities);

	if (obj->dual.GetValue()) to_return = new Volume_Tree_Unary_Op(SGDLNot, to_return, obj->bboxSize.GetValue());

	return to_return;
}

double *QuadricProxy::Get_Current_Coeff(Quadric* obj, double coeff[10])
{
	Vec3f bboxSize = obj->bboxSize.GetValue();
	Vec4f P0 = obj->P0.GetValue();
	Vec4f P1 = obj->P1.GetValue();
	Vec4f P2 = obj->P2.GetValue();
	Vec4f P3 = obj->P3.GetValue();
	Vec4f P4 = obj->P4.GetValue();
	Vec4f P5 = obj->P5.GetValue();
	Bool dual = obj->dual.GetValue();
	Bool solid = obj->solid.GetValue();

	double v6v4[24];

	double bx = bboxSize.x;
	double by = bboxSize.y;
	double bz = bboxSize.z;

    v6v4 [0] = (P0.x * bx) / 2.0;
    v6v4 [1] = (P0.y * by) / 2.0;
    v6v4 [2] = (P0.z * bz) / 2.0;
    v6v4 [3] = (P0.q);

    v6v4 [4] = (P1.x * bx) / 2.0;
    v6v4 [5] = (P1.y * by) / 2.0;
    v6v4 [6] = (P1.z * bz) / 2.0;
    v6v4 [7] = (P1.q);

    v6v4 [8] = (P2.x * bx) / 2.0;
    v6v4 [9] = (P2.y * by) / 2.0;
    v6v4[10] = (P2.z * bz) / 2.0;
    v6v4[11] = (P2.q);

    v6v4[12] = (P3.x * bx) / 2.0;
    v6v4[13] = (P3.y * by) / 2.0;
    v6v4[14] = (P3.z * bz) / 2.0;
    v6v4[15] = (P3.q);

    v6v4[16] = (P4.x * bx) / 2.0;
    v6v4[17] = (P4.y * by) / 2.0;
    v6v4[18] = (P4.z * bz) / 2.0;
    v6v4[19] = (P4.q);

    v6v4[20] = (P5.x * bx) / 2.0;
    v6v4[21] = (P5.y * by) / 2.0;
    v6v4[22] = (P5.z * bz) / 2.0;
    v6v4[23] = (P5.q);

	SGDL_z2_coo( coeff, v6v4);

	return coeff;
}

///nodefaultlib:"msvcrt libcd"
QuadricProxy::~QuadricProxy() {}

bool QuadricProxy::checkUpdate() {
	DWORD lastUpdate = GetQuadric()->GetUpdateTime();
	if (updateTime_!=-1 && updateTime_==lastUpdate) return true;
	this->updateTime_ = lastUpdate;

    Volume_Tree_Node *TheTree = GetVolumeTreeNode();

	Vec3f bboxSize = GetQuadric()->bboxSize.GetValue();
    DensityMatrix densMatrix;
	densMatrix.dimension[0] = bboxSize.x + 1;
	densMatrix.dimension[1] = bboxSize.y + 1;
	densMatrix.dimension[2] = bboxSize.z + 1;

    //Add a ling to the tree
    TheTree->Ref();

    setDisplayList(NewDLzK_coe(densMatrix,
                                Global_Grid,
                                *TheTree,
                                TextureURL,
                                DensityArray(),
                                true));
    //Remove the link form the tree
    TheTree->UnRef();

	return true;
}

// is p outside the box ?
Bool QuadricProxy::isOutside(const Vec3f &p)
{
    return (!GetVolumeTreeNode()->isInside(p.x,p.y,p.z));
}

void QuadricProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
	GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};
	glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Quadric");
}

// Render object
BOOL QuadricProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
	if (!checkUpdate()) return false;

  	if (glIsEnabled(GL_LIGHTING) && !glIsEnabled(GL_LIGHT0)) glEnable(GL_LIGHT0);
	glCallList(m_nDisplayList);

  TRACE("[Quadric] DisplayList %4d \n", m_nDisplayList);

  return TRUE;
}

BOOL QuadricProxy::RenderBox(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  return TRUE;
}

BOOL QuadricProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      return TRUE;
    }
  return FALSE;
}

void QuadricProxy::setDisplayList(int dspLst) {
	if (this->m_nDisplayList) {
		glDeleteLists(this->m_nDisplayList, 1);
	}
	this->m_nDisplayList = dspLst;
}
