/***********************************************************************
This software module was originally developed by Chloe Mignot,Eric Baillargeon
and Eric Paquet, (SGDL Systme Inc. ) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (SGDL Systme Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************/

/***********************************************************************/
//
//						SolidRepProxy.cpp
//
// This is the implementation of a new VRML node by SGDL Systems
// SolidRep node :  capable of creating a volume by operating on other volumes.
//
// last modified: November 19th, 2001
/***********************************************************************/

#include "OpenGL/OpenGL.h"
#include "OpenGL/base/OpenGLVisualRenderer.h"

#include "ImplicitProxy.h"
#include "QuadricProxy.h"
#include "SolidRepProxy.h"
#include "../../group/TransformProxy.h"
//sgdl primitives
#include "z2_coo.h"

// For use in indexedFace set generation
CString SolidRepProxy::TextureURL = "NONE";

//*******************************************************************
//  This function sets the default value of the parameters of a SolidRep
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
SolidRepProxy::SolidRepProxy(SolidRep* pSolidRep)
 : GeometryProxy(pSolidRep), updateTime_(-1)
{
	this->m_nDisplayList = 0;
    //dual = FALSE;
    My_Tree = NULL;
}

//*******************************************************************
//  This function destroys My_Tree if there is one created
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
SolidRepProxy::~SolidRepProxy()
{
    if ( My_Tree != NULL )
    {
        //Remove a link to My_Tree
        My_Tree->UnRef();
    }
}

//*******************************************************************
//  This function sets the variable My_Tree properly
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void SolidRepProxy::SetMyTree(Volume_Tree_Node *the_other)
{
   if ( My_Tree != NULL )
       My_Tree->UnRef();

    My_Tree = the_other;

    //Add a link to the VTN
    if(the_other != NULL)
        the_other->Ref();
}

//*******************************************************************
//  This function returns the SolidRep associated Volume_Tree_Node
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node *SolidRepProxy::GetVolumeTreeNode()
{
	return SolidRepProxy::GetVolumeTreeNode(GetSolidRep(), My_Tree);
}

Volume_Tree_Node *SolidRepProxy::GetVolumeTreeNode(SolidRep* obj, Volume_Tree_Node*& My_Tree)
{
    Volume_Tree_Node *to_return = NULL;

    if ( My_Tree != NULL ) {
        to_return = My_Tree;
    } else if (obj->solidTree.GetNode() != NULL) {
        to_return = Give_Volume_Tree_Node(obj->solidTree.GetNode());

        //To add a doing nothing node that compute the SolidRep bounding box // temporary patch
		Matrix mat;
		mat.LoadIdentityTransform();
        My_Tree = new Volume_Tree_Matrix(to_return, obj->bboxSize.GetValue(), mat);
		My_Tree->Ref();
        to_return = My_Tree;
    }

    return to_return;
}

//*******************************************************************
//  This function builds the geometry of a SolidRep
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
bool SolidRepProxy::checkUpdate()
{
	DWORD lastUpdate = GetSolidRep()->GetUpdateTime();
	if (updateTime_!=-1 && updateTime_==lastUpdate) return true;

	float SR_color[3] = {-1, -1, -1};
    bool apply_color = true;

	Volume_Tree_Node *Solid_Rep_VTN = GetVolumeTreeNode();

	//We've been able to create the tree
	if ( Solid_Rep_VTN ) {
      if ((Solid_Rep_VTN->GetLocalColor())[0] == -1)
         Solid_Rep_VTN->Set_Color(SR_color);

	    Vec3f bboxSize = GetSolidRep()->bboxSize.GetValue();
		//+1 to round
	    DensityMatrix densMatrix;
		densMatrix.dimension[0] = bboxSize.x + 1;
		densMatrix.dimension[1] = bboxSize.y + 1;
		densMatrix.dimension[2] = bboxSize.z + 1;
		MF<int>& dL = GetSolidRep()->densityList;
        DensityArray MyArray;
		if (dL.GetLength()>0) {
			DensityArray tmp(dL.GetLength(), dL.GetArray().GetBuffer());
			MyArray = tmp;
            MyArray.Sort();
		}

        //Add a link to the VTN
        Solid_Rep_VTN->Ref();

        bool manage_normals = true;

		setDisplayList(NewDLzK_coe(densMatrix,
                            Global_Grid,
                            *Solid_Rep_VTN,
                            TextureURL,
                            MyArray,
                            manage_normals));

        //Remove a link to the VTN
        Solid_Rep_VTN->UnRef();

		this->updateTime_ = lastUpdate;
		return true;
	}
	return false;
}

//*******************************************************************
//  This function tests only the bbox.  Maybe we should use our powerfull
//  VTN...
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Bool SolidRepProxy::isOutside(const Vec3f &p) const
{
    bool to_return = true;
    Volume_Tree_Node *the_tree = const_cast<SolidRepProxy*>(this)->GetVolumeTreeNode();
    if ( the_tree )
    {
		MF<int>& dL = GetSolidRep()->densityList;

        if ( dL.GetLength() > 0 )
        {
	        DensityArray MyArray(dL.GetLength(), dL.GetArray().GetBuffer());
		    MyArray.Sort();

            the_tree->currentDensity = MyArray;
        }

        to_return = !(the_tree->isInside(p.x, p.y, p.z));
    }
    return to_return;
}

//****************************
//Functions used in vsexpr.cpp
//****************************

//***************************************************
//
//  Set the first bounding box with th higest x, y an z of the two bounding boxs
//
//	Author : Eric Paquet, SGDL Systems
//	Date : October 17th, 2001
//
//  Input  : float First[]       , the to be modify bounding box
//           const float Second[], the other bounding box
//
//  Output : Maybe a modify First bounding box
//
/***************************************************/
Node_Type Give_Node_Type(MediaObject *The_Node)
{
    Node_Type to_return = NODE_Unknown;

	if (The_Node) {
		int code = The_Node->GetNodeCode();
		if (code == Group     ::nodeCode) to_return = NODE_Group;
		if (code == Transform ::nodeCode) to_return = NODE_Transform;
		if (code == Shape     ::nodeCode) to_return = NODE_Shape;
		if (code == Implicit  ::nodeCode) to_return = NODE_Implicit;
		if (code == Quadric   ::nodeCode) to_return = NODE_Quadric;
		if (code == SolidRep  ::nodeCode) to_return = NODE_SolidRep;
		if (code == Sphere    ::nodeCode) to_return = NODE_Sphere;
		if (code == Cone      ::nodeCode) to_return = NODE_Cone;
		if (code == Cylinder  ::nodeCode) to_return = NODE_Cylinder;
		if (code == Box       ::nodeCode) to_return = NODE_Box;
	}

    return to_return;
}

/***************************************************
//
//  Return the Volume_Tree_Node associated with a
//	Node
//
//	Author : Eric Paquet, SGDL Systems
//	Date : October 17th, 2001
//
//  Input  : The_Node     , the current working node

//
//  Output : A Volume_Tree_Node corresponding to the Node

//
/***************************************************/
Volume_Tree_Node *Give_Volume_Tree_Node(MediaObject *The_Node)
{
    ASSERT( The_Node != NULL );

    Volume_Tree_Node *to_return         = NULL;
    Volume_Tree_Node *child_s_Tree_Node = NULL;
    MFNodeType<SF3DNode> *children_ptr      = NULL;
    MediaObject      *temp_Children     = NULL;
    ZArray<Volume_Tree_Node*> All_Children;

    int Number_of_Child   = 0;
    int default_Density[] = {1,2};

    switch ( Give_Node_Type(The_Node) )
    {
        case NODE_Group:
        {
            //Construct a SR with all child of the group
            children_ptr = &(((Group*)The_Node)->children);
            Verify_Ptr( children_ptr );

            Number_of_Child = children_ptr->GetLength();

            if ( Number_of_Child > 0 )
            {
                for ( int i = 0; i < Number_of_Child; i++ )
                {
                    temp_Children = children_ptr->GetObject(i);
                    Verify_Ptr( temp_Children );

                    child_s_Tree_Node = Give_Volume_Tree_Node(temp_Children);

                    if (child_s_Tree_Node)
                        All_Children += child_s_Tree_Node;
                }

                if (All_Children.GetLength() == 0)
                    break; //We haven't found any child that are compatible node
                else if ( All_Children.GetLength() == 1 )
                    to_return = All_Children[0];
                else if ( All_Children.GetLength() == 2 )
                    to_return = new Volume_Tree_Binary_Op(SGDLOr, All_Children[0], All_Children[1]);
                else
                    to_return = new Volume_Tree_Multiple_Op(SGDLMOr, All_Children);
            }
            break;
        }

        case NODE_Transform:
        {
            //Get the matrix and set all the child with it
            children_ptr = &(((Transform*)The_Node)->children);
            Verify_Ptr( children_ptr );

            Number_of_Child = children_ptr->GetLength();
            if ( Number_of_Child > 0 )
            {
                for ( int i = 0; i < Number_of_Child; i++ )
                {
                    temp_Children = children_ptr->GetObject(i);
                    Verify_Ptr( temp_Children );

                    child_s_Tree_Node = Give_Volume_Tree_Node(temp_Children);

                    if (child_s_Tree_Node)
                        All_Children += child_s_Tree_Node;
                }

                //A bit of optimization
                if (All_Children.GetLength() == 0)
                    break;
                else if ( All_Children.GetLength() == 1 )
                    to_return = All_Children[0];
                else if ( All_Children.GetLength() == 2 )
                    to_return = new Volume_Tree_Binary_Op(SGDLOr, All_Children[0], All_Children[1]);
                else
                    to_return = new Volume_Tree_Multiple_Op(SGDLMOr, All_Children);

				TransformProxy transProxy((Transform*)The_Node);
				Matrix mat;
				transProxy.ConcatenateTransform(mat);
				to_return = Verified_Volume_Tree_Matrix(mat, to_return);
            }

            break;
        }
        case NODE_Shape:
        {
            //Search for a diffuse color and if it's not black set the VTN to this color
            Appearance *tried_appearance = 0;
            Material   *tried_material   = 0;
            Color3f    temp_color;
            Shape      *tried_Shape      = (Shape *)(The_Node);
            Verify_Ptr( tried_Shape );

            tried_appearance = (Appearance*)tried_Shape->appearance.GetNode();

            if (tried_appearance)
                tried_material = (Material*)tried_appearance->material.GetNode();

            if (tried_material)
                temp_color = tried_material->diffuseColor;

            to_return = Give_Volume_Tree_Node(((Shape*)The_Node)->geometry.GetNode());
            if (to_return && temp_color)
                to_return->Set_Color(temp_color);
            break;
        }

        case NODE_Implicit:
        {
            //No problemo
            to_return = ImplicitProxy::GetVolumeTreeNode((Implicit*)The_Node);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
            break;
        }

        case NODE_Quadric:
        {
            //No problemo
            to_return = QuadricProxy::GetVolumeTreeNode((Quadric*)The_Node);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
            break;
        }

        case NODE_SolidRep:
        {
			SolidRep* solidRep = (SolidRep*)The_Node;
			if (solidRep->data) {
				to_return = (Volume_Tree_Node*)solidRep->data;
	            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
			} else {
				assert(false);
			}
            break;
        }

        case NODE_Sphere:
        {
            //if it's a VRML Sphere convert it to an Implicit sphere
            Sphere *The_Sphere = (Sphere*)(The_Node);
            float rayon = The_Sphere->radius;

            double v6v4[24];
            double coeff[10];

            double diameter = 2.0 * rayon;

            v6v4 [0] = (-1.0 * diameter) / 2.0;
            v6v4 [1] = (0.0 * diameter) / 2.0;
            v6v4 [2] = (0.0 * diameter) / 2.0;
            v6v4 [3] = (1.0);

            v6v4 [4] = (1.0 * diameter) / 2.0;
            v6v4 [5] = (0.0 * diameter) / 2.0;
            v6v4 [6] = (0.0 * diameter) / 2.0;
            v6v4 [7] = (1.0);

            v6v4 [8] = (0.0 * diameter) / 2.0;
            v6v4 [9] = (1.0 * diameter) / 2.0;
            v6v4[10] = (0.0 * diameter) / 2.0;
            v6v4[11] = (0.0);

            v6v4[12] = (0.0 * diameter) / 2.0;
            v6v4[13] = (0.0 * diameter) / 2.0;
            v6v4[14] = (1.0 * diameter) / 2.0;
            v6v4[15] = (0.0);

            v6v4[16] = (0.0 * diameter) / 2.0;
            v6v4[17] = (1.0 * diameter) / 2.0;
            v6v4[18] = (0.0 * diameter) / 2.0;
            v6v4[19] = (1.0);

            v6v4[20] = (0.0 * diameter) / 2.0;
            v6v4[21] = (0.0 * diameter) / 2.0;
            v6v4[22] = (1.0 * diameter) / 2.0;
            v6v4[23] = (1.0);

            SGDL_z2_coo( coeff, v6v4);

            //Get the VTN
            float the_bbox[] = {2.0 * rayon, 2.0 * rayon, 2.0 * rayon};
            to_return = new SGDLNode(coeff, 10, true, the_bbox, default_Density);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
			break;
		}

	    case NODE_Cone:
        {
			//if it's a VRML Cone convert it to an implicit Cone
			Cone *The_Cone = (Cone*)(The_Node);
		    float Cone_Bottom_rayon = The_Cone->bottomRadius;
			float Cone_Height = The_Cone->height;

			double v6v4[24];
			double coeff[10];

			double diameter = 2.0 * Cone_Bottom_rayon;

			v6v4 [0] = (-1.0 * diameter) / 2.0;
			v6v4 [1] = (-1.0 * Cone_Height) / 2.0;
			v6v4 [2] = (0.0 * diameter) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * diameter) / 2.0;
			v6v4 [5] = (-1.0 * Cone_Height) / 2.0;
			v6v4 [6] = (0.0 * diameter) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * diameter) / 2.0;
			v6v4 [9] = (1.0 * Cone_Height) / 2.0;
			v6v4[10] = (0.0 * diameter) / 2.0;
			v6v4[11] = (1.0);

			v6v4[12] = (0.0 * diameter) / 2.0;
			v6v4[13] = (0.0 * Cone_Height) / 2.0;
			v6v4[14] = (1.0 * diameter) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * diameter) / 2.0;
			v6v4[17] = (1.0 * Cone_Height) / 2.0;
			v6v4[18] = (0.0 * diameter) / 2.0;
			v6v4[19] = (1.0);

			v6v4[20] = (0.0 * diameter) / 2.0;
			v6v4[21] = (-1.0 * Cone_Height) / 2.0;
			v6v4[22] = (1.0 * diameter) / 2.0;
			v6v4[23] = (1.0);

			SGDL_z2_coo( coeff, v6v4);

			//Get the VTN
			float the_bbox[] = {2.0 * Cone_Bottom_rayon, Cone_Height, 2.0 * Cone_Bottom_rayon};
			to_return = new SGDLNode(coeff, 10, true, the_bbox, default_Density);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
			break;
		}

        case NODE_Cylinder:
		{
			//if it's a VRML Cylinder convert it to an implicit Cylinder
			Cylinder *The_Cylinder = (Cylinder*)(The_Node);
			float Cyl_Rayon = The_Cylinder->radius;
			float Cyl_Height = The_Cylinder->height;

			double v6v4[24];
			double coeff[10];

			double diameter = 2.0 * Cyl_Rayon;

			v6v4 [0] = (-1.0 * diameter) / 2.0;
			v6v4 [1] = (0.0 * Cyl_Height) / 2.0;
			v6v4 [2] = (0.0 * diameter) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * diameter) / 2.0;
			v6v4 [5] = (0.0 * Cyl_Height) / 2.0;
			v6v4 [6] = (0.0 * diameter) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * diameter) / 2.0;
			v6v4 [9] = (1.0 * Cyl_Height) / 2.0;
			v6v4[10] = (0.0 * diameter) / 2.0;
			v6v4[11] = (0.0);

			v6v4[12] = (0.0 * diameter) / 2.0;
			v6v4[13] = (0.0 * Cyl_Height) / 2.0;
			v6v4[14] = (1.0 * diameter) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * diameter) / 2.0;
			v6v4[17] = (1.0 * Cyl_Height) / 2.0;
			v6v4[18] = (0.0 * diameter) / 2.0;
			v6v4[19] = (0.0);

			v6v4[20] = (0.0 * diameter) / 2.0;
			v6v4[21] = (0.0 * Cyl_Height) / 2.0;
			v6v4[22] = (1.0 * diameter) / 2.0;
			v6v4[23] = (1.0);

			SGDL_z2_coo( coeff, v6v4);

			//Get the VTN
			float the_bbox[] = {2.0 * Cyl_Rayon, Cyl_Height, 2.0 * Cyl_Rayon};
			to_return = new SGDLNode(coeff, 10, true, the_bbox, default_Density);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
			break;
		}
	    case NODE_Box:
		{
			//if it's a VRML Box convert it to an implicit Box
			Box *The_Box = (Box*)(The_Node);
			Vec3f size = The_Box->size.GetValue();
			float delta_x = size.x;
			float delta_y = size.y;
			float delta_z = size.z;

			double v6v4[24];
			double coeff[10];

			v6v4 [0] = (-1.0 * delta_x) / 2.0;
			v6v4 [1] = (0.0 * delta_y) / 2.0;
			v6v4 [2] = (0.0 * delta_z) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * delta_x) / 2.0;
			v6v4 [5] = (0.0 * delta_y) / 2.0;
			v6v4 [6] = (0.0 * delta_z) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * delta_x) / 2.0;
			v6v4 [9] = (1.0 * delta_y) / 2.0;
			v6v4[10] = (0.0 * delta_z) / 2.0;
			v6v4[11] = (0.0);

			v6v4[12] = (0.0 * delta_x) / 2.0;
			v6v4[13] = (0.0 * delta_y) / 2.0;
			v6v4[14] = (1.0 * delta_z) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * delta_x) / 2.0;
			v6v4[17] = (1.0 * delta_y) / 2.0;
			v6v4[18] = (0.0 * delta_z) / 2.0;
			v6v4[19] = (0.0);

			v6v4[20] = (0.0 * delta_x) / 2.0;
			v6v4[21] = (0.0 * delta_y) / 2.0;
			v6v4[22] = (1.0 * delta_z) / 2.0;
			v6v4[23] = (0.0);

			SGDL_z2_coo( coeff, v6v4);

			//Get the VTN
			float the_bbox[] = {delta_x, delta_y, delta_z};
			to_return = new SGDLNode(coeff, 10, true, the_bbox, default_Density);
            SGDL_Node_In_Solid_Rep_Must_Be_Solid(to_return);
			break;
		}
        default:
		{
			//there is a problem
			ASSERT( false );
			break;
		}
    }
    return to_return;
}

/*******************************************************************\
Function: SGDL_Node_In_Solid_Rep_Must_Be_Solid(Volume_Tree_Node *To_verify_Solidity)

  Solidify a SGDLNode

    Input: The possible SGDLNode* to solidify
    Output: The possible SGDLNode* to solidify

      Author:     Eric Paquet SGDL systems
      Created:    2001/11/6


*********************************************************************/
void SGDL_Node_In_Solid_Rep_Must_Be_Solid(Volume_Tree_Node *To_verify_Solidity)
{
	if (To_verify_Solidity && To_verify_Solidity->getId()==SGDL_NODE) {
		((SGDLNode *)To_verify_Solidity)->Set_Solidity(true);
	}
}

//*******************************************************************
//  Exhausted of doing the check I create a function doing it
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Verify_Ptr( void* the_ptr )
{
	if ( the_ptr == NULL )
		ASSERT( false );
}

void SolidRepProxy::setDisplayList(int dspLst) {
	if (this->m_nDisplayList) {
		glDeleteLists(this->m_nDisplayList, 1);
	}
	this->m_nDisplayList = dspLst;
}

void SolidRepProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
	GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};
	glColor4fv(color);

  RenderDescriptionBox(pRenderer, "SolidRep");
}

// Render object
BOOL SolidRepProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
	if (!checkUpdate()) return false;

  	if (glIsEnabled(GL_LIGHTING)) glEnable(GL_LIGHT0);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glCallList(m_nDisplayList);

	TRACE("[SolidRep] DisplayList %4d \n", m_nDisplayList);

	return TRUE;
}

BOOL SolidRepProxy::RenderBox(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  return TRUE;
}

BOOL SolidRepProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      return TRUE;
    }
  return FALSE;
}
