
#ifndef _SOLIDREP_PROXY_
#define _SOLIDREP_PROXY_

#include "../../geometry/GeometryProxy.h"

#include "DensityTLS.h"
#include "VolumeTreeNode/SGDLTools.h"
#include "SGDLPolygonise.h"

/*******************************************************************
    This class is used to implement operation over volumes

  by : Eric Paquet
  date : December 17th, 2001

*******************************************************************/
class SolidRepProxy : public GeometryProxy {
public:
	SolidRepProxy(SolidRep* pSolidRep);
	virtual ~SolidRepProxy();

	// is p outside the shape
	Bool isOutside(const Vec3f &p) const;

	Volume_Tree_Node *GetVolumeTreeNode();
	static Volume_Tree_Node* GetVolumeTreeNode(SolidRep* obj, Volume_Tree_Node*& My_Tree);

	virtual BOOL	Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
	virtual BOOL	RenderBox(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
	virtual BOOL	Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);
	virtual void	RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:
    inline SolidRep* GetSolidRep() const { return (SolidRep*) GetMediaObject(); }

	DWORD updateTime_;
	bool checkUpdate();

	Volume_Tree_Node *My_Tree;

    void SetMyTree(Volume_Tree_Node *the_other);
	void setDisplayList(int dispLst);

protected:
    static CString TextureURL;
};

//Enum
enum Node_Type
{
	NODE_Transform,
	NODE_Shape,
	NODE_SolidRep,
	NODE_Implicit,
	NODE_Quadric,
	NODE_Sphere,
	NODE_Cone,
	NODE_Cylinder,
	NODE_Box,
	NODE_Unknown,
    NODE_Group
};

//FUNCTIONS
Node_Type Give_Node_Type(MediaObject *The_Node);
Volume_Tree_Node *Give_Volume_Tree_Node(MediaObject *The_Node);

void SGDL_Node_In_Solid_Rep_Must_Be_Solid(Volume_Tree_Node *To_verify_Solidity);
void Verify_Ptr( void* the_ptr );

#endif /*_SOLIDREP_PROXY_*/
