//*********************************************************************************
//  This class contains all the information that is necessary to build 
//  an implicit volume by his coefficient
//
//	(input)
//	sgdl_Script_Def : a string containing a SGDLScript definition of the volume			
//	dual : true you get the solid dual, false you get the solid
//	solid : true the solid will be closed else it's open
//	coeff : Table  containing the coefficients
//	transfo_Mat : Transfo to apply to the solid
//
//	Author : Eric Paquet
//	Date : October 3th, 2001
//
//	Modif:
//*********************************************************************************

#ifndef _SGDL_NODE_H_
#define _SGDL_NODE_H_

#include "VolumeTreeNode.h"

class SGDLNode : public Volume_Tree_Node
{
    //public members
public:
    SGDLNode(const double coeff_ptr[],
			 int          coeff_number,
			 bool         is_solid,
			 const float  bbox_ptr[],
			 int          Node_Density[2]);

    SGDLNode(const float coeff_ptr[],
			 int         coeff_number,
			 bool        is_solid,
			 const float bbox_ptr[],
			 int         Node_Density[2]);
    
    int     EvaluateDensity(const double& x, const double& y, const double& z)const;
    
    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Gradient,
                             float        *Color = 0) const;

    double  *GetCoeff    (double x, double y, double z);
    int     GetNumOfCoeff(double x, double y, double z);

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;

    void    UnRef();
    
	//Accessor
	void    Set_Solidity(bool to_set);
    
private:
    //Protected member	
    bool    solid; 
    double  *coeff;	
    int     nCoeff;
	int density[2];
    
	void Manage_At_The_Top();

     ~SGDLNode(void);
};


#endif  //_SGDL_NODE_H_