/*
	This software module was originally developed by SGDL Systems Inc. in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license
to this software module or modifications thereof for use in hardware or software
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1).
Those intending to use this software module in hardware or software products are
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability for use of
this software module or modifications thereof in an implementation. Copyright is
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
	SGDL Systems Inc. retains full right to use the code for his/her own purpose, assign or
donate the code to a third party and to inhibit third parties from using the code
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
						Copyright  2001
*/

//*******************************************************************
//Volume_Tree_Node
//*******************************************************************

#include "Opengl/AFX/Solid/VolumeTreeNode/SGDLTools.h"

DensityArray Volume_Tree_Node::currentDensity;

//*******************************************************************
//  Volume_Tree_Node constructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node::Volume_Tree_Node(int id, const float bbox_ptr[]):
ref(0), bbox(0), id_(id)
{
	if ( bbox_ptr ) {
		bbox = new float[3];
		Set_Bbox(bbox_ptr);
	}
	float temp[] = {-1.0, -1.0, -1.0};
	Set_Color(temp);
    ShowAllDensities = true;
}

//*******************************************************************
//  Volume_Tree_Node destructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node::~Volume_Tree_Node(void)
{
	assert(!ref);
	if ( bbox != NULL && ref == 0)
	{
		delete[] bbox;
	}
}

//*******************************************************************
//  This function sets the member currentDensity
//
//  By Eric Baillargeon
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void  Volume_Tree_Node::init()
{
    currentDensity.SetDefault(K_DensityDefaultMaximum,K_DensityDefaultValues);
}

//*******************************************************************
//  This function sets the bbox of s VTN 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Set_Bbox(const float bbox_ptr[])
{
	if ( bbox == NULL )
	{
		bbox = new float[3];
	}

	bbox[0] = bbox_ptr[0];
	bbox[1] = bbox_ptr[1];
	bbox[2] = bbox_ptr[2];
}

//*******************************************************************
//  This function sets the color of a VTN
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Set_Color(const float to_set[])
{
	color[0] = to_set[0];
	color[1] = to_set[1];
	color[2] = to_set[2];
}

//*******************************************************************
//  This function adjusts x, y or z to the current bbox if they are outside
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Adjust_With_BBox(double& x, double& y, double& z)const
{
	if ( bbox != NULL && At_The_Top)
	{
		double xdiv2 = bbox[0]/2.0;
	    if (x > xdiv2 )
		{	     
		    x = xdiv2;
		}
	    else if (x < -xdiv2 )
		{
		    x = -xdiv2;
		}

		double ydiv2 = bbox[1]/2.0;
	    if ( y > ydiv2 )
		{
		    y = ydiv2;
		}
	    else if ( y < -ydiv2 )
		{
		    y = -ydiv2;
		}
        
		double zdiv2 = bbox[2]/2.0;
	    if ( z > zdiv2 )
		{
            z = zdiv2;
		}
	    else if ( z < -zdiv2 )
		{
		    z = -zdiv2;
		}
	}
}

/********************************************************************\
Function:  

  Input:     const double& x;
	     const double& y;
	     const double& z;
  Output:    bool :	      if the point is inside the volume;
  
    Author:	Eric Baillargeon SGDL Systme Inc.
    Created:	2001/11/23	
     
      
*********************************************************************/
bool Volume_Tree_Node::isInside(const double& x, const double& y, const double& z)const
{   
    int resulEval = EvaluateDensity(x,y,z);

    if (ShowAllDensities) 
        return (resulEval > 0);
    else
        return CheckDensity(resulEval ,currentDensity );
}

/********************************************************************\
Function:  

  Input:     const double& x;
	     const double& y;
	     const double& z;
  Output:    bool :	      if the density is different of 0
  
    Author:	Eric Paquet SGDL Systme Inc.
    Created:	2001/11/28	
     
      
*********************************************************************/
bool Volume_Tree_Node::isInsideWithoutDensityCheck(const double& x, const double& y, const double& z)const
{
    return EvaluateDensity(x,y,z) > 0;
}

//EOF*****************************************************************