#ifndef _VOLUME_TREE_NODE_H
#define _VOLUME_TREE_NODE_H

//**************************************************************
// Class Volume_Tree_Node - This is the abstract class of all
// the nodes needed to build a local Volume Tree with primitives,
// transformation matrix and operations. It's mainly used by the
// polygonizer.
//**************************************************************
class DensityArray;
class Volume_Tree_Binary_Op;
class Volume_Tree_Unary_Op;

#include <ztl.h>
#include "OpenGL/base/Matrix.h"

const int SGDL_VT_CONST = 1;
const int SGDL_VT_BINOP = 2;
const int SGDL_VT_UNOP = 3;
const int SGDL_VT_MULOP = 4;
const int SGDL_VT_MATRIX = 5;
const int SGDL_NODE = 6;

//***************************************************************
//	 Here is an example of a tree
//  class Point
//						   Volume_Tree_Op
//			  /				       \
//	       Volume_Tree_Op				     Volume_Tree_Op
//	       /	     \				     /		   \
//Volume_Tree_Leaf	   Volume_Tree_Leaf	Volume_Tree_Leaf	 Volume_Tree_Op
//									 /	       \
//							  Volume_Tree_Leaf		Volume_Tree_Leaf
//*****************************************************************

class Volume_Tree_Node
{
protected:
    Volume_Tree_Node(int id, const float bbox_ptr[]=0);

public:
    virtual ~Volume_Tree_Node();

	//Accessor
	void Set_Bbox(const float bbox_ptr[]);

	//Functions

    //Eb, here we do some initialisation;
    void init();
    
    virtual int EvaluateDensity(const double& x, const double& y, const double& z)const = 0;    

    // SL Fonction to evaluate the normal (here, we need to traverse the tree because
    // no API function that return the normal at one point is available).
    virtual bool   EvaluateGradient(const double &x,
                                    const double &y,
                                    const double &z,
                                    double       &Distance,
                                    double       *Gradient,
                                    float        *Color = 0) const = 0;

    virtual bool    isInside( const double& x, const double& y, const double& z)  const;

    bool            isInsideWithoutDensityCheck(const double& x, 
                                                const double& y, 
                                                const double& z) const;

    virtual double  *GetCoeff(double x, double y , double z)     = 0;

    virtual int     GetNumOfCoeff(double x, double y , double z) = 0;

    virtual bool    GetColor(const double &x, 
                             const double &y, 
                             const double &z, 
                             float set_color[])const = 0;

    virtual void    Ref()   {ref++;}
    virtual void    UnRef() {ref--;}

	void            Set_Color(const float to_set[]);
	const float     *GetLocalColor() const {return color;}

    void Adjust_With_BBox(double& x, double& y, double& z)const;    

    //Public members
    static DensityArray currentDensity;
	bool                At_The_Top;
    bool                ShowAllDensities;

	int getId() { return this->id_; }

protected:
    //Protected member
    float *bbox; 
	float        color[3]; // default = -1.0 = Unknown
    int          ref;    
	virtual void Manage_At_The_Top()=0;

private:
	int id_;
};

#endif //_VOLUME_TREE_NODE_H