#ifdef FIXME_OB

/*=============================================================================

This code is licensed under the Web3D-blaxxun Community Source License,
provided in distribution file LICENSE.TXT and available online at
http://www.web3D.org/TaskGroups/x3d/blaxxun/Web3D-blaxxunCommunitySourceAgreement.html
and may only be used for non-commercial use as specified in that license.

THE WEB3D CONSORTIUM AND BLAXXUN DO NOT MAKE AND HEREBY DISCLAIM ANY EXPRESS
OR IMPLIED WARRANTIES RELATING TO THIS CODE, INCLUDING BUT NOT LIMITED TO,
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR
PURPOSE, OR ANY WARRANTIES THAT MIGHT ARISE FROM A COURSE OF DEALING, USAGE
OR TRADE PRACTICE.  THE COMMUNITY SOURCE CODE IS PROVIDED UNDER THIS
AGREEMENT "AS IS," WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE COMMUNITY SOURCE CODE ARE
FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR
NON-INFRINGING OR IN ANY WAY CONSTITUTE THE COMPLETE PRODUCT MARKETED UNDER
THE NAMES GIVEN SAID CODE.

==============================================================================*/
// ===================================================================
// matrix.cpp
// 20.10.96     denom = v.x * M.x[0][3] + v.y * M.x[1][3] + v.z * M.x[2][3] + M.x[3][3];

#else

#include "defs.h"
#include "ztypes.h"

#define Point Vec3f

#include "SL_matrix.h"

inline Point CrossProd (const Point& A, const Point& B)
{
	return Point(A.y * B.z - A.z * B.y,
			A.z * B.x - A.x * B.z,
			A.x * B.y - A.y * B.x);
}

inline float DotProd (const Point& A, const Point& B)
{
	return A.x * B.x + A.y * B.y + A.z * B.z;
}

// From hquat.c

/*
  unmatrix : Spencer W. Thomas, University of Michigan
  Graphic Gems II, Page 600 + 320
*/
#define XY x
#define XZ y
#define YZ z

int SL_Matrix::Decompose(Point &scale,Point &shear, Point &rotate, Point &translate) const
{ register int i;
  SL_Matrix locmat(*this);
  Point row[3];
  Point cross12;

  /* assume
     det of upper 3*3 matrix non zero
     no perspective part
  */
  /* normalize */

  translate.x = locmat.x[3][0]; locmat.x[3][0]= 0;
  translate.y = locmat.x[3][1]; locmat.x[3][1]= 0;
  translate.z = locmat.x[3][2]; locmat.x[3][2]= 0;

  /* now get scale and shear */
  for(i=0; i< 3; i++) { row[i] = Vec3f(locmat.x[i][0],locmat.x[i][1],locmat.x[i][2]); }

  /* Compute x scale factor and normalize first row */
  scale.x = row[0].GetLength();
  row[0].Normalize();

  shear.XY = DotProd(row[0],row[1]);
  
  row[1]+=row[0] * -shear.XY;

  /* Compute y scale factor and normalize second row */
  scale.y = row[1].GetLength();
  row[1].Normalize();

  if (scale.y != 0.0) shear.XY /= scale.y;

  /* compute xz and yz shears, orthogonalize 3rd row */
  shear.XZ = DotProd(row[0],row[2]);
  row[2]+=row[0]* -shear.XZ;

  shear.YZ = DotProd(row[1],row[2]);
  row[2]+=row[1]*-shear.YZ;

  scale.z = row[2].GetLength();
  row[2].Normalize();

  if (scale.z != 0.0) shear.XZ /= scale.z;
  if (scale.z != 0.0) shear.YZ /= scale.z;

  /* at this point the matrix in rows is orthonormal.
     Check for a coordinate system flip. if the determinat is -1
     then negate the matrix and the scaling factors.
  */
  cross12 = CrossProd(row[1],row[2]);

  if (DotProd(row[0],cross12) < 0)
     for(i=0; i <3;i++) {
        row[i]*= -1.0;
		if (i==0) scale.x *= -1; else
		if (i==1) scale.y *= -1; else
		if (i==2) scale.z *= -1;
     }

  /* rotations */
  rotate.y = asin(-row[0].z);

  if (cos(rotate.y) != 0) {
        rotate.x = atan2(row[1].z,row[2].z);
        rotate.z = atan2(row[0].y,row[0].x);
  } else {
        rotate.x = atan2(row[1].x,row[1].y);
        rotate.z = 0;
  }

  return(1);
}

// Transform a point by a SL_Matrix.
Point
operator* (const SL_Matrix& M, const Point& v)
{
    Point ret;
    float denom;

    ret.x = v.x * M.x[0][0] + v.y * M.x[1][0] + v.z * M.x[2][0] + M.x[3][0];
    ret.y = v.x * M.x[0][1] + v.y * M.x[1][1] + v.z * M.x[2][1] + M.x[3][1];
    ret.z = v.x * M.x[0][2] + v.y * M.x[1][2] + v.z * M.x[2][2] + M.x[3][2];
    denom = v.x * M.x[0][3] + v.y * M.x[1][3] + v.z * M.x[2][3] + M.x[3][3];

	//ASSERT(denom != 0.0f);
	if (denom == 0.0f) 
		return ret;

    if (denom != 1.0)
		ret *= (1.0f / denom);

    return ret;
}

SL_Matrix
operator* (const SL_Matrix& A, const SL_Matrix& B)
{
    SL_Matrix ret;
    for (int i = 0; i < 4; i++)	{
        ret.x[i][0] = A.x[i][0] * B.x[0][0] + A.x[i][1] * B.x[1][0] + A.x[i][2] * B.x[2][0] + A.x[i][3] * B.x[3][0] ;
        ret.x[i][1] = A.x[i][0] * B.x[0][1] + A.x[i][1] * B.x[1][1] + A.x[i][2] * B.x[2][1] + A.x[i][3] * B.x[3][1] ;
        ret.x[i][2] = A.x[i][0] * B.x[0][2] + A.x[i][1] * B.x[1][2] + A.x[i][2] * B.x[2][2] + A.x[i][3] * B.x[3][2] ;
        ret.x[i][3] = A.x[i][0] * B.x[0][3] + A.x[i][1] * B.x[1][3] + A.x[i][2] * B.x[2][3] + A.x[i][3] * B.x[3][3] ;
	}

	ret.contents = A.contents | B.contents;
    return ret;
}

// Multiply two matrices.
SL_Matrix&
SL_Matrix::operator*= (const SL_Matrix& A)
{
    SL_Matrix ret = *this;

    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++) {
	    float subt = 0;
	    for (int k = 0; k < 4; k++)
			subt += ret.x[i][k] * A.x[k][j];
	    x[i][j] = subt;
	}
	contents |= A.contents;    
	return *this;
}

// Construct an identity matrix.
SL_Matrix
IdentityMatrix()
{
    SL_Matrix M;

	const float one = 1.0f;
	const float zero = 0.0f;
    // help compiler ?
    M.x[0][0] = one;  M.x[1][0] =		M.x[2][0] =		M.x[3][0] =
    M.x[0][1] = zero; M.x[1][1] = one;  M.x[2][1] =		M.x[3][1] =
    M.x[0][2] =       M.x[1][2] = zero;	M.x[2][2] = one;  M.x[3][2] =
    M.x[0][3] =       M.x[1][3] =       M.x[2][3] = zero; M.x[3][3] =one;
/*
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    M.x[i][j] = (i == j) ? 1.0 : 0.0;
*/
	M.contents = 0;
    return M;
}

static inline double MINOR(const SL_Matrix& M,
                    int r0, int r1, int r2, int c0, int c1, int c2)
{
  return M.x[r0][c0] * (M.x[r1][c1] * M.x[r2][c2] - M.x[r2][c1] * M.x[r1][c2]) -
	 M.x[r0][c1] * (M.x[r1][c0] * M.x[r2][c2] - M.x[r2][c0] * M.x[r1][c2]) +
	 M.x[r0][c2] * (M.x[r1][c0] * M.x[r2][c1] - M.x[r2][c0] * M.x[r1][c1]);
}

void 
Adjoint(const SL_Matrix& m,SL_Matrix &A) 
{
  
  A.x[0][0] =  MINOR(m, 1, 2, 3, 1, 2, 3);
  A.x[0][1] = -MINOR(m, 0, 2, 3, 1, 2, 3);
  A.x[0][2] =  MINOR(m, 0, 1, 3, 1, 2, 3);
  A.x[0][3] = -MINOR(m, 0, 1, 2, 1, 2, 3);
  A.x[1][0] = -MINOR(m, 1, 2, 3, 0, 2, 3);
  A.x[1][1] =  MINOR(m, 0, 2, 3, 0, 2, 3);
  A.x[1][2] = -MINOR(m, 0, 1, 3, 0, 2, 3);
  A.x[1][3] =  MINOR(m, 0, 1, 2, 0, 2, 3);
  A.x[2][0] =  MINOR(m, 1, 2, 3, 0, 1, 3);
  A.x[2][1] = -MINOR(m, 0, 2, 3, 0, 1, 3);
  A.x[2][2] =  MINOR(m, 0, 1, 3, 0, 1, 3);
  A.x[2][3] = -MINOR(m, 0, 1, 2, 0, 1, 3);
  A.x[3][0] = -MINOR(m, 1, 2, 3, 0, 1, 2);
  A.x[3][1] =  MINOR(m, 0, 2, 3, 0, 1, 2);
  A.x[3][2] = -MINOR(m, 0, 1, 3, 0, 1, 2);
  A.x[3][3] =  MINOR(m, 0, 1, 2, 0, 1, 2);

  A.contents = m.contents; // ?? what to return

}

double Determinant(const SL_Matrix& m)
{
/*		5.5.97 
  return 
	m[0][0] * MINOR(m, 1, 2, 3, 1, 2, 3) -
	 m[0][1] * MINOR(m, 1, 2, 3, 0, 2, 3) +
	 m[0][2] * MINOR(m, 1, 2, 3, 0, 1, 3) -
	 m[0][3] * MINOR(m, 1, 2, 3, 0, 1, 2);
*/
  double ret = 0.0;
  if (m.x[0][0] != 0.0f) ret = m.x[0][0] * MINOR(m, 1, 2, 3, 1, 2, 3);
  if (m.x[0][1] != 0.0f) ret -= m.x[0][1] * MINOR(m, 1, 2, 3, 0, 2, 3);
  if (m.x[0][2] != 0.0f) ret += m.x[0][2] * MINOR(m, 1, 2, 3, 0, 1, 3);
  if (m.x[0][3] != 0.0f) ret -= m.x[0][3] * MINOR(m, 1, 2, 3, 0, 1, 2);
  return ret;

}

void
SL_Matrix::Invert()
{
	// identity SL_Matrix, is the inverse for itself
	if (contents ==0)
		return;

	SL_Matrix m;

	float det =  (1.0/ Determinant(*this));
   
	Adjoint(*this,m);

	for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] = m.x[i][j]*det;
    
	contents = contents; // ?? what to return
}

SL_Matrix
Invert(const SL_Matrix& M)
{
    SL_Matrix InvertMe = M;
    InvertMe.Invert();
    return InvertMe;
}

// Construct a rotation matrix.  Rotates Angle radians about the
// X axis.
SL_Matrix
RotationXMatrix(float Angle)
{
    SL_Matrix M(IdentityMatrix());
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[1][1] = Cosine;
    M.x[2][1] = -Sine;
    M.x[1][2] = Sine;
    M.x[2][2] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}

// Construct a rotation matrix.  Rotates Angle radians about the
// Y axis.
SL_Matrix
RotationYMatrix(float Angle)
{
    SL_Matrix M(IdentityMatrix());
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[0][0] = Cosine;
    M.x[2][0] = Sine;
    M.x[0][2] = -Sine;
    M.x[2][2] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}

// Construct a rotation SL_Matrix.  Rotates Angle radians about the
// Z axis.
SL_Matrix
RotationZMatrix(float Angle)
{
    SL_Matrix M(IdentityMatrix());
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[0][0] = Cosine;
    M.x[1][0] = -Sine;
    M.x[0][1] = Sine;
    M.x[1][1] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}

// can be optimized
SL_Matrix 
RotationMatrix(const Point& Rotation)
{
   SL_Matrix M = RotationXMatrix(Rotation.x);
   if (Rotation.y != 0.0) M *= RotationYMatrix(Rotation.y);
   if (Rotation.z != 0.0) M *= RotationZMatrix(Rotation.z);
   M.contents = SL_Matrix::HAS_ROTATION;
   return M;
}

#endif // FIXME_OB