#if 1 //def FIXME_OB

/*=============================================================================

This code is licensed under the Web3D-blaxxun Community Source License,
provided in distribution file LICENSE.TXT and available online at
http://www.web3D.org/TaskGroups/x3d/blaxxun/Web3D-blaxxunCommunitySourceAgreement.html
and may only be used for non-commercial use as specified in that license.

THE WEB3D CONSORTIUM AND BLAXXUN DO NOT MAKE AND HEREBY DISCLAIM ANY EXPRESS
OR IMPLIED WARRANTIES RELATING TO THIS CODE, INCLUDING BUT NOT LIMITED TO,
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR
PURPOSE, OR ANY WARRANTIES THAT MIGHT ARISE FROM A COURSE OF DEALING, USAGE
OR TRADE PRACTICE.  THE COMMUNITY SOURCE CODE IS PROVIDED UNDER THIS
AGREEMENT "AS IS," WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE COMMUNITY SOURCE CODE ARE
FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR
NON-INFRINGING OR IN ANY WAY CONSTITUTE THE COMPLETE PRODUCT MARKETED UNDER
THE NAMES GIVEN SAID CODE.

==============================================================================*/
#ifndef _SL_Matrix_h_
#define _SL_Matrix_h_

/******************************************************************************
@doc

@module SL_Matrix.h - 4*4 Matrix class	|


Purpose:

Classes:
<c Matrix>
<c MatrixAndInverse>
<c DecomposedMatrix>

Notes:

Changes:

$Revision: 1.1.1.1 $
$Log: matrix.h,v $
Revision 1.1.1.1  2002/07/17 21:05:07  mikeseven
Initial check in of July 17, 2002 version

Revision 1.2  2002/06/14 16:33:20  web3d-mpeg
no message

Revision 1.1.1.1  2001/04/03 03:12:40  web3d-mpeg
no message

Revision 1.14  1999/07/06 16:55:17  tom
..

Revision 1.13  1999/04/07 15:20:54  tom
empty message



Todo :

******************************************************************************/

// Forward-declare some other classes
//class Point;

class AStream;

#define Point Vec3f

//! a 4*4 float transformation matrix
//! many operations are designed to not use itermediate or stack return values for speed
class SL_Matrix {
public:
	enum Contents {
		HAS_TRANSLATION = 1,		//!< matrix has translation part

		HAS_SCALE		= 2,		//!< a scale part 
		
		HAS_NEGATIVE_SCALE = 4,		//!< negative scale 
		HAS_ROTATION	= 8,		//!< any rotation
		
		HAS_PERSPECTIVE	= 16,		//!< perspective part
		
		HAS_NON_UNIFORM_SCALE = 32,	//!< non uniform scale 
	
		HAS_ANY_SCALE = 2+4+32,
		HAS_ANY_TRANSFORM	= 1+2+4+8+32,
		HAS_ANYTHING	= 1+2+4+8+16+32,
	};	
	
	union {
		//! the 4*4 matrix
		float x[4][4];
	};
	unsigned char contents;

#ifdef FIXME_OB
	// identity : contents = 0
	bool IsIdentity() const { return contents == 0; }
	bool HasScale() const { return (contents & HAS_SCALE) !=0; }
	bool HasRotation() const { return (contents & HAS_ROTATION) !=0; }
	bool Has(int what) const { return (contents & what) !=0; }

	//! constructor, do nothing !
    SL_Matrix() {  contents = HAS_ANYTHING; }

	//! constructor, with matrix 
    SL_Matrix(const SL_Matrix &m);


	//! Acess row
	float*  operator[] (int row) { return(x[row]);}
	//! Acess row
	const float*  operator[] (int row) const { return(x[row]);}

    // Assignment operators
    SL_Matrix& operator+= (const SL_Matrix& A);	//!< add-to
    SL_Matrix& operator-= (const SL_Matrix& A);	//!< subtract-from
    SL_Matrix& operator*= (const SL_Matrix& A);	//!< multiply by matrix

    SL_Matrix& MultiplyLeft(const SL_Matrix& A);	//!< multiply by SL_Matrix, A is left, this is right
    SL_Matrix& MultiplyRight(const SL_Matrix& A);	//!< multiply by SL_Matrix, A is right, this is left

	//! translation SL_Matrix 
	SL_Matrix& TranslationMatrix(const Point& Location);
	SL_Matrix& ScaleByLeft(const Point& scale);
    
	//! set to A*B, neither A or B is this !!!!!!
	SL_Matrix& Multiply(const SL_Matrix& A,const SL_Matrix& B);	
    
	SL_Matrix& operator*= (float A);		//!< scale by scalar

    // Fundamental operations
    void Invert();					//!< Invert the matrix
    void Transpose();				//!< Transpose the matrix
    
    friend SL_Matrix Invert(const SL_Matrix& M);		//!< Invert a given SL_Matrix
    friend SL_Matrix Transpose(const SL_Matrix& M);	//!< Transpose a given SL_Matrix

    void SetToInverse(const SL_Matrix& M);	//!< set this to the inverse of a given SL_Matrix
	
	//! set to identiy matrix 
	SL_Matrix& Identity();
	
	// *= assigment

	SL_Matrix& Scale(float X, float Y, float Z);
	SL_Matrix& Scale(const Point& scale);
	
	SL_Matrix& Translate(const Point& Location);
	SL_Matrix& Translate(float X, float Y, float Z);
	
	SL_Matrix& Rotate(const Point& Rotation);
	SL_Matrix& Rotate(float X, float Y, float Z);

    // Create various types of SL_Matrix.
    friend SL_Matrix IdentityMatrix();
    friend SL_Matrix ZeroMatrix();
    friend SL_Matrix TranslationMatrix(const Point& Location);
	friend SL_Matrix TranslationMatrix(float X, float Y, float Z);

    friend SL_Matrix RotationXMatrix(float Angle);
    friend SL_Matrix RotationYMatrix(float Angle);
    friend SL_Matrix RotationZMatrix(float Angle);
    friend SL_Matrix RotationMatrix(const Point& Rotation);
	friend SL_Matrix RotationMatrix (float X, float Y, float Z);

    friend SL_Matrix RotationYPRMatrix(float Yaw, float Pitch, float Roll);
    friend SL_Matrix RotationAxisMatrix(const Point& axis, float Angle);
	friend SL_Matrix RotationAxisMatrix(const Point &AxisPt, const Point& AxisDir, float angle);

    friend SL_Matrix ScaleMatrix(float X, float Y, float Z);
    friend SL_Matrix ScaleMatrix(const Point & Scale);
    
    friend SL_Matrix GenRotation(const Point& x, const Point& y, const Point& z);
	
    friend SL_Matrix ViewMatrix(const Point& LookAt, const Point& Viewer, const Point& Up);
    friend SL_Matrix QuadricMatrix(float a, float b, float c, float d, float e,
				float f, float g, float h, float j, float k);
    friend SL_Matrix MirrorX();
    friend SL_Matrix MirrorY();
    friend SL_Matrix MirrorZ();
    friend SL_Matrix RotationOnly(const SL_Matrix& x);

    // Binary operators
    friend SL_Matrix operator+ (const SL_Matrix& A, const SL_Matrix& B);
    friend SL_Matrix operator- (const SL_Matrix& A, const SL_Matrix& B);
    friend SL_Matrix operator* (const SL_Matrix& A, float B);
    friend SL_Matrix operator* (const SL_Matrix& A, const SL_Matrix& B);
	//added by Chloe Mignot 2001/10/26 SGDL Systemes Inc
	friend bool operator== (const SL_Matrix& A, const SL_Matrix& B);
    
	friend void Multiply (const SL_Matrix& A, const SL_Matrix& B,SL_Matrix &ret);

    friend class Point operator* (const SL_Matrix& M, const Point& v);
    friend class Point RotateOnly(const SL_Matrix& M, const Point& v);
	friend class Vector operator* (const SL_Matrix& M, const Vector&v); 

	// get a scale factor to transform values 
	float ScaleFactor() const ;

    int Decompose(Point &scale,Point &shear, Point &rotate, Point &translate) const;
	int Compose(const Point &scale, const Point &shear,const Point &rotate,const Point &translate);
	
	void PostTranslate(float tx,float ty,float tz); 	
	void PostScale(float sx,float sy,float sz); 


    /*
    friend class PointArray  PointArray& operator* (const SL_Matrix& M, const PointArray& A);
    friend class PointArray PointArray& operator*= (const SL_Matrix& M);
    */

    // Overloaded output operator.
    //friend ostream& operator<< (ostream& s, const SL_Matrix& M);
	int io(AStream &a);	// input/output with AStreams
#else
	//! constructor, do nothing !
    SL_Matrix() {  contents = HAS_ANYTHING; }

    void Invert();		//!< Invert the matrix
    friend SL_Matrix Invert(const SL_Matrix& M);		//!< Invert a given SL_Matrix
    friend SL_Matrix IdentityMatrix();
    friend SL_Matrix operator* (const SL_Matrix& A, const SL_Matrix& B);
    friend class Point operator* (const SL_Matrix& M, const Point& v);
    SL_Matrix& operator*= (const SL_Matrix& A);	//!< multiply by matrix

	bool IsIdentity() const { return contents == 0; }
	bool HasRotation() const { return (contents & HAS_ROTATION) !=0; }

    int Decompose(Point &scale,Point &shear, Point &rotate, Point &translate) const;
    friend SL_Matrix RotationMatrix(const Point& Rotation);
#endif
};

#endif

#endif // FIXME_OB