/*
	This software module was originally developed by SGDL Systems Inc. in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license
to this software module or modifications thereof for use in hardware or software
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1).
Those intending to use this software module in hardware or software products are
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability for use of
this software module or modifications thereof in an implementation. Copyright is
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
	SGDL Systems Inc. retains full right to use the code for his/her own purpose, assign or
donate the code to a third party and to inhibit third parties from using the code
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
						Copyright  2001 - 2004
*/

#include <math.h>
#include "z2_coo.h"

static bool
SGDL_prevec(const double x[], int n) {
  bool isOk = true;

  while (n-->0) {
    isOk &= (x[n] == 0);
  }
  return isOk ? false : true;
}

static double*
SGDL_x__euc(double ret[4], const double x[4]) {
  if (x[3]!=0) {
    ret[0] = x[0]/x[3];
    ret[1] = x[1]/x[3];
    ret[2] = x[2]/x[3];
    ret[3] = 1;
  } else {
	ret[0] = x[0];
	ret[1] = x[1];
	ret[2] = x[2];
	ret[3] = x[3];
  }
  return ret;
}

static double*
SGDL_x__pro(double ret[4], const double vec[4], const double mat[16]) {
  double A0x0 = vec[0];
  double A0x1 = vec[1];
  double A0x2 = vec[2];
  double A0x3 = vec[3];
  const double* m0 = &mat[0];
  const double* m1 = &mat[4];
  const double* m2 = &mat[8];
  const double* m3 = &mat[12];
  ret[0] = A0x0*m0[0] + A0x1*m1[0] + A0x2*m2[0] + A0x3*m3[0];
  ret[1] = A0x0*m0[1] + A0x1*m1[1] + A0x2*m2[1] + A0x3*m3[1];
  ret[2] = A0x0*m0[2] + A0x1*m1[2] + A0x2*m2[2] + A0x3*m3[2];
  ret[3] = A0x0*m0[3] + A0x1*m1[3] + A0x2*m2[3] + A0x3*m3[3];
  return ret;
}

float*
SGDL_z__pro(float ret[4], const float vec[4], const float mat[16]) {
  float A0x0 = vec[0];
  float A0x1 = vec[1];
  float A0x2 = vec[2];
  float A0x3 = vec[3];
  const float* m0 = &mat[0];
  const float* m1 = &mat[4];
  const float* m2 = &mat[8];
  const float* m3 = &mat[12];
  ret[0] = A0x0*m0[0] + A0x1*m0[1] + A0x2*m0[2] + A0x3*m0[3];
  ret[1] = A0x0*m1[0] + A0x1*m1[1] + A0x2*m1[2] + A0x3*m1[3];
  ret[2] = A0x0*m2[0] + A0x1*m2[1] + A0x2*m2[2] + A0x3*m2[3];
  ret[3] = A0x0*m3[0] + A0x1*m3[1] + A0x2*m3[2] + A0x3*m3[3];
  return ret;
}

static void
SGDL_matadj(double ret[16], const double tabV[16]) {
  double a = tabV[0];
  double b = tabV[1];
  double c = tabV[2];
  double d = tabV[3];
  double e = tabV[4];
  double f = tabV[5];
  double g = tabV[6];
  double h = tabV[7];
  double i = tabV[8];
  double j = tabV[9];
  double k = tabV[10];
  double l = tabV[11];
  double m = tabV[12];
  double n = tabV[13];
  double o = tabV[14];
  double p = tabV[15];
  /* v0 */
  ret[0] =   f*k*p - f*l*o - j*g*p + j*h*o + n*g*l - n*h*k;
  ret[1] = - b*k*p + b*l*o + j*c*p - j*d*o - n*c*l + n*d*k;
  ret[2] = - b*h*o + b*g*p - f*c*p + f*d*o + n*c*h - n*d*g;
  ret[3] = - b*g*l + b*h*k + f*c*l - f*d*k - j*c*h + j*d*g;
  /* v1 */
  ret[4] = - e*k*p + e*l*o + i*g*p - i*h*o - m*g*l + m*h*k;
  ret[5] = - a*l*o + a*k*p + i*d*o - i*c*p - m*d*k + m*c*l;
  ret[6] = - a*g*p + a*h*o + e*c*p - e*d*o - m*c*h + m*d*g;
  ret[7] = - a*h*k + a*g*l + e*d*k - e*c*l - i*d*g + i*c*h;
  /* v2 */
  ret[8] = - e*l*n + e*j*p + i*h*n - i*f*p - m*h*j + m*f*l;
  ret[9] = - a*j*p + a*l*n + i*b*p - i*d*n - m*b*l + m*d*j;
  ret[10] = - a*h*n + a*f*p + e*d*n - e*b*p - m*d*f + m*b*h;
  ret[11] = - a*f*l + a*h*j + e*b*l - e*d*j - i*b*h + i*d*f;
  /* v3 */
  ret[12] = - e*j*o + e*k*n + i*f*o - i*g*n - m*f*k + m*g*j;
  ret[13] = - a*k*n + a*j*o + i*c*n - i*b*o - m*c*j + m*b*k;
  ret[14] = - a*f*o + a*g*n + e*b*o - e*c*n - m*b*g + m*c*f;
  ret[15] = - a*g*j + a*f*k + e*c*j - e*b*k - i*c*f + i*b*g;
}

bool
SGDL_isIdentity(const float mat[16]) {
	return
	mat[0]==1 && mat[1]==0 && mat[2]==0 && mat[3]==0 &&
	mat[4]==0 && mat[5]==1 && mat[6]==0 && mat[7]==0 &&
	mat[8]==0 && mat[9]==0 && mat[10]==1 && mat[11]==0 &&
	mat[12]==0 && mat[13]==0 && mat[14]==0 && mat[15]==1;
}

void
SGDL_matInv(float ret[16], const float tabV[16]) {
  float a = tabV[0];
  float b = tabV[1];
  float c = tabV[2];
  float d = tabV[3];
  float e = tabV[4];
  float f = tabV[5];
  float g = tabV[6];
  float h = tabV[7];
  float i = tabV[8];
  float j = tabV[9];
  float k = tabV[10];
  float l = tabV[11];
  float m = tabV[12];
  float n = tabV[13];
  float o = tabV[14];
  float p = tabV[15];
  float det = 1.0;
  /* v0 */
  ret[0] =   f*k*p - f*l*o - j*g*p + j*h*o + n*g*l - n*h*k;
  ret[1] = - b*k*p + b*l*o + j*c*p - j*d*o - n*c*l + n*d*k;
  ret[2] = - b*h*o + b*g*p - f*c*p + f*d*o + n*c*h - n*d*g;
  ret[3] = - b*g*l + b*h*k + f*c*l - f*d*k - j*c*h + j*d*g;
  /* v1 */
  ret[4] = - e*k*p + e*l*o + i*g*p - i*h*o - m*g*l + m*h*k;
  ret[5] = - a*l*o + a*k*p + i*d*o - i*c*p - m*d*k + m*c*l;
  ret[6] = - a*g*p + a*h*o + e*c*p - e*d*o - m*c*h + m*d*g;
  ret[7] = - a*h*k + a*g*l + e*d*k - e*c*l - i*d*g + i*c*h;
  /* v2 */
  ret[8] = - e*l*n + e*j*p + i*h*n - i*f*p - m*h*j + m*f*l;
  ret[9] = - a*j*p + a*l*n + i*b*p - i*d*n - m*b*l + m*d*j;
  ret[10] = - a*h*n + a*f*p + e*d*n - e*b*p - m*d*f + m*b*h;
  ret[11] = - a*f*l + a*h*j + e*b*l - e*d*j - i*b*h + i*d*f;
  /* v3 */
  ret[12] = - e*j*o + e*k*n + i*f*o - i*g*n - m*f*k + m*g*j;
  ret[13] = - a*k*n + a*j*o + i*c*n - i*b*o - m*c*j + m*b*k;
  ret[14] = - a*f*o + a*g*n + e*b*o - e*c*n - m*b*g + m*c*f;
  ret[15] = - a*g*j + a*f*k + e*c*j - e*b*k - i*c*f + i*b*g;

	if (a!=0) det = a*ret[0];
	else if (b!=0) det = b*ret[1];
	else if (c!=0) det = c*ret[2];
	else if (d!=0) det = d*ret[3];

	if (det!=0) {
  ret[0] /= det;
  ret[1] /= det;
  ret[2] /= det;
  ret[3] /= det;

  ret[4] /= det;
  ret[5] /= det;
  ret[6] /= det;
  ret[7] /= det;

  ret[8] /= det;
  ret[9] /= det;
  ret[10] /= det;
  ret[11] /= det;

  ret[12] /= det;
  ret[13] /= det;
  ret[14] /= det;
  ret[15] /= det;
	}
}

static double*
SGDL_z2_pro(double ret[10], const double v10z2[10], const double MatP[16]) {
  if (MatP) {
    const double* v16_0 = &MatP[0];
    const double* v16_1 = &MatP[4];
    const double* v16_2 = &MatP[8];
    const double* v16_3 = &MatP[12];

    double a00 = v16_0[0];
    double a01 = v16_0[1];
    double a02 = v16_0[2];
    double a03 = v16_0[3];
    /* =================== */
    double a10 = v16_1[0];
    double a11 = v16_1[1];
    double a12 = v16_1[2];
    double a13 = v16_1[3];
    /* =================== */
    double a20 = v16_2[0];
    double a21 = v16_2[1];
    double a22 = v16_2[2];
    double a23 = v16_2[3];
    /* =================== */
    double a30 = v16_3[0];
    double a31 = v16_3[1];
    double a32 = v16_3[2];
    double a33 = v16_3[3];

    double c0 = v10z2[0];
    double c1 = v10z2[1];
    double c2 = v10z2[2];
    double c3 = v10z2[3];
    double c4 = v10z2[4];
    double c5 = v10z2[5];
    double c6 = v10z2[6];
    double c7 = v10z2[7];
    double c8 = v10z2[8];
    double c9 = v10z2[9];

    double a0q0 = c0*a00 + c1*a01 + c3*a02 + c6*a03;
    double a0q1 =          c2*a01 + c4*a02 + c7*a03;
    double a0q2 =                   c5*a02 + c8*a03;
    double a0q3 =                            c9*a03;
    /* ===================== */
    double a1q0 = c0*a10 + c1*a11 + c3*a12 + c6*a13;
    double a1q1 =          c2*a11 + c4*a12 + c7*a13;
    double a1q2 =                   c5*a12 + c8*a13;
    double a1q3 =                            c9*a13;
    /* ===================== */
    double a2q0 = c0*a20 + c1*a21 + c3*a22 + c6*a23;
    double a2q1 =          c2*a21 + c4*a22 + c7*a23;
    double a2q2 =                   c5*a22 + c8*a23;
    double a2q3 =                            c9*a23;
    /* ===================== */
    double a3q0 = c0*a30 + c1*a31 + c3*a32 + c6*a33;
    double a3q1 =          c2*a31 + c4*a32 + c7*a33;
    double a3q2 =                   c5*a32 + c8*a33;
    double a3q3 =                            c9*a33;

    double* C_XiXj = ret;

    C_XiXj[0] = /* x0 */
      a00*a0q0 +
      a01*a0q1 + 
      a02*a0q2 + 
      a03*a0q3;
    C_XiXj[1] = /* x0.x1 */
      a10*a0q0 + a00*a1q0 + 
      a11*a0q1 + a01*a1q1 + 
      a12*a0q2 + a02*a1q2 +
      a13*a0q3 + a03*a1q3;
    C_XiXj[2] = /* x1 */
      a10*a1q0 +
      a11*a1q1 +
      a12*a1q2 +
      a13*a1q3;
    C_XiXj[3] = /* x0.x2 */
      a20*a0q0 + a00*a2q0 + 
      a21*a0q1 + a01*a2q1 + 
      a22*a0q2 + a02*a2q2 +
      a23*a0q3 + a03*a2q3;
    C_XiXj[4] = /* x1.x2 */
      a20*a1q0 + a10*a2q0 +
      a21*a1q1 + a11*a2q1 +
      a22*a1q2 + a12*a2q2 +
      a23*a1q3 + a13*a2q3;
    C_XiXj[5] = /* x2 */
      a20*a2q0 +
      a21*a2q1 +
      a22*a2q2 +
      a23*a2q3;
    C_XiXj[6] = /* x0.x3 */
      a30*a0q0 + a00*a3q0 +
      a31*a0q1 + a01*a3q1 +
      a32*a0q2 + a02*a3q2 +
      a33*a0q3 + a03*a3q3;
    C_XiXj[7] = /* x1.x3 */
      a30*a1q0 + a10*a3q0 +
      a31*a1q1 + a11*a3q1 +
      a32*a1q2 + a12*a3q2 +
      a33*a1q3 + a13*a3q3;
    C_XiXj[8] = /* x2.x3 */
      a30*a2q0 + a20*a3q0 +
      a31*a2q1 + a21*a3q1 +
      a32*a2q2 + a22*a3q2 +
      a33*a2q3 + a23*a3q3;
    C_XiXj[9] = /* x3 */
      a30*a3q0 +
      a31*a3q1 +
      a32*a3q2 +
      a33*a3q3;
  } else {
	  for (int i=0; i<10; ++i) ret[i] = v10z2[i];
  }
  return ret;
}

static void
SGDL_v4v4_in0_1(double ret[16], const double vec[16]) {
	double max = 0;
	int i;
	for (i=0; i<16; ++i) {
		if (fabs(vec[i])>max) max = fabs(vec[i]);
	}
	if (max>0) max = 1.0 / max;
	for (i=0; i<16; ++i) {
		ret[i] = vec[i]*max;
	}
}

static void
SGDL_v10_in0_1(double ret[10], const double vec[10]) {
	double max = 0;
	int i;
	for (i=0; i<9; ++i) {
		if (fabs(vec[i])>max) max = fabs(vec[i]);
	}
	if (max>0) max = 1.0 / max;
	for (i=0; i<10; ++i) {
		ret[i] = vec[i]*max;
	}
}

double*
SGDL_z2_coo(double ret[10], const double vec[24]) {
  double vec0[16];
  double B4[4];
  double B5[4];

  for (int i=0; i<16; ++i) vec0[i] = vec[i];
  SGDL_matadj(vec0, vec0);

  /*
   * vrifie que mat dfini bien un simplex (4 sommets non coplanaires), donc
   * l'adjointe pour tre valide ne doit pas avoir de ligne nulle
   */
  if (!SGDL_prevec(&vec0[0], 4) ||
      !SGDL_prevec(&vec0[4], 4) ||
      !SGDL_prevec(&vec0[8], 4) ||
      !SGDL_prevec(&vec0[12], 4)) {
    /*
     * on dgnre en une densit 0 : z2(x) < 0 quelque soit x
     *
     * seuls des points imaginaires appartiennent  la z2
     */
	ret[0] = ret[2] = ret[5] = ret[9] = -1;
	ret[1] = ret[3] = ret[4] = ret[6] = ret[7] = ret[8] = 0;
  } else {
    // conditionnement numerique pour vec0, puis vec[4] et vec[5]
    SGDL_v4v4_in0_1(vec0, vec0);
    SGDL_x__euc(B4, &vec[16]);
    SGDL_x__euc(B5, &vec[20]);
    
    SGDL_x__pro(B4, B4, vec0);
    SGDL_x__pro(B5, B5, vec0);

    double a =   B4[2]*B4[2] * B5[3]*B5[3];
    double b = - B4[0]*B4[1] * B5[3]*B5[3];
    double c = - B4[2]*B4[2] * B5[0]*B5[1];
    
    /* conditionnement et annule les coeffs. si DEGN */
    double max = fabs(a);
    double abs;
    
    if ((abs = fabs(b)) > max) max = abs;
    if ((abs = fabs(c)) > max) max = abs;
    
    if (max>0) {
	  const double EPSILON = 1e-15*max;
      if (fabs(a) < EPSILON) a = 0.0; else a /= max;
      if (fabs(b) < EPSILON) b = 0.0; else b /= max;
      if (fabs(c) < EPSILON) c = 0.0; else c /= max;

	  ret[0] = ret[2] = ret[3] = ret[4] = ret[6] = ret[7] = ret[8] = 0;
	  ret[1] = a;
	  ret[5] = b;
	  ret[9] = c;

      SGDL_z2_pro(ret, ret, vec0);
      SGDL_v10_in0_1(ret, ret);
    } else {
      /*
       * on dgnre en une densit 0 : z2(x) < 0 quelque soit x
       *
       * seuls des points imaginaires appartiennent  la z2
       */
      ret[0] = ret[2] = ret[5] = ret[9] = -1;
	  ret[1] = ret[3] = ret[4] = ret[6] = ret[7] = ret[8] = 0;
    }
  }

  return ret;
}
