
//	This software module was originally developed by James King (CRL), 
//  Leon Sucharov (CRL) and edited by Massimo Balestri (Telecom
//	Italia Lab S.p.A.) in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
////
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	CRL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 
//
//
//
//	Version developed by James King (CRL), Leon Sucharov (CRL) on behalf of MOSES
//	More information on MOSES can be found at http://www.crl.co.uk/projects/moses/


// IPMPToolES_DecoderConfig.cpp: implementation of the IPMPToolES_DecoderConfig class.
//
//////////////////////////////////////////////////////////////////////

#include "IPMPToolES_DecoderConfig.h"
#include <memory.h>

USING_MOSES_NAMESPACE

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IPMPToolES_DecoderConfig::IPMPToolES_DecoderConfig()
:DecoderSpecificInfo( )
{

}

IPMPToolES_DecoderConfig::~IPMPToolES_DecoderConfig()
{

}

void IPMPToolES_DecoderConfig::setIPMP_ToolID( unsigned char *toolID )
{
	memcpy( IPMP_ToolID, toolID, 16 ) ;
}

void IPMPToolES_DecoderConfig::setTool_Format_ID( unsigned int nFormat )
{
	Tool_Format_ID = nFormat ;
}

void IPMPToolES_DecoderConfig::setTool_Package_ID( unsigned short nID )
{
	Tool_Package_ID = nID ;
}

void IPMPToolES_DecoderConfig::getIPMP_ToolID( unsigned char *toolID )
{
	memcpy( toolID, IPMP_ToolID, 16 ) ;
}

unsigned int IPMPToolES_DecoderConfig::getTool_Format_ID( )
{
	return Tool_Format_ID ;
}

unsigned short IPMPToolES_DecoderConfig::getTool_Package_ID( )
{
	return Tool_Package_ID ;
}

int IPMPToolES_DecoderConfig::toBuffer( byte *buf )
{
	int nbits = BaseDescriptor::toBuffer( buf ) ;

	memcpy( buf + (nbits/8), IPMP_ToolID, 16 ) ;
	nbits += 128 ;

	put( Tool_Format_ID, buf, nbits, 32 ) ;
	nbits += 32 ;

	put( Tool_Package_ID, buf, nbits, 16 ) ;
	nbits += 16 ;

	return nbits ;
}

int IPMPToolES_DecoderConfig::fromBuffer( const byte* buf)
{
	int nbits = BaseDescriptor::fromBuffer( buf ) ;

	byte buffer[256] ;
	memcpy( buffer, buf, buf[1] ) ;

	memcpy( IPMP_ToolID, buf + (nbits/8), 16 ) ;
	nbits += 128 ;

	Tool_Format_ID = get( buf, nbits, 32 ) ;
	nbits += 32 ;

	Tool_Package_ID = (short) get( buf, nbits, 16 ) ;
	nbits += 16 ;

	return nbits ;
}

unsigned int IPMPToolES_DecoderConfig::sizeOfInstance() const
{
	return BaseDescriptor::sizeOfInstance() + 22 ;
}

IPMPToolES_DecoderConfig::IPMPToolES_DecoderConfig( const IPMPToolES_DecoderConfig& copy )
{
	memcpy( IPMP_ToolID, copy.IPMP_ToolID, 16 ) ;
	Tool_Format_ID = copy.Tool_Format_ID ;
	Tool_Package_ID = copy.Tool_Package_ID ;
}

IPMPToolES_DecoderConfig& IPMPToolES_DecoderConfig::operator= ( const IPMPToolES_DecoderConfig& copy )
{
	memcpy( IPMP_ToolID, copy.IPMP_ToolID, 16 ) ;
	Tool_Format_ID = copy.Tool_Format_ID ;
	Tool_Package_ID = copy.Tool_Package_ID ;
	return *this ;
}
