
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.), in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include "IPMP_ByteArray.h"
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <assert.h>

USING_MOSES_NAMESPACE


////////////////////////////////////////////////////////////
/////    class IPMP_ByteArray
////////////////////////////////////////////////////////////
IPMP_ByteArray::IPMP_ByteArray():
	ExpandableBaseClass(0)
{
	sizeOfArray = 0;
	data = NULL;
}

IPMP_ByteArray::~IPMP_ByteArray()
{
	if (data != NULL)
		delete[] data;
	data = NULL ;
}

IPMP_ByteArray::IPMP_ByteArray(const IPMP_ByteArray &ba):
	ExpandableBaseClass(ba)
{
	sizeOfArray = 0;
	data = NULL;
	setData(ba.getData(), ba.getDataSize());
}

IPMP_ByteArray & IPMP_ByteArray::operator= ( const IPMP_ByteArray & rval )
{
	setData(rval.getData(), rval.getDataSize());
	return *this ;
}


void IPMP_ByteArray::setData(const byte* datasource, unsigned int size)
{
	sizeOfArray = size;					// set new size
	if (data != NULL)
	{
		delete[] data;
		data = NULL;
	}
	
	if (sizeOfArray > 0)
	{	
		data = new byte[sizeOfArray];
		if (data != NULL)
			memcpy(data, datasource, sizeOfArray);
	}
}


byte * IPMP_ByteArray::getData() const
{
	return data;
}

unsigned int IPMP_ByteArray::getDataSize() const
{
	return sizeOfArray;
}


int IPMP_ByteArray::fromBuffer(const unsigned char *buf)
{
	int nbits = ExpandableBaseClass::fromBuffer(buf);	// read tag and size
	sizeOfArray = ExpandableBaseClass::getSizeOfInstance();
	setData(buf + nbits/8, sizeOfArray);					// read the data
	return (nbits + 8*sizeOfArray);					// return the num of bits
}

int IPMP_ByteArray::toBuffer(unsigned char *buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);	// write tag and size
	memcpy(buf + nbits/8, data, sizeOfArray);		// write the data
	return (nbits + 8*sizeOfArray);			// return the num of bits
}


unsigned int IPMP_ByteArray::sizeOfInstance() const
{
	return sizeOfArray;
}

