
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 



#ifndef __IPMP_BYTEARRAY__
#define __IPMP_BYTEARRAY__
#include "namespace.h"

#include "ExpandableBaseClass.h"

BEGIN_MOSES_NAMESPACE

/**
 * @class IPMP_ByteArray
 * This class stores an array of bytes of known size, and converts it into binary format.
 * @author Massimo Balestri
 * @date 2002
 */
class IPMP_ByteArray : public ExpandableBaseClass
{
private:
	unsigned int sizeOfArray;	// size of data
	byte* data;					// data

public:
	IPMP_ByteArray();											///< constructor
	~IPMP_ByteArray();											///< destructor
	IPMP_ByteArray(const IPMP_ByteArray &ba);					///< copy constructor
	IPMP_ByteArray & operator=(const IPMP_ByteArray &rval);		///< assignment operator

	virtual int fromBuffer(const unsigned char *buf);	
	virtual int toBuffer(unsigned char *buf);
	virtual unsigned int sizeOfInstance() const;


	/**
	 * Copies the content of the data paramenter into the byte array
	 * @param datasource the source buffer
	 * @param size the source size
	 */
	void setData(const byte* datasource, unsigned int size);

	/**
	 * Get access to the data stored in the byte array.
	 * @return a pointer to the byte array
	 */
	byte* getData() const;

	/**
	 * Get the size of data stored in the byte array.
	 * @return the data size in bytes.
	 */
	unsigned int getDataSize() const;
};

END_MOSES_NAMESPACE

#endif
