
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolDescriptor.cpp

#include "IPMP_Descriptor.h"
#include "IPMP_Data_BaseClass.h"
#include <assert.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>

USING_MOSES_NAMESPACE

IPMP_Descriptor::IPMP_Descriptor():
	BaseDescriptor(IPMP_DescrTag)			// set the correct tag
{
	init() ;
}	

IPMP_Descriptor::IPMP_Descriptor(const IPMP_Descriptor & descriptor):	// copy constructor
	BaseDescriptor(descriptor)
{
	init();
	switch (descriptor.getType())
	{
		case HOOKS_TYPE:
		{
			bool b = setHooksType(
				(byte) descriptor.getIPMP_DescriptorID(), 
				descriptor.getIPMPS_Type(),
				descriptor.getIPMP_data(),
				descriptor.getSizeOfData());
			assert(b);
			break;
		}
		case URL_TYPE:
		{
			bool b = setUrlType((byte) descriptor.getIPMP_DescriptorID(), descriptor.getURLString());
			assert(b);
			break;		
		}
		case IPMPX_TYPE:
		{
			bool b = setIPMPXType(
				descriptor.getIPMP_DescriptorID(),
				descriptor.getToolID(),
				descriptor.getControlPointCode(),
				descriptor.getSequenceCode());
			
			assert(b);
			
			for(int x=0;x < descriptor.getSizeOfData(); x++ )
			{
				bool b = addIPMPXMessage( *descriptor.getIPMPXMessage(x) );
				assert(b);
			}

			break;
		}
	}	// end switch
}

IPMP_Descriptor::~IPMP_Descriptor() 
{
	cleanUp() ;
}

void IPMP_Descriptor::init()
{
	descrType = IPMPX_TYPE;					// default is IPMPX, not hooks
	IPMP_ToolId = new byte[16];
	IPMP_DescriptorID = 0 ;
	IPMPS_Type = 0;	
	memset(IPMP_ToolId, 0, sizeof(IPMP_ToolId));
	controlPointCode = 0 ;
	sequenceCode = 0 ;
	IPMPX_data = new IPMP_Data_BaseClass * [MAX_IPMPX_DATA_MESSAGES] ;
	
	for (int i=0; i<MAX_IPMPX_DATA_MESSAGES; i++)
		IPMPX_data[i] = NULL;

	URLString = NULL ;
	IPMP_data = NULL ;
	sizeOfData = 0 ;
}

void IPMP_Descriptor::cleanUp() 
{
	delete[] IPMP_ToolId;

	if (URLString != NULL) 
		delete[] URLString ;

	if (IPMP_data != NULL)
		delete[] IPMP_data ;
	
	if (IPMPX_data != NULL)
	{
		for (unsigned int i=0;i<sizeOfData;i++)
			delete IPMPX_data[i];

		delete[] IPMPX_data ;
	}
}


int IPMP_Descriptor::fromBuffer(const byte * buf)
{
	BaseDescriptor::fromBuffer(buf);		// read the base descriptor and set "buf" as read buffer for this class.
	cleanUp() ;	
	init() ;

	descrType = HOOKS_TYPE;
	IPMP_DescriptorID = (byte) get(8) ;					// 8 bits
	IPMPS_Type = get(16 ) ;								// 16 bits

	if( IPMP_DescriptorID == 0xff && IPMPS_Type == 0xffff )
	{
		descrType = IPMPX_TYPE;
		IPMP_DescriptorID = get( 16 ) ;					// 16 bits
		getBuffer(IPMP_ToolId, 128 ) ;					// 128 bits
		controlPointCode = get( 8 ) ;					// 8 bits

		if( controlPointCode )
		{
			sequenceCode = get( 8 );
		}

		sizeOfData=0;
		int buffersize = getSizeOfInstance() - 3;
		while( (getReadCounter() / 8) < buffersize )
		{
			unsigned int nbit;
			IPMPX_data[sizeOfData++] = IPMP_Data_BaseClass::createMessage(getReadPtr(), nbit);
			skipReading( nbit );
		}
	}
	else if( IPMPS_Type == 0 )
	{
		descrType = URL_TYPE;
		sizeOfData = getSizeOfInstance() - 3;
		URLString = new char[sizeOfData];
		getBuffer(URLString, sizeOfData * 8);
	}
	else
	{
		descrType = HOOKS_TYPE;
		sizeOfData = getSizeOfInstance() - 3;
		IPMP_data = new byte[sizeOfData];
		getBuffer(IPMP_data,  sizeOfData * 8);
	}
	return getReadCounter();				// return the number of read bits
}


int IPMP_Descriptor::toBuffer(byte * buf)
{
	BaseDescriptor::toBuffer(buf);		// read the base descriptor and set "buf" as read buffer for this class.

	if (descrType == IPMPX_TYPE)
	{ 
		put(0xFF, 8);			// IPMP_DescriptorID (escape for IPMPX)
		put(0xFFFF, 16);		// IPMPS_TYPE (escape for IPMPX)
	}
	else
	{
		put( IPMP_DescriptorID, 8 ) ;				// 8 bits
		put( IPMPS_Type, 16 ) ;						// 16 bits
	}

	if (descrType == IPMPX_TYPE)
	{
		put( IPMP_DescriptorID, 16 ) ;				// 16 bits
		putBuffer(IPMP_ToolId, 128 ) ;				// 128 bits
		put( controlPointCode, 8 ) ;
		if( controlPointCode )
		{
			put( sequenceCode, 8 ) ;
		}
		
		for( unsigned int x=0;x<sizeOfData;x++ )
		{
			skipWriting( IPMPX_data[x]->toBuffer( getWritePtr() )) ;
		}
	} 
	else if( IPMPS_Type == 0 )
	{
		putBuffer(URLString, 8 * (strlen(URLString) + 1));
	} 
	else
	{
		putBuffer(IPMP_data, 8 * sizeOfData);
	}
	return getWriteCounter();				// return the number of read bits
}

byte * IPMP_Descriptor::getIPMP_data() const
{
	return IPMP_data;
}

char * IPMP_Descriptor::getURLString( ) const
{
	return URLString;
}

int IPMP_Descriptor::getSizeOfData() const
{
	return sizeOfData ;
}

byte IPMP_Descriptor::getSequenceCode() const
{
	return (byte) sequenceCode ;
}

byte IPMP_Descriptor::getControlPointCode() const
{
	return (byte) controlPointCode ;
}

byte * IPMP_Descriptor::getToolID() const
{
	return IPMP_ToolId;
}

unsigned short IPMP_Descriptor::getIPMP_DescriptorID() const
{
	return (unsigned short) IPMP_DescriptorID ;	
}

unsigned short IPMP_Descriptor::getIPMPS_Type( ) const
{
	return (unsigned short) IPMPS_Type ;	
}

unsigned int IPMP_Descriptor::sizeOfInstance() const
{
	unsigned int size = BaseDescriptor::sizeOfInstance() ;
	size += 3 ;

	if (descrType == IPMPX_TYPE)
	{
		size += 2 ;
		size += 16 ;
		size += 1 ;
		if( controlPointCode )
		{
			size += 1 ;
		}
		
		for( unsigned int x=0;x<sizeOfData;x++ )
		{
			size += IPMPX_data[x]->getSize();
		}
	}
	else if( IPMPS_Type == 0 )
	{
		size += (strlen(URLString) + 1);
	}
	else
	{
		size += sizeOfData ;
	}

	return size;
}

// copy the message into the local storage
bool IPMP_Descriptor::addIPMPXMessage(IPMP_Data_BaseClass &msg )
{
	if (descrType != IPMPX_TYPE)
		return false;

	if( sizeOfData < MAX_IPMPX_DATA_MESSAGES )
	{
		IPMPX_data[ sizeOfData++ ] = IPMP_Data_BaseClass::createMessage(msg);		// copy the message
		return true ;
	}
	else
		return false ;
}

IPMP_Data_BaseClass* IPMP_Descriptor::getIPMPXMessage( unsigned int nIndex ) const
{
	if (descrType != IPMPX_TYPE)
		return NULL;

	if( nIndex < sizeOfData )
	{
		return IPMPX_data[ nIndex ] ;
	}
	else
		return NULL;
}

bool IPMP_Descriptor::setHooksType(byte descrId, unsigned short mytype, byte * ipmpdata, int ipmpdatasize)
{
	// check param values
	if (mytype < 0x2000)
		return false;				// cannot be zero (=URL) or reserved values (1..0x2000)

	descrType = HOOKS_TYPE;
	IPMP_DescriptorID = descrId;
	IPMPS_Type = mytype;

	if( IPMP_data )
		delete[] IPMP_data ;
	IPMP_data = new byte[ ipmpdatasize ] ;
	memcpy( IPMP_data, ipmpdata, ipmpdatasize) ;	
	sizeOfData = ipmpdatasize;
	return true;
}

bool IPMP_Descriptor::setUrlType(byte descrId, const char *URL)
{
	if (URL == NULL)				// check if the URL exists
		return false;

	size_t size = strlen(URL) + 1;
	if (size > 2048)				// check if the URL size is reasonable
		return false;				// return without changing anything

	sizeOfData = size;
	descrType = URL_TYPE;
	IPMP_DescriptorID = descrId;
	IPMPS_Type = 0x0000;

	if( URLString )
		delete[] URLString ;

	URLString = new char[ sizeOfData ] ;
	strncpy( URLString, URL, sizeOfData ) ;
	return true;
}

bool IPMP_Descriptor::setIPMPXType(unsigned short descrId, byte *toolID, byte cpCode, byte sc)
{
	if ((descrId == 0) || (descrId == 0xFFFF))
		return false;								// forbidden

	descrType = IPMPX_TYPE;
	IPMP_DescriptorID = descrId;
	memcpy(IPMP_ToolId, toolID, 16) ;
	controlPointCode = cpCode;
	sequenceCode = sc;
	IPMPS_Type = 0xFFFF;
	return true;
}



IPMP_Descriptor::DescrTypeEnum IPMP_Descriptor::getType() const
{
	return descrType;
}
