
//	This software module was originally developed by ADETTI in the course of
//  development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_MutualAuthentication.cpp

#include "IPMP_MutualAuthentication.h"

#include <assert.h>
#include <string.h>

/////////////////////////////////////////////////////////////////////////////
//
// IPMP_MutualAuthentication
//

USING_MOSES_NAMESPACE

IPMP_MutualAuthentication::IPMP_MutualAuthentication()
	:	IPMP_Data_BaseClass(IPMP_MutualAuthentication_tag)
{
	requestNegotiation = false;
	successNegotiation = false;
	failedNegotiation = false;
	inclAuthenticationData = false;
	inclAuthCodes = false;
	nCandidate = 0;
	ppCandidateAlgorithms = 0;
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;
	authenticationDataSize = 0;
	pAuthenticationData = 0;
	pAuthCodes = 0;
}

IPMP_MutualAuthentication::~IPMP_MutualAuthentication()
{
	clear();
}

IPMP_MutualAuthentication::IPMP_MutualAuthentication(const IPMP_MutualAuthentication & ma)
:	IPMP_Data_BaseClass(ma)
{
	requestNegotiation = ma.requestNegotiation;		// copy flags
	successNegotiation = ma.successNegotiation;
	failedNegotiation = ma.failedNegotiation;
	inclAuthenticationData = ma.inclAuthenticationData;
	inclAuthCodes = ma.inclAuthCodes;
	nCandidate = 0;
	ppCandidateAlgorithms = 0;
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;
	authenticationDataSize = 0;
	pAuthenticationData = 0;
	pAuthCodes = 0;

	if (requestNegotiation)
	{
		setRequestNegotiation(ma.getCandidateAlgorithms(), ma.getNumCandidateAlgorithms());
	}
	if (successNegotiation)
	{
		setSuccessNegotiation(ma.getAgreedAlgorithms(), ma.getNumAgreedAlgorithms());
	}
	if (inclAuthenticationData)
	{
		setAuthenticationData(ma.getAuthenticationData(), ma.getAuthenticationDataSize());
	}
	if (inclAuthCodes)
	{
		setAuthCodes( ma.getAuthCodes());
	}
}


void IPMP_MutualAuthentication::clear()
{
	requestNegotiation = false;
	successNegotiation = false;
	failedNegotiation = false;
	inclAuthenticationData = false;
	inclAuthCodes = false;
	if (ppCandidateAlgorithms != 0) {
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			delete ppCandidateAlgorithms[nc];
		}
		delete [] ppCandidateAlgorithms;
		ppCandidateAlgorithms = 0;
	}
	nCandidate = 0;
	if (ppAgreedAlgorithms != 0) {
		for (word32 na = 0; na < nAlgorithms; ++na) {
			delete ppAgreedAlgorithms[na];
		}
		delete [] ppAgreedAlgorithms;
		ppAgreedAlgorithms = 0;
	}
	nAlgorithms = 0;
	delete [] pAuthenticationData;
	pAuthenticationData = 0;
	authenticationDataSize = 0;
	delete [] pAuthCodes;
	pAuthCodes = 0;
}

int IPMP_MutualAuthentication::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	assert((nbits % 8) == 0);

	byte * mabuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	put(requestNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(successNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(failedNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(inclAuthenticationData, mabuf, bitindex, 1);
	bitindex += 1;
	put(inclAuthCodes, mabuf, bitindex, 1);
	bitindex += 1;

	// reserved
	put(0, mabuf, bitindex, 3);
	bitindex += 3;

	if (requestNegotiation) {
		assert(nCandidate == 0 || ppCandidateAlgorithms != 0);
		// nCandidate
		put(nCandidate, mabuf, bitindex, 32);
		bitindex += 32;
		// candidate algorithms
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			assert(ppCandidateAlgorithms[nc] != 0);
			bitindex += ppCandidateAlgorithms[nc]->toBuffer(&mabuf[bitindex / 8]);
		}
	}

	if (successNegotiation) {
		assert(nAlgorithms == 0 || ppAgreedAlgorithms != 0);
		// nAlgorithms
		put(nAlgorithms, mabuf, bitindex, 32);
		bitindex += 32;
		// agreed algorithms
		for (word32 na = 0; na < nAlgorithms; ++na) {
			assert(ppAgreedAlgorithms[na] != 0);
			bitindex += ppAgreedAlgorithms[na]->toBuffer(&mabuf[bitindex / 8]);
		}
	}

	if (inclAuthenticationData) {
		assert(authenticationDataSize == 0 || pAuthenticationData != 0);
		put(authenticationDataSize, mabuf, bitindex, 32);
		bitindex += 32;
		for (word32 ad_idx = 0; ad_idx < authenticationDataSize; ++ad_idx) {
			put(pAuthenticationData[ad_idx], mabuf, bitindex, 8);
			bitindex += 8;
		}
	}

	if (inclAuthCodes) {
		assert(pAuthCodes != 0);
		bitindex += pAuthCodes->toBuffer(&mabuf[bitindex / 8]);
	}

	return (nbits + bitindex);
}

int IPMP_MutualAuthentication::fromBuffer(const byte * buf)
{
	clear();

	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	assert((nbits % 8) == 0);
	
	const byte * mabuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	requestNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	successNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	failedNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	inclAuthenticationData = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	inclAuthCodes = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;

	// reserved
	get(mabuf, bitindex, 3);
	bitindex += 3;

	if (requestNegotiation) {
		// nCandidate
		nCandidate = get(mabuf, bitindex, 32);
		bitindex += 32;
		// candidate algorithms
		if (nCandidate > 0) {
			ppCandidateAlgorithms = new AlgorithmDescriptor *[nCandidate];
			assert(ppCandidateAlgorithms != 0);
			for (word32 nc = 0; nc < nCandidate; ++nc) {
				ppCandidateAlgorithms[nc] = new AlgorithmDescriptor;
				assert(ppCandidateAlgorithms[nc] != 0);
				bitindex += ppCandidateAlgorithms[nc]->fromBuffer(&mabuf[bitindex / 8]);
			}
		}
	}

	if (successNegotiation) {
		// nAlgorithms
		nAlgorithms = get(mabuf, bitindex, 32);
		bitindex += 32;
		// agreed algorithms
		if (nAlgorithms > 0) {
			ppAgreedAlgorithms = new AlgorithmDescriptor *[nAlgorithms];
			assert(ppAgreedAlgorithms != 0);
			for (word32 na = 0; na < nAlgorithms; ++na) {
				ppAgreedAlgorithms[na] = new AlgorithmDescriptor;
				assert(ppAgreedAlgorithms[na] != 0);
				bitindex += ppAgreedAlgorithms[na]->fromBuffer(&mabuf[bitindex / 8]);
			}
		}
	}

	if (inclAuthenticationData) {
		authenticationDataSize = get(mabuf, bitindex, 32);
		bitindex += 32;
		if (authenticationDataSize > 0) {
			pAuthenticationData = new byte[authenticationDataSize];
			assert(pAuthenticationData != 0);
			for (word32 ad_idx = 0; ad_idx < authenticationDataSize; ++ad_idx) {
				pAuthenticationData[ad_idx] = (byte)get(mabuf, bitindex, 8);
				bitindex += 8;
			}
		}
	}

	if (inclAuthCodes) {
		pAuthCodes = new AuthCodes;
		assert(pAuthCodes != 0);
		bitindex += pAuthCodes->fromBuffer(&mabuf[bitindex / 8]);
	}

	return (nbits + bitindex);
}


// compute size, in BYTES!
unsigned int IPMP_MutualAuthentication::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();
	size += 1; // control bits
	if (requestNegotiation) {
		assert(nCandidate == 0 || ppCandidateAlgorithms != 0);
		size += 4; // nCandidate
		// candidate algorithms
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			assert(ppCandidateAlgorithms[nc] != 0);
			size += ppCandidateAlgorithms[nc]->getSize();
		}
	}
	if (successNegotiation) {
		assert(nAlgorithms == 0 || ppAgreedAlgorithms != 0);
		size += 4; // nAlgorithms
		// candidate algorithms
		for (word32 nc = 0; nc < nAlgorithms; ++nc) {
			assert(ppAgreedAlgorithms[nc] != 0);
			size += ppAgreedAlgorithms[nc]->getSize();
		}
	}
	if (inclAuthenticationData) {
		assert(authenticationDataSize == 0 || pAuthenticationData != 0);
		size += 4;
		size += authenticationDataSize;
	}
	if (inclAuthCodes) {
		assert(pAuthCodes != 0);
		size += pAuthCodes->getSize();
	}
	return size;
}

bool IPMP_MutualAuthentication::getRequestNegotiation() const
{
	return requestNegotiation;
}

void IPMP_MutualAuthentication::setRequestNegotiation(AlgorithmDescriptor * * ppCandidateAlgorithms, word32 nCandidate)
{
	word32 nc;
	
	if (this->ppCandidateAlgorithms != 0) {
		for (nc = 0; nc < this->nCandidate; ++nc) {
			delete this->ppCandidateAlgorithms[nc];
		}
		delete [] this->ppCandidateAlgorithms;
	}

	if (ppCandidateAlgorithms != 0 && nCandidate > 0) {

		this->nCandidate = nCandidate;
		this->ppCandidateAlgorithms = new AlgorithmDescriptor *[nCandidate];
		
		AlgorithmDescriptor * pAD;
		for (nc = 0; nc < nCandidate; ++nc) {
			if (ppCandidateAlgorithms[nc] != 0) {
				pAD = this->ppCandidateAlgorithms[nc] = new AlgorithmDescriptor(*(ppCandidateAlgorithms[nc]));
			} else {
				pAD = this->ppCandidateAlgorithms[nc] = 0;
			}
		}

	} else {
		this->nCandidate = 0;
		this->ppCandidateAlgorithms = 0;
	}

	requestNegotiation = true;
}

void IPMP_MutualAuthentication::clearRequestNegotiation()
{
	word32 nc;
	
	if (ppCandidateAlgorithms != 0) {
		for (nc = 0; nc < nCandidate; ++nc) {
			delete ppCandidateAlgorithms[nc];
		}
		delete [] ppCandidateAlgorithms;
	}

	nCandidate = 0;
	ppCandidateAlgorithms = 0;

	requestNegotiation = false;
}

word32 IPMP_MutualAuthentication::getNumCandidateAlgorithms() const
{
	return nCandidate;
}

AlgorithmDescriptor * * IPMP_MutualAuthentication::getCandidateAlgorithms() const
{
	return ppCandidateAlgorithms;
}


bool IPMP_MutualAuthentication::getSuccessNegotiation() const
{
	return successNegotiation;
}

void IPMP_MutualAuthentication::setSuccessNegotiation(AlgorithmDescriptor * * ppAgreedAlgorithms, word32 nAlgorithms)
{
	word32 na;
	
	if (this->ppAgreedAlgorithms != 0) {
		for (na = 0; na < this->nAlgorithms; ++na) {
			delete this->ppAgreedAlgorithms[na];
		}
		delete [] this->ppAgreedAlgorithms;
	}
	
	if (ppAgreedAlgorithms != 0) {

		this->nAlgorithms = nAlgorithms;
		this->ppAgreedAlgorithms = new AlgorithmDescriptor *[nAlgorithms];
		
		AlgorithmDescriptor * pAD;
		for (na = 0; na < nAlgorithms; ++na) {
			if (ppAgreedAlgorithms[na] != 0) {
				pAD = this->ppAgreedAlgorithms[na] = new AlgorithmDescriptor(*(ppAgreedAlgorithms[na]));
			} else {
				pAD = this->ppAgreedAlgorithms[na] = 0;
			}
		}

	} else {
		this->nAlgorithms = 0;
		this->ppAgreedAlgorithms = 0;
	}

	successNegotiation = true;
}

void IPMP_MutualAuthentication::clearSuccessNegotiation()
{
	word32 na;
	
	if (ppAgreedAlgorithms != 0) {
		for (na = 0; na < nAlgorithms; ++na) {
			delete ppAgreedAlgorithms[na];
		}
		delete [] ppAgreedAlgorithms;
	}
	
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;

	successNegotiation = false;
}

word32 IPMP_MutualAuthentication::getNumAgreedAlgorithms() const
{
	return nAlgorithms;
}

AlgorithmDescriptor * * IPMP_MutualAuthentication::getAgreedAlgorithms() const
{
	return ppAgreedAlgorithms;
}

bool IPMP_MutualAuthentication::getFailedNegotiation() const
{
	return failedNegotiation;
}

void IPMP_MutualAuthentication::setFailedNegotiation()
{
	failedNegotiation = true;
}

void IPMP_MutualAuthentication::clearFailedNegotiation()
{
	failedNegotiation = false;
}

bool IPMP_MutualAuthentication::hasAuthenticationData() const
{
	return inclAuthenticationData;
}

void IPMP_MutualAuthentication::setAuthenticationData(const byte * pAuthenticationData, word32 authenticationDataSize)
{
	delete [] this->pAuthenticationData;

	this->pAuthenticationData = new byte[authenticationDataSize];
	assert(this->pAuthenticationData != 0);
	this->authenticationDataSize = authenticationDataSize;
	memcpy(this->pAuthenticationData, pAuthenticationData, authenticationDataSize);

	inclAuthenticationData = true;
}

void IPMP_MutualAuthentication::clearAuthenticationData()
{
	delete [] this->pAuthenticationData;
	this->pAuthenticationData = 0;
	authenticationDataSize = 0;

	inclAuthenticationData = true;
}

byte * IPMP_MutualAuthentication::getAuthenticationData() const
{
	return pAuthenticationData;
}

word32 IPMP_MutualAuthentication::getAuthenticationDataSize() const
{
	return authenticationDataSize;
}


bool IPMP_MutualAuthentication::hasAuthCodes() const
{
	return inclAuthCodes;
}

void IPMP_MutualAuthentication::setAuthCodes(AuthCodes * pAuthCodes)
{
	delete this->pAuthCodes;

	if (pAuthCodes != 0) {
		this->pAuthCodes = new AuthCodes(*pAuthCodes);
	} else {
		this->pAuthCodes = 0;
	}

	inclAuthCodes = true;
}

AuthCodes * IPMP_MutualAuthentication::getAuthCodes() const
{
	return pAuthCodes;
}

/////////////////////////////////////////////////////////////////////////////
//
// BaseAuthenticationDescriptor
//

#ifdef IPMP_AlgorithmDescr_tag
#undef IPMP_AlgorithmDescr_tag
#endif
#define IPMP_AlgorithmDescr_tag 0x01

#ifdef IPMP_KeyDescr_tag
#undef IPMP_KeyDescr_tag
#endif
#define IPMP_KeyDescr_tag 0x02

BaseAuthenticationDescriptor::BaseAuthenticationDescriptor(byte tag)
	: ExpandableBaseClass(tag)
{

}

BaseAuthenticationDescriptor::~BaseAuthenticationDescriptor()
{

}

void BaseAuthenticationDescriptor::clear()
{

}

int BaseAuthenticationDescriptor::toBuffer(byte * buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);

	return nbits;
}

int BaseAuthenticationDescriptor::fromBuffer(const byte * buf)
{
	int nbits = ExpandableBaseClass::fromBuffer(buf);

	return nbits;
}

unsigned int BaseAuthenticationDescriptor::sizeOfInstance() const
{
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//
// AlgorithmDescriptor
//

AlgorithmDescriptor::AlgorithmDescriptor(byte tag)
	: BaseAuthenticationDescriptor(tag)
{
	isRegistered = false;
	regAuthId = 0;
	specAlgoIdSize = 0;
	pSpecAlgoId = 0;
	opaqueDataSize = 0;
	pOpaqueData = 0;
}

AlgorithmDescriptor::AlgorithmDescriptor()
	: BaseAuthenticationDescriptor(IPMP_AlgorithmDescr_tag)
{
	isRegistered = false;
	regAuthId = 0;
	specAlgoIdSize = 0;
	pSpecAlgoId = 0;
	opaqueDataSize = 0;
	pOpaqueData = 0;
}

AlgorithmDescriptor::~AlgorithmDescriptor()
{
	clear();
}

// clear all internal data
void AlgorithmDescriptor::clear()
{
	BaseAuthenticationDescriptor::clear();

	isRegistered = false;
	regAuthId = 0;
	specAlgoIdSize = 0;
	delete [] pSpecAlgoId;
	pSpecAlgoId = 0;
	opaqueDataSize = 0;
	delete [] pOpaqueData;
	pOpaqueData = 0;
}

AlgorithmDescriptor::AlgorithmDescriptor(const AlgorithmDescriptor & ad)
	: BaseAuthenticationDescriptor(const_cast<AlgorithmDescriptor &>(ad).getObjectId())
{
	isRegistered = ad.isRegistered;
	regAuthId = ad.regAuthId;
	if (ad.pSpecAlgoId != 0 && ad.specAlgoIdSize > 0) {
		pSpecAlgoId = new byte[ad.specAlgoIdSize];
		memcpy(pSpecAlgoId, ad.pSpecAlgoId, ad.specAlgoIdSize);
		specAlgoIdSize = ad.specAlgoIdSize;
	} else {
		pSpecAlgoId = 0;
		specAlgoIdSize = 0;
	}
	if (ad.pOpaqueData != 0 && ad.opaqueDataSize > 0) {
		pOpaqueData = new byte[ad.opaqueDataSize];
		memcpy(pOpaqueData, ad.pOpaqueData, ad.opaqueDataSize);
		opaqueDataSize = ad.opaqueDataSize;
	} else {
		pOpaqueData = 0;
		opaqueDataSize = 0;
	}
}

bool AlgorithmDescriptor::operator==(const AlgorithmDescriptor & a_ad) const
{
	return equals(a_ad);
}

bool AlgorithmDescriptor::equals(const AlgorithmDescriptor & a_ad) const
{
	if (&a_ad == this) return true;
	if (a_ad.isRegistered == isRegistered) {
		if (isRegistered) {
			if (a_ad.regAuthId == regAuthId) {
				return true;
			}
		} else {
			if (a_ad.specAlgoIdSize == specAlgoIdSize) {
				if (memcmp(a_ad.pSpecAlgoId, pSpecAlgoId, specAlgoIdSize) == 0) {
					return true;
				}
			}
		}
	}
	return false;
}

int AlgorithmDescriptor::toBuffer(byte * buf)
{
	int nbits = BaseAuthenticationDescriptor::toBuffer(buf);
	assert((nbits % 8) == 0);

	byte * adbuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	put(isRegistered, adbuf, bitindex, 1);
	bitindex += 1;

	// reserved
	put(0, adbuf, bitindex, 7);
	bitindex += 7;

	if (isRegistered) {
		// regAuthId
		put(regAuthId, adbuf, bitindex, 16);
		bitindex += 16;
	} else {
		// specAlgoId
		assert(specAlgoIdSize == 0 || pSpecAlgoId != 0);
		put(specAlgoIdSize, adbuf, bitindex, 32);
		bitindex += 32;
		for (word32 sai_idx = 0; sai_idx < specAlgoIdSize; ++sai_idx) {
			put(pSpecAlgoId[sai_idx], adbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	// opaqueData
	assert(opaqueDataSize == 0 || pOpaqueData != 0);
	put(opaqueDataSize, adbuf, bitindex, 32);
	bitindex += 32;
	for (word32 od_idx = 0; od_idx < opaqueDataSize; ++od_idx) {
		put(pOpaqueData[od_idx], adbuf, bitindex, 8);
		bitindex += 8;
	}

	return (nbits + bitindex);
}

int AlgorithmDescriptor::fromBuffer(const byte * buf)
{
	clear();

	int nbits = BaseAuthenticationDescriptor::fromBuffer(buf);
	assert((nbits % 8) == 0);
	
	const byte * adbuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	isRegistered = (get(adbuf, bitindex, 1) != 0);
	bitindex += 1;

	// reserved
	get(adbuf, bitindex, 7);
	bitindex += 7;

	if (isRegistered) {
		// regAuthId
		regAuthId = get(adbuf, bitindex, 16);
		bitindex += 16;
	} else {
		// specAlgoId
		specAlgoIdSize = get(adbuf, bitindex, 32);
		bitindex += 32;
		if (specAlgoIdSize > 0) {
			pSpecAlgoId = new byte[specAlgoIdSize];
			assert(pSpecAlgoId != 0);
			for (word32 sai_idx = 0; sai_idx < specAlgoIdSize; ++sai_idx) {
				pSpecAlgoId[sai_idx] = (byte)get(adbuf, bitindex, 8);
				bitindex += 8;
			}
		}
	}

	// opaqueData
	opaqueDataSize = get(adbuf, bitindex, 32);
	bitindex += 32;
	if (opaqueDataSize > 0) {
		pOpaqueData = new byte[opaqueDataSize];
		assert(pOpaqueData != 0);
		for (word32 od_idx = 0; od_idx < opaqueDataSize; ++od_idx) {
			pOpaqueData[od_idx] = (byte)get(adbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	return (nbits + bitindex);
}


unsigned int AlgorithmDescriptor::sizeOfInstance() const
{
	unsigned int size = BaseAuthenticationDescriptor::sizeOfInstance();

	// control bits
	size += 1;

	if (isRegistered) {
		// regAuthId
		size += 2;
	} else {
		// specAlgoId
		assert(specAlgoIdSize == 0 || pSpecAlgoId != 0);
		size += 4;
		size += specAlgoIdSize;
	}

	// opaqueData
	assert(opaqueDataSize == 0 || pOpaqueData != 0);
	size += 4;
	size += opaqueDataSize;

	return size;
}

bool AlgorithmDescriptor::getIsRegistered() const
{
	return isRegistered;
}

void AlgorithmDescriptor::setRegAuthId(word32 regAuthId)
{
	delete [] pSpecAlgoId;
	pSpecAlgoId = 0;
	specAlgoIdSize = 0;

	this->regAuthId = regAuthId;

	isRegistered = true;
}

word32 AlgorithmDescriptor::getRegAuthId() const
{
	return regAuthId;
}

void AlgorithmDescriptor::setSpecAlgoId(const byte * pSpecAlgoId, word32 specAlgoIdSize)
{
	regAuthId = 0;

	delete [] this->pSpecAlgoId;
	if (pSpecAlgoId != 0 && specAlgoIdSize > 0) {
		this->pSpecAlgoId = new byte[specAlgoIdSize];
		assert(this->pSpecAlgoId != 0);
		memcpy(this->pSpecAlgoId, pSpecAlgoId, specAlgoIdSize);
		this->specAlgoIdSize = specAlgoIdSize;
	} else {
		this->pSpecAlgoId = 0;
		this->specAlgoIdSize = 0;
	}

	isRegistered = false;
}

word32 AlgorithmDescriptor::getSpecAlgoIdSize() const
{
	return specAlgoIdSize;
}

byte * AlgorithmDescriptor::getSpecAlgoId() const
{
	return pSpecAlgoId;
}

void AlgorithmDescriptor::setOpaqueData(const byte * pOpaqueData, word32 opaqueDataSize)
{
	delete [] this->pOpaqueData;
	if (pOpaqueData != 0 && opaqueDataSize > 0) {
		this->pOpaqueData = new byte[opaqueDataSize];
		assert(this->pOpaqueData != 0);
		memcpy(this->pOpaqueData, pOpaqueData, opaqueDataSize);
		this->opaqueDataSize = opaqueDataSize;
	} else {
		this->pOpaqueData = 0;
		this->opaqueDataSize = 0;
	}
}

void AlgorithmDescriptor::clearOpaqueData()
{
	delete [] pOpaqueData;
	pOpaqueData = 0;
	opaqueDataSize = 0;
}

word32 AlgorithmDescriptor::getOpaqueDataSize() const
{
	return opaqueDataSize;
}

byte * AlgorithmDescriptor::getOpaqueData() const
{
	return pOpaqueData;
}
	
/////////////////////////////////////////////////////////////////////////////
//
// KeyDescriptor
//

KeyDescriptor::KeyDescriptor(byte tag)
	: AlgorithmDescriptor(tag)
{
	keyBodySize = 0;
	pKeyBody = 0;
}

KeyDescriptor::KeyDescriptor()
	: AlgorithmDescriptor(IPMP_KeyDescr_tag)
{
	keyBodySize = 0;
	pKeyBody = 0;
}

KeyDescriptor::KeyDescriptor(const KeyDescriptor & kd)
	: AlgorithmDescriptor(kd)
{
	if (kd.keyBodySize > 0 && pKeyBody != 0) {
		keyBodySize = kd.keyBodySize;
		pKeyBody = new byte[keyBodySize];
		memcpy(pKeyBody, kd.pKeyBody, keyBodySize);
	} else {
		keyBodySize = 0;
		pKeyBody = 0;
	}
}

KeyDescriptor::~KeyDescriptor()
{
	clear();
}

void KeyDescriptor::clear()
{
	AlgorithmDescriptor::clear();

	delete [] pKeyBody;
	pKeyBody = 0;
	keyBodySize = 0;
}

bool KeyDescriptor::equals(const AlgorithmDescriptor & a_ad) const
{
	if (const_cast<AlgorithmDescriptor &>(a_ad).getObjectId() ==
		const_cast<KeyDescriptor *>(this)->getObjectId()) {
		// Both instances are KeyDescriptors;
		if (AlgorithmDescriptor::equals(a_ad)) {
			// describe the same algorithm;
			KeyDescriptor & kd = (KeyDescriptor &)a_ad;
			if (kd.keyBodySize == keyBodySize) {
				if (memcmp(kd.pKeyBody, pKeyBody, keyBodySize) == 0) {
					// and have the same key.
					return true;
				}
			}
		}
	}
	return false;
}

int KeyDescriptor::toBuffer(byte * buf)
{
	int nbits = AlgorithmDescriptor::toBuffer(buf);
	assert((nbits % 8) == 0);

	byte * kdbuf = buf + (nbits/8);
	long bitindex = 0;

	// keyBody
	assert(keyBodySize == 0 || pKeyBody != 0);
	put(keyBodySize, kdbuf, bitindex, 32);
	bitindex += 32;
	for (word32 kb_idx = 0; kb_idx < keyBodySize; ++kb_idx) {
		put(pKeyBody[kb_idx], kdbuf, bitindex, 8);
		bitindex += 8;
	}

	return (nbits + bitindex);
}

int KeyDescriptor::fromBuffer(const byte * buf)
{
	clear();

	int nbits = AlgorithmDescriptor::fromBuffer(buf);
	assert((nbits % 8) == 0);
	
	const byte * kdbuf = buf + (nbits/8);
	long bitindex = 0;

	// keyBody
	keyBodySize = get(kdbuf, bitindex, 32);
	bitindex += 32;
	if (keyBodySize > 0) {
		pKeyBody = new byte[keyBodySize];
		assert(pKeyBody != 0);
		for (word32 kb_idx = 0; kb_idx < keyBodySize; ++kb_idx) {
			pKeyBody[kb_idx] = (byte)get(kdbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	return (nbits + bitindex);
}


unsigned int KeyDescriptor::sizeOfInstance() const
{
	unsigned int size = AlgorithmDescriptor::sizeOfInstance();
	
	// keyBodySize
	assert(keyBodySize == 0 || pKeyBody != 0);
	size += 4;
	size += keyBodySize;

	return size;
}

void KeyDescriptor::setKeyBody(const byte * pKeyBody, word32 keyBodySize)
{
	delete [] this->pKeyBody;
	if (pKeyBody != 0 && keyBodySize > 0) {
		this->pKeyBody = new byte[keyBodySize];
		assert(this->pKeyBody != 0);
		memcpy(this->pKeyBody, pKeyBody, keyBodySize);
		this->keyBodySize = keyBodySize;
	} else {
		this->pKeyBody = 0;
		this->keyBodySize = 0;
	}
}

word32 KeyDescriptor::getKeyBodySize() const
{
	return keyBodySize;
}

byte * KeyDescriptor::getKeyBody() const
{
	return pKeyBody;
}

/////////////////////////////////////////////////////////////////////////////
//
// Certificate
//

Certificate::Certificate()
{
	dataSize = 0;
	data = 0;
}

Certificate::~Certificate()
{
	clear();
}

Certificate::Certificate(const Certificate & c)
{
	assert(c.dataSize == 0 || c.data != 0);
	dataSize = c.dataSize;
	if (dataSize > 0) {
		data = new byte[dataSize];
		memcpy(data, c.data, dataSize);
	} else {
		data = 0;
	}
}

void Certificate::clear()
{
	delete [] data;
	data = 0;
	dataSize = 0;
}

int Certificate::toBuffer(byte * buf)
{
	byte * cbuf = buf;
	long bitindex = 0;

	// data
	assert(dataSize == 0 || data != 0);
	put(dataSize, cbuf, bitindex, 32);
	bitindex += 32;
	for (word32 d_idx = 0; d_idx < dataSize; ++d_idx) {
		put(data[d_idx], cbuf, bitindex, 8);
		bitindex += 8;
	}

	return bitindex;
}

int Certificate::fromBuffer(const byte * buf)
{
	clear();

	const byte * cbuf = buf;
	long bitindex = 0;

	// data
	dataSize = get(cbuf, bitindex, 32);
	bitindex += 32;
	if (dataSize > 0) {
		data = new byte[dataSize];
		assert(data != 0);
		for (word32 d_idx = 0; d_idx < dataSize; ++d_idx) {
			data[d_idx] = (byte)get(cbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	return bitindex;
}
	
bool Certificate::checkSyntax() const
{
	bool isValid = false;
	
	if (dataSize == 0 || data != 0) {
		isValid = true;
	}

	return isValid;
}

unsigned int Certificate::getSize() const
{
	unsigned int size = 0;
	
	// dataSize
	assert(dataSize == 0 || data != 0);
	size += 4;
	size += dataSize;

	return size;
}

void Certificate::setData(const byte * data, word32 dataSize)
{
	delete [] this->data;
	if (data != 0 && dataSize > 0) {
		this->data = new byte[dataSize];
		assert(this->data != 0);
		memcpy(this->data, data, dataSize);
		this->dataSize = dataSize;
	} else {
		this->data = 0;
		this->dataSize = 0;
	}
}

word32 Certificate::getDataSize() const
{
	return dataSize;
}

byte * Certificate::getData() const
{
	return data;
}

/////////////////////////////////////////////////////////////////////////////
//
// AuthCodes
//

AuthCodes::AuthCodes()
{
	type = 0;
	nCertificate = 0;
	certType = 0;
	ppCertificates = 0;
	pPublicKey = 0;
	opaqueSize = 0;
	pOpaque = 0;
	authCodesSize = 0;
	pAuthCodes = 0;
}

AuthCodes::~AuthCodes()
{
	clear();
}

AuthCodes::AuthCodes(const AuthCodes & ac)
{
	type = ac.type;

	assert(ac.nCertificate == 0 || ac.ppCertificates != 0);
	if (ac.nCertificate != 0) {
		nCertificate = ac.nCertificate;
		ppCertificates = new Certificate *[nCertificate];
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ac.ppCertificates[nc] != 0);
			ppCertificates[nc] = new Certificate(*ac.ppCertificates[nc]);
			assert(ppCertificates[nc] != 0);
		}
	} else {
		nCertificate = 0;
		ppCertificates = 0;
	}

	if (ac.pPublicKey != 0) {
		pPublicKey = new KeyDescriptor(*ac.pPublicKey);
	} else {
		pPublicKey = 0;
	}

	assert(ac.opaqueSize == 0 || ac.pOpaque != 0);
	if (ac.opaqueSize != 0) {
		opaqueSize = ac.opaqueSize;
		pOpaque = new byte[opaqueSize];
		memcpy(pOpaque, ac.pOpaque, opaqueSize);
	} else {
		opaqueSize = 0;
		pOpaque = 0;
	}

	assert(ac.authCodesSize == 0 || ac.pAuthCodes != 0);
	if (ac.authCodesSize != 0) {
		authCodesSize = ac.authCodesSize;
		pAuthCodes = new byte[authCodesSize];
		memcpy(pAuthCodes, ac.pAuthCodes, authCodesSize);
	} else {
		authCodesSize = 0;
		pAuthCodes = 0;
	}
}

void AuthCodes::clear()
{
	type = 0;
	certType = 0;
	if (ppCertificates != 0) {
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			delete ppCertificates[nc];
		}
		delete [] ppCertificates;
		ppCertificates = 0;
	}
	nCertificate = 0;
	delete pPublicKey;
	pPublicKey = 0;
	opaqueSize = 0;
	delete [] pOpaque;
	pOpaque = 0;
	authCodesSize = 0;
	delete [] pAuthCodes;
	pAuthCodes = 0;
}

int AuthCodes::toBuffer(byte * buf)
{
	byte * acbuf = buf;
	long bitindex = 0;

	// type
	put(type, acbuf, bitindex, 8);
	bitindex += 8;

	if (type == 0x01) {
		assert(nCertificate == 0 || ppCertificates != 0);
		// nCertificate
		put(nCertificate, acbuf, bitindex, 32);
		bitindex += 32;
		// certType
		put(certType, acbuf, bitindex, 32);
		bitindex += 32;
		// ppCertificates
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc] != 0);
			bitindex += ppCertificates[nc]->toBuffer(&acbuf[bitindex / 8]);
		}
	} else if (type == 0x02) {
		// publicKey
		assert(pPublicKey != 0);
		bitindex += pPublicKey->toBuffer(&acbuf[bitindex / 8]);
	} else if (type == 0xFE) {
		// opaque
		assert(opaqueSize == 0 || pOpaque != 0);
		put(opaqueSize, acbuf, bitindex, 32);
		bitindex += 32;
		for (word32 o_idx = 0; o_idx < opaqueSize; ++o_idx) {
			put(pOpaque[o_idx], acbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	// authCodes
	assert(authCodesSize == 0 || pAuthCodes != 0);
	put(authCodesSize, acbuf, bitindex, 32);
	bitindex += 32;
	for (word32 ac_idx = 0; ac_idx < authCodesSize; ++ac_idx) {
		put(pAuthCodes[ac_idx], acbuf, bitindex, 8);
		bitindex += 8;
	}

	return bitindex;
}

int AuthCodes::fromBuffer(const byte * buf)
{
	clear();

	const byte * acbuf = buf;
	long bitindex = 0;

	// type
	type = get(acbuf, bitindex, 8);
	bitindex += 8;

	if (type == 0x01) {
		// nCertificate
		nCertificate = get(acbuf, bitindex, 32);
		bitindex += 32;
		// certType
		certType = get(acbuf, bitindex, 32);
		bitindex += 32;
		// ppCertificates
		if (nCertificate > 0) {
			ppCertificates = new Certificate *[nCertificate];
			assert(ppCertificates != 0);
			for (word32 nc = 0; nc < nCertificate; ++nc) {
				ppCertificates[nc] = new Certificate;
				assert(ppCertificates[nc] != 0);
				bitindex += ppCertificates[nc]->fromBuffer(&acbuf[bitindex / 8]);
			}
		}
	} else if (type == 0x02) {
		// publicKey
		pPublicKey = new KeyDescriptor;
		assert(pPublicKey != 0);
		bitindex += pPublicKey->fromBuffer(&acbuf[bitindex / 8]);
	} else if (type == 0xFE) {
		// opaque
		opaqueSize = get(acbuf, bitindex, 32);
		bitindex += 32;
		if (opaqueSize > 0) {
			pOpaque = new byte[opaqueSize];
			assert(pOpaque != 0);
			for (word32 o_idx = 0; o_idx < opaqueSize; ++o_idx) {
				pOpaque[o_idx] = (byte)get(acbuf, bitindex, 8);
				bitindex += 8;
			}
		}
	}

	// authCodes
	authCodesSize = get(acbuf, bitindex, 32);
	bitindex += 32;
	if (authCodesSize > 0) {
		pAuthCodes = new byte[authCodesSize];
		assert(pAuthCodes != 0);
		for (word32 ac_idx = 0; ac_idx < authCodesSize; ++ac_idx) {
			pAuthCodes[ac_idx] = (byte)get(acbuf, bitindex, 8);
			bitindex += 8;
		}
	}

	return bitindex;
}
	
bool AuthCodes::checkSyntax() const
{
	return true; // not implemented yet
}

unsigned int AuthCodes::getSize() const 
{
	unsigned int size = 0;

	// type
	size += 1;

	if (type == 0x01) {
		assert(nCertificate == 0 || ppCertificates != 0);
		// nCertificate
		size += 4;
		// certType
		size += 4;
		// ppCertificates
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc] != 0);
			size += ppCertificates[nc]->getSize();
		}
	} else if (type == 0x02) {
		// publicKey
		assert(pPublicKey != 0);
		size += pPublicKey->getSize();		
	} else if (type == 0xFE) {
		// opaque
		assert(opaqueSize == 0 || pOpaque != 0);
		size += 4;
		size += opaqueSize;
	}

	// authCodes
	assert(authCodesSize == 0 || pAuthCodes != 0);
	size += 4;
	size += authCodesSize;

	return size;
}

word32 AuthCodes::getType() const
{
	return type;
}

void AuthCodes::clearType()
{
	word32 nc;

	// clear previous 0x01 data (if existing)
	if (this->ppCertificates != 0) {
		for (nc = 0; nc < this->nCertificate; ++ nc) {
			delete this->ppCertificates[nc];
		}
		delete [] this->ppCertificates;
		this->ppCertificates = 0;
	}
	this->nCertificate = 0;
	this->certType = 0;
	// clear previous 0x02 data (if existing)
	delete pPublicKey;
	pPublicKey = 0;
	// clear previous 0xFE data (if existing)
	delete [] pOpaque;
	pOpaque = 0;
	opaqueSize = 0;

	type = 0x00;
}

void AuthCodes::setCertificates(word32 certType, Certificate * * ppCertificates, word32 nCertificate)
{
	assert(nCertificate == 0 || ppCertificates != 0);

	word32 nc;

	clearType();

	// set new data
	this->certType = certType;
	this->nCertificate = nCertificate;
	if (nCertificate > 0) {
		this->ppCertificates = new Certificate *[nCertificate];
		assert(this->ppCertificates != 0);
		for (nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc]);
			this->ppCertificates[nc] = new Certificate(*(ppCertificates[nc]));
			assert(this->ppCertificates[nc] != 0);
		}
	}

	// set appropriate type
	type = 0x01;
}

word32 AuthCodes::getCertificateType() const
{
	return certType;
}

word32 AuthCodes::getNumCertificates() const
{
	return nCertificate;
}

Certificate * * AuthCodes::getCertificates() const
{
	return ppCertificates;
}

void AuthCodes::setPublicKey(KeyDescriptor * pPublicKey)
{
	clearType();

	if (pPublicKey != 0) {
		this->pPublicKey = new KeyDescriptor(*pPublicKey);
	}

	type = 0x02;
}

KeyDescriptor * AuthCodes::getPublicKey() const
{
	return pPublicKey;
}

void AuthCodes::setOpaque(const byte * pOpaque, word32 opaqueSize)
{
	assert(opaqueSize > 0 || pOpaque != 0);

	clearType();

	if (opaqueSize > 0) {
		this->opaqueSize = opaqueSize;
		this->pOpaque = new byte[opaqueSize];
		assert(this->pOpaque != 0);
		memcpy(this->pOpaque, pOpaque, opaqueSize);
	} else {
		this->opaqueSize = 0;
		this->pOpaque = 0;
	}

	type = 0xFE;
}

word32 AuthCodes::getOpaqueSize() const
{
	return opaqueSize;
}

byte * AuthCodes::getOpaque() const
{
	return pOpaque;
}

void AuthCodes::setAuthCodes(const byte * pAuthCodes, word32 authCodesSize)
{
	assert(authCodesSize > 0 || pAuthCodes != 0);

	if (authCodesSize > 0) {
		this->authCodesSize = authCodesSize;
		this->pAuthCodes = new byte[authCodesSize];
		assert(this->pAuthCodes != 0);
		memcpy(this->pAuthCodes, pAuthCodes, authCodesSize);
	} else {
		this->authCodesSize = 0;
		this->pAuthCodes = 0;
	}
}

void AuthCodes::clearAuthCodes()
{
	clearType();
	
	delete [] pAuthCodes;
	pAuthCodes = 0;
	authCodesSize = 0;
}

word32 AuthCodes::getAuthCodesSize() const
{
	return authCodesSize;
}

byte * AuthCodes::getAuthCodes() const
{
	return pAuthCodes;
}



