
//	This software module was originally developed by Claudio Alberti (EPFL)
//  and modified by Massimo Balestri (Telecom Italia Lab)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_SendAudioWatermark.cpp

#include "IPMP_SendAudioWatermark.h"
#include <memory.h>


USING_MOSES_NAMESPACE

IPMP_SendAudioWatermark::IPMP_SendAudioWatermark()
:IPMP_Data_BaseClass(IPMP_SendAudioWatermark_tag),
 hasOpaqueData( false )
{

}	


IPMP_SendAudioWatermark::~IPMP_SendAudioWatermark() 
{

}


int IPMP_SendAudioWatermark::fromBuffer(const byte * buf)
{
	// Do base class tags etc.
	word32 bits = IPMP_Data_BaseClass::fromBuffer(buf);

	// read flags
	compression_status = get( buf, bits, 2 ) ;
	wm_status = get( buf, bits+=2, 2 ) ;
	hasOpaqueData = (1 == get( buf, bits+=2, 1 )) ;

	bits += 4 ; // padding

	// if payload read it from buffer
	if (wm_status == WM_PAYLOAD)
	{
		payload.fromBuffer( buf + ( bits / 8 ) ) ;
		bits+=payload.getSize() * 8 ;
	}

	// if opaque data read it from buffer
	if (hasOpaqueData)
	{
		opaqueData.fromBuffer( buf + ( bits / 8 ) ) ;
		bits+=opaqueData.getSize() * 8 ;
	}

	return bits ;
}


int IPMP_SendAudioWatermark::toBuffer(byte * buf)
{
	// Do base class tags etc.
	word32 bits = IPMP_Data_BaseClass::toBuffer(buf);

	// write flags
	put( compression_status, buf, bits, 2 ) ;
	put( wm_status, buf, bits+=2, 2 ) ;
	put( hasOpaqueData, buf, bits+=2, 1 ) ;
	
	bits+=4 ; // padding

	// if payload write it to buffer
	if (wm_status == WM_PAYLOAD)
	{
		payload.toBuffer( buf + (bits/8) ) ;
		bits+=payload.getSize() * 8 ;
	}

	// if opaque data write it to buffer
	if (hasOpaqueData)
	{
		opaqueData.toBuffer( buf + (bits/8) ) ;
		bits+=opaqueData.getSize() * 8 ;
	}

	return bits ; // return size of buffer in bits
}


// compute size, in BYTES!
unsigned int IPMP_SendAudioWatermark::sizeOfInstance() const
{
	// now compute the size of this instance.
	
	// 1 byte for flags
	int nSize = 1 + IPMP_Data_BaseClass::sizeOfInstance() ;

	if (wm_status == WM_PAYLOAD)
	{
		nSize+= payload.getSize();
	}

	if (hasOpaqueData)
	{
		nSize+= opaqueData.getSize();
	}

	return nSize;
}

void IPMP_SendAudioWatermark::setPayload( const byte * buf, word32 len )
{
	payload.setData( buf, len ) ;
}

void IPMP_SendAudioWatermark::setOpaqueData ( const byte * buf, word32 len )
{
	opaqueData.setData( buf, len ) ;
	hasOpaqueData = true ;
}

void IPMP_SendAudioWatermark::setWMStatus( enum WmStatusTable status )
{
	wm_status = status ;
}

void IPMP_SendAudioWatermark::setCompressionStatus( CompressionStatusTable status )
{
	compression_status = status ;
}

const byte * IPMP_SendAudioWatermark::getPayload() const
{
	byte *p = payload.getData() ; // chop of preceding size
	return p ;
}

word32 IPMP_SendAudioWatermark::getPayloadSize() const
{
	return payload.getDataSize() ;
}

const byte* IPMP_SendAudioWatermark::getOpaqueData() const
{
	byte *p = opaqueData.getData() ; // chop of preceding size
	return p ;
}

word32 IPMP_SendAudioWatermark::getOpaqueDataSize() const
{
	return opaqueData.getDataSize() ;	
}


IPMP_SendAudioWatermark::WmStatusTable IPMP_SendAudioWatermark::getWMStatus() const
{
	return (WmStatusTable) wm_status ;
}

enum IPMP_SendAudioWatermark::CompressionStatusTable IPMP_SendAudioWatermark::getCompressionStatus() const
{
	return (CompressionStatusTable) compression_status ;
}

