
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolDescriptor.cpp

#include "IPMP_Tool.h"
#include <assert.h>
#include <stdlib.h>
#include <memory.h>

USING_MOSES_NAMESPACE

IPMP_Tool::IPMP_Tool():
	BaseDescriptor(IPMP_ToolTag),			// set the correct tag
	toolParamDesc()							// init the byte array
{
	init() ;		// initialize all member vars
}	

IPMP_Tool::~IPMP_Tool() 
{	
	if (IPMP_ToolID != NULL)
		delete[] IPMP_ToolID;

	for (unsigned int i=0;i<numAlternates;i++)
		delete [] specificToolID[i];

	if (specificToolID != NULL)
		delete [] specificToolID ;
}


int IPMP_Tool::fromBuffer(const byte * buf)
{
	int nbits = BaseDescriptor::fromBuffer(buf);		// read the base descriptor and set "buf" as read buffer for this class.
	assert((nbits % 8) == 0);

	setIPMP_ToolID( getReadPtr() );						// read the IPMP tool ID
	nbits += 128;
	
	isAltGroup = get(buf, nbits, 1);
	nbits++;
	isParametric = get(buf, nbits, 1);
	nbits++;
	nbits+=6;

	if(isAltGroup)
	{
		if (numAlternates)
			for (unsigned int i=0;i<numAlternates;i++)
				delete [] specificToolID[i];
		if (specificToolID != NULL)
			delete[] specificToolID ;

		numAlternates = get(buf, nbits, 8);
		nbits+=8;

		specificToolID=new  byte * [numAlternates];
		for (unsigned int i=0;i<numAlternates;i++)
		{
			specificToolID[i]=new byte [16];
			for( int x=0;x<16;x++ )
			{
				specificToolID[i][x] = (byte) get( buf, nbits, 8 ) ;
				nbits+=8 ;
			}
		}
	}
	if(isParametric)
	{
		nbits += toolParamDesc.fromBuffer(buf+nbits/8);
	}

	int mysize = getSizeOfInstance();
	while (nbits/8 < mysize)
	{
		IPMP_ByteArray ba;
		nbits += ba.fromBuffer(buf+nbits/8);
		toolURL.addObject(ba);
	}

	return nbits;				// return the number of read bits
}

int IPMP_Tool::toBuffer(byte * buf)
{
	int nbits = BaseDescriptor::toBuffer(buf);		// read the base descriptor and set "buf" as read buffer for this class.

	memcpy(buf+nbits/8, IPMP_ToolID, 16);
	nbits+=128 ;
	put(isAltGroup, buf, nbits, 1); 
	nbits++;
	put(isParametric, buf, nbits, 1); 
	nbits++;
	nbits+=6;

	if(isAltGroup)
	{
		put(numAlternates, buf, nbits, 8); 
		nbits+=8;

		for (unsigned int i=0;i<numAlternates;i++){
			memcpy(buf+nbits/8, specificToolID[i], 16);
			nbits+=128 ;
		}
	}

	if(isParametric)
	{
		nbits+=toolParamDesc.toBuffer(buf+nbits/8);
	}

	for (unsigned int i=0;i<toolURL.getObjectNum();i++){
		nbits += toolURL.getObject(i).toBuffer(buf+nbits/8);
	}
	return nbits;				// return the number of read bits
}


void IPMP_Tool::setIPMP_ToolID(const byte *toolId)
{
	if (IPMP_ToolID != NULL)
		delete[] IPMP_ToolID;

	IPMP_ToolID = new byte[16];
	memcpy(IPMP_ToolID, toolId, 16) ;
}

byte * IPMP_Tool::getIPMP_ToolID() const
{
	return IPMP_ToolID;
}

void IPMP_Tool::setSpecificToolID(const byte *toolId, unsigned int index)
{
	if (index <numAlternates)
		memcpy(specificToolID[index], toolId, 16) ;
}

void IPMP_Tool::setToolParamDesc( const IPMP_ParametricDescription & paramDescr )
{
	toolParamDesc = paramDescr;
}

void IPMP_Tool::addToolURL( byte *desc, int length )
{
	IPMP_ByteArray ba;
	ba.setData(desc, length);

	toolURL.addObject(ba);
}

byte * IPMP_Tool::getToolURL(unsigned int index) const
{
	if (index <toolURL.getObjectNum())
		return toolURL.getObject(index).getData();
	else 
		return NULL;
}

unsigned int IPMP_Tool::getToolURLsize(unsigned int index) const
{
	if (index <toolURL.getObjectNum())
		return toolURL.getObject(index).getDataSize();
	else 
		return 0;
}

const IPMP_ParametricDescription & IPMP_Tool::getToolParamDesc() const
{
	return toolParamDesc;
}

unsigned int IPMP_Tool::getToolParamDescSize() const
{
	return toolParamDesc.sizeOfInstance();
}


byte * IPMP_Tool::getSpecificToolID(unsigned int index) const
{
	if (index <numAlternates)
		return specificToolID[index];
	else
		return NULL;
}

unsigned int IPMP_Tool::sizeOfInstance() const
{
	int size = BaseDescriptor::sizeOfInstance() ;
	size+=16 ;
	size+=1 ;

	if(isAltGroup)
	{
		size+=1 ;
		size+=(16*numAlternates);
	}
	if(isParametric)
	{
		size+=toolParamDesc.getSize();
	}
	for (unsigned int i=0;i < toolURL.getObjectNum();i++){
		size += toolURL.getObject(i).getSize();
	}

	return size;
}

void IPMP_Tool::setIsAltGroup(bool alt)
{
	isAltGroup=alt;
}

bool IPMP_Tool::getIsAltGroup() const
{
	return (isAltGroup != 0);
}

void IPMP_Tool::setIsParametric(bool parametric)
{
	isParametric=parametric;
}

bool IPMP_Tool::getIsParametric() const
{
	return (isParametric != 0);
}

void IPMP_Tool::setNumAlternates(unsigned int num)
{
	for (unsigned int i=0;i<numAlternates;i++)
		delete [] specificToolID[i];

	if (specificToolID != NULL)
		delete[] specificToolID ;

	numAlternates=num;

	specificToolID=new  byte * [numAlternates];
	for (i=0; i<numAlternates; i++)
		specificToolID[i]=new byte [16];
}

unsigned int IPMP_Tool::getNumAlternates() const 
{
	return numAlternates;
}

unsigned int IPMP_Tool::getNumURLs() const
{
	return toolURL.getObjectNum();
}

void IPMP_Tool::init() 
{
	IPMP_ToolID = NULL;
	isAltGroup=0;
	isParametric=0;
	numAlternates=0;
	specificToolID = NULL;
}

// copy constructor
IPMP_Tool::IPMP_Tool(const IPMP_Tool &tool):
	BaseDescriptor(tool),						// set the correct tag
	toolParamDesc(tool.toolParamDesc)			// copy the byte array

{
	init();
	setIPMP_ToolID( tool.getIPMP_ToolID() );
	setIsAltGroup( tool.getIsAltGroup() );
	setIsParametric( tool.getIsParametric() );
	if (isAltGroup)
	{
		setNumAlternates(tool.getNumAlternates());
		for (unsigned int i=0; i<numAlternates; i++)
		{
			setSpecificToolID(tool.getSpecificToolID(i),i);
		}
	}
	
	// toolParaDesc has been copy-constructed, so no copy is needed here

	for (unsigned int i=0; i<tool.getNumURLs(); i++)
	{
		addToolURL( tool.getToolURL(i), tool.getToolURLsize(i) );
	}
}

