
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolDescriptor.cpp

#include "IPMP_ToolListDescriptor.h"
#include <assert.h>
#include <stdlib.h>
#include <memory.h>
#include "IPMP_Tool.h"

USING_MOSES_NAMESPACE

IPMP_ToolListDescriptor::IPMP_ToolListDescriptor()
:BaseDescriptor( IPMP_ToolListDescrTag )
{
	init() ;
}

IPMP_ToolListDescriptor::~IPMP_ToolListDescriptor()
{
	clearUp() ;
}

void IPMP_ToolListDescriptor::clearUp()
{
	if( m_Tools )
	{
		for( int x=0;x<m_nCount;x++ )
			delete m_Tools[x] ;
		delete[] m_Tools ;
	}
}

void IPMP_ToolListDescriptor::init()
{
	m_nCount = 0 ;
	m_Tools = NULL ;
}

int IPMP_ToolListDescriptor::fromBuffer( const byte* buf )
{
	int nbits = BaseDescriptor::fromBuffer( buf ) ;

	IPMP_Tool **pTools = new IPMP_Tool *[256] ;
	int nsize = buf[1] ;

	clearUp() ;
	init() ;

	while( nbits/8 < nsize )
	{
		pTools[m_nCount] = new IPMP_Tool ;
		nbits+=pTools[m_nCount++]->fromBuffer( buf + (nbits/8) ) ;
	}

	// copy into member variable

	m_Tools = new IPMP_Tool *[m_nCount] ;
	for( int x=0;x<m_nCount;x++ )
		m_Tools[x] = pTools[x] ;

	delete[] pTools ;


	return nbits ;
}

int IPMP_ToolListDescriptor::toBuffer( byte*buf )
{
	int nbits = BaseDescriptor::toBuffer( buf ) ;

	for( int x=0;x<m_nCount;x++ )
		nbits += m_Tools[x]->toBuffer( buf + nbits/8 ) ;

	return nbits ;
}

unsigned int IPMP_ToolListDescriptor::sizeOfInstance() const
{
	int nbytes = BaseDescriptor::sizeOfInstance() ;

	for( int x=0;x<m_nCount;x++ )
		nbytes += m_Tools[x]->getSize() ;

	return nbytes ;
}


void IPMP_ToolListDescriptor::setToolNum( int nNumTools )
{
	clearUp();
	m_nCount = nNumTools;
	m_Tools = new IPMP_Tool *[nNumTools];
	for (int i=0; i<nNumTools; i++)
		m_Tools[i] = NULL;				// not yet allocated
}

int IPMP_ToolListDescriptor::getToolNum() const 
{
	return m_nCount ;
}

bool IPMP_ToolListDescriptor::setIPMP_Tool(const IPMP_Tool * pTool, int nIndex )
{
	if( nIndex < m_nCount )
	{
		if (pTool != NULL)
		{
			if (m_Tools[nIndex] != NULL) 
				delete m_Tools[nIndex];

			m_Tools[nIndex] = new IPMP_Tool(*pTool);		// use copy constructor
			return true ;
		}
	}
	return false;
}

IPMP_Tool* IPMP_ToolListDescriptor::getIPMP_Tool( int nIndex ) const
{
	if( nIndex >= m_nCount )
		return NULL ;
	else
		return m_Tools[nIndex];
}

// copy constructor
IPMP_ToolListDescriptor::IPMP_ToolListDescriptor(const IPMP_ToolListDescriptor &desc)
:BaseDescriptor( IPMP_ToolListDescrTag )
{
	init() ;
	setToolNum( desc.getToolNum() );
	for (int i=0; i<m_nCount; i++)
		setIPMP_Tool( desc.getIPMP_Tool(i), i);
}
