
//	This software module was originally developed by James King (CRL) and edited by
//  Massimo Balestri (Telecom Italia Lab)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	CRL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_ToolListDescriptor.h

#ifndef	IPMP_TOOLLISTDESCRIPTOR_H
#define	IPMP_TOOLLISTDESCRIPTOR_H

#include "namespace.h"
#include "BaseDescriptor.h"

BEGIN_MOSES_NAMESPACE

class IPMP_Tool ;

/** 
 *  @class IPMP_ToolListDescriptor
 *  this message conveys the list of IPMP tools required to access the content
 *	associated with the IOD in which it is described. 
 *  
 *  @author James King
 *  @date 15/11/2002
 */
class IPMP_ToolListDescriptor: public BaseDescriptor 
{
private:
	IPMP_Tool **m_Tools ;
	int m_nCount ;

	void init();			// initialize all member vars
	void clearUp() ;

public:
	IPMP_ToolListDescriptor(const IPMP_ToolListDescriptor & desc);
	/**
	 * constructor.
	 */
	IPMP_ToolListDescriptor();	
	
	/**
	 * destructor.
	 */
	virtual ~IPMP_ToolListDescriptor();		
	
	/**
	 * method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	
	/**
	 * method for reading the contents of the message.
	 * @param buf message storage location for the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * get the total size of the class,when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance() const;

	/**
	 * set the number of IPMP tool messages contaned in this object.
	 * @param numTools the number of tool messages
	 */
	void setToolNum( int numTools ) ;

	/**
	 * get the number of IPMP tool messages contaned in this object.
	 * @return the number of tool messages
	 */
	int getToolNum() const;

	/**
	 * set one IPMP tool message in the list of messages contained in this object.
	 * @param pTool the message to be inserted in the list
	 * @param nIndex the index of teh message in the list
	 * @return TRUE if successful
	 */
	bool setIPMP_Tool( const IPMP_Tool * pTool, int nIndex ) ;


	/**
	 * get one IPMP tool message from the list of messages contained in this object.
	 * @param nIndex the index of the message in the list
	 * @return a pointer to the tool if the index is in range; NULL otherwise.
	 */
	IPMP_Tool* getIPMP_Tool( int nIndex ) const;
};


END_MOSES_NAMESPACE

#endif