#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#include "zerotree.h"
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "proto.h"
#include "wavecore.h"



//------------------------------------ Implmentation zerotree ------------------------------------

//constructeur
zerotree::zerotree() {
	m_maxn = 15;
	m_nextstreamind = 0;
	m_streamingON = 1;
	//m_maxn = 11;
	m_nbvertex = 0;
	m_nbtriangle = 0;
	m_depth = 0;
	m_maxcoeff = 0;
	m_ind = 0;
	m_tabsub_zerotree = 0;

	m_vertexparent = NULL;
	m_tabvertex = NULL;
	m_triangles = NULL;
	m_tabsub_zerotree = NULL;

	m_nomf = NULL;

	m_treecoeff=NULL;
	m_treecoeff_loc=NULL;

	m_EC=new EncoderConfig();

	m_currentcoeffnumber=0;

	#ifdef LOGZT
	logfile=fopen("logzt.log", "w");
	#endif


}



void zerotree::reset() {
	m_maxn = 15;
	m_nextstreamind = 0;
	m_streamingON = 1;
	//m_maxn = 11;
	m_nbvertex = 0;
	m_nbtriangle = 0;
	m_depth = 0;
	m_maxcoeff = 0;
	m_ind = 0;
	m_tabsub_zerotree = 0;

	if (m_vertexparent) {
		delete[] m_vertexparent;
	}
	m_vertexparent = NULL;

	if (m_tabvertex) {
		delete[] m_tabvertex;
	}
	m_tabvertex = NULL;

	if (m_triangles) {
		delete[] m_triangles;
	}
	m_triangles = NULL;

	if (m_tabsub_zerotree) {
		delete[] m_tabsub_zerotree;
	}
	m_tabsub_zerotree = NULL;

	m_nomf = NULL;

	noeudcoeff* pnc;
	while (m_treecoeff) {
		pnc = m_treecoeff;
		m_treecoeff = m_treecoeff->frere;
		deletefils(pnc);
		delete pnc;
	}
	m_treecoeff=NULL;

	m_o.reset();
	m_i.reset();
}

//destructeur
zerotree::~zerotree() {
	reset();

	#ifdef LOGZT
	fclose(logfile);
	#endif
}






void zerotree::start_decode_conf(char* nomf, char *dsif, Mesh *BM) {
	char buf[500];
	char* extractbuf;
	char* tok;
	int i;
	noeudcoeff* ncroot;
	entry* pentry;



	if (nomf==NULL) {
		return;
	}

	m_nomf = nomf;
	//m_i.open(nombase,ios::in|ios::binary);
	m_EC = new EncoderConfig(dsif, 0);

	// destruction maillage prcdent
	reset();

	m_depth=m_EC->NumberOfLevels;

	
	m_nbbasetriangle = BM->NbFace;
	
	m_nbvertex=BM->NbVertex;

	m_nbtriangle = m_nbbasetriangle*(((1 << (m_depth+m_depth+2))-1)/3);

	m_triangles = new triangle[m_nbtriangle];
	for (i = 0; i<m_nbtriangle; i++) {
		m_triangles[i].tind = -1;
		m_triangles[i].root = 0;
		m_triangles[i].v1 = -1;
		m_triangles[i].v2 = -1;
		m_triangles[i].v3 = -1;
	}
	m_triangles_voisins = new voisins[m_nbtriangle];

	//lecture des triangles
	i=0;
	int pas = m_nbtriangle/m_nbbasetriangle;
	while (i<m_nbtriangle) {
	

		m_triangles[i].tind =0;			
		m_triangles[i].root =1;			
		m_triangles[i].v1 =BM->Face[i/pas][0];			
		m_triangles[i].v2 =BM->Face[i/pas][1];			
		m_triangles[i].v3 =BM->Face[i/pas][2];			

		i += pas;
	}

	for (i=0; i<m_nbtriangle; i+=pas) {
		for (int j = i+ pas; j<m_nbtriangle; j+=pas) {
			if (setvoisin(i,j)) {
				j = m_nbtriangle;
			}
		}
	}



	unsigned char* niv=(unsigned char*)malloc(2000000*sizeof(unsigned char));
	for (i=0; i<m_depth; i++) {
		diviseniveau(i, niv);
	}

	//creation m_tabvertex et m_vertexparent
	m_tabvertex = new vertex[m_nbvertex];
	m_vertexparent = new int[m_nbvertex];

	for (i=0; i<m_nbvertex; i++) {
		m_tabvertex[i].depth = niv[i];
		m_tabvertex[i].x = 0;
		m_tabvertex[i].y = 0;
		m_tabvertex[i].z = 0;
		m_vertexparent[i] = i;
	}
	free(niv);	

 
	delete[] m_triangles_voisins;
	m_triangles_voisins = NULL;

	
	for (i=0; i<BM->NbVertex; i++) {
		m_tabvertex[i].depth = m_depth+1;
		m_tabvertex[i].x = BM->Vertex[i][0];
		m_tabvertex[i].y = BM->Vertex[i][1];
		m_tabvertex[i].z = BM->Vertex[i][2];
	}

	buildvertexparent();



	//Fossile
	//p_WaveMesh->initMesh(this);

	m_nbsub_zerotree = buildcoefftree_conf();

	//initialisation sub_zerotrees
	m_tabsub_zerotree = new sub_zerotree[m_nbsub_zerotree];

	ncroot = m_treecoeff;

	for (i=0; i<m_nbsub_zerotree/3; i++) {

		//Z component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPZ.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISZ.add(pentry);
		}
		ncroot = ncroot->frere;

		//Y component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPY.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISY.add(pentry);
		}
		ncroot = ncroot->frere;

		//X component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPX.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISX.add(pentry);
		}
		ncroot = ncroot->frere;

		

		
	}

	return;
}

void zerotree::buildvertexparent() {
	m_ind=0;
	while (m_ind<m_nbtriangle) {
		iterbasetriangle();
	}
}

void zerotree::iterbasetriangle() {
	if (m_triangles[m_ind].root!=1) {
		cout<<"warning : erreur root triangle"<<endl;
		exit(0);
	}
	else {
		int a,b,c;
		if (iterinit(a,b,c)) {
			iter0(0,a,b,c);
			iter0(1,a,b,c);
			iter0(2,a,b,c);
			iter0(3,a,b,c);
		}
		m_ind++;
	}
	return;
}

int zerotree::iterinit(int & a, int & b, int & c) {
	m_ind++;
	if ((m_ind>=m_nbtriangle)||(m_triangles[m_ind].tind!=0)||(m_triangles[m_ind].root==1)) {
		m_ind--;
		return 0;
	}
	else {
		a = m_triangles[m_ind].v1;
		b = m_triangles[m_ind].v2;
		c = m_triangles[m_ind].v3;
		m_ind--;
		return 1;
	}
}

void zerotree::iter0(int tind, int a, int b, int c) {
	int ap,bp,cp;

	m_ind++;
	if (m_ind>=m_nbtriangle) {
		return;
	}

	if (iterinit(ap,bp,cp)) {
		switch (tind) {
			case 0 :
				m_vertexparent[ap] = a;
				m_vertexparent[bp] = b;
				m_vertexparent[cp] = c;
			break;
			case 1 :
				m_vertexparent[ap] = a;
				m_vertexparent[bp] = c;
				m_vertexparent[cp] = b;
			break;
			case 2 :
				m_vertexparent[ap] = c;
				m_vertexparent[bp] = b;
				m_vertexparent[cp] = a;
			break;
			case 3 :
				m_vertexparent[ap] = b;
				m_vertexparent[bp] = a;
				m_vertexparent[cp] = c;
			break;
			default:
			break;
		}

		iter0(0,ap,bp,cp);
		iter0(1,ap,bp,cp);
		iter0(2,ap,bp,cp);
		iter0(3,ap,bp,cp);
		return;
	}
}

void zerotree::aplativertexparent() {
	//arbre aplati, tous les coefficient ont pour pere un coefficient racine
	for (int i=0; i<m_nbvertex; i++) {
		int parent = i;
		while (m_vertexparent[parent]!=parent) {
			parent = m_vertexparent[parent];
		}
		m_vertexparent[i] = parent;
	}
}



	
int zerotree::buildcoefftree_conf() {

	int count = 0;
	int k, l;
	noeudcoeff* c;
	noeudcoeff** coeffs = new noeudcoeff*[3*m_nbvertex];
	for (int i = 0; i<m_nbvertex; i++) {
		if (m_tabvertex[i].depth != m_depth+1) {

			coeffs[3*i] = new noeudcoeff();
			coeffs[3*i]->coeffvalue = &(m_tabvertex[i].x);
			coeffs[3*i]->quantvalue = quantize(m_tabvertex[i].x,m_EC->NbBPX, 0);
			coeffs[3*i]->ind = 3*i;

			coeffs[3*i+1] = new noeudcoeff();
			coeffs[3*i+1]->coeffvalue = &(m_tabvertex[i].y);
			coeffs[3*i+1]->quantvalue = quantize(m_tabvertex[i].y,m_EC->NbBPY, 1);
			coeffs[3*i+1]->ind = 3*i+1;

			coeffs[3*i+2] = new noeudcoeff();
			coeffs[3*i+2]->coeffvalue = &(m_tabvertex[i].z);
			coeffs[3*i+2]->quantvalue = quantize(m_tabvertex[i].z,m_EC->NbBPZ, 2);
			coeffs[3*i+2]->ind = 3*i+2;
		}
	}

	bool *djv=(bool*)malloc(m_nbvertex*sizeof(bool));
	for (i=0; i<m_nbvertex; i++)
		djv[i]=0;
	for (i=0; i<m_nbtriangle; i++) {
		for (l=0; l<3; l++) { 
			k=(l==0)*m_triangles[i].v1+(l==1)*m_triangles[i].v2+(l==2)*m_triangles[i].v3;
			if (!djv[k]) {
				if (m_vertexparent[k]==k && m_tabvertex[k].depth != m_depth+1 && coeffs[3*k]->frere==NULL ) {
					coeffs[3*k]->frere = m_treecoeff;
					m_treecoeff = coeffs[3*k];

					coeffs[3*k+1]->frere = m_treecoeff;
					m_treecoeff = coeffs[3*k+1];
					
					coeffs[3*k+2]->frere = m_treecoeff;
					m_treecoeff = coeffs[3*k+2];
					count = count+3;
				} else if (m_vertexparent[k]!=k) {
					c = coeffs[3*m_vertexparent[k]];
					coeffs[3*k]->frere = c->fils;
					c->fils = coeffs[3*k];

					c = coeffs[3*m_vertexparent[k]+1];
					coeffs[3*k+1]->frere = c->fils;
					c->fils = coeffs[3*k+1];

					c = coeffs[3*m_vertexparent[k]+2];
					coeffs[3*k+2]->frere = c->fils;
					c->fils = coeffs[3*k+2];
				}
				djv[k]=1;
			}
		}

	}

	ConformantOrdering(count);

	delete djv;
	delete[] coeffs;

	delete[] m_vertexparent;
	m_vertexparent = NULL;


	return count;
}

void zerotree::MakeConformantRootOrder(int nbzt) {

	int i, count;

	// Ne marche pas encore, ne pas utiliser tel quel

	// On dresse la correspondance (indice de root) |-> (indice de sommet)
	// On l'a dja ?? Oui sans doute

	// On dresse la correspondance (indice de sommet) |-> (indice de root vis)
	int* vertex2aimedroot=(int*)malloc(sizeof(int)*m_nbvertex);
	for (i=0; i<m_nbvertex; i++) 
		vertex2aimedroot[i]=-1;
	for (count=0, i=1; i<m_nbtriangle; i++) {
		//Si on est sur un triangle de niveau 1 on extrait ses coins
		if (m_triangles[i-1].root==1) {
			if (vertex2aimedroot[m_triangles[i].v2]==-1) {//Pas deja parcouru
				vertex2aimedroot[m_triangles[i].v2]=count;
				count++;
			}
			if (vertex2aimedroot[m_triangles[i].v3]==-1) {//Pas deja parcouru
				vertex2aimedroot[m_triangles[i].v3]=count;
				count++;
			}
			if (vertex2aimedroot[m_triangles[i].v1]==-1) {//Pas deja parcouru
				vertex2aimedroot[m_triangles[i].v1]=count;
				count++;
			}
		}
	}

	// On en dduit (indice de root) |-> (indice de root vis)
	noeudcoeff** root2aimedroot=(noeudcoeff**)malloc(sizeof(noeudcoeff*)*3*nbzt);
	noeudcoeff** root=(noeudcoeff**)malloc(sizeof(noeudcoeff*)*3*nbzt);
	noeudcoeff* m_run=m_treecoeff;
	for (i=0; i<3*nbzt; i++) {
		root[i]=m_run;
		m_run=m_run->frere;
	}
	for (i=0; i<3*nbzt; i++) {
		root2aimedroot[i]=root[vertex2aimedroot[root[i]->ind]];
	}

	// On permute les indices
	m_treecoeff=root2aimedroot[0];//tete
	for (i=0; i<3*nbzt-1; i++) {
		root2aimedroot[i]->frere=root2aimedroot[i+1];
	}
	root2aimedroot[i]->frere=m_run;//queue

	//DEBUG
	/*FILE *f=fopen("log", "w");
	noeudcoeff* m_run=m_treecoeff;
	for (i=0; i<nbzt; i++) {
		fprintf(f, "%d", m_run->ind/3);
		m_run=m_run->frere;
	}
	fclose(f);
*/
	//the end
	free(vertex2aimedroot);
	free(root2aimedroot);
	free(root);
}

void zerotree::ConformantOrdering(int nbzt) {
	reverseRoots(nbzt);
	//MakeConformantRootOrder(nbzt);
	permuteOffspring();
}

void zerotree::reverseRoots(int nbzt) {

	noeudcoeff* running=m_treecoeff, *newlist=NULL, *tmp;

	//inversion
	for(int i=0; i<nbzt; i++) {
		
		tmp=running;
		running=running->frere;
		tmp->frere=newlist;
		newlist=tmp;
	}

	//rattachement
	m_treecoeff->frere=running;
	m_treecoeff=newlist;

}



void zerotree::initmaxcoeff_conf() {

	m_EC = new EncoderConfig();
	m_maxcoeffX = m_maxcoeffY = m_maxcoeffZ = -1;
	for (int i=0; i<m_nbvertex; i++) {
		if (m_tabvertex[i].depth!=m_depth+1) {
			if (fabs(m_tabvertex[i].x)>m_maxcoeffX) {
				m_maxcoeffX = m_EC->Xmax = (float)(fabs(m_tabvertex[i].x));
			}
			if (fabs(m_tabvertex[i].y)>m_maxcoeffY) {
				m_maxcoeffY = m_EC->Ymax = (float)(fabs(m_tabvertex[i].y));
			}
			if (fabs(m_tabvertex[i].z)>m_maxcoeffZ) {
				m_maxcoeffZ = m_EC->Zmax = (float)(fabs(m_tabvertex[i].z));
			}
		}
	}
	//!!!
	//m_maxcoeff = (float)(-m_maxcoeff/(2*(pow(0.5,m_maxn+1)-1)));

}

int zerotree::SN(int n, noeudcoeff* nc) {
	return ((nc->quantvalue&(32768 >> (1+m_maxn-n)))>0);
}

int zerotree::SN(int n, noeudcoeff* nc, int bidon) {
	
		return (((nc->quantvalue)&(1<<(n-1)))>0);
		
}

void zerotree::setSN(int n, int sn, noeudcoeff* pnc) {
	unsigned short int bits = 32768 >> (1+m_maxn-n);
	if (sn) {
		pnc->quantvalue = pnc->quantvalue|bits;
	}
	else {
		pnc->quantvalue = pnc->quantvalue&(bits^65535);
	}
}

void zerotree::setSN(int n, int sn, noeudcoeff_loc* pnc) {
	unsigned short int bits = 32768 >> (1+m_maxn-n);
	if (sn) {
		pnc->quantvalue = pnc->quantvalue|bits;
	}
	else {
		pnc->quantvalue = pnc->quantvalue&(bits^65535);
	}
}

int zerotree::SND(int n, entry* e, int flag) {

	int res = 0;
	entry* ne;
	noeudcoeff* p = e->pnc->fils;
	unsigned short int i,bits;

	while (p) {
		if (p->quantvalue&(1<<(n-1))) {
			p = NULL;
			res = 1;
		}
		else {
			p = p->frere;
		}
	}

	if (res==0) {
		p = e->pnc->fils;
		while (p) {
			ne = new entry;
			ne->pnc = p;
			res = SND(n,ne, flag);
			delete ne;
			if (res==1) {
				p = NULL;
			}
			else {
				p = p->frere;
			}
		}
	}

	return res;
}

int zerotree::SND(int n, entry* e) {
	int res = 0;
	entry* ne;
	noeudcoeff* p = e->pnc->fils;
	unsigned short int i,bits;

	while (p) {
		i = p->quantvalue;
		bits = 32768;
		bits = bits >> (1+m_maxn-n);
		if (i&bits) {
			p = NULL;
			res = 1;
		}
		else {
			p = p->frere;
		}
	}

	if (res==0) {
		p = e->pnc->fils;
		while (p) {
			ne = new entry;
			ne->pnc = p;
			res = SND(n,ne);
			delete ne;
			if (res==1) {
				p = NULL;
			}
			else {
				p = p->frere;
			}
		}
	}

	return res;
}

int zerotree::SNL(int n, entry* e) {
	int res = 0;
	entry* ne;
	noeudcoeff* p = e->pnc->fils;

	while (p) {
		ne = new entry;
		ne->pnc = p;
		res = SND(n,ne);
		delete ne;
		if (res==1) {
			p = NULL;
		}
		else {
			p = p->frere;
		}
	}

	return res;
}

int zerotree::SNL(int n, entry* e, int flag) {
	int res = 0;
	entry* ne;
	noeudcoeff* p = e->pnc->fils;

	while (p) {
		ne = new entry;
		ne->pnc = p;
		res = SND(n,ne, flag);
		delete ne;
		if (res==1) {
			p = NULL;
		}
		else {
			p = p->frere;
		}
	}

	return res;
}

int zerotree::posorneg(noeudcoeff* pnc) {
	if (*(pnc->coeffvalue)<0) {
		return 0;
	}
	else {
		return 1;
	}
}

int zerotree::sign(noeudcoeff* pnc) {
	if (*(pnc->coeffvalue)<0) {
		return 0;
	}
	else {
		return 1;
	}
}

void zerotree::setsign(noeudcoeff* pnc,int sign) {
	if (sign) {
		pnc->quantvalue = pnc->quantvalue|32768;
	}
	else {
		pnc->quantvalue = pnc->quantvalue&32767;
	}
}

void zerotree::setsign(noeudcoeff_loc* pnc,int sign) {
	if (sign) {
		pnc->quantvalue = pnc->quantvalue|32768;
	}
	else {
		pnc->quantvalue = pnc->quantvalue&32767;
	}
}

int zerotree::Lijvide(entry* e) {
	noeudcoeff* p = e->pnc->fils;
	while (p) {
		if (p->fils) {
			p = NULL;
			return 0;
		}
		else {
			p = p->frere;
		}
	}
	return 1;
}


//Cette fonction ne marche pas dans le cas "sparse tree"
#ifdef marche pas
int zerotree::Lijvide(entry_loc* e) {
	noeudcoeff_loc* p = e->pnc->fils();
	while (p) {
		if (p->m_fils) {
			p = NULL;
			return 0;
		}
		else {
			p = p->m_frere;
		}
	}
	return 1;
}
#endif


int zerotree::Lijvide(entry_loc* e) {


	int dpl=e->pnc->alpha+e->pnc->beta+e->pnc->gamma;
	int dplm=1<<(m_EC->NumberOfLevels);


	return 2*dpl>=dplm;

}




void zerotree::threeD_Wavelet_Mesh_Object_Scale_coeff() {

	//Class of the same name (see the spec)

	//TO DO

}





unsigned short int zerotree::quantize(float f, unsigned char nbit) {
	unsigned short int res = 0;
	unsigned short int bits = 32768;

	if (f==0) {
		return 32768;
	}
	
	if (f>=0) {
		res = res|bits;
	}
	else {
		f = -f;
	}
	bits = bits/2;

	// derniere modif
	bits = bits>>(15-nbit);

	float fmax = m_maxcoeff;
	for (int i=0; i<nbit; i++) {
		if (f>=fmax) {
			f = f-fmax;
			res = res|bits;
		}
		else {
		}
		bits = bits/2;
		fmax = fmax/2;
	}

	if (res) {
		return res;
	}

	return 32768;
}

unsigned short int zerotree::quantize(float f, unsigned char nbit, int flag) {
	unsigned short int res = 0;
	unsigned short int bits = 32768;

	if (f==0) {
		return 32768;
	}
	
	if (f<0) {
		f = -f;
		bits=0;
	}
	//bits = bits/2;

	// derniere modif
	//bits = bits>>(15-nbit);

	float fmax;
	switch(flag) {
	case 0:
		fmax = m_maxcoeffX;
		break;
	case 1:
		fmax = m_maxcoeffY;
		break;
	case 2:
		fmax = m_maxcoeffZ;
		break;
	}
	
	
	f=f/fmax;
	for (int i=0; i<nbit; i++)
		f*=2;
	res=(unsigned short int)f;

	if (res>=(1<<nbit))
		res=res-1;
	
	res=res|bits;

	return res;
/*	for (int i=0; i<nbit; i++) {
		if (f>=fmax) {
			f = f-fmax;
			res = res|bits;
		}
		else {
		}
		bits = bits/2;
		fmax = fmax/2;
	}

	if (res) {
		return res;
	}

	return 32768;
	*/

}


float zerotree::unquantize(unsigned short int usi) {
	float res=0;
	unsigned short int bits = 32768/2;

	//derniere modif
	//bits = bits>>(15-m_maxn);

	float fmax = m_maxcoeff;
	for (int i=0; i<m_maxn; i++) {
		if (usi&bits) {
			res += fmax;
		}
		bits = bits/2;
		fmax = fmax/2;
	}
	if (!(usi&32768)) {
		res = -res;
	}
	return res;
}

float zerotree::unquantize(unsigned short int usi, int flag) {
	float res=0;
	unsigned short int bits = 32768/2;

	//derniere modif
	//bits = bits>>(15-m_maxn);

	float fmax;
	switch (flag) {
	case 0:
		fmax=m_EC->Xmax;
		m_maxn=m_EC->NbBPX;
		break;
	case 1:
		fmax=m_EC->Ymax;
		m_maxn=m_EC->NbBPY;
		break;
	case 2:
		fmax=m_EC->Zmax;
		m_maxn=m_EC->NbBPZ;
		break;
	}
/*	for (int i=0; i<m_maxn; i++) {
		if (usi&bits) {
			res += fmax;
		}
		bits = bits/2;
		fmax = fmax/2;
	}
	if (!(usi&32768)) {
		res = -res;
	}
	return res;*/

	//remplac par
	int maxbin=1<<m_maxn;
	int sg=0;
	if (usi>=32768) {
		sg=1;
		usi-=32768;
	}
	if (sg)
		return usi*fmax/maxbin;
	else 
		return -usi*fmax/maxbin;
}

float zerotree::unquantize(unsigned short int usi, unsigned char nbit) {
	float res=0;
	unsigned short int bits = 32768/2;

	//derniere modif
	bits = bits>>(15-nbit);

	float fmax = m_maxcoeff;
	for (int i=0; i<nbit; i++) {
		if (usi&bits) {
			res += fmax;
		}
		bits = bits/2;
		fmax = fmax/2;
	}
	if (!(usi&32768)) {
		res = -res;
	}
	return res;
}


void zerotree::unquantize(noeudcoeff* pnc) {
	float f;
	noeudcoeff* pfils;
	f = unquantize(pnc->quantvalue);
	*(pnc->coeffvalue) = f;
	pfils = pnc->fils;
	while (pfils) {
		unquantize(pfils);
		pfils = pfils->frere;
	}
	return;
}

void zerotree::unquantize(noeudcoeff* pnc, int flag) {
	float f;
	int ind;
	int comp;

	noeudcoeff* pfils;
	f = unquantize(pnc->quantvalue, flag);
	*(pnc->coeffvalue) = f;
	ind=(pnc->ind)/3;
	comp=(pnc->ind)%3;
	switch(comp) {
	case 0: 
		m_tabvertex[ind].x=*(pnc->coeffvalue);
		break;
	case 1: 
		m_tabvertex[ind].y=*(pnc->coeffvalue);
		break;
	case 2: 
		m_tabvertex[ind].z=*(pnc->coeffvalue);
		break;
	}
	pfils = pnc->fils;
	while (pfils) {
		unquantize(pfils, flag);
		pfils = pfils->frere;
	}
	return;
}

void zerotree::unquantize(noeudcoeff* pnc, unsigned char nbit) {
	float f;
	noeudcoeff* pfils;
	f = unquantize(pnc->quantvalue,nbit);
	*(pnc->coeffvalue) = f;
	pfils = pnc->fils;
	while (pfils) {
		unquantize(pfils,nbit);
		pfils = pfils->frere;
	}
	return;
}

void zerotree::unquantize(noeudcoeff_loc* pnc, int flag) {
	float f;
	noeudcoeff_loc* pfils;
	f = unquantize(pnc->quantvalue,flag);

//DEBUG
if (pnc->F0==4 && pnc->alpha==1 && pnc->beta==0 && pnc->gamma==1)
	pnc->F0=pnc->F0;
//OK

	if (!pnc->coeffvalue)
		pnc->coeffvalue=(float*)malloc(sizeof(float));
	*(pnc->coeffvalue) = f;
	pfils = pnc->m_fils;
	while (pfils) {
		unquantize(pfils,flag);
		pfils = pfils->m_frere;
	}
	return;
}

void zerotree::dumpquantize(noeudcoeff* pnc) {
	noeudcoeff* pfils;
	m_dumpstream<<pnc->quantvalue<<endl;;
	pfils = pnc->fils;
	while (pfils) {
		dumpquantize(pfils);
		pfils = pfils->frere;
	}
	return;
}

void zerotree::dumpquantize(noeudcoeff_loc* pnc) {
	noeudcoeff_loc* pfils;
	m_dumpstream<<pnc->quantvalue<<endl;;
	pfils = pnc->m_fils;
	while (pfils) {
		dumpquantize(pfils);
		pfils = pfils->m_frere;
	}
	return;
}

void zerotree::dump() {
	noeudcoeff* nc = m_treecoeff;

	m_dumpstream.open("dump",ios::in);
	m_dumpstream<<"Multires data file\n";
	m_dumpstream<<"depth "<<(int)m_depth<<"\n";
	m_dumpstream<<"Vertices\n";
	for (int i = 0; i<m_nbvertex; i++) {
		m_dumpstream<<(int)m_tabvertex[i].depth<<" "<<m_tabvertex[i].x<<" "<<m_tabvertex[i].y<<" "<<m_tabvertex[i].z<<"\n";
	}
	m_dumpstream<<"\n";
	m_dumpstream<<"Triangles\n";
	for (i = 0; i<m_nbtriangle; i++) {
		m_dumpstream<<"name: ";
		m_dumpstream<<(int)m_triangles[i].tind<<" "<<(int)m_triangles[i].root<<" ";
		m_dumpstream<<m_triangles[i].v1<<" "<<m_triangles[i].v2<<" "<<m_triangles[i].v3<<"\n";
	}
	m_dumpstream<<"end\n";
	m_dumpstream.close();
	return;
}

void zerotree::dumpZ2Dat(DatMesh *DM) {

	
	noeudcoeff* nc = m_treecoeff;


	for (int i = 0; i<m_nbvertex; i++) {
		DM->V[i][0]=(int)m_tabvertex[i].depth;
		DM->V[i][1]=m_tabvertex[i].x;
		DM->V[i][2]=m_tabvertex[i].y;
		DM->V[i][3]=m_tabvertex[i].z;
	}

	for (i = 0; i<m_nbtriangle; i++) {
		DM->F[i][0]=(int)m_triangles[i].tind;
		DM->F[i][1]=(int)m_triangles[i].root;
		DM->F[i][2]=(int)m_triangles[i].v1;
		DM->F[i][3]=(int)m_triangles[i].v2;
		DM->F[i][4]=(int)m_triangles[i].v3;
	
	}

	DM->NbVertices=m_nbvertex;
	DM->NbFaces=m_nbtriangle;

	return;
}	
	
int zerotree::readplane_conf(int i, unsigned char n, ibitstream& ibs, int flag) {
	int resSN, sign;
	entry* pentry, *emove, *newentry;
	noeudcoeff* p, *p2;
	liste_entry *m_pLSP, *m_pLSPadd, *m_pLIP, *m_pLIS;
	int cmpt=0;


	switch(flag) {
	case 0:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitX) {
			return -1 ;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPX);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPXadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPX);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISX);
		break;
	case 1:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitY) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPY);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPYadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPY);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISY);
		break;
	case 2:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitZ) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPZ);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPZadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPZ);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISZ);
		break;
	}
	//2 Sorting Pass
	//2.1
	pentry = m_pLIP->startiter();
			
	while (pentry) {
		//input Sn(i,j)
		ibs.readbit(resSN);
		cmpt++;
		setSN(n,resSN,pentry->pnc);
				
		if (resSN==1) {
				
			//input sign of ci,j 2.1.2
			ibs.readbit(sign);
			cmpt++;
			setsign(pentry->pnc,sign);
					
			//move entry to LSP
			emove = pentry;
			pentry = m_pLIP->removeiter();
			m_pLSPadd->add(emove);
		}
		else {
			pentry = m_pLIP->iter();
		}
	}
			
	//2.2
	pentry = m_pLIS->startiter();
	while (pentry) {
				
		//2.2.1
		if (pentry->entrytype==1) {
			//input Sn(D(i,j))
			ibs.readbit(resSN);
			cmpt++;
					
			if (resSN==1) {
				p = pentry->pnc->fils;
				while (p) {
					//input Sn(k,l)
					ibs.readbit(resSN);
					cmpt++;
					setSN(n,resSN,p);
							
					if (resSN==1) {
						//input sign of ck,l
						ibs.readbit(sign);
						cmpt++;
						setsign(p,sign);

						//add entry to LSP
						entry* ne = new entry;
						ne->pnc = p;
						m_pLSPadd->add(ne);
					}
					else {
						entry* ne = new entry;
						ne->pnc = p;
						m_pLIP->addend(ne);
					}
					p = p->frere;
				}
						
				//L(i,j) vide?
				if (Lijvide(pentry)) {
						
					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
							
					//move entry to the end of LIS as an entry of type B
					pentry->entrytype = 2;
					ibs.readbit(resSN);
					cmpt++;
					if (resSN) {
						p2 = pentry->pnc->fils;
						while (p2) {
							entry* ne = new entry;
							ne->entrytype = 1;
							ne->pnc = p2;
							m_pLIS->addend(ne);
							p2 = p2->frere;
						}
						pentry = m_pLIS->removeiter();
					}
					else {
						emove = pentry;
						pentry = m_pLIS->removeiter();
						m_pLIS->addend(emove);
					}
				}
			}
			else {
				pentry = m_pLIS->iter();
			}
		}
				
		//2.2.2
		else {
			if (pentry->entrytype==2) {

				//input Sn(L(i,j))
				ibs.readbit(resSN);
				cmpt++;
						
				if (resSN) {
					p = pentry->pnc->fils;
					while (p) {
						// add to the end of LIS as an entry of type A
						newentry = new entry;
						newentry->entrytype = 1;
						newentry->pnc = p;
						m_pLIS->addend(newentry);
						p = p->frere;
					}

					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					pentry = m_pLIS->iter();
				}
			}	
		}
	}
	
	//3 Refinement Pass
	pentry = m_pLSP->startiter();

	//active le plan de bits correspondant ,& logique avec puissance de 2
	unsigned short int bits = 32768;
	bits = bits >> (1+m_maxn-n);

	while (pentry) {
		int readbit;
		unsigned short int usi = pentry->pnc->quantvalue;
		//input the nth most significant bit of |ci,j|
		ibs.readbit(readbit);
		cmpt++;
		if (readbit) {
			usi = usi|bits;
		}
		else {
			usi = usi&(bits^65535);
		}
		pentry->pnc->quantvalue = usi;
		pentry = m_pLSP->iter();	
	}
		
	//append m_LSP m_LSPadd
	m_pLSP->append(*m_pLSPadd);

	switch(flag) {
	case 0:
		m_tabsub_zerotree[i].m_lastplanbitX++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPX) {
			m_tabsub_zerotree[i].m_LIPX.reset();
			m_tabsub_zerotree[i].m_LISX.reset();
			m_tabsub_zerotree[i].m_LSPX.reset();
			m_tabsub_zerotree[i].m_LSPXadd.reset();
		}
		break;
	case 1:
		m_tabsub_zerotree[i].m_lastplanbitY++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPY) {
			m_tabsub_zerotree[i].m_LIPY.reset();
			m_tabsub_zerotree[i].m_LISY.reset();
			m_tabsub_zerotree[i].m_LSPY.reset();
			m_tabsub_zerotree[i].m_LSPYadd.reset();
		}
		break;
	case 2:
		m_tabsub_zerotree[i].m_lastplanbitZ++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPZ) {
			m_tabsub_zerotree[i].m_LIPZ.reset();
			m_tabsub_zerotree[i].m_LISZ.reset();
			m_tabsub_zerotree[i].m_LSPZ.reset();
			m_tabsub_zerotree[i].m_LSPZadd.reset();
		}
		break;
	}
	return cmpt;
}






#if 0
int zerotree::readplane_conf(int i, unsigned char n, ibitstream& ibs, int flag) {
	int resSN, sign;
	entry* pentry, *emove, *newentry;
	noeudcoeff* p, *p2;
	liste_entry *m_pLSP, *m_pLSPadd, *m_pLIP, *m_pLIS;
	int cmpt=0;


	switch(flag) {
	case 0:
		if (m_EC->NbBPX-n!=m_tabsub_zerotree[i].m_lastplanbitX) {
			return -1 ;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPX);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPXadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPX);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISX);
		break;
	case 1:
		if (m_EC->NbBPY-n!=m_tabsub_zerotree[i].m_lastplanbitY) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPY);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPYadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPY);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISY);
		break;
	case 2:
		if (m_EC->NbBPZ-n!=m_tabsub_zerotree[i].m_lastplanbitZ) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPZ);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPZadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPZ);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISZ);
		break;
	}
	//2 Sorting Pass
	//2.1
	pentry = m_pLIP->startiter();
			
	while (pentry) {
		//input Sn(i,j)
		ibs.readbit(resSN);
		cmpt++;
		setSN(n,resSN,pentry->pnc);
				
		if (resSN==1) {
				
			//input sign of ci,j 2.1.2
			ibs.readbit(sign);
			cmpt++;
			setsign(pentry->pnc,sign);
					
			//move entry to LSP
			emove = pentry;
			pentry = m_pLIP->removeiter();
			m_pLSPadd->add(emove);
		}
		else {
			pentry = m_pLIP->iter();
		}
	}
			
	//2.2
	pentry = m_pLIS->startiter();
	while (pentry) {
				
		//2.2.1
		if (pentry->entrytype==1) {
			//input Sn(D(i,j))
			ibs.readbit(resSN);
			cmpt++;
					
			if (resSN==1) {
				p = pentry->pnc->fils;
				while (p) {
					//input Sn(k,l)
					ibs.readbit(resSN);
					cmpt++;
					setSN(n,resSN,p);
							
					if (resSN==1) {
						//input sign of ck,l
						ibs.readbit(sign);
						cmpt++;
						setsign(p,sign);

						//add entry to LSP
						entry* ne = new entry;
						ne->pnc = p;
						m_pLSPadd->add(ne);
					}
					else {
						entry* ne = new entry;
						ne->pnc = p;
						m_pLIP->addend(ne);
					}
					p = p->frere;
				}
						
				//L(i,j) vide?
				if (Lijvide(pentry)) {
						
					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
							
					//move entry to the end of LIS as an entry of type B
					pentry->entrytype = 2;
					ibs.readbit(resSN);
					cmpt++;
					if (resSN) {
						p2 = pentry->pnc->fils;
						while (p2) {
							entry* ne = new entry;
							ne->entrytype = 1;
							ne->pnc = p2;
							m_pLIS->addend(ne);
							p2 = p2->frere;
						}
						pentry = m_pLIS->removeiter();
					}
					else {
						emove = pentry;
						pentry = m_pLIS->removeiter();
						m_pLIS->addend(emove);
					}
				}
			}
			else {
				pentry = m_pLIS->iter();
			}
		}
				
		//2.2.2
		else {
			if (pentry->entrytype==2) {

				//input Sn(L(i,j))
				ibs.readbit(resSN);
				cmpt++;
						
				if (resSN) {
					p = pentry->pnc->fils;
					while (p) {
						// add to the end of LIS as an entry of type A
						newentry = new entry;
						newentry->entrytype = 1;
						newentry->pnc = p;
						m_pLIS->addend(newentry);
						p = p->frere;
					}

					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					pentry = m_pLIS->iter();
				}
			}	
		}
	}
	
	//3 Refinement Pass
	pentry = m_pLSP->startiter();

	//active le plan de bits correspondant ,& logique avec puissance de 2
	unsigned short int bits = 32768;
	bits = bits >> (1+m_maxn-n);

	while (pentry) {
		int readbit;
		unsigned short int usi = pentry->pnc->quantvalue;
		//input the nth most significant bit of |ci,j|
		ibs.readbit(readbit);
		cmpt++;
		if (readbit) {
			usi = usi|bits;
		}
		else {
			usi = usi&(bits^65535);
		}
		pentry->pnc->quantvalue = usi;
		pentry = m_pLSP->iter();	
	}
		
	//append m_LSP m_LSPadd
	m_pLSP->append(*m_pLSPadd);

	switch(flag) {
	case 0:
		m_tabsub_zerotree[i].m_lastplanbitX++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPX) {
			m_tabsub_zerotree[i].m_LIPX.reset();
			m_tabsub_zerotree[i].m_LISX.reset();
			m_tabsub_zerotree[i].m_LSPX.reset();
			m_tabsub_zerotree[i].m_LSPXadd.reset();
		}
		break;
	case 1:
		m_tabsub_zerotree[i].m_lastplanbitY++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPY) {
			m_tabsub_zerotree[i].m_LIPY.reset();
			m_tabsub_zerotree[i].m_LISY.reset();
			m_tabsub_zerotree[i].m_LSPY.reset();
			m_tabsub_zerotree[i].m_LSPYadd.reset();
		}
		break;
	case 2:
		m_tabsub_zerotree[i].m_lastplanbitZ++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPZ) {
			m_tabsub_zerotree[i].m_LIPZ.reset();
			m_tabsub_zerotree[i].m_LISZ.reset();
			m_tabsub_zerotree[i].m_LSPZ.reset();
			m_tabsub_zerotree[i].m_LSPZadd.reset();
		}
		break;
	}
	return cmpt;
}



void zerotree::decodemp4_conf(char* nomf, char *dsif, Mesh *BM) {
	entry* pentry;
	noeudcoeff *ncroot;
	noeudcoeff *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;

	char drive[_MAX_DRIVE];   
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];   
	char ext[_MAX_EXT];
	char nombase[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char nomfichier[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int FBP, LBP;
	unsigned int comp;

	if (nomf) {

		
		start_decode_conf(nomf, dsif, BM);
		
		m_nomf=nomf;
		//extraction nom du fichier sans extension
		_splitpath(m_nomf, drive, dir, fname, ext);
		strcpy(nombase,drive);
		strcat(nombase,dir);
		strcat(nombase,fname);
		//construction nom fichier
		strcat(nombase,".mp4");
		m_i.open(nombase,ios::in|ios::binary|ios::nocreate);

		int mid = m_nbsub_zerotree;///2;

		//Class 3D_Wavelet_Mesh_Object (see the spec)

		//WMOL
		m_i.readnbit(tmp, 1);// Temporaire

		m_EC->isBase=(bool)tmp;
	
		//isInLocalCoordinates
		m_i.readnbit(tmp, 1);
		m_EC->isInLocalCoordinates=tmp;

		//Scale Coeffs
		if (m_EC->isBase && m_EC->hasScaleCoeff)
			threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

		
		//Class ReadZT (see the spec)

		//LengthNbBits
		m_i.readnbit(tmp,5);
		m_EC->LengthNbBits=tmp;

		//isPartial
		m_i.readnbit(tmp,1);
		m_EC->isPartial=tmp;

		//NbZT
		//m_i.writenbit((unsigned short int)m_nbsub_zerotree,16);

		if (!m_EC->isPartial) {

			ncroot = m_treecoeff;
			i = 0;
			while (ncroot!=NULL) {

				ncz=ncroot;
				ncy=ncroot->frere;
				ncx=ncy->frere;

				ncroot=ncx;

				//bit(1) X;
				m_i.readnbit(tmp,1);
				m_EC->X=tmp;

				if (m_EC->X) {
					
					for (unsigned char j=m_EC->NbBPX; j>0; j--) {


						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);

						//Readplane
						nbbits = readplane_conf(i,j,m_i, 0);
						if (nbbits!=tmp)
							printf("Bug\n");
	
						
						
					}
					ncroot = ncy;

				}
				//bit(1) Y;
				m_i.readnbit(tmp,1);
				m_EC->Y=tmp;

				if (m_EC->Y) {
					
					for (unsigned char j=m_EC->NbBPY; j>0; j--) {


						
						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);

						//Readplane
						nbbits = readplane_conf(i,j,m_i, 1);
						if (nbbits!=tmp)
							printf("Bug\n");



						

						
					}
					ncroot = ncz; 

				}
				//bit(1) Z;
				m_i.readnbit(tmp,1);
				m_EC->Z=tmp;

				if (m_EC->Z) {
					
					for (unsigned char j=m_EC->NbBPZ; j>0; j--) {

						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);
						
						//Readplane
						nbbits = readplane_conf(i,j,m_i, 2);
						if (nbbits!=tmp)
							printf("Bug\n");



						

						
					}
					i++;
					ncroot = ncx->frere; 

				}
			}
			m_i.close();
		}
		else //Partial


		{
			ncroot = m_treecoeff;
			i = 0;
			while (ncroot) {

				ncz=ncroot;
				ncy=ncroot->frere;
				ncx=ncy->frere;

				ncroot=ncx;
				if (m_EC->X) {
			
					//ZtId
					m_i.readnbit(tmp,16);
					i=tmp;

					//FBP
					m_i.readnbit(tmp,5);
					m_EC->NbBPX=tmp;

					//LBP
					m_i.readnbit(LBP,5);
				
					// component
					m_i.readnbit(comp,2);
					
					for (unsigned char j=m_EC->NbBPX; j>0; j--) {


						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);

						//Encodage
						nbbits = readplane_conf(i,j,m_i, 0);
		


						

					}
					

				}

				ncroot = ncy;
				if (m_EC->Y) {
			
					//ZtId
					m_i.readnbit(tmp,16);
					i=tmp;

					//FBP
					//m_i.writenbit((unsigned char)15,5);
					m_i.readnbit(tmp,5);
					m_EC->NbBPY=tmp;

					//LBP
					m_i.readnbit(LBP,5);
				
					// component
					m_i.readnbit(comp,2);
					
					for (unsigned char j=m_EC->NbBPY; j>0; j--) {

						

						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);
						nbbits=tmp;
						
						//Encodage
						nbbits = readplane_conf(i,j,m_i, 1);
	
		

					}
					

				}
				ncroot = ncz;

				if (m_EC->Z) {
			
					//ZtId
					m_i.readnbit(tmp,16);
					i=tmp;

					//FBP
					//m_i.readnbit((unsigned char)15,5);
					m_i.readnbit(tmp,5);
					m_EC->NbBPZ=tmp;

					//LBP
					m_i.readnbit(LBP,5);
				
					// component
					m_i.readnbit(comp,2);
					
					for (unsigned char j=m_EC->NbBPZ; j>0; j--) {


						//BPLength
						m_i.readnbit(tmp,m_EC->LengthNbBits);
						nbbits=tmp;

						//Encodage
						nbbits = readplane_conf(i,j,m_i, 2);
	

					}
					i++;
					
				}
				ncroot = ncx->frere;
			}		
		}
	}

	m_i.close();

	//TRACE
	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->frere;
	}

	//Dquantif
	ncroot = m_treecoeff;
	while (ncroot!=NULL) {
		unquantize(ncroot, (ncroot->ind)%3);
		ncroot = ncroot->frere;
		unquantize(ncroot, (ncroot->ind)%3);
		ncroot = ncroot->frere;
		unquantize(ncroot, (ncroot->ind)%3);
		ncroot = ncroot->frere;
	}


	return;	
}
#endif

void zerotree::decodemp4_conf(char* nomf, char *dsif, Mesh *BM) {
	entry* pentry;
	noeudcoeff *ncroot;
	noeudcoeff *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;

	char drive[_MAX_DRIVE];   
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];   
	char ext[_MAX_EXT];
	char nombase[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char nomfichier[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int comp;
	int FT, LT;

	if (nomf) {

		
		start_decode_conf(nomf, dsif, BM);
		
		m_nomf=nomf;
		//extraction nom du fichier sans extension
		_splitpath(m_nomf, drive, dir, fname, ext);
		strcpy(nombase,drive);
		strcat(nombase,dir);
		strcat(nombase,fname);
		//construction nom fichier
		strcat(nombase,".mp4");
		m_i.open(nombase,ios::in|ios::binary|ios::nocreate);

		int mid = m_nbsub_zerotree;///2;

		//Class 3D_Wavelet_Mesh_Object (see the spec)

		//WMOL
		m_i.readnbit(tmp, 1);// Temporaire

		m_EC->isBase=(bool)tmp;
	
		//isInLocalCoordinates
		m_i.readnbit(tmp, 1);
		m_EC->isInLocalCoordinates=tmp;

		//Scale Coeffs
		if (m_EC->isBase && m_EC->hasScaleCoeff)
			threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

		
		//Class 3D_Wavelet_Mesh_Object (see the spec)


		//LengthNbBits
		m_i.readnbit(tmp,5);
		m_EC->LengthNbBits=tmp;

		//FBP
		m_i.readnbit(tmp,5);
		m_EC->FBP=tmp;

		//LBP
		m_i.readnbit(tmp,5);
		m_EC->LBP=tmp;

		//bit(1) X;
		m_i.readnbit(tmp,1);
		m_EC->X=tmp;

		//bit(1) Y;
		m_i.readnbit(tmp,1);
		m_EC->Y=tmp;

		//bit(1) Z;
		m_i.readnbit(tmp,1);
		m_EC->Z=tmp;

		//isPartial
		m_i.readnbit(tmp,1);
		m_EC->isPartial=tmp;

		//NbZT
		//m_i.writenbit((unsigned short int)m_nbsub_zerotree,16);

		if (m_EC->isPartial) {
			m_i.readnbit(tmp,16);
			FT = tmp;
			LT = FT + 1;
		} else { // Full transmission
			FT = 0;
			LT = m_nbsub_zerotree/3;
		}
	



		for (int j=m_EC->LBP; j>=m_EC->FBP; j--) {

			ncroot = m_treecoeff;
			i = 0;
			while (ncroot!=NULL) {

				ncz=ncroot;
				ncy=ncroot->frere;
				ncx=ncy->frere;

				ncroot=ncx;

					

				if (m_EC->X && i>=FT && i<=LT) {
				

					//BPLength
					m_i.readnbit(tmp,m_EC->LengthNbBits);

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 0);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
					
				}
				ncroot = ncy;

				if (m_EC->Y && i>=FT && i<=LT) {


					
					//BPLength
					m_i.readnbit(tmp,m_EC->LengthNbBits);

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 1);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				ncroot = ncz; 


				if (m_EC->Z && i>=FT && i<=LT) {
				
				
					//BPLength
					m_i.readnbit(tmp,m_EC->LengthNbBits);
					
					//Readplane
					nbbits = readplane_conf(i,j,m_i, 2);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				i++;
				ncroot = ncx->frere; 
			}
			
		}
		m_i.close();

	//TRACE
	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->frere;
	}


		//Dquantif
		ncroot = m_treecoeff;
		while (ncroot!=NULL) {
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
		}
	}

	return;	
}



void zerotree::deletefils(noeudcoeff* pnc) {
	noeudcoeff* pfils, *pnnc;
	pfils = pnc->fils;
	while (pfils) {
		pnnc = pfils;
		pfils = pfils->frere;
		deletefils(pnnc);
		delete pnnc;
	}
	return;
}


void zerotree::diviseniveau(int niveau) {
	int it0, it1, it2, it3, itparent;

	//int nbz = (int)(pow(4,niveau));
	int nbz = 1 << (niveau + niveau);

	//int pas =  (int)((pow(4,m_depth+1)-1)/3);
	int pas =  ((1 << (m_depth + m_depth + 2)) -1)/3;

	//int zecart =  (int)((pow(4,m_depth-niveau)-1)/3);
	int zecart = ((1 << (m_depth + m_depth - niveau - niveau))-1)/3;

	for (int i=0; i<m_nbbasetriangle; i++) {
		itparent = i*pas + niveau;

		for (int j=0; j<nbz; j++) {
			it0 = itparent + 1;
			it1 = it0 + zecart;
			it2 = it1 + zecart;
			it3 = it2 + zecart;

			m_triangles[it0].tind = 0;
			m_triangles[it1].tind = 1;
			m_triangles[it2].tind = 2;
			m_triangles[it3].tind = 3;

			m_triangles_voisins[it0].vo1 = it3;
			m_triangles_voisins[it3].vo1 = it0;
			m_triangles_voisins[it0].vo2 = it1;
			m_triangles_voisins[it1].vo2 = it0;
			m_triangles_voisins[it0].vo3 = it2;
			m_triangles_voisins[it2].vo3 = it0;

			if (m_triangles[it0].v1==-1) {
				m_triangles[it0].v1 = m_nbvertex;
				m_triangles[it1].v1 = m_triangles[itparent].v1;
				m_triangles[it2].v1 = m_nbvertex;
				m_triangles[it3].v1 = m_nbvertex;
				propage(itparent,1,zecart);
				m_nbvertex++;
			}
		
			if (m_triangles[it0].v2==-1) {
				m_triangles[it0].v2 = m_nbvertex;
				m_triangles[it1].v2 = m_nbvertex;
				m_triangles[it2].v2 = m_triangles[itparent].v2;
				m_triangles[it3].v2 = m_nbvertex;
				propage(itparent,2,zecart);
				m_nbvertex++;
			}
		
			if (m_triangles[it0].v3==-1) {
				m_triangles[it0].v3 = m_nbvertex;
				m_triangles[it1].v3 = m_nbvertex;
				m_triangles[it2].v3 = m_nbvertex;
				m_triangles[it3].v3 = m_triangles[itparent].v3;
				propage(itparent,3,zecart);
				m_nbvertex++;
			}

			itparent = it3 + zecart + f(j);
		}
	}
	return;
}

void zerotree::diviseniveau(int niveau, unsigned char*niv) {
	int it0, it1, it2, it3, itparent;

	//int nbz = (int)(pow(4,niveau));
	int nbz = 1 << (niveau + niveau);

	//int pas =  (int)((pow(4,m_depth+1)-1)/3);
	int pas =  ((1 << (m_depth + m_depth + 2)) -1)/3;

	//int zecart =  (int)((pow(4,m_depth-niveau)-1)/3);
	int zecart = ((1 << (m_depth + m_depth - niveau - niveau))-1)/3;

	for (int i=0; i<m_nbbasetriangle; i++) {
		itparent = i*pas + niveau;

		for (int j=0; j<nbz; j++) {
			it0 = itparent + 1;
			it1 = it0 + zecart;
			it2 = it1 + zecart;
			it3 = it2 + zecart;

			m_triangles[it0].tind = 0;
			m_triangles[it1].tind = 1;
			m_triangles[it2].tind = 2;
			m_triangles[it3].tind = 3;

			m_triangles_voisins[it0].vo1 = it3;
			m_triangles_voisins[it3].vo1 = it0;
			m_triangles_voisins[it0].vo2 = it1;
			m_triangles_voisins[it1].vo2 = it0;
			m_triangles_voisins[it0].vo3 = it2;
			m_triangles_voisins[it2].vo3 = it0;

			if (m_triangles[it0].v1==-1) {
				m_triangles[it0].v1 = m_nbvertex;
				m_triangles[it1].v1 = m_triangles[itparent].v1;
				m_triangles[it2].v1 = m_nbvertex;
				m_triangles[it3].v1 = m_nbvertex;
				propage(itparent,1,zecart);
				niv[m_nbvertex]=m_depth-niveau;
				m_nbvertex++;
			}
		
			if (m_triangles[it0].v2==-1) {
				m_triangles[it0].v2 = m_nbvertex;
				m_triangles[it1].v2 = m_nbvertex;
				m_triangles[it2].v2 = m_triangles[itparent].v2;
				m_triangles[it3].v2 = m_nbvertex;
				propage(itparent,2,zecart);
				niv[m_nbvertex]=m_depth-niveau;
				m_nbvertex++;
			}
		
			if (m_triangles[it0].v3==-1) {
				m_triangles[it0].v3 = m_nbvertex;
				m_triangles[it1].v3 = m_nbvertex;
				m_triangles[it2].v3 = m_nbvertex;
				m_triangles[it3].v3 = m_triangles[itparent].v3;
				propage(itparent,3,zecart);
				niv[m_nbvertex]=m_depth-niveau;
				m_nbvertex++;
			}

			itparent = it3 + zecart + f(j);
		}
	}
	return;
}

int zerotree::setvoisin(int tind1, int tind2) {
	triangle* t1 = &(m_triangles[tind1]);
	triangle* t2 = &(m_triangles[tind2]);
	voisins* vo = &(m_triangles_voisins[tind1]);
	int* p_vo;

	if (vo->vo1==-1) {
		p_vo = NULL;
		if (egalecouple(t1->v1,t1->v2,t2->v1,t2->v2)) {
			p_vo = &(m_triangles_voisins[tind2].vo1);
		}
		if ((!p_vo)&&(egalecouple(t1->v1,t1->v2,t2->v2,t2->v3))) {
			p_vo = &(m_triangles_voisins[tind2].vo2);
		}
		if ((!p_vo)&&(egalecouple(t1->v1,t1->v2,t2->v3,t2->v1))) {
			p_vo = &(m_triangles_voisins[tind2].vo3);
		}
		if (p_vo) {
			vo->vo1 = tind2;
			*p_vo = tind1;
		}
	}

	if (vo->vo2==-1) {
		p_vo = NULL;
		if (egalecouple(t1->v2,t1->v3,t2->v1,t2->v2)) {
			p_vo = &(m_triangles_voisins[tind2].vo1);
		}
		if ((!p_vo)&&(egalecouple(t1->v2,t1->v3,t2->v2,t2->v3))) {
			p_vo = &(m_triangles_voisins[tind2].vo2);
		}
		if ((!p_vo)&&(egalecouple(t1->v2,t1->v3,t2->v3,t2->v1))) {
			p_vo = &(m_triangles_voisins[tind2].vo3);
		}
		if (p_vo) {
			vo->vo2 = tind2;
			*p_vo = tind1;
		}
	}

	if (vo->vo3==-1) {
		p_vo = NULL;
		if (egalecouple(t1->v3,t1->v1,t2->v1,t2->v2)) {
			p_vo = &(m_triangles_voisins[tind2].vo1);
		}
		if ((!p_vo)&&(egalecouple(t1->v3,t1->v1,t2->v2,t2->v3))) {
			p_vo = &(m_triangles_voisins[tind2].vo2);
		}
		if ((!p_vo)&&(egalecouple(t1->v3,t1->v1,t2->v3,t2->v1))) {
			p_vo = &(m_triangles_voisins[tind2].vo3);
		}
		if (p_vo) {
			vo->vo3 = tind2;
			*p_vo = tind1;
		}
	}
	return ((vo->vo1==-1)&&(vo->vo2==-1)&&(vo->vo3==-1));
}

void zerotree::propage(int tind, int vind, int ecart) {
	// indtv : indice du triangle voisin 
	int indtv;
	// indtvt0, indtvt1, indtvt2, indtvt3, indice des sous triangles du triangle d'indice indtv
	int indtvt0, indtvt1, indtvt2, indtvt3;
	// tinda, tindb indice des 2 sous triangles  de tind avec une arete commune a indtv 
	int tinda, tindb;
	// pva pvb voisins  de tinda, tindb
	int* pva, *pvb;
	// tindvi indice voisin
	int tindvi;
	// vi sommet partag
	int vi;

	switch (vind) {
		case 1:
			indtv = m_triangles_voisins[tind].vo2;
			vi = m_triangles[tind+1].v1;
			tindvi = m_triangles[tind].v2;
			tinda = tind + 1 + 2*ecart; 
			tindb = tind + 1 + 3*ecart;
			pva = &(m_triangles_voisins[tinda].vo1);
			pvb = &(m_triangles_voisins[tindb].vo3);
			break;
		case 2:
			indtv = m_triangles_voisins[tind].vo3;
			vi = m_triangles[tind+1].v2;
			tindvi = m_triangles[tind].v3;
			tinda = tind + 1 + 3*ecart; 
			tindb = tind + 1 + 1*ecart; 
			pva = &(m_triangles_voisins[tinda].vo2);
			pvb = &(m_triangles_voisins[tindb].vo1);
			break;
		case 3:
			indtv = m_triangles_voisins[tind].vo1;
			vi = m_triangles[tind+1].v3;
			tindvi = m_triangles[tind].v1;
			tinda = tind + 1 + 1*ecart; 
			tindb = tind + 1 + 2*ecart; 
			pva = &(m_triangles_voisins[tinda].vo3);
			pvb = &(m_triangles_voisins[tindb].vo2);
			break;
		default:
			break;
	}

	//il peut manquer des voisins a un triangle
	if (indtv==-1) {
		return;
	}

	indtvt0 = indtv + 1;
	indtvt1 = indtvt0 + ecart;
	indtvt2 = indtvt1 + ecart;
	indtvt3 = indtvt2 + ecart;

	if (m_triangles_voisins[indtv].vo1 == tind) {
		m_triangles[indtvt0].v3 = vi;
		m_triangles[indtvt1].v3 = vi;
		m_triangles[indtvt2].v3 = vi;
		m_triangles[indtvt3].v3 = m_triangles[indtv].v3;

		if (m_triangles[indtv].v1==tindvi) {
			m_triangles_voisins[indtvt1].vo3 = tinda;
			m_triangles_voisins[indtvt2].vo2 = tindb;
			*pva = indtvt1;
			*pvb = indtvt2;
		}
		else {
			m_triangles_voisins[indtvt1].vo3 = tindb;
			m_triangles_voisins[indtvt2].vo2 = tinda;
			*pvb = indtvt1;
			*pva = indtvt2;
		}
		return;
	}

	if (m_triangles_voisins[indtv].vo2 == tind) {
		m_triangles[indtvt0].v1 = vi;
		m_triangles[indtvt1].v1 = m_triangles[indtv].v1;
		m_triangles[indtvt2].v1 = vi;
		m_triangles[indtvt3].v1 = vi;

		if (m_triangles[indtv].v2==tindvi) {
			m_triangles_voisins[indtvt2].vo1 = tinda;
			m_triangles_voisins[indtvt3].vo3 = tindb;
			*pva = indtvt2;
			*pvb = indtvt3;
		}
		else {
			m_triangles_voisins[indtvt2].vo1 = tindb;
			m_triangles_voisins[indtvt3].vo3 = tinda;
			*pvb = indtvt2;
			*pva = indtvt3;
		}
		return;
	}

	if (m_triangles_voisins[indtv].vo3 == tind) {
		m_triangles[indtvt0].v2 = vi;
		m_triangles[indtvt1].v2 = vi;
		m_triangles[indtvt2].v2 = m_triangles[indtv].v2;
		m_triangles[indtvt3].v2 = vi;

		if (m_triangles[indtv].v3==tindvi) {
			m_triangles_voisins[indtvt3].vo2 = tinda;
			m_triangles_voisins[indtvt1].vo1 = tindb;
			*pva = indtvt3;
			*pvb = indtvt1;
		}
		else {
			m_triangles_voisins[indtvt3].vo2 = tindb;
			m_triangles_voisins[indtvt1].vo1 = tinda;
			*pvb = indtvt3;
			*pva = indtvt1;
		}
		return;
	}
}

int zerotree::f(int x) {
	x++;
	int res = 0;
	while (x%4==0) {
		res++;
		x=x/4;
	}
	return res;
}

int zerotree::egalecouple(int a1, int b1, int a2, int b2) {
	if ((a1==a2)&&(b1==b2)) {
		return 1;
	}
	if ((a1==b2)&&(b1==a2)) {
		return 1;
	}
	return 0;
}


int zerotree::GetWType() {
	return 2*(m_EC->Wtype[0])+m_EC->Wtype[1];
}

int zerotree::read_encode(DatMesh*DM) {
	ifstream f;
	char buf[500];
	char* extractbuf;
	char* tok;

	if (DM==NULL) {
		return 0;
	}

	// destruction maillage prcdent
	if (m_vertexparent) {
		delete m_vertexparent;
	}
	if (m_tabvertex) {
		delete m_tabvertex;
	}
	if (m_triangles) {
		delete m_triangles;
	}

	//extraction nombre de niveau du maillage
	m_depth = (int)DM->V[0][0]-1;

	//extraction nombre de vertex du maillage
	m_nbvertex = DM->NbVertices;

	//creation m_tabvertex et m_vertexparent
	m_tabvertex = new vertex[m_nbvertex];
	m_vertexparent = new int[m_nbvertex];
	for (int i=0; i<m_nbvertex; i++) {
		m_vertexparent[i] = i;
	}

	//extraction nombre de triangles
	m_nbtriangle = DM->NbFaces;

	//cration m_triangles;
	m_triangles = new triangle[m_nbtriangle];
	
	//lecture des vertex 
	for (i=0; i<DM->NbVertices; i++) {

		m_tabvertex[i].depth = DM->V[i][0];				
		m_tabvertex[i].x = DM->V[i][1];
		m_tabvertex[i].y = DM->V[i][2];
		m_tabvertex[i].z = DM->V[i][3];

	}

	//lecture des triangles
	m_nbbasetriangle=0;
	for (i=0; i<DM->NbFaces; i++) {

		if (DM->F[i][1]==1)
			m_nbbasetriangle++;
		m_triangles[i].tind =DM->F[i][0];
		m_triangles[i].root =DM->F[i][1];		
		m_triangles[i].v1 =DM->F[i][2];		
		m_triangles[i].v2 =DM->F[i][3];			
		m_triangles[i].v3 =DM->F[i][4];			
	
	}

	

	buildvertexparent();
	return 1;
}

int zerotree::read_decode(DatMesh*DM) {
	ifstream f;
	char buf[500];
	char* extractbuf;
	char* tok;

	if (DM==NULL) {
		return 0;
	}

	// destruction maillage prcdent
	if (m_vertexparent) {
		delete m_vertexparent;
	}
	if (m_tabvertex) {
		delete m_tabvertex;
	}
	if (m_triangles) {
		delete m_triangles;
	}

	//extraction nombre de niveau du maillage
	m_depth = (int)DM->V[0][0]-1;

	//extraction nombre de vertex du maillage
	m_nbvertex = DM->NbVertices;

	//creation m_tabvertex et m_vertexparent
	m_tabvertex = new vertex[m_nbvertex];
	m_vertexparent = new int[m_nbvertex];
	for (int i=0; i<m_nbvertex; i++) {
		m_vertexparent[i] = i;
	}

	//extraction nombre de triangles
	m_nbtriangle = DM->NbFaces;

	//cration m_triangles;
	m_triangles = new triangle[m_nbtriangle];
	
	//lecture des vertex 
	for (i=0; i<DM->NbVertices; i++) {

		m_tabvertex[i].depth = DM->V[i][0];				
		m_tabvertex[i].x = 0;
		m_tabvertex[i].y = 0;
		m_tabvertex[i].z = 0;

	}

	//lecture des triangles
	m_nbbasetriangle=0;
	for (i=0; i<DM->NbFaces; i++) {

		if (DM->F[i][1]==1)
			m_nbbasetriangle++;
		m_triangles[i].tind =DM->F[i][0];
		m_triangles[i].root =DM->F[i][1];		
		m_triangles[i].v1 =DM->F[i][2];		
		m_triangles[i].v2 =DM->F[i][3];			
		m_triangles[i].v3 =DM->F[i][4];			
	
	}

	

	buildvertexparent();
	return 1;
}

int zerotree::read_encode(char* nomf) {
	ifstream f;
	this->m_nomf = nomf;
	char buf[500];
	char* extractbuf;
	char* tok;

	if (nomf==NULL) {
		return 0;
	}

	// destruction maillage prcdent
	if (m_vertexparent) {
		delete m_vertexparent;
	}
	if (m_tabvertex) {
		delete m_tabvertex;
	}
	if (m_triangles) {
		delete m_triangles;
	}

	f.open(nomf);

	//extraction nombre de niveau du maillage
	f.getline(buf,500);
	f.getline(buf,500);
	extractbuf = &(buf[5]);
	tok = strtok(extractbuf," \t");
	m_depth = atoi(tok);

	f.getline(buf,500);

	//extraction nombre de vertex du maillage
	m_nbvertex = 0;
	int t=1;
	while (t) {
		f.getline(buf,500);
		if (buf[0]=='T') {
			t=0;
		}
		else {
			if (buf[0]!='\0') {
				m_nbvertex++;
			}
		}
	}

	//creation m_tabvertex et m_vertexparent
	m_tabvertex = new vertex[m_nbvertex];
	m_vertexparent = new int[m_nbvertex];
	for (int i=0; i<m_nbvertex; i++) {
		m_vertexparent[i] = i;
	}

	//extraction nombre de triangles
	m_nbtriangle = 0;
	t=1;
	while (t) {
		f.getline(buf,500);
		if (buf[0]!='n') {
			t=0;
		}
		else {
			m_nbtriangle++;
		}
	}

	//cration m_triangles;
	m_triangles = new triangle[m_nbtriangle];
	f.close();
	
	f.open(nomf);
	f.getline(buf,500);
	f.getline(buf,500);
	f.getline(buf,500);

	//lecture des vertex 
	t = 1;
	i = 0;
	while (t) {
		f.getline(buf,500);
		if (buf[0]=='T') {
			t=0;
		}
		else {
			if (buf[0]!='\0') {
				extractbuf = &(buf[0]);
				tok = strtok(extractbuf, " \t");
				m_tabvertex[i].depth = atoi(tok);			

				tok = strtok(NULL, " \t");
				m_tabvertex[i].x = (float)(atof(tok));			

				tok = strtok(NULL, " \t");
				m_tabvertex[i].y = (float)(atof(tok));			

				tok = strtok(NULL, " \t");
				m_tabvertex[i].z = (float)(atof(tok));			

				i++;
			}
		}
	}

	//lecture des triangles
	t=1;
	i=0;
	while (t) {
		f.getline(buf,500);
		if (buf[0]!='n') {
			t=0;
		}
		else {
			extractbuf = &(buf[5]);
			tok = strtok(extractbuf, " \t");
			m_triangles[i].tind =atoi(tok);			

			tok = strtok(NULL, " \t");
			m_triangles[i].root =atoi(tok);			

			tok = strtok(NULL, " \t");
			m_triangles[i].v1 =atoi(tok);			

			tok = strtok(NULL, " \t");
			m_triangles[i].v2 =atoi(tok);			

			tok = strtok(NULL, " \t");
			m_triangles[i].v3 =atoi(tok);			

			i++;
		}
	}
	f.close();

	buildvertexparent();
	return 1;
}

int zerotree::writeplanem_conf(int i, unsigned char n, mbitstream& obs, int flag, WaveRep* WR, Mesh* BM) {

	//returns the number of bits written

	int resSN;
	int nbw=0;
	entry* pentry, *emove, *newentry;
	noeudcoeff* p, *p2;
	liste_entry *m_pLSP, *m_pLSPadd, *m_pLIP, *m_pLIS;


	//Check coherency
	switch(flag) {
	case 0:
		if (m_EC->NbBPX-n!=m_tabsub_zerotree[i].m_lastplanbitX) {
			return 0;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPX);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPXadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPX);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISX);
		break;
	case 1:
		if (m_EC->NbBPY-n!=m_tabsub_zerotree[i].m_lastplanbitY) {
			return 0;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPY);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPYadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPY);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISY);
		break;
	case 2:
		if (m_EC->NbBPZ-n!=m_tabsub_zerotree[i].m_lastplanbitZ) {
			return 0;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPZ);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPZadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPZ);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISZ);
		break;
	}
	#ifdef LOGZT
	fprintf(logfile, "Etat de LSP:\n");
	entry* parcours=m_pLSP->m_tete;
	while (parcours) {
		int ind=parcours->pnc->ind;
		fprintf(logfile, "%d, %d, %d, %d\n", WR->tab[ind]->F0, WR->tab[ind]->alpha, WR->tab[ind]->beta, WR->tab[ind]->gamma);
		parcours=parcours->next;
	}

	fprintf(logfile, "Etat de LIP:\n");
	parcours=m_pLIP->m_tete;
	while (parcours) {
		int ind=parcours->pnc->ind;
		fprintf(logfile, "%d, %d, %d, %d\n", WR->tab[ind]->F0, WR->tab[ind]->alpha, WR->tab[ind]->beta, WR->tab[ind]->gamma);
		parcours=parcours->next;
	}

	fprintf(logfile, "Etat de LIS:\n");
	parcours=m_pLIS->m_tete;
	while (parcours) {
		int ind=parcours->pnc->ind;
		fprintf(logfile, "%d, %d, %d, %d\n", WR->tab[ind]->F0, WR->tab[ind]->alpha, WR->tab[ind]->beta, WR->tab[ind]->gamma);
		parcours=parcours->next;
	}
	#endif

	//2 Sorting Pass
	//2.1
	pentry = m_pLIP->startiter();
			
	while (pentry) {
			
		resSN = SN(n,pentry->pnc, flag);

		//output Sn(i,j)
		obs.writebit(resSN);
		#ifdef LOGZT
		fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
		#endif

		nbw++;
				
		if (resSN==1) {
					
			//output sign of ci,j 2.1.2
			obs.writebit(sign(pentry->pnc));
			#ifdef LOGZT
			fprintf(logfile, "Son signe est %d\n", sign(pentry->pnc));
			#endif
			nbw++;
					
			//move entry to LSP
			#ifdef LOGZT
			fprintf(logfile, "move entry to LSP\n");
			#endif
			emove = pentry;
			pentry = m_pLIP->removeiter();
			m_pLSPadd->add(emove);
		}
		else {
			pentry = m_pLIP->iter();
		}
	}
			
	//2.2
	pentry = m_pLIS->startiter();
	while (pentry) {
			
		//2.2.1
		if (pentry->entrytype==1) {
				
			resSN = SND(n,pentry, flag);
					
			//output Sn(D(i,j))
			obs.writebit(resSN);
			#ifdef LOGZT
			fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
			#endif
			nbw++;
					
			if (resSN==1) {
						
				p = pentry->pnc->fils;
				#ifdef LOGZT
				fprintf(logfile, "p=pentry->fils\n");
				if (p)
				fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
				#endif
				while (p) {
						
					resSN = SN(n,p, flag);
						
					//output Sn(k,l)
					obs.writebit(resSN);
					#ifdef LOGZT
					fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
					#endif
					nbw++;
							
					if (resSN==1) {
								
						//output sign of ck,l
						obs.writebit(sign(p));
						#ifdef LOGZT
						fprintf(logfile, "Son signe est %d\n", sign(p));
						#endif
						nbw++;
								
						//add entry to LSP
						entry* ne = new entry;
						ne->pnc = p;
						m_pLSPadd->add(ne);
					}
					else {
						//add entry to the end of the LIP
						entry* ne = new entry;
						ne->pnc = p;
						m_pLIP->addend(ne);
					}
					p = p->frere;
					#ifdef LOGZT
					fprintf(logfile, "p=p->frere\n");
					if (p)
					fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
					#endif
				}
						
				//L(i,j) vide?
				if (Lijvide(pentry)) {
					#ifdef LOGZT
					fprintf(logfile, "Lijvide\n");
					#endif
						
					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					//move entry to the end of LIS as an entry of type B
					#ifdef LOGZT
					fprintf(logfile, "Lijnonvide\n");
					#endif
					pentry->entrytype = 2;
					resSN = SNL(n,pentry, flag);
					obs.writebit(resSN);
					#ifdef LOGZT
					fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
					#endif
					nbw++;

					if (resSN) {
						p2 = pentry->pnc->fils;
						#ifdef LOGZT
						fprintf(logfile, "pentry->fils\n");
						if (p)
						fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
						#endif
						while (p2) {
							entry* ne = new entry;
							ne->entrytype = 1;
							ne->pnc = p2;
							m_pLIS->addend(ne);
							p2 = p2->frere;
							#ifdef LOGZT
							fprintf(logfile, "pentry->frere\n");
							if (p)
							fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
							#endif
						}
						pentry = m_pLIS->removeiter();
					}
					else {
						emove = pentry;
						pentry = m_pLIS->removeiter();
						m_pLIS->addend(emove);
					}
				}
			}
			else {
				pentry = m_pLIS->iter();
			}
		}
				
		//2.2.2
		else {
			if (pentry->entrytype==2) {

				resSN = SNL(n,pentry, flag);
						
				//output Sn(L(i,j))
				obs.writebit(resSN);
				#ifdef LOGZT
				fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
				#endif
				nbw++;
						
				if (resSN) {

					p = pentry->pnc->fils;
					#ifdef LOGZT
					fprintf(logfile, "pentry->fils\n");
					if (p)
					fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
					#endif
					while (p) {
						// add to the end of LIS as an entry of type A
						newentry = new entry;
						newentry->entrytype = 1;
						newentry->pnc = p;
						m_pLIS->addend(newentry);
						p = p->frere;
						#ifdef LOGZT
						fprintf(logfile, "pentry->frere\n");
						if (p)
						fprintf(logfile, "%d %d %d %d\n", WR->tab[p->ind]->F0, WR->tab[p->ind]->alpha, WR->tab[p->ind]->beta, WR->tab[p->ind]->gamma);
						#endif
					}

					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					pentry = m_pLIS->iter();
				}
			}	
		}
	}

	//3 Refinement Pass
	pentry = m_pLSP->startiter();

	//active le plan de bits correspondant ,& logique avec puissance de 2
	unsigned short int bits = 32768;
	bits = bits >> (1+m_maxn-n);

	while (pentry) {
		unsigned short int usi = pentry->pnc->quantvalue;

		//output the nth most significant bit of |ci,j|
		obs.writebit(((usi&bits)>0));
		#ifdef LOGZT
		fprintf(logfile, "refinement output Sn(i,j) = %d\n", ((usi&bits)>0));
		#endif
		nbw++;
		pentry = m_pLSP->iter();	
	}
		
	m_pLSP->append(*m_pLSPadd);

	switch(flag) {
	case 0:
		m_tabsub_zerotree[i].m_lastplanbitX++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->LBP) {
			m_tabsub_zerotree[i].m_LIPX.reset();
			m_tabsub_zerotree[i].m_LISX.reset();
			m_tabsub_zerotree[i].m_LSPX.reset();
			m_tabsub_zerotree[i].m_LSPXadd.reset();
		}
		break;
	case 1:
		m_tabsub_zerotree[i].m_lastplanbitY++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->LBP) {
			m_tabsub_zerotree[i].m_LIPY.reset();
			m_tabsub_zerotree[i].m_LISY.reset();
			m_tabsub_zerotree[i].m_LSPY.reset();
			m_tabsub_zerotree[i].m_LSPYadd.reset();
		}
		break;
	case 2:
		m_tabsub_zerotree[i].m_lastplanbitZ++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->LBP) {
			m_tabsub_zerotree[i].m_LIPZ.reset();
			m_tabsub_zerotree[i].m_LISZ.reset();
			m_tabsub_zerotree[i].m_LSPZ.reset();
			m_tabsub_zerotree[i].m_LSPZadd.reset();
		}
		break;
	}
	
	return nbw;
}









void zerotree::writeDSI() {

	char drive[_MAX_DRIVE];   
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];   
	char ext[_MAX_EXT];
	char nombase[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char nomfichier[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	FILE *f;

	//extraction nom du fichier sans extension
	_splitpath(m_nomf, drive, dir, fname, ext);
	strcpy(nombase,drive);
	strcat(nombase,dir);
	strcat(nombase,fname);
	//construction nom fichier
	strcat(nombase,".dsi");
	f=fopen(nombase, "w");

	fprintf(f, "%d\n", m_EC->hasScaleCoeff);
	fprintf(f, "%d\n", m_EC->NbBpSC);
	fprintf(f, "%d\n", m_EC->NbBPX+1);
	fprintf(f, "%d\n", m_EC->NbBPY+1);
	fprintf(f, "%d\n", m_EC->NbBPZ+1);
	fprintf(f, "%d\n", 0);
	fprintf(f, "%d\n", 1);
	fprintf(f, "%d\n", m_EC->lift);
	fprintf(f, "%d\n", m_depth);
	//To be added
	fprintf(f, "%f\n", m_maxcoeffX);
	fprintf(f, "%f\n", m_maxcoeffY);
	fprintf(f, "%f\n", m_maxcoeffZ);

	fclose(f);
}

void zerotree::writeDSI(char *filename_wo_extention) {

	char nombase[512];
	FILE *f;

	
	//construction nom fichier
	strcpy(nombase, filename_wo_extention);
	strcat(nombase,".dsi");
	f=fopen(nombase, "w");

	fprintf(f, "%d\n", m_EC->hasScaleCoeff);
	fprintf(f, "%d\n", m_EC->NbBpSC);
	fprintf(f, "%d\n", m_EC->NbBPX+1);
	fprintf(f, "%d\n", m_EC->NbBPY+1);
	fprintf(f, "%d\n", m_EC->NbBPZ+1);
	fprintf(f, "%d\n", 0);
	fprintf(f, "%d\n", 1);
	fprintf(f, "%d\n", m_EC->lift);
	fprintf(f, "%d\n", m_EC->NumberOfLevels);
	//To be added
	fprintf(f, "%f\n", m_maxcoeffX);
	fprintf(f, "%f\n", m_maxcoeffY);
	fprintf(f, "%f\n", m_maxcoeffZ);

	fclose(f);
}




void zerotree::SetName(char*s) {

	if (m_nomf==NULL)
		m_nomf=(char*)malloc(512*sizeof(char));
	strcpy(m_nomf, s);
}

Mesh* zerotree::GetBaseMesh() {

	Mesh *res=(Mesh*)malloc(sizeof(Mesh));
	int i,j;

	//Sommets
	i=0;
	while (m_tabvertex[i].depth==m_depth+1) {
		res->Vertex[i][0]=m_tabvertex[i].x;	
		res->Vertex[i][1]=m_tabvertex[i].y;
		res->Vertex[i][2]=m_tabvertex[i].z;
		i++;
	}

	res->NbVertex=i;

	//Facettes
	j=0;
	for (i = 0; i<m_nbtriangle; i++) {
		if (m_triangles[i].root) {
			res->Face[j][0]=m_triangles[i].v1;
			res->Face[j][1]=m_triangles[i].v2;
			res->Face[j][2]=m_triangles[i].v3;
			j++;
		}
	}

	res->NbFace=j;

	return res;

}
void zerotree::writeNode() {
	char drive[_MAX_DRIVE];   
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];   
	char ext[_MAX_EXT];
	char nombase[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char nomfichier[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	char nomdsi[_MAX_DRIVE+_MAX_DIR+_MAX_FNAME];
	FILE *f;
	int i;
	int j;

	//extraction nom du fichier sans extension
	_splitpath(m_nomf, drive, dir, fname, ext);
	strcpy(nombase,drive);
	strcat(nombase,dir);
	strcat(nombase,fname);
	//construction nom fichier
	strcpy(nomfichier, nombase);
	strcpy(nomdsi, nombase);
	strcat(nombase,".wrl");
	strcat(nomfichier,".mp4");
	strcat(nomdsi,".dsi");
	f=fopen(nombase, "w");

	//Ecriture du corps
	fprintf(f, "#VRML V2.0 utf8\n\n");
	fprintf(f, "Shape {\n");
	fprintf(f, "	appearance Appearance {\n");
	fprintf(f, "		material Material {\n");
	fprintf(f, "			diffuseColor 1.0 .8 .3\n");
	fprintf(f, "		}\n		}\n");
	fprintf(f, "	geometry WaveletSubdivisionSurface {\n");
	fprintf(f, "		DecoderSpecificInfo ""%s""\n", nomdsi);
	fprintf(f, "		WSBitstream ""%s""\n", nomfichier);
	fprintf(f, "			coord Coordinate {\n");
	fprintf(f, "				point [\n");

	//Sommets
	i=0;
	while (m_tabvertex[i].depth==m_depth+1) {
		fprintf(f, "%f %f %f", m_tabvertex[i].x, m_tabvertex[i].y, m_tabvertex[i].z);
		i++;
		if (m_tabvertex[i].depth==m_depth+1)
			fprintf(f, ",\n");
		else
			fprintf(f, "\n");
	}

	fprintf(f, "				]			\n}\n");
	fprintf(f, "			coordIndex [\n");

	//Facettes
	j=0;
	for (i = 0; i<m_nbtriangle; i++) {
		if (m_triangles[i].root) {
			fprintf(f, "%d, %d, %d, -1,\n", m_triangles[i].v1, m_triangles[i].v2, m_triangles[i].v3);
		}
	}


	fprintf(f, "			]\n			}\n		}\n");

	fclose(f);
}

int zerotree::permuteOffspring() {
	//rem : there are always at least 3 sons. 4rth one is fixed if exists.
	//rem : first son has brother NULL;

	noeudcoeff* ncroot;
	noeudcoeff* tmp1;
	noeudcoeff* tmp2;
	noeudcoeff* tmp3;


	ncroot=m_treecoeff;
	while (ncroot) {

		//permutation
		if (ncroot->fils->frere->frere->frere==NULL) {//Only 3 sons
			tmp3=ncroot->fils;
			tmp2=tmp3->frere;
			tmp1=tmp2->frere;

			ncroot->fils=tmp1;
			ncroot->fils->frere=tmp3;
			ncroot->fils->frere->frere=tmp2;
			ncroot->fils->frere->frere->frere=NULL;
		} else {//4 sons
			tmp3=ncroot->fils->frere;
			tmp2=tmp3->frere;
			tmp1=tmp2->frere;

			ncroot->fils->frere=tmp1;
			ncroot->fils->frere->frere=tmp3;
			ncroot->fils->frere->frere->frere=tmp2;
			ncroot->fils->frere->frere->frere->frere=NULL;
		}
			
		//iteration
		ncroot=ncroot->frere;
	}

	return 1;
}

int zerotree::encodemp4_conf(Mesh *BM, WaveRep *WR, unsigned char* bufferout, int& bufsize, int bufmaxsize, BSD* BSDesc) {

	//OoC but slow

	int nbzt;
	int i, j;
	mbitstream *m_o=new mbitstream(bufferout);

	//init
	m_treecoeff=NULL;

	// Determination des nombres de zerotree et des coeffs max. 
	m_maxcoeffX = m_maxcoeffY = m_maxcoeffZ = -1;
	for (nbzt=0, i=0; i<WR->NbCoeff; i++) {
		//nbzt
		if ((WR->tab[i]->alpha+WR->tab[i]->beta+WR->tab[i]->gamma)==2)
			nbzt++;
		//max
		if (WR->tab[i]->x>m_maxcoeffX)
			m_maxcoeffX=WR->tab[i]->x;
		if (WR->tab[i]->y>m_maxcoeffY)
			m_maxcoeffY=WR->tab[i]->y;
		if (WR->tab[i]->z>m_maxcoeffZ)
			m_maxcoeffZ=WR->tab[i]->z;
	}

	//On remplit le m_EC avec les max
	m_EC->Xmax=m_maxcoeffX;
	m_EC->Ymax=m_maxcoeffY;
	m_EC->Zmax=m_maxcoeffZ;

	//On alloue les BSD roots
	BSDesc->roots=(BSDRoot**)malloc(sizeof(BSDRoot*)*nbzt);

	// Pour chaque indice de zerotree
	for(i=0; i<nbzt; i++) {


		//On alloue les BSD bitplanes
		BSDesc->roots[i]=(BSDRoot*)malloc(sizeof(BSDRoot));
		BSDesc->roots[i]->BPD=(BPDesc**)malloc(sizeof(BPDesc*)*(m_EC->LBP+1));
		for (j=0; j<m_EC->LBP+1; j++) {
			BSDesc->roots[i]->BPD[j]=(BPDesc*)malloc(sizeof(BPDesc));
		}
		BSDesc->nbroots=nbzt;
		BSDesc->nbbp=m_EC->LBP+1;


		// On cre la hirarchie grce au WR
		createZTHierarchy(i, WR, nbzt, BM);

		// On encode le zerotree et on gnre le BSD
		encodeAndBSD(i, BM, WR, m_o, bufsize, bufmaxsize, BSDesc);

		deleteZTHierarchy(m_treecoeff);
		m_treecoeff=NULL;

	}
	
	// Suivant les options, on cre le bitstream AFX avec le BSD et la sortie binaire
	//makeAFXstream();

	bufferout=m_o->octets;

	return 0;
}

int zerotree::deleteZTHierarchy(noeudcoeff* p) {

	//algo:
	//1. Si tu as des fils, libre les
	//2. Si tu as un frre, libre le
	//3. Tu n'as plus ni frere ni fils, libre toi


	//1.
	if (p->fils!=NULL) {
		deleteZTHierarchy(p->fils);
		p->fils=NULL;
	}

	//2.
	if (p->frere!=NULL) {
		deleteZTHierarchy(p->frere);
		p->frere=NULL;
	}
	
	//3. 
	free(p);


	return 0;
}


int zerotree::createZTHierarchy(int root, WaveRep* WR, int nbzt, Mesh *BM) {

	int currentBary[4];
	int i;
	int direction;
	int cnt;
	noeudcoeff* treeroot, *treeroot1, *treeroot2, *treeroot3;
	noeudcoeff* tmp;

	// On repre la racine et sa direction
	cnt=0;
	for (i=0; i< WR->NbCoeff && cnt<=root; i++) {
		if (WR->tab[i]->alpha+WR->tab[i]->beta+WR->tab[i]->gamma==2)
			cnt++;
	}
	i--;

	currentBary[0]=WR->tab[i]->F0;
	currentBary[1]=WR->tab[i]->alpha;
	currentBary[2]=WR->tab[i]->beta;
	currentBary[3]=WR->tab[i]->gamma;

	direction=(currentBary[1]==0)+2*(currentBary[2]==0)+3*(currentBary[3]==0)-1;

	// On cre la racine et on la met  la fin de la liste des roots
	tmp=m_treecoeff;
	if (m_treecoeff)
		while (tmp->frere!=NULL)
			tmp=tmp->frere;

	treeroot=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	treeroot->ind=root;
	treeroot->coeffvalue=&WR->tab[i]->x;
	treeroot->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	treeroot->frere=NULL;
	treeroot->fils=NULL;
	if (tmp) 
		tmp->frere=treeroot;
	else m_treecoeff=treeroot;
	tmp=treeroot;
	treeroot1=tmp;

	treeroot=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	treeroot->ind=root;
	treeroot->coeffvalue=&WR->tab[i]->y;
	treeroot->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	treeroot->frere=NULL;
	treeroot->fils=NULL;
	tmp->frere=treeroot;
	tmp=treeroot;
	treeroot2=tmp;

	treeroot=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	treeroot->ind=root;
	treeroot->coeffvalue=&WR->tab[i]->z;
	treeroot->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	treeroot->frere=NULL;
	treeroot->fils=NULL;
	tmp->frere=treeroot;
	tmp=treeroot;
	treeroot3=tmp;

	
	// On trouve et on ajoute ses quatre fils et on rcurse
	iterbary(currentBary, BM, WR, treeroot1, treeroot2, treeroot3);

	return 0;
}

#ifdef Cette_mthode_contient_un_bug
int zerotree::iterbary(int bary[4], Mesh *BM, WaveRep *WR, int direction, noeudcoeff* fatherx, noeudcoeff* fathery, noeudcoeff* fatherz) {
	
	int level=bary[1]+bary[2]+bary[3];
	int found, i;
	int bary1[4], bary2[4], bary3[4], bary4[4];
	noeudcoeff* f1x, *f2x, *f3x, *f4x;
	noeudcoeff* f1y, *f2y, *f3y, *f4y;
	noeudcoeff* f1z, *f2z, *f3z, *f4z;
	float x, y, z;
	int al, be, ga, F0;
	int only3=0;

	switch(direction) {
		case 0:	// tourns vers alpha
			//premier fils
			bary1[0]=bary[0];
			bary1[1]=bary[1];
			bary1[2]=bary[2]+level;
			bary1[3]=bary[3];

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=bary[1];
			bary2[2]=bary[2];
			bary2[3]=bary[3]+level;

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=bary[1]+level;
			bary3[2]=bary[2];
			bary3[3]=bary[3];

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=bary[1]-level;
			bary4[2]=bary[2];
			bary4[3]=bary[3];

			break;

			case 1:	// tourns vers beta
			//premier fils
			bary1[0]=bary[0];
			bary1[1]=bary[1];
			bary1[2]=bary[2];
			bary1[3]=bary[3]+level;

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=bary[1]+level;
			bary2[2]=bary[2];
			bary2[3]=bary[3];

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=bary[1];
			bary3[2]=bary[2]+level;
			bary3[3]=bary[3];

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=bary[1];
			bary4[2]=bary[2]-level;
			bary4[3]=bary[3];

			break;

			case 2:	// tourns vers gamma
			//premier fils
			bary1[0]=bary[0];
			bary1[1]=bary[1]+level;
			bary1[2]=bary[2];
			bary1[3]=bary[3];

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=bary[1];
			bary2[2]=bary[2]+level;
			bary2[3]=bary[3];

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=bary[1];
			bary3[2]=bary[2];
			bary3[3]=bary[3]+level;

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=bary[1];
			bary4[2]=bary[2];
			bary4[3]=bary[3]-level;

			break;
		
	}


	
	// Rtablissement des coordonnes ngatives
	if(bary1[1]<0||bary1[2]<0||bary1[3]<0) {
		changeFace(bary1, BM, &al, &be, &ga, &F0);
		bary1[0]=F0;
		bary1[1]=al;
		bary1[2]=be;
		bary1[3]=ga;
	}

	if(bary2[1]<0||bary2[2]<0||bary2[3]<0) {
		changeFace(bary2, BM, &al, &be, &ga, &F0);
		bary2[0]=F0;
		bary2[1]=al;
		bary2[2]=be;
		bary2[3]=ga;
	}

	if(bary3[1]<0||bary3[2]<0||bary3[3]<0) {
		changeFace(bary3, BM, &al, &be, &ga, &F0);
		bary3[0]=F0;
		bary3[1]=al;
		bary3[2]=be;
		bary3[3]=ga;
	}

	if(bary4[1]<0||bary4[2]<0||bary4[3]<0) {
		only3=changeFace(bary4, BM, &al, &be, &ga, &F0);
		bary4[0]=F0;
		bary4[1]=al;
		bary4[2]=be;
		bary4[3]=ga;
	}

	// Recherche dans le WR
	if (!only3) {
		found=0;
		for (i=0; i<WR->NbCoeff; i++) {
			if (WR->tab[i]->F0==bary4[0] && WR->tab[i]->alpha==bary4[1] && WR->tab[i]->beta==bary4[2] && WR->tab[i]->gamma==bary4[3]) {
				found=1;
				break;
			}
		}
		if (!found)
			return 1;	// Le coefficient n'a pas t trouv
		


		// Ajout dans la hirarchie

		f4x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4x->ind=i;
		f4x->coeffvalue=&WR->tab[i]->x;
		f4x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
		f4x->frere=NULL;
		f4x->fils=NULL;
		fatherx->fils=f4x;

		f4y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4y->ind=i;
		f4y->coeffvalue=&WR->tab[i]->y;
		f4y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
		f4y->frere=NULL;
		f4y->fils=NULL;
		fathery->fils=f4y;

		f4z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4z->ind=i;
		f4z->coeffvalue=&WR->tab[i]->z;
		f4z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
		f4z->frere=NULL;
		f4z->fils=NULL;
		fatherz->fils=f4z;
	}
	else 
		f4x=f4y=f4z=NULL;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
		if (WR->tab[i]->F0==bary3[0] && WR->tab[i]->alpha==bary3[1] && WR->tab[i]->beta==bary3[2] && WR->tab[i]->gamma==bary3[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f3x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3x->ind=i;
	f3x->coeffvalue=&WR->tab[i]->x;
	f3x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	if (only3) 
		f3x->frere=NULL;
	else
		f3x->frere=f4x;
	f3x->fils=NULL;
	fatherx->fils=f3x;

	f3y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3y->ind=i;
	f3y->coeffvalue=&WR->tab[i]->y;
	f3y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	if (only3) 
		f3y->frere=NULL;
	else
		f3y->frere=f4y;
	f3y->fils=NULL;
	fathery->fils=f3y;

	f3z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3z->ind=i;
	f3z->coeffvalue=&WR->tab[i]->z;
	f3z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	if (only3) 
		f3z->frere=NULL;
	else
		f3z->frere=f4z;
	f3z->fils=NULL;
	fatherz->fils=f3z;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
		if (WR->tab[i]->F0==bary2[0] && WR->tab[i]->alpha==bary2[1] && WR->tab[i]->beta==bary2[2] && WR->tab[i]->gamma==bary2[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f2x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2x->ind=i;
	f2x->coeffvalue=&WR->tab[i]->x;
	f2x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	f2x->frere=f3x;
	f2x->fils=NULL;
	fatherx->fils=f2x;
	
	f2y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2y->ind=i;
	f2y->coeffvalue=&WR->tab[i]->y;
	f2y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	f2y->frere=f3y;
	f2y->fils=NULL;
	fathery->fils=f2y;
	
	f2z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2z->ind=i;
	f2z->coeffvalue=&WR->tab[i]->z;
	f2z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	f2z->frere=f3z;
	f2z->fils=NULL;
	fatherz->fils=f2z;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
 		if (WR->tab[i]->F0==bary1[0] && WR->tab[i]->alpha==bary1[1] && WR->tab[i]->beta==bary1[2] && WR->tab[i]->gamma==bary1[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f1x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1x->ind=i;
	f1x->coeffvalue=&WR->tab[i]->x;
	f1x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	f1x->frere=f2x;
	f1x->fils=NULL;
	fatherx->fils=f1x;

	f1y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1y->ind=i;
	f1y->coeffvalue=&WR->tab[i]->y;
	f1y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	f1y->frere=f2y;
	f1y->fils=NULL;
	fathery->fils=f1y;

	f1z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1z->ind=i;
	f1z->coeffvalue=&WR->tab[i]->z;
	f1z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	f1z->frere=f2z;
	f1z->fils=NULL;
	fatherz->fils=f1z;

	// Rcursion sur les quatre fils
	if (iterbary(bary1, BM, WR, f1x, f1y, f1z))
		Beep(440, 50);
	if (iterbary(bary1, BM, WR, f2x, f2y, f2z))
		Beep(440, 50)
	if (iterbary(bary1, BM, WR, f3x, f3y, f3z))
		Beep(440, 50);
	if (iterbary(bary1, BM, WR, f4x, f4y, f4z))
		Beep(440, 50);

	
	
	return 0;
}
#endif

int zerotree::iterbary(int bary[4], Mesh *BM, WaveRep *WR, noeudcoeff* fatherx, noeudcoeff* fathery, noeudcoeff* fatherz) {
	
	if (fatherx == NULL)
		return 1;

	int level=bary[1]+bary[2]+bary[3];
	int found, i;
	int bary1[4], bary2[4], bary3[4], bary4[4];
	noeudcoeff* f1x, *f2x, *f3x, *f4x;
	noeudcoeff* f1y, *f2y, *f3y, *f4y;
	noeudcoeff* f1z, *f2z, *f3z, *f4z;
	float x, y, z;
	int al, be, ga, F0;
	int only3=0;

	int direction = (bary[1]%2==0)+2*(bary[2]%2==0)+3*(bary[3]%2==0)-1;

	switch(direction) {
		case 0:	// tourns vers alpha
			//premier fils
			bary1[0]=bary[0];
			bary1[1]=2*bary[1];
			bary1[2]=2*bary[2]+1;
			bary1[3]=2*bary[3]-1;

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=2*bary[1];
			bary2[2]=2*bary[2]-1;
			bary2[3]=2*bary[3]+1;

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=2*bary[1]+2;
			bary3[2]=2*bary[2]-1;
			bary3[3]=2*bary[3]-1;

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=2*bary[1]-2;
			bary4[2]=2*bary[2]+1;
			bary4[3]=2*bary[3]+1;

			break;

		case 1:	// tourns vers beta
			bary1[0]=bary[0];
			bary1[1]=2*bary[1]-1;
			bary1[2]=2*bary[2];
			bary1[3]=2*bary[3]+1;

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=2*bary[1]+1;
			bary2[2]=2*bary[2];
			bary2[3]=2*bary[3]-1;

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=2*bary[1]-1;
			bary3[2]=2*bary[2]+2;
			bary3[3]=2*bary[3]-1;

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=2*bary[1]+1;
			bary4[2]=2*bary[2]-2;
			bary4[3]=2*bary[3]+1;

			break;

		case 2:	// tourns vers gamma
			//premier fils
			bary1[0]=bary[0];
			bary1[1]=2*bary[1]+1;
			bary1[2]=2*bary[2]-1;
			bary1[3]=2*bary[3];

			//deuxime fils
			bary2[0]=bary[0];
			bary2[1]=2*bary[1]-1;
			bary2[2]=2*bary[2]+1;
			bary2[3]=2*bary[3];

			//troisime fils
			bary3[0]=bary[0];
			bary3[1]=2*bary[1]-1;
			bary3[2]=2*bary[2]-1;
			bary3[3]=2*bary[3]+2;

			//quatrime fils
			bary4[0]=bary[0];
			bary4[1]=2*bary[1]+1;
			bary4[2]=2*bary[2]+1;
			bary4[3]=2*bary[3]-2;

			break;
		
	}


	
	// Rtablissement des coordonnes ngatives
	if(bary1[1]<0||bary1[2]<0||bary1[3]<0) {
		changeFace(bary1, BM, &al, &be, &ga, &F0);
		bary1[0]=F0;
		bary1[1]=al;
		bary1[2]=be;
		bary1[3]=ga;
	}

	if(bary2[1]<0||bary2[2]<0||bary2[3]<0) {
		changeFace(bary2, BM, &al, &be, &ga, &F0);
		bary2[0]=F0;
		bary2[1]=al;
		bary2[2]=be;
		bary2[3]=ga;
	}

	if(bary3[1]<0||bary3[2]<0||bary3[3]<0) {
		changeFace(bary3, BM, &al, &be, &ga, &F0);
		bary3[0]=F0;
		bary3[1]=al;
		bary3[2]=be;
		bary3[3]=ga;
	}

	if(bary4[1]<0||bary4[2]<0||bary4[3]<0) {
		only3=changeFace(bary4, BM, &al, &be, &ga, &F0);
		bary4[0]=F0;
		bary4[1]=al;
		bary4[2]=be;
		bary4[3]=ga;
	}

	// Recherche dans le WR
	if (!only3) {
		found=0;
		for (i=0; i<WR->NbCoeff; i++) {
			if (WR->tab[i]->F0==bary4[0] && WR->tab[i]->alpha==bary4[1] && WR->tab[i]->beta==bary4[2] && WR->tab[i]->gamma==bary4[3]) {
				found=1;
				break;
			}
		}
		if (!found) 
			return 1;	// Le coefficient n'a pas t trouv
		


		// Ajout dans la hirarchie

		f4x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4x->ind=i;
		f4x->coeffvalue=&WR->tab[i]->x;
		f4x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
		f4x->frere=NULL;
		f4x->fils=NULL;
		fatherx->fils=f4x;

		f4y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4y->ind=i;
		f4y->coeffvalue=&WR->tab[i]->y;
		f4y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
		f4y->frere=NULL;
		f4y->fils=NULL;
		fathery->fils=f4y;

		f4z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
		f4z->ind=i;
		f4z->coeffvalue=&WR->tab[i]->z;
		f4z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
		f4z->frere=NULL;
		f4z->fils=NULL;
		fatherz->fils=f4z;
	}
	else 
		f4x=f4y=f4z=NULL;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
		if (WR->tab[i]->F0==bary3[0] && WR->tab[i]->alpha==bary3[1] && WR->tab[i]->beta==bary3[2] && WR->tab[i]->gamma==bary3[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f3x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3x->ind=i;
	f3x->coeffvalue=&WR->tab[i]->x;
	f3x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	if (only3) 
		f3x->frere=NULL;
	else
		f3x->frere=f4x;
	f3x->fils=NULL;
	fatherx->fils=f3x;

	f3y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3y->ind=i;
	f3y->coeffvalue=&WR->tab[i]->y;
	f3y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	if (only3) 
		f3y->frere=NULL;
	else
		f3y->frere=f4y;
	f3y->fils=NULL;
	fathery->fils=f3y;

	f3z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f3z->ind=i;
	f3z->coeffvalue=&WR->tab[i]->z;
	f3z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	if (only3) 
		f3z->frere=NULL;
	else
		f3z->frere=f4z;
	f3z->fils=NULL;
	fatherz->fils=f3z;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
		if (WR->tab[i]->F0==bary2[0] && WR->tab[i]->alpha==bary2[1] && WR->tab[i]->beta==bary2[2] && WR->tab[i]->gamma==bary2[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f2x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2x->ind=i;
	f2x->coeffvalue=&WR->tab[i]->x;
	f2x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	f2x->frere=f3x;
	f2x->fils=NULL;
	fatherx->fils=f2x;
	
	f2y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2y->ind=i;
	f2y->coeffvalue=&WR->tab[i]->y;
	f2y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	f2y->frere=f3y;
	f2y->fils=NULL;
	fathery->fils=f2y;
	
	f2z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f2z->ind=i;
	f2z->coeffvalue=&WR->tab[i]->z;
	f2z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	f2z->frere=f3z;
	f2z->fils=NULL;
	fatherz->fils=f2z;

	// Recherche dans le WR
	found=0;
	for (i=0; i<WR->NbCoeff; i++) {
 		if (WR->tab[i]->F0==bary1[0] && WR->tab[i]->alpha==bary1[1] && WR->tab[i]->beta==bary1[2] && WR->tab[i]->gamma==bary1[3]) {
			found=1;
			break;
		}
	}
	if (!found)
		return 1;	//Erreur : le coefficient n'a pas t trouv

	// Ajout dans la hirarchie
	f1x=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1x->ind=i;
	f1x->coeffvalue=&WR->tab[i]->x;
	f1x->quantvalue=quantize(WR->tab[i]->x,m_EC->NbBPX, 0);
	f1x->frere=f2x;
	f1x->fils=NULL;
	fatherx->fils=f1x;

	f1y=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1y->ind=i;
	f1y->coeffvalue=&WR->tab[i]->y;
	f1y->quantvalue=quantize(WR->tab[i]->y,m_EC->NbBPY, 1);
	f1y->frere=f2y;
	f1y->fils=NULL;
	fathery->fils=f1y;

	f1z=(noeudcoeff*)malloc(sizeof(noeudcoeff));
	f1z->ind=i;
	f1z->coeffvalue=&WR->tab[i]->z;
	f1z->quantvalue=quantize(WR->tab[i]->z,m_EC->NbBPZ, 2);
	f1z->frere=f2z;
	f1z->fils=NULL;
	fatherz->fils=f1z;



	// Rcursion sur les quatre fils
	iterbary(bary1, BM, WR, f1x, f1y, f1z);
	iterbary(bary2, BM, WR, f2x, f2y, f2z);
	iterbary(bary3, BM, WR, f3x, f3y, f3z);
	iterbary(bary4, BM, WR, f4x, f4y, f4z);

	
	
	return 0;
}



#ifdef vieille_version_ne fonctionne_plus
int zerotree::changeFace(int bary[4], Mesh *BM, int *al, int *be, int *ga, int *F0) {

	int aprime, bprime, cprime;
	int b1, b2;
	int ok;

	// Reprage de l'arte
	ok = 1*(bary[1]<0)+2*(bary[2]<0)+4*(bary[3]<0);
	switch (ok) {
		case 1:
			b1=BM->Face[bary[0]][1];
			b2=BM->Face[bary[0]][2];
			break;
		case 2:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][2];
			break;
		case 4:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][1];
			break;
	}

	// Reprage des faces voisines
	Lint *run=BM->FaceVoisine[b1];

	// On parcourt la liste des voisins
	while (run!=NULL) {


		ok = ((b2==BM->Face[run->facette][0])||
			(b2==BM->Face[run->facette][1])||
			(b2==BM->Face[run->facette][2])) && run->facette!=bary[0];
		if (ok) {
			aprime=BM->Face[run->facette][0];
			bprime=BM->Face[run->facette][1];
			cprime=BM->Face[run->facette][2];
			goto facetfound;
		}
		run=run->suivant;
	}
	//Not found
	return 1;
facetfound:;

	chCoord3f(bary[1], bary[2], bary[3], BM->Face[bary[0]], BM->Face[run->facette], al, be, ga);

	//redressement des coordonnes ngatives
	if (*al<0) 
		*al=-*al;
	if (*be<0) 
		*be=-*be;
	if (*ga<0) 
		*ga=-*ga;

	*F0=run->facette;

	return 0;

}
#endif

int zerotree::changeFace(int bary[4], Mesh *BM, int *al, int *be, int *ga, int *F0) {

	int aprime, bprime, cprime;
	int b1, b2;
	int ok;

	// Reprage de l'arte
	ok = 1*(bary[1]<0)+2*(bary[2]<0)+4*(bary[3]<0);
	switch (ok) {
		case 1:
			b1=BM->Face[bary[0]][1];
			b2=BM->Face[bary[0]][2];
			break;
		case 2:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][2];
			break;
		case 4:
			b1=BM->Face[bary[0]][0];
			b2=BM->Face[bary[0]][1];
			break;
	}

	// Reprage des faces voisines
	Lint *run=BM->FaceVoisine[b1];

	// On parcourt la liste des voisins
	while (run!=NULL) {


		ok = ((b2==BM->Face[run->facette][0])||
			(b2==BM->Face[run->facette][1])||
			(b2==BM->Face[run->facette][2])) && run->facette!=bary[0];
		if (ok) {
			aprime=BM->Face[run->facette][0];
			bprime=BM->Face[run->facette][1];
			cprime=BM->Face[run->facette][2];
			goto facetfound;
		}
		run=run->suivant;
	}
	//Not found
	return 1;
facetfound:;

	//Changement de coordonnes
	zerotree::chCoord(bary[1], bary[2], bary[3], BM->Face[bary[0]], BM->Face[run->facette], al, be, ga);

	//Rsultat
	*F0=run->facette;

	return 0;

}

int zerotree::chCoord(int alpha, int beta, int gamma, int F0[3], int F1[3], int* al, int* be, int* ga) {

	//Passage en positif
	if (alpha<0) {
		alpha=-alpha;
		beta-=alpha;
		gamma-=alpha;
	}
	if (beta<0) {
		beta=-beta;
		alpha-=beta;
		gamma-=beta;
	}
	if (gamma<0) {
		gamma=-gamma;
		alpha-=gamma;
		beta-=gamma;
	}

	//Changement d'ordre
	chCoord3f(alpha, beta, gamma, F0, F1, al, be, ga);

	return 0;
}

int zerotree::encodeAndBSD(int nzt, Mesh *BM, WaveRep* WR, mbitstream *m_o, int &bufsize, int bufmaxsize, BSD* BSDesc) {

	//the bitstream is in bufferout
	//the markers are in the text output
	//returns the ZT length (in bits)
	//La cellule du BSD doit tre alloue (root + BPDesc)

	int firstbit, lastbit;
	entry* pentry;
	noeudcoeff *ncroot;
	noeudcoeff *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;
	int nbbits;
	mbitstream m_ot;

	int FT, LT;

	//extraction maximum et construction arbre de coefficients
	//m_nbsub_zerotree = buildcoefftree_conf();

	//initialisation sub_zerotrees
	m_tabsub_zerotree = new sub_zerotree;

	m_statin=m_statout=0;

	// On se place a la bonne racine
	ncroot = m_treecoeff;
	//for(i=0;i<nzt; i++) {
	//	ncroot=ncroot->frere->frere->frere;
	//}

	//Cration des listes

	//X component
	m_tabsub_zerotree->m_ztID = nzt;
	m_tabsub_zerotree->m_ncroot = ncroot;

	pentry = new entry();
	pentry->pnc = ncroot;
	m_tabsub_zerotree->m_LIPX.add(pentry);

	if (ncroot->fils!=NULL) {
		pentry = new entry();
		pentry->pnc = ncroot;
		pentry->entrytype = 1;
		m_tabsub_zerotree->m_LISX.add(pentry);
	}

	ncroot = ncroot->frere;


	//Y component
	m_tabsub_zerotree->m_ztID = nzt;
	m_tabsub_zerotree->m_ncroot = ncroot;

	pentry = new entry();
	pentry->pnc = ncroot;
	m_tabsub_zerotree->m_LIPY.add(pentry);

	if (ncroot->fils!=NULL) {
		pentry = new entry();
		pentry->pnc = ncroot;
		pentry->entrytype = 1;
		m_tabsub_zerotree->m_LISY.add(pentry);
	}
	ncroot = ncroot->frere;

	
	//Z component
	m_tabsub_zerotree->m_ztID = nzt;
	m_tabsub_zerotree->m_ncroot = ncroot;

	pentry = new entry();
	pentry->pnc = ncroot;
	m_tabsub_zerotree->m_LIPZ.add(pentry);

	if (ncroot->fils!=NULL) {
		pentry = new entry();
		pentry->pnc = ncroot;
		pentry->entrytype = 1;
		m_tabsub_zerotree->m_LISZ.add(pentry);
	}
	ncroot = ncroot->frere;

	//BSD update
	
	BSDesc->roots[nzt]->offset=8*m_o->top+m_o->offset;
		
	
	//Naked bitplanes

	for (int j=m_EC->LBP; j>=0; j--) {
		ncroot = m_treecoeff;
		i = 0;
		
		ncx=ncroot;

		// X		
		


		//Encodage
		#ifdef LOGZT
		fprintf(logfile, "Codage zerotree %d plan de bit %d composante X\n", nzt, j);
		#endif
		m_ot.octets=(unsigned char*)malloc(100);
		nbbits = writeplanem_conf(i,j,m_ot, 0, WR, BM);
		m_statin+=nbbits;
		if (BSDesc->BPlengthMax<nbbits)
			BSDesc->BPlengthMax=nbbits;

		//BPLength
		//m_o->writenbit((unsigned short)nbbits,m_EC->LengthNbBits);
		//m_statout+=m_EC->LengthNbBits;

		//BSD output
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->offsetX=8*m_o->top+m_o->offset;
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->lengthX=nbbits;

		//Readplane
		m_ot.dump(*m_o);
		m_ot.reset();									
		
		

			
		// Y

		ncroot = ncroot->frere; 
		
		
		//Encodage
		#ifdef LOGZT
		fprintf(logfile, "Codage zerotree %d plan de bit %d composante Y\n", nzt, j);
		#endif
		m_ot.octets=(unsigned char*)malloc(100);
		nbbits = writeplanem_conf(i,j,m_ot, 1, WR, BM);
		m_statin+=nbbits;
		if (BSDesc->BPlengthMax<nbbits)
			BSDesc->BPlengthMax=nbbits;

		//BPLength
		//m_o->writenbit((unsigned short)nbbits,m_EC->LengthNbBits);
		//m_statout+=m_EC->LengthNbBits;

		//BSD output
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->offsetY=8*m_o->top+m_o->offset;
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->lengthY=nbbits;
		
		//Readplane
		m_ot.dump(*m_o);
		m_ot.reset();

		
		
		
		// Z
//DEBUG
if (nzt==58 && j==9)
j=9;
		
		ncroot=ncroot->frere;
	

		//Encodage
		#ifdef LOGZT
		fprintf(logfile, "Codage zerotree %d plan de bit %d composante Z\n", nzt, j);
		#endif
		m_ot.octets=(unsigned char*)malloc(100);
		nbbits = writeplanem_conf(i,j,m_ot, 2, WR, BM);
		m_statin+=nbbits;
		if (BSDesc->BPlengthMax<nbbits)
			BSDesc->BPlengthMax=nbbits;

		//BPLength
		//m_o->writenbit((unsigned short)nbbits,m_EC->LengthNbBits);
		//m_statout+=m_EC->LengthNbBits;

		//BSD output
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->offsetZ=8*m_o->top+m_o->offset;
		BSDesc->roots[nzt]->BPD[m_EC->LBP-j]->lengthZ=nbbits;
		
		//Readplane
		m_ot.dump(*m_o);
		m_ot.reset();


		i++;
		ncroot = ncx; 

		//BSD update
		lastbit=8*m_o->top+m_o->offset;
		BSDesc->roots[nzt]->length=lastbit-BSDesc->roots[nzt]->offset;

	}
		
	m_o->close();
	
	
	bufsize=m_o->top*8;

	if (bufsize>bufmaxsize)
		return 1; // Error - buffer overflow

	return 0;
}

// Tout en mmoire ------------------------------------------------------------------------------------
#if 0	//Old version kept during work
void zerotree::decodemp4_conf(unsigned char* bufferin, int bufsize, Mesh *BM) {
	entry* pentry;
	noeudcoeff *ncroot;
	noeudcoeff *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;

	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int comp;
	int FT, LT;
	mbitstream *m_i = new mbitstream(bufferin);

	if (1) {

		
		start_decode_conf_v2(BM); 

		int mid = m_nbsub_zerotree;///2;

		//Class 3D_Wavelet_Mesh_Object (see the spec)

		//WMOL
		m_i->readnbit(tmp, 1);// Temporaire

		m_EC->isBase=(bool)tmp;
	
		//isInLocalCoordinates
		m_i->readnbit(tmp, 1);
		m_EC->isInLocalCoordinates=tmp;

		//Scale Coeffs
		if (m_EC->isBase && m_EC->hasScaleCoeff)
			threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

		
		//Class 3D_Wavelet_Mesh_Object (see the spec)


		//LengthNbBits
		m_i->readnbit(tmp,5);
		m_EC->LengthNbBits=tmp;

		//FBP
		m_i->readnbit(tmp,5);
		m_EC->FBP=tmp;

		//LBP
		m_i->readnbit(tmp,5);
		m_EC->LBP=tmp;

		//bit(1) X;
		m_i->readnbit(tmp,1);
		m_EC->X=tmp;

		//bit(1) Y;
		m_i->readnbit(tmp,1);
		m_EC->Y=tmp;

		//bit(1) Z;
		m_i->readnbit(tmp,1);
		m_EC->Z=tmp;

		//isPartial
		m_i->readnbit(tmp,1);
		m_EC->isPartial=tmp;

		//NbZT
		//m_i->writenbit((unsigned short int)m_nbsub_zerotree,16);

		if (m_EC->isPartial) {
			m_i->readnbit(tmp,16);
			FT = tmp;
			LT = FT + 1;
		} else { // Full transmission
			FT = 0;
			LT = m_nbsub_zerotree/3;
		}
	



		for (int j=m_EC->LBP; j>=m_EC->FBP; j--) {

			ncroot = m_treecoeff;
			i = 0;
			while (ncroot!=NULL) {

				ncz=ncroot;
				ncy=ncroot->frere;
				ncx=ncy->frere;

				ncroot=ncx;

					

				if (m_EC->X && i>=FT && i<=LT) {
					

					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 0);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
					
				}
				ncroot = ncy;

				if (m_EC->Y && i>=FT && i<=LT) {


					
					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 1);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				ncroot = ncz; 


				if (m_EC->Z && i>=FT && i<=LT) {
			
				
					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 2);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				i++;
				ncroot = ncx->frere;
			}
			
		}
		//m_i->close();

	//TRACE
	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->frere;
	}


		//Dquantif
		ncroot = m_treecoeff;
		while (ncroot!=NULL) {
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
		}
	}

	return;	
}
#endif //end of old version of decodemp4_conf in memory

void zerotree::decodemp4_conf(unsigned char* bufferin, int bufsize, Mesh *BM) {
	entry* pentry;
	noeudcoeff *ncroot;
	noeudcoeff *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;

	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int comp;
	int FT, LT;
	mbitstream *m_i = new mbitstream(bufferin);

	if (1) {

		
		start_decode_conf_v2(BM); 

		int mid = m_nbsub_zerotree;///2;

		//Class 3D_Wavelet_Mesh_Object (see the spec)

		//WMOL
		m_i->readnbit(tmp, 1);// Temporaire

		m_EC->isBase=(bool)tmp;
	
		//isInLocalCoordinates
		m_i->readnbit(tmp, 1);
		m_EC->isInLocalCoordinates=tmp;

		//Scale Coeffs
		if (m_EC->isBase && m_EC->hasScaleCoeff)
			threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

		
		//Class 3D_Wavelet_Mesh_Object (see the spec)


		//LengthNbBits
		m_i->readnbit(tmp,5);
		m_EC->LengthNbBits=tmp;

		//FBP
		m_i->readnbit(tmp,5);
		m_EC->FBP=tmp;

		//LBP
		m_i->readnbit(tmp,5);
		m_EC->LBP=tmp;

		//bit(1) X;
		m_i->readnbit(tmp,1);
		m_EC->X=tmp;

		//bit(1) Y;
		m_i->readnbit(tmp,1);
		m_EC->Y=tmp;

		//bit(1) Z;
		m_i->readnbit(tmp,1);
		m_EC->Z=tmp;

		//isPartial
		m_i->readnbit(tmp,1);
		m_EC->isPartial=tmp;

		//NbZT
		//m_i->writenbit((unsigned short int)m_nbsub_zerotree,16);

		if (m_EC->isPartial) {
			m_i->readnbit(tmp,16);
			FT = tmp;
			LT = FT + 1;
		} else { // Full transmission
			FT = 0;
			LT = m_nbsub_zerotree/3;
		}
	



		for (int j=m_EC->LBP; j>=m_EC->FBP; j--) {

			ncroot = m_treecoeff;
			i = 0;
			while (ncroot!=NULL) {

				ncz=ncroot;
				ncy=ncroot->frere;
				ncx=ncy->frere;

				ncroot=ncx;

					

				if (m_EC->X && i>=FT && i<=LT) {
					

					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);
					if (tmp==0) {
						m_tabsub_zerotree[i].m_lastplanbitX++;			
						break;
					}

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 0);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
					
				}
				ncroot = ncy;

				if (m_EC->Y && i>=FT && i<=LT) {


					
					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);
					if (tmp==0) {
						m_tabsub_zerotree[i].m_lastplanbitY++;
						break;
					}

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 1);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				ncroot = ncz; 


				if (m_EC->Z && i>=FT && i<=LT) {
			
				
					//BPLength
					m_i->readnbit(tmp,m_EC->LengthNbBits);
					if (tmp==0) {
						m_tabsub_zerotree[i].m_lastplanbitX++;
						break;
					}

					//Readplane
					nbbits = readplane_conf(i,j,m_i, 2);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);

					
				}
				i++;
				ncroot = ncx->frere;
			}
			
		}
		//m_i->close();

	//TRACE
	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->frere;
	}


		//Dquantif
		ncroot = m_treecoeff;
		while (ncroot!=NULL) {
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
			unquantize(ncroot, (ncroot->ind)%3);
			ncroot = ncroot->frere;
		}
	}

	return;	
}

void zerotree::start_decode_conf(Mesh *BM) {
	char buf[500];
	char* extractbuf;
	char* tok;
	int i;
	noeudcoeff* ncroot;
	entry* pentry;



	if (m_EC==NULL) {
		return;
	}

	//  partir d'ici m_EC doit tre rempli

	// destruction maillage prcdent
	reset();

	m_depth=m_EC->NumberOfLevels;

	
	m_nbbasetriangle = BM->NbFace;
	
	m_nbvertex=BM->NbVertex;

	m_nbtriangle = m_nbbasetriangle*(((1 << (m_depth+m_depth+2))-1)/3);

	m_triangles = new triangle[m_nbtriangle];
	for (i = 0; i<m_nbtriangle; i++) {
		m_triangles[i].tind = -1;
		m_triangles[i].root = 0;
		m_triangles[i].v1 = -1;
		m_triangles[i].v2 = -1;
		m_triangles[i].v3 = -1;
	}
	m_triangles_voisins = new voisins[m_nbtriangle];

	//lecture des triangles
	i=0;
	int pas = m_nbtriangle/m_nbbasetriangle;
	while (i<m_nbtriangle) {
	

		m_triangles[i].tind =0;			
		m_triangles[i].root =1;			
		m_triangles[i].v1 =BM->Face[i/pas][0];			
		m_triangles[i].v2 =BM->Face[i/pas][1];			
		m_triangles[i].v3 =BM->Face[i/pas][2];			

		i += pas;
	}

	for (i=0; i<m_nbtriangle; i+=pas) {
		for (int j = i+ pas; j<m_nbtriangle; j+=pas) {
			if (setvoisin(i,j)) {
				j = m_nbtriangle;
			}
		}
	}



	unsigned char* niv=(unsigned char*)malloc(2000000*sizeof(unsigned char));
	for (i=0; i<m_depth; i++) {
		diviseniveau(i, niv);
	}

	//creation m_tabvertex et m_vertexparent
	m_tabvertex = new vertex[m_nbvertex];
	m_vertexparent = new int[m_nbvertex];

	for (i=0; i<m_nbvertex; i++) {
		m_tabvertex[i].depth = niv[i];
		m_tabvertex[i].x = 0;
		m_tabvertex[i].y = 0;
		m_tabvertex[i].z = 0;
		m_vertexparent[i] = i;
	}
	free(niv);	

 
	delete[] m_triangles_voisins;
	m_triangles_voisins = NULL;

	
	for (i=0; i<BM->NbVertex; i++) {
		m_tabvertex[i].depth = m_depth+1;
		m_tabvertex[i].x = BM->Vertex[i][0];
		m_tabvertex[i].y = BM->Vertex[i][1];
		m_tabvertex[i].z = BM->Vertex[i][2];
	}

	buildvertexparent();



	//Fossile
	//p_WaveMesh->initMesh(this);

	m_nbsub_zerotree = buildcoefftree_conf();

	//initialisation sub_zerotrees
	m_tabsub_zerotree = new sub_zerotree[m_nbsub_zerotree];

	ncroot = m_treecoeff;

	for (i=0; i<m_nbsub_zerotree/3; i++) {

		//Z component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPZ.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISZ.add(pentry);
		}
		ncroot = ncroot->frere;

		//Y component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPY.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISY.add(pentry);
		}
		ncroot = ncroot->frere;

		//X component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPX.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISX.add(pentry);
		}
		ncroot = ncroot->frere;

		

		
	}

	return;
}


void zerotree::start_decode_conf_v2(Mesh *BM) {
	char buf[500];
	char* extractbuf;
	char* tok;
	int i;
	noeudcoeff* ncroot;
	entry* pentry;



	if (m_EC==NULL) {
		return;
	}

	//  partir d'ici m_EC doit tre rempli

	// destruction maillage prcdent
	reset();


	m_depth=m_EC->NumberOfLevels;


	m_nbbasetriangle = BM->NbFace;
	

	m_nbvertex=BM->NbVertex;

	
	m_nbtriangle = m_nbbasetriangle*(((1 << (m_depth+m_depth+2))-1)/3);


	DatMesh* DM=makeDMsubdividedFromBaseMesh(BM, m_depth);

	
	//passage du dat en m_triangle et libration du dat
	read_decode(DM);

	free(DM);

	for (i=0; i<BM->NbVertex; i++) {
		m_tabvertex[i].depth = m_depth+1;
		m_tabvertex[i].x = BM->Vertex[i][0];
		m_tabvertex[i].y = BM->Vertex[i][1];
		m_tabvertex[i].z = BM->Vertex[i][2];
	}

	m_nbsub_zerotree = buildcoefftree_conf();

	//initialisation sub_zerotrees
	m_tabsub_zerotree = new sub_zerotree[m_nbsub_zerotree];

	ncroot = m_treecoeff;

	for (i=0; i<m_nbsub_zerotree/3; i++) {

		//Z component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPZ.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISZ.add(pentry);
		}
		ncroot = ncroot->frere;

		//Y component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPY.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISY.add(pentry);
		}
		ncroot = ncroot->frere;

		//X component
		m_tabsub_zerotree[i].m_ztID = i;
		m_tabsub_zerotree[i].m_ncroot = ncroot;

		pentry = new entry();
		pentry->pnc = ncroot;
		m_tabsub_zerotree[i].m_LIPX.add(pentry);

		if (ncroot->fils!=NULL) {
			pentry = new entry();
			pentry->pnc = ncroot;
			pentry->entrytype = 1;
			m_tabsub_zerotree[i].m_LISX.add(pentry);
		}
		ncroot = ncroot->frere;

		

		
	}

	return;
}

int zerotree::readplane_conf(int i, unsigned char n, mbitstream *ibs, int flag) {
	int resSN, sign;
	entry* pentry, *emove, *newentry;
	noeudcoeff* p, *p2;
	liste_entry *m_pLSP, *m_pLSPadd, *m_pLIP, *m_pLIS;
	int cmpt=0;


	switch(flag) {
	case 0:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitX) {
			return -1 ;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPX);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPXadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPX);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISX);
		break;
	case 1:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitY) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPY);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPYadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPY);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISY);
		break;
	case 2:
		if (m_EC->LBP-n!=m_tabsub_zerotree[i].m_lastplanbitZ) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree[i].m_LSPZ);
		m_pLSPadd = &(m_tabsub_zerotree[i].m_LSPZadd);
		m_pLIP = &(m_tabsub_zerotree[i].m_LIPZ);
		m_pLIS = &(m_tabsub_zerotree[i].m_LISZ);
		break;
	}
	//2 Sorting Pass
	//2.1
	pentry = m_pLIP->startiter();
			
	while (pentry) {
		//input Sn(i,j)
		ibs->readbit(resSN);
		cmpt++;
		setSN(n,resSN,pentry->pnc);
				
		if (resSN==1) {
				
			//input sign of ci,j 2.1.2
			ibs->readbit(sign);
			cmpt++;
			setsign(pentry->pnc,sign);
					
			//move entry to LSP
			emove = pentry;
			pentry = m_pLIP->removeiter();
			m_pLSPadd->add(emove);
		}
		else {
			pentry = m_pLIP->iter();
		}
	}
			
	//2.2
	pentry = m_pLIS->startiter();
	while (pentry) {
				
		//2.2.1
		if (pentry->entrytype==1) {
			//input Sn(D(i,j))
			ibs->readbit(resSN);
			cmpt++;
					
			if (resSN==1) {
				p = pentry->pnc->fils;
				while (p) {
					//input Sn(k,l)
					ibs->readbit(resSN);
					cmpt++;
					setSN(n,resSN,p);
							
					if (resSN==1) {
						//input sign of ck,l
						ibs->readbit(sign);
						cmpt++;
						setsign(p,sign);

						//add entry to LSP
						entry* ne = new entry;
						ne->pnc = p;
						m_pLSPadd->add(ne);
					}
					else {
						entry* ne = new entry;
						ne->pnc = p;
						m_pLIP->addend(ne);
					}
					p = p->frere;
				}
						
				//L(i,j) vide?
				if (Lijvide(pentry)) {
						
					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
							
					//move entry to the end of LIS as an entry of type B
					pentry->entrytype = 2;
					ibs->readbit(resSN);
					cmpt++;
					if (resSN) {
						p2 = pentry->pnc->fils;
						while (p2) {
							entry* ne = new entry;
							ne->entrytype = 1;
							ne->pnc = p2;
							m_pLIS->addend(ne);
							p2 = p2->frere;
						}
						pentry = m_pLIS->removeiter();
					}
					else {
						emove = pentry;
						pentry = m_pLIS->removeiter();
						m_pLIS->addend(emove);
					}
				}
			}
			else {
				pentry = m_pLIS->iter();
			}
		}
				
		//2.2.2
		else {
			if (pentry->entrytype==2) {

				//input Sn(L(i,j))
				ibs->readbit(resSN);
				cmpt++;
						
				if (resSN) {
					p = pentry->pnc->fils;
					while (p) {
						// add to the end of LIS as an entry of type A
						newentry = new entry;
						newentry->entrytype = 1;
						newentry->pnc = p;
						m_pLIS->addend(newentry);
						p = p->frere;
					}

					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					pentry = m_pLIS->iter();
				}
			}	
		}
	}
	
	//3 Refinement Pass
	pentry = m_pLSP->startiter();

	//active le plan de bits correspondant ,& logique avec puissance de 2
	unsigned short int bits = 32768;
	bits = bits >> (1+m_maxn-n);

	while (pentry) {
		int readbit;
		unsigned short int usi = pentry->pnc->quantvalue;
		//input the nth most significant bit of |ci,j|
		ibs->readbit(readbit);
		cmpt++;
		if (readbit) {
			usi = usi|bits;
		}
		else {
			usi = usi&(bits^65535);
		}
		pentry->pnc->quantvalue = usi;
		pentry = m_pLSP->iter();	
	}
		
	//append m_LSP m_LSPadd
	m_pLSP->append(*m_pLSPadd);

	switch(flag) {
	case 0:
		m_tabsub_zerotree[i].m_lastplanbitX++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPX) {
			m_tabsub_zerotree[i].m_LIPX.reset();
			m_tabsub_zerotree[i].m_LISX.reset();
			m_tabsub_zerotree[i].m_LSPX.reset();
			m_tabsub_zerotree[i].m_LSPXadd.reset();
		}
		break;
	case 1:
		m_tabsub_zerotree[i].m_lastplanbitY++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPY) {
			m_tabsub_zerotree[i].m_LIPY.reset();
			m_tabsub_zerotree[i].m_LISY.reset();
			m_tabsub_zerotree[i].m_LSPY.reset();
			m_tabsub_zerotree[i].m_LSPYadd.reset();
		}
		break;
	case 2:
		m_tabsub_zerotree[i].m_lastplanbitZ++;
		if (m_tabsub_zerotree[i].m_lastplanbit==m_EC->NbBPZ) {
			m_tabsub_zerotree[i].m_LIPZ.reset();
			m_tabsub_zerotree[i].m_LISZ.reset();
			m_tabsub_zerotree[i].m_LSPZ.reset();
			m_tabsub_zerotree[i].m_LSPZadd.reset();
		}
		break;
	}
	return cmpt;
}

// Tout en local

void zerotree::decodemp4_conf_loc(unsigned char* bufferin, int bufsize, Mesh *BM, int NbCoeff) {
	entry* pentry;
	noeudcoeff_loc *ncroot;
	noeudcoeff_loc *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;
	int Nbcoeff;
	noeudcoeff_loc **coeffs;

	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int comp;
	int FT, LT;
	mbitstream *m_i = new mbitstream(bufferin);

	//Allocation des coeffs
	coeffs=(noeudcoeff_loc**)calloc(3*NbCoeff, sizeof(noeudcoeff_loc*));

	start_decode_conf_loc(BM); 


	int mid = m_nbsub_zerotree;///2;

	//Class 3D_Wavelet_Mesh_Object (see the spec)

	//WMOL
	m_i->readnbit(tmp, 1);// Temporaire
	m_EC->isBase=(bool)tmp;

	//IsInBad
	m_i->readnbit(tmp, 1);
	m_EC->isInBand=(bool)tmp;

	//DSI in the stream in case of inband transmission
	if (tmp) {
		m_i->readnbit(tmp, 1);
		m_EC->hasScaleCoeff=(bool)tmp;
		if (m_EC->hasScaleCoeff) {
			m_i->readnbit(tmp,5);
			m_EC->NbBpSC=(unsigned char)tmp;
		}
		m_i->readnbit(tmp,5);		
		m_EC->NbBPX=(unsigned char)tmp;
		m_i->readnbit(tmp,5);		
		m_EC->NbBPY=(unsigned char)tmp;
		m_i->readnbit(tmp,5);		
		m_EC->NbBPZ=(unsigned char)tmp;

		m_i->readnbit(tmp, 1);
		m_EC->Wtype[0]=(bool)tmp;
		m_i->readnbit(tmp, 1);
		m_EC->Wtype[1]=(bool)tmp;
		m_i->readnbit(tmp, 1);
		m_EC->lift=(bool)tmp;
		m_i->readnbit(tmp,4);		
		m_EC->NumberOfLevels=(unsigned char)tmp;
		m_i->readnbit(m_EC->Xmax,32);
		m_i->readnbit(m_EC->Ymax,32);
		m_i->readnbit(m_EC->Zmax,32);
	}

	//isInLocalCoordinates
	m_i->readnbit(tmp, 1);
	m_EC->isInLocalCoordinates=tmp;

	//Scale Coeffs
	if (m_EC->isBase && m_EC->hasScaleCoeff)
		threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

	
	//Class 3D_Wavelet_Mesh_Object (see the spec)


	//LengthNbBits
	m_i->readnbit(tmp,5);
	m_EC->LengthNbBits=tmp;

	//FBP
	m_i->readnbit(tmp,5);
	m_EC->FBP=tmp;

	//LBP
	m_i->readnbit(tmp,5);
	m_EC->LBP=tmp;

	//bit(1) X;
	m_i->readnbit(tmp,1);
	m_EC->X=tmp;

	//bit(1) Y;
	m_i->readnbit(tmp,1);
	m_EC->Y=tmp;

	//bit(1) Z;
	m_i->readnbit(tmp,1);
	m_EC->Z=tmp;

	//isPartial
	m_i->readnbit(tmp,1);
	m_EC->isPartial=tmp;

	//NbZT
	//m_i->writenbit((unsigned short int)m_nbsub_zerotree,16);

	if (m_EC->isPartial) {
		m_i->readnbit(tmp,16);
		FT = tmp;
		LT = FT + 1;
	} else { // Full transmission
		FT = 0;
		LT = m_nbsub_zerotree/3;
	}




	for (int j=m_EC->LBP; j>=m_EC->FBP; j--) {

		ncroot = m_treecoeff_loc;
		i = 0;
		while (ncroot!=NULL) {


			
			ncz=ncroot;
			ncy=ncroot->m_frere;
			ncx=ncy->m_frere;

			ncroot=ncx;

				

			if (m_EC->X && i>=FT && i<=LT) {
				
				#ifdef LOGZT
				fprintf(logfile, "Decodage zerotree %d plan de bit %d composante X\n", i, j);
				#endif

				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitX++;			
				} else {


					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 0, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}
				
			}
			ncroot = ncy;

			if (m_EC->Y && i>=FT && i<=LT) {

				#ifdef LOGZT
				fprintf(logfile, "Decodage zerotree %d plan de bit %d composante Y\n", i, j);
				#endif
				
				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitY++;			
				} else {

					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 1, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}

				
			}
			ncroot = ncz; 


			if (m_EC->Z && i>=FT && i<=LT) {

				#ifdef LOGZT
				fprintf(logfile, "Decodage zerotree %d plan de bit %d composante Z\n", i, j);
				#endif
		
			
				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitZ++;			
				} else {

					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 2, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}

		
			}
			i++;
			ncroot = ncx->m_frere;
		}
		
	}
	//m_i->close();

	//TRACE
/*	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff_loc;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->m_frere;
*/
		//Dquantif
		ncroot = m_treecoeff_loc;
		while (ncroot!=NULL) {
			if (m_EC->X) {
				unquantize(ncroot, (int)0);
				ncroot = ncroot->m_frere;
			}
			if (m_EC->Y) {
				unquantize(ncroot, (int)1);
				ncroot = ncroot->m_frere;
			}
			if (m_EC->Z) {
				unquantize(ncroot, (int)2);
				ncroot = ncroot->m_frere;
			}
		}
	//}

	return;	
}

int zerotree::readplane_conf_loc(int i, unsigned char n, mbitstream *ibs, int flag, Mesh *BM) {
	int resSN, sign;
	entry_loc* pentry, *emove, *newentry;
	noeudcoeff_loc* p, *p2;
	liste_entry_loc *m_pLSP, *m_pLSPadd, *m_pLIP, *m_pLIS;
	int cmpt=0;


	switch(flag) {
	case 0:
		if (m_EC->LBP-n!=m_tabsub_zerotree_loc[i].m_lastplanbitX) {
			return -1 ;
		}
		m_pLSP = &(m_tabsub_zerotree_loc[i].m_LSPX);
		m_pLSPadd = &(m_tabsub_zerotree_loc[i].m_LSPXadd);
		m_pLIP = &(m_tabsub_zerotree_loc[i].m_LIPX);
		m_pLIS = &(m_tabsub_zerotree_loc[i].m_LISX);
		break;
	case 1:
		if (m_EC->LBP-n!=m_tabsub_zerotree_loc[i].m_lastplanbitY) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree_loc[i].m_LSPY);
		m_pLSPadd = &(m_tabsub_zerotree_loc[i].m_LSPYadd);
		m_pLIP = &(m_tabsub_zerotree_loc[i].m_LIPY);
		m_pLIS = &(m_tabsub_zerotree_loc[i].m_LISY);
		break;
	case 2:
		if (m_EC->LBP-n!=m_tabsub_zerotree_loc[i].m_lastplanbitZ) {
			return -1;
		}
		m_pLSP = &(m_tabsub_zerotree_loc[i].m_LSPZ);
		m_pLSPadd = &(m_tabsub_zerotree_loc[i].m_LSPZadd);
		m_pLIP = &(m_tabsub_zerotree_loc[i].m_LIPZ);
		m_pLIS = &(m_tabsub_zerotree_loc[i].m_LISZ);
		break;
	}

	#ifdef LOGZT
	fprintf(logfile, "Etat de LSP:\n");
	entry_loc* parcours=m_pLSP->m_tete;
	while (parcours) {
		fprintf(logfile, "%d, %d, %d, %d\n", parcours->pnc->F0, parcours->pnc->alpha, parcours->pnc->beta, parcours->pnc->gamma);
		parcours=parcours->next;
	}

	fprintf(logfile, "Etat de LIP:\n");
	parcours=m_pLIP->m_tete;
	while (parcours) {
		fprintf(logfile, "%d, %d, %d, %d\n", parcours->pnc->F0, parcours->pnc->alpha, parcours->pnc->beta, parcours->pnc->gamma);
		parcours=parcours->next;
	}

	fprintf(logfile, "Etat de LIS:\n");
	parcours=m_pLIS->m_tete;
	while (parcours) {
		fprintf(logfile, "%d, %d, %d, %d\n", parcours->pnc->F0, parcours->pnc->alpha, parcours->pnc->beta, parcours->pnc->gamma);
		parcours=parcours->next;
	}
	#endif

	//2 Sorting Pass
	//2.1
	pentry = m_pLIP->startiter();
			
	while (pentry) {
		//input Sn(i,j)
		ibs->readbit(resSN);
		#ifdef LOGZT
		fprintf(logfile, "Input Sn(i,j) = %d\n", resSN);
		#endif
		cmpt++;
		setSN(n,resSN,pentry->pnc);
				
		if (resSN==1) {
				
			//input sign of ci,j 2.1.2
			ibs->readbit(sign);
			#ifdef LOGZT
			fprintf(logfile, "Son signe est %d\n", sign);
			#endif
			cmpt++;
			setsign(pentry->pnc,sign);
					
			//move entry to LSP
			emove = pentry;
			#ifdef LOGZT
			fprintf(logfile, "move entry to LSP\n");
			#endif
			pentry = m_pLIP->removeiter();
			m_pLSPadd->add(emove);
		}
		else {
			pentry = m_pLIP->iter();
		}
	}
			
	//2.2
	pentry = m_pLIS->startiter();
	while (pentry) {
				
		//2.2.1
		if (pentry->entrytype==1) {
			//input Sn(D(i,j))
			ibs->readbit(resSN);
			#ifdef LOGZT
			fprintf(logfile, "Input Sn(i,j) = %d\n", resSN);
			#endif
			cmpt++;
					
			if (resSN==1) {
				p = pentry->pnc->fils();
				#ifdef LOGZT
				fprintf(logfile, "p=pentry->fils\n");
				fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
				#endif
				while (p) {
					//input Sn(k,l)
					ibs->readbit(resSN);
					#ifdef LOGZT
					fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
					#endif
					cmpt++;
					setSN(n,resSN,p);
							
					if (resSN==1) {
						//input sign of ck,l
						ibs->readbit(sign);
						#ifdef LOGZT
						fprintf(logfile, "Son signe est %d\n", sign);
						#endif
						cmpt++;
						setsign(p,sign);

						//add entry to LSP
						entry_loc* ne = new entry_loc;
						ne->pnc = p;
						m_pLSPadd->add(ne);
					}
					else {
						entry_loc* ne = new entry_loc;
						ne->pnc = p;
						m_pLIP->addend(ne);
					}
					p = p->frere(BM);
					#ifdef LOGZT
					fprintf(logfile, "p=p->frere\n");
					if (p)
						fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
					#endif
				}
						
				//L(i,j) vide?
				if (Lijvide(pentry)) {
					#ifdef LOGZT
					fprintf(logfile, "Lijvide\n");
					#endif
						
					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					#ifdef LOGZT
					fprintf(logfile, "Lijnonvide\n");
					#endif
							
					//move entry to the end of LIS as an entry of type B
					pentry->entrytype = 2;
					ibs->readbit(resSN);
					#ifdef LOGZT
					fprintf(logfile, "output Sn(i,j) = %d\n", resSN);
					#endif
					cmpt++;
					if (resSN) {
						p2 = pentry->pnc->fils();
						#ifdef LOGZT
						fprintf(logfile, "pentry->fils\n");
						if (p)
						fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
						#endif
						while (p2) {
							entry_loc* ne = new entry_loc;
							ne->entrytype = 1;
							ne->pnc = p2;
							m_pLIS->addend(ne);
							p2 = p2->frere(BM);
							#ifdef LOGZT
							fprintf(logfile, "pentry->frere\n");
							if (p)
							fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
							#endif
						}
						pentry = m_pLIS->removeiter();
					}
					else {
						emove = pentry;
						pentry = m_pLIS->removeiter();
						m_pLIS->addend(emove);
					}
				}
			}
			else {
				pentry = m_pLIS->iter();
			}
		}
				
		//2.2.2
		else {
			if (pentry->entrytype==2) {

				//input Sn(L(i,j))
				ibs->readbit(resSN);
				#ifdef LOGZT
				fprintf(logfile, "Input Sn(i,j) = %d\n", resSN);
				#endif
				cmpt++;
						
				if (resSN) {
					p = pentry->pnc->fils();
					#ifdef LOGZT
					fprintf(logfile, "pentry->fils\n");
					if (p)
					fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
					#endif
					while (p) {
						// add to the end of LIS as an entry of type A
						newentry = new entry_loc;
						newentry->entrytype = 1;
						newentry->pnc = p;
						m_pLIS->addend(newentry);
						p = p->frere(BM);
						#ifdef LOGZT
						fprintf(logfile, "pentry->frere\n");
						if (p)
						fprintf(logfile, "%d %d %d %d\n", p->F0, p->alpha, p->beta, p->gamma);
						#endif
					}

					//remove entry from the LIS
					emove = pentry;
					pentry = m_pLIS->removeiter();
					delete emove;
				}
				else {
					pentry = m_pLIS->iter();
				}
			}	
		}
	}
	
	//3 Refinement Pass
	pentry = m_pLSP->startiter();

	//active le plan de bits correspondant ,& logique avec puissance de 2
	unsigned short int bits = 32768;
	bits = bits >> (1+m_maxn-n);

	while (pentry) {
		int readbit;
		unsigned short int usi = pentry->pnc->quantvalue;
		//input the nth most significant bit of |ci,j|
		ibs->readbit(readbit);
		#ifdef LOGZT
		fprintf(logfile, "refinement input Sn(i,j) = %d\n", readbit);
		#endif
		cmpt++;
		if (readbit) {
			usi = usi|bits;
		}
		else {
			usi = usi&(bits^65535);
		}
		pentry->pnc->quantvalue = usi;
		pentry = m_pLSP->iter();	
	}
		
	//append m_LSP m_LSPadd
	m_pLSP->append(*m_pLSPadd);

	switch(flag) {
	case 0:
		m_tabsub_zerotree_loc[i].m_lastplanbitX++;
		if (m_tabsub_zerotree_loc[i].m_lastplanbit==m_EC->NbBPX) {
			m_tabsub_zerotree_loc[i].m_LIPX.reset();
			m_tabsub_zerotree_loc[i].m_LISX.reset();
			m_tabsub_zerotree_loc[i].m_LSPX.reset();
			m_tabsub_zerotree_loc[i].m_LSPXadd.reset();
		}
		break;
	case 1:
		m_tabsub_zerotree_loc[i].m_lastplanbitY++;
		if (m_tabsub_zerotree_loc[i].m_lastplanbit==m_EC->NbBPY) {
			m_tabsub_zerotree_loc[i].m_LIPY.reset();
			m_tabsub_zerotree_loc[i].m_LISY.reset();
			m_tabsub_zerotree_loc[i].m_LSPY.reset();
			m_tabsub_zerotree_loc[i].m_LSPYadd.reset();
		}
		break;
	case 2:
		m_tabsub_zerotree_loc[i].m_lastplanbitZ++;
		if (m_tabsub_zerotree_loc[i].m_lastplanbit==m_EC->NbBPZ) {
			m_tabsub_zerotree_loc[i].m_LIPZ.reset();
			m_tabsub_zerotree_loc[i].m_LISZ.reset();
			m_tabsub_zerotree_loc[i].m_LSPZ.reset();
			m_tabsub_zerotree_loc[i].m_LSPZadd.reset();
		}
		break;
	}
	return cmpt;
}

void zerotree::start_decode_conf_loc(Mesh *BM) {
	char buf[500];
	char* extractbuf;
	char* tok;
	int i;
	noeudcoeff_loc* ncroot;
	entry_loc* pentry;
	int coeffindice;



	if (m_EC==NULL) {
		return;
	}

	//  partir d'ici m_EC doit tre rempli

	// destruction maillage prcdent
	reset();

	m_depth=m_EC->NumberOfLevels;
	m_nbbasetriangle = BM->NbFace;
	m_nbvertex=BM->NbVertex;
	m_nbtriangle = m_nbbasetriangle*(((1 << (m_depth+m_depth+2))-1)/3);

	
	//cration des racines m_treecoeff (niveau 0 seulement), le reste est implicite
	//TODO
	createRootTab(BM);

	// Le maillage de base est mis dans les coefficients d'chelles
	// TODO

	//initialisation sub_zerotrees
	m_tabsub_zerotree_loc = new sub_zerotree_loc[m_nbsub_zerotree];

	ncroot = m_treecoeff_loc;

	for (i=0; ncroot!=NULL; i++) {

		//X component
		if (m_EC->X) {
			m_tabsub_zerotree_loc[i].m_ztID = i;
			m_tabsub_zerotree_loc[i].m_ncroot = ncroot;

			pentry = new entry_loc();
			pentry->pnc = ncroot;
			m_tabsub_zerotree_loc[i].m_LIPX.add(pentry);

			if (/*m_depth>*/1) {
				pentry = new entry_loc();
				pentry->pnc = ncroot;
				pentry->entrytype = 1;
				m_tabsub_zerotree_loc[i].m_LISX.add(pentry);
			}
			ncroot = ncroot->m_frere;
		}
		if (m_EC->Y) {
			//Y component
			m_tabsub_zerotree_loc[i].m_ztID = i;
			m_tabsub_zerotree_loc[i].m_ncroot = ncroot;

			pentry = new entry_loc();
			pentry->pnc = ncroot;
			m_tabsub_zerotree_loc[i].m_LIPY.add(pentry);

			if (/*m_depth>*/1) {
				pentry = new entry_loc();
				pentry->pnc = ncroot;
				pentry->entrytype = 1;
				m_tabsub_zerotree_loc[i].m_LISY.add(pentry);
			}
			ncroot = ncroot->m_frere;
		}

		//Z component
		if (m_EC->Z) {
			m_tabsub_zerotree_loc[i].m_ztID = i;
			m_tabsub_zerotree_loc[i].m_ncroot = ncroot;

			pentry = new entry_loc();
			pentry->pnc = ncroot;
			m_tabsub_zerotree_loc[i].m_LIPZ.add(pentry);

			if (/*m_depth>*/1) {
				pentry = new entry_loc();
				pentry->pnc = ncroot;
				pentry->entrytype = 1;
				m_tabsub_zerotree_loc[i].m_LISZ.add(pentry);
			}
			ncroot = ncroot->m_frere;
		}
	}

	m_nbsub_zerotree=i*3;
	
	this->m_nbvertex=BM->NbVertex;

	return;
}

void zerotree::createRootTab(Mesh *BM) {

	Lint** Dejavu=(Lint**)calloc(BM->NbVertex, sizeof(Lint*));
	Lint* run;
	int a;
	int b;
	int i;
	int trouve;

	//Pour chaque face de BM
	noeudcoeff_loc* run2=NULL;
	int coeffindice=0;
	for (i=0; i<m_nbbasetriangle; i++) {

		//On parcourt les artes dans l'ordre
		//Pour chaque arte on cre un coefficient et on dclare que le sommet de plus petit indice
		//a deja contribu avec celui du plus grand indice
		
		//Premire arte
		
		//Test de dejavu
		if (BM->Face[i][0]<BM->Face[i][1]) {
			a=BM->Face[i][0];
			b=BM->Face[i][1];
		} else {
			b=BM->Face[i][0];
			a=BM->Face[i][1];
		}
		run=Dejavu[a];
		trouve=0;
		while (run!=NULL) {
			if (run->facette==b) {
				trouve=1;
				break;
			}
			run=run->suivant;
		}
		if (!trouve) {

			//Creation du coeff
			//X
			noeudcoeff_loc* newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=1;
			newcoeff->gamma=0;
			newcoeff->Age=0;
			if (!m_treecoeff_loc) {
				m_treecoeff_loc=newcoeff;
				run2=newcoeff;
			}
			else {
				run2->m_frere=newcoeff;
				run2=run2->m_frere;
			}
			//Y
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=1;
			newcoeff->gamma=0;
			newcoeff->Age=0;
			run2->m_frere=newcoeff;
			run2=run2->m_frere;
			//Z
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=1;
			newcoeff->gamma=0;
			newcoeff->Age=0;
			coeffindice++;
			run2->m_frere=newcoeff;
			run2=run2->frere(BM);

			//Dejavu
			Lint *newint=(Lint*)malloc(sizeof(Lint));
			newint->facette=b;
			newint->suivant=Dejavu[a];
			Dejavu[a]=newint;
		}
		
		//Deuxime arte
		//Test de dejavu
		if (BM->Face[i][1]<BM->Face[i][2]) {
			a=BM->Face[i][1];
			b=BM->Face[i][2];
		} else {
			b=BM->Face[i][1];
			a=BM->Face[i][2];
		}
		run=Dejavu[a];
		trouve=0;
		while (run!=NULL) {
			if (run->facette==b) {
				trouve=1;
				break;
			}
			run=run->suivant;
		}
		if (!trouve) {
		
			//Creation du coeff
			//X
			noeudcoeff_loc* newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=0;
			newcoeff->beta=1;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			if (!m_treecoeff_loc)
				m_treecoeff_loc=newcoeff;
			else {
				run2->m_frere=newcoeff;
				run2=run2->m_frere;
			}
			//Y
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=0;
			newcoeff->beta=1;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			run2->m_frere=newcoeff;
			run2=run2->m_frere;
			//Z
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=0;
			newcoeff->beta=1;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			coeffindice++;
			run2->m_frere=newcoeff;
			run2=run2->m_frere;

			//Dejavu
			Lint *newint=(Lint*)malloc(sizeof(Lint));
			newint->facette=b;
			newint->suivant=Dejavu[a];
			Dejavu[a]=newint;
		}

		//Test de dejavu
		if (BM->Face[i][0]<BM->Face[i][2]) {
			a=BM->Face[i][0];
			b=BM->Face[i][2];
		} else {
			b=BM->Face[i][0];
			a=BM->Face[i][2];
		}
		run=Dejavu[a];
		trouve=0;
		while (run!=NULL) {
			if (run->facette==b) {
				trouve=1;
				break;
			}
			run=run->suivant;
		}
		if (!trouve) {

			//Troisime arte
			
			//Creation du coeff
			//X
			noeudcoeff_loc* newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=0;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			if (!m_treecoeff_loc)
				m_treecoeff_loc=newcoeff;
			else {
				run2->m_frere=newcoeff;
				run2=run2->m_frere;
			}
			//Y
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=0;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			run2->m_frere=newcoeff;
			run2=run2->m_frere;
			//Z
			newcoeff=new noeudcoeff_loc();
			newcoeff->coeffvalue=0;
			newcoeff->quantvalue=0;
			newcoeff->m_fils=NULL;
			newcoeff->m_frere=NULL;
			newcoeff->ind=coeffindice;
			newcoeff->F0=i;
			newcoeff->alpha=1;
			newcoeff->beta=0;
			newcoeff->gamma=1;
			newcoeff->Age=0;
			coeffindice++;
			run2->m_frere=newcoeff;
			run2=run2->m_frere;

			//Dejavu
			Lint *newint=(Lint*)malloc(sizeof(Lint));
			newint->facette=b;
			newint->suivant=Dejavu[a];
			Dejavu[a]=newint;

		}

	}

	//On libre la table de verif
	free(Dejavu);

	//Mise  jour du nombre de coefficients
	m_currentcoeffnumber = coeffindice;

}

int zerotree::dumpZ2WR(WaveRep* WR, Mesh* BM) {

	//Parcours en profondeur, remplissage en zig-zag
	
	int i, k;
	noeudcoeff_loc* ncroot;

	//Initialisation de WR
	WR->NbCoeff=BM->NbFace*(((1<<m_depth)+1)*((1<<m_depth)+2))/2+m_nbvertex;
	for (i=0; i<WR->NbCoeff; i++) {
		WR->tab[i]=NULL;
	}

	//On copie les sommets de niveau 0 (coefficients d'chelle)
	for (i=0; i<BM->NbFace; i++) {
		//premier sommet
		k=BM->Face[i][0];
		if (!WR->tab[k]) {
			WR->tab[k]=(WaveCo*)malloc(sizeof(WaveCo));
			WR->tab[k]->F0=i;
			WR->tab[k]->alpha=1;
			WR->tab[k]->beta=0;
			WR->tab[k]->gamma=0;
			WR->tab[k]->x=BM->Vertex[k][0];
			WR->tab[k]->y=BM->Vertex[k][1];
			WR->tab[k]->z=BM->Vertex[k][2];
		}

		//deuxime sommet
		k=BM->Face[i][1];
		if (!WR->tab[k]) {
			WR->tab[k]=(WaveCo*)malloc(sizeof(WaveCo));
			WR->tab[k]->F0=i;
			WR->tab[k]->alpha=0;
			WR->tab[k]->beta=1;
			WR->tab[k]->gamma=0;
			WR->tab[k]->x=BM->Vertex[k][0];
			WR->tab[k]->y=BM->Vertex[k][1];
			WR->tab[k]->z=BM->Vertex[k][2];
		}

		//troisime sommet
		k=BM->Face[i][2];
		if (!WR->tab[k]) {
			WR->tab[k]=(WaveCo*)malloc(sizeof(WaveCo));
			WR->tab[k]->F0=i;
			WR->tab[k]->alpha=0;
			WR->tab[k]->beta=0;
			WR->tab[k]->gamma=1;
			WR->tab[k]->x=BM->Vertex[k][0];
			WR->tab[k]->y=BM->Vertex[k][1];
			WR->tab[k]->z=BM->Vertex[k][2];
		}
	}
	//fin copie des sommmets de niveau 0

	//Initialisation de la file
	ncroot=m_treecoeff_loc;

	//On rcurse sur chaque root
	for (i=0; i<m_nbsub_zerotree; i++) {
		recDump2WR(ncroot, WR, i%3);
		ncroot=ncroot->m_frere;

	}

	//On supprime les trous dans le tableau
	int cmpt=0;
	for (i=0; i<WR->NbCoeff; i++) {
		if (WR->tab[i]) {
			//recopie du i-eme dns le cmpt-eme
			if (!WR->tab[cmpt])
				WR->tab[cmpt]=(WaveCo*)malloc(sizeof(WaveCo));
			WR->tab[cmpt]->F0=WR->tab[i]->F0;
			WR->tab[cmpt]->alpha=WR->tab[i]->alpha;
			WR->tab[cmpt]->beta=WR->tab[i]->beta;
			WR->tab[cmpt]->gamma=WR->tab[i]->gamma;
			WR->tab[cmpt]->x=WR->tab[i]->x;
			WR->tab[cmpt]->y=WR->tab[i]->y;
			WR->tab[cmpt]->z=WR->tab[i]->z;
			cmpt++;
		}
	}
	WR->NbCoeff=cmpt;

	return 0;
}

void zerotree::recDump2WR(noeudcoeff_loc* coeff, WaveRep* WR, int composante) {

	int indice;		//L'indice dans le tableau
	noeudcoeff_loc* ncroot;

	//le coefficient courant
	indice=coeff->ImplicitIndex(m_depth)+m_nbvertex;
	if (WR->tab[indice]==NULL)
		WR->tab[indice]=(WaveCo*)calloc(1, sizeof(WaveCo));
	WR->tab[indice]->F0=coeff->F0;
	WR->tab[indice]->alpha=coeff->alpha;
	WR->tab[indice]->beta=coeff->beta;
	WR->tab[indice]->gamma=coeff->gamma;
	switch(composante) {
		case 0:
			WR->tab[indice]->x=*(coeff->coeffvalue);
			break;
		case 1:
			WR->tab[indice]->y=*(coeff->coeffvalue);
			break;
		case 2:
			WR->tab[indice]->z=*(coeff->coeffvalue);
	}

	//on rcurse sur les trois ou quatre fils
	ncroot=coeff->m_fils;
	while (ncroot) {
		recDump2WR(ncroot, WR, composante);
		ncroot=ncroot->m_frere;
	}



}

int zerotree::mp4decode(unsigned char* bufferin, int bufsize, Mesh *BM, int NbCoeff) {

	//Comme decodemp4_conf mais retourne le numro du zerotree dcod si transmission partielle
	entry* pentry;
	noeudcoeff_loc *ncroot;
	noeudcoeff_loc *ncx, *ncy, *ncz;
	int indfichier = -1;
	int i;
	int Nbcoeff;
	noeudcoeff_loc **coeffs;

	char indchar[10];
	unsigned short tmp;
	int nbbits;
	unsigned int comp;
	int FT, LT;
	mbitstream *m_i = new mbitstream(bufferin);

	if (m_firsttime) {

		m_firsttime = FALSE;

		//Allocation des coeffs
		coeffs=(noeudcoeff_loc**)calloc(3*NbCoeff, sizeof(noeudcoeff_loc*));

		start_decode_conf_loc(BM); 

	}


	int mid = m_nbsub_zerotree;///2;

	//Class 3D_Wavelet_Mesh_Object (see the spec)

	//WMOL
	m_i->readnbit(tmp, 1);// Temporaire

	m_EC->isBase=(bool)tmp;

	//isInLocalCoordinates
	m_i->readnbit(tmp, 1);
	m_EC->isInLocalCoordinates=tmp;

	//Scale Coeffs
	if (m_EC->isBase && m_EC->hasScaleCoeff)
		threeD_Wavelet_Mesh_Object_Scale_coeff();//See the spec

	
	//Class 3D_Wavelet_Mesh_Object (see the spec)


	//LengthNbBits
	m_i->readnbit(tmp,5);
	m_EC->LengthNbBits=tmp;

	//FBP
	m_i->readnbit(tmp,5);
	m_EC->FBP=tmp;

	//LBP
	m_i->readnbit(tmp,5);
	m_EC->LBP=tmp;

	//bit(1) X;
	m_i->readnbit(tmp,1);
	m_EC->X=tmp;

	//bit(1) Y;
	m_i->readnbit(tmp,1);
	m_EC->Y=tmp;

	//bit(1) Z;
	m_i->readnbit(tmp,1);
	m_EC->Z=tmp;

	//isPartial
	m_i->readnbit(tmp,1);
	m_EC->isPartial=tmp;

	//NbZT
	//m_i->writenbit((unsigned short int)m_nbsub_zerotree,16);

	if (m_EC->isPartial) {
		m_i->readnbit(tmp,16);
		FT = tmp;
		LT = FT;
	} else { // Full transmission
		FT = 0;
		LT = m_nbsub_zerotree/3;
	}




	for (int j=m_EC->LBP; j>=m_EC->FBP; j--) {

		ncroot = m_treecoeff_loc;
		i = 0;
		while (ncroot!=NULL) {


			
			ncz=ncroot;
			ncy=ncroot->m_frere;
			ncx=ncy->m_frere;

			ncroot=ncx;

				

			if (m_EC->X && i>=FT && i<=LT) {
				

				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitX++;			
				} else {


					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 0, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}
				
			}
			ncroot = ncy;

			if (m_EC->Y && i>=FT && i<=LT) {


				
				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitY++;			
				} else {

					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 1, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}

				
			}
			ncroot = ncz; 


			if (m_EC->Z && i>=FT && i<=LT) {
		
			
				//BPLength
				m_i->readnbit(tmp,m_EC->LengthNbBits);

				if (tmp==0) {
					m_tabsub_zerotree_loc[i].m_lastplanbitZ++;			
				} else {

					//Readplane
					nbbits = readplane_conf_loc(i,j,m_i, 2, BM);
					if (nbbits!=tmp)
						//printf("Bug\n");
						Beep(440, 100);
				}

		
			}
			i++;
			ncroot = ncx->m_frere;
		}
		
	}
	//m_i->close();

	//TRACE
/*	m_dumpstream.open("dump",ios::in);
	ncroot=m_treecoeff_loc;
	while (ncroot) {
		dumpquantize(ncroot);
		ncroot=ncroot->m_frere;
*/
		//Dquantif
		ncroot = m_treecoeff_loc;
		while (ncroot!=NULL) {
			if (m_EC->X) {
				unquantize(ncroot, (int)0);
				ncroot = ncroot->m_frere;
			}
			if (m_EC->Y) {
				unquantize(ncroot, (int)1);
				ncroot = ncroot->m_frere;
			}
			if (m_EC->Z) {
				unquantize(ncroot, (int)2);
				ncroot = ncroot->m_frere;
			}
		}
	//}

		if (m_EC->isPartial) 
			return FT;
		else
			return -1;	
}

int zerotree::recMoveCoeffs(noeudcoeff_loc* coeff, int composante, int* baryCoord[4], float* Z, unsigned int& nbcoeff) {

	noeudcoeff_loc* ncroot;


	//Ecriture du coefficient
	baryCoord[0][nbcoeff]=coeff->F0;
	baryCoord[1][nbcoeff]=coeff->alpha;
	baryCoord[2][nbcoeff]=coeff->beta;
	baryCoord[3][nbcoeff]=coeff->gamma;
	switch(composante) {
		case 0:
			Z[nbcoeff]=*(coeff->coeffvalue);
			break;
		case 1:
			Z[nbcoeff]=*(coeff->coeffvalue);
			break;
		case 2:
			Z[nbcoeff]=*(coeff->coeffvalue);
	}
	nbcoeff++;

	//on rcurse sur les trois ou quatre fils
	ncroot=coeff->m_fils;
	while (ncroot) {
		recMoveCoeffs(ncroot, composante, baryCoord, Z, nbcoeff);
		ncroot=ncroot->m_frere;
	}

	return 0;

}

int zerotree::DumpZTHierarchy(char *filename) {

	FILE* f=fopen(filename, "a");
	static int indent=1;
	int i;

	//On parcourt chaque noeud et on rcurse
	noeudcoeff* run = m_treecoeff;
	while (run!= NULL) {
		
		//Affichage et rcursion
		DumpNoeud(f, run, indent);
		run = run->frere;
	}

	fclose(f);

	return 0;
}

int zerotree::DumpNoeud(FILE*f, noeudcoeff* run, int indent) {

	if (run == NULL)
		return 1;

	int i;

	//Indentation
	for (i=0; i<indent; i++) {
		fprintf(f, "  ");
	}

	//Affichage du noeud 
	fprintf(f, "%f\n", *(run->coeffvalue));

	//Rcursion
	if (run->fils!=NULL) {
		DumpNoeud(f, run->fils, indent+1);
		DumpNoeud(f, run->fils->frere, indent+1);
		DumpNoeud(f, run->fils->frere->frere, indent+1);
		if (run->fils!=NULL) 
			DumpNoeud(f, run->fils->frere->frere->frere, indent+1);
	}

	return 0;
}

//------------------------------------ Implmentation sub_zerotree ------------------------------------
sub_zerotree::sub_zerotree() {
	m_lastplanbit = 0;
	m_lastplanbitX = 0;
	m_lastplanbitY = 0;
	m_lastplanbitZ = 0;
	return;
}

sub_zerotree::sub_zerotree(int ztID) {
	m_ztID = ztID;
	m_lastplanbit = 0;
	return;
}

sub_zerotree::~sub_zerotree() {
	return;
}

sub_zerotree_loc::sub_zerotree_loc() {
	m_lastplanbit = 0;
	m_lastplanbitX = 0;
	m_lastplanbitY = 0;
	m_lastplanbitZ = 0;
	return;
}

sub_zerotree_loc::sub_zerotree_loc(int ztID) {
	m_ztID = ztID;
	m_lastplanbit = 0;
	return;
}

sub_zerotree_loc::~sub_zerotree_loc() {
	return;
}